# --- Automatic generated test file  ---
# Generator    : generate_raise_order.py
# Date         : 2016-02-01
# Git revision : 221d808f111c2a4c61e75d16b3b1b5f1d600a6ab

import numpy as np
from splipy import Volume, Surface, Curve, BSplineBasis
from math import sqrt
import unittest


class TestRaiseOrder(unittest.TestCase):
    def test_curve_2D_p3_rational(self):
        controlpoints = np.array([[ -2. ,   1. ,   1. ],
       [ 18. ,   4. ,   1.3],
       [ 35. ,   2. ,   1.1],
       [ 48. ,   1. ,   0.6],
       [ 64. ,   4. ,   0.8],
       [ 79. ,  -2. ,   0.6],
       [ 95. ,  -1. ,   1.3]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.2,  1.8,  2.8,  4. ,  5. ,  5. ,  5. ]))
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 4)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p23_rational(self):
        controlpoints = np.array([[   3. ,   -1. ,    0.8],       [  18. ,   -3. ,    1. ],       [  44. ,    0. ,    0.7],       [  61. ,   -5. ,    1.4],       [  83. ,    0. ,    0.7],       [ 100. ,   -5. ,    1.4],       [   4. ,    9. ,    1.2],       [  22. ,   11. ,    1.3],       [  38. ,   11. ,    0.9],       [  61. ,   17. ,    0.6],       [  79. ,    9. ,    1.5],       [  96. ,   18. ,    1.4],       [  -2. ,   30. ,    1. ],       [  23. ,   24. ,    1. ],       [  43. ,   25. ,    1. ],       [  64. ,   31. ,    1.2],       [  76. ,   30. ,    1.2],       [  97. ,   32. ,    1. ],       [  -3. ,   45. ,    1.3],       [  24. ,   42. ,    0.8],       [  41. ,   38. ,    1.3],       [  60. ,   46. ,    0.6],       [  81. ,   40. ,    0.5],       [ 104. ,   38. ,    0.6],       [  -2. ,   55. ,    1.2],       [  16. ,   59. ,    1.3],       [  38. ,   58. ,    0.8],       [  60. ,   52. ,    1. ],       [  82. ,   55. ,    0.6],       [ 103. ,   60. ,    0.5],       [  -1. ,   70. ,    0.8],       [  17. ,   73. ,    1.4],       [  41. ,   69. ,    1.1],       [  64. ,   70. ,    0.5],       [  83. ,   67. ,    0.6],       [  99. ,   75. ,    0.8],       [  -2. ,   84. ,    1.1],       [  23. ,   85. ,    1.1],       [  39. ,   86. ,    0.9],       [  60. ,   83. ,    0.9],       [  76. ,   90. ,    1. ],       [ 104. ,   89. ,    1.3],       [   1. ,   95. ,    1.4],       [  16. ,   98. ,    1.1],       [  43. ,   97. ,    1.2],       [  55. ,  104. ,    1.2],       [  80. ,   98. ,    0.6],       [ 101. ,  104. ,    1.1]])
        basis0 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.9,  1.6,  3. ,  3.8,  5. ,  5. ]))
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1. ,  1.7,  2.9,  4.1,  4.7,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(2, 3)
        self.assertEqual(surf2.order(direction=0), 4)
        self.assertEqual(surf2.order(direction=1), 6)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p4_rational_C0_periodic(self):
        controlpoints = np.array([[ 101. ,   -4. ,    1.4],
       [  78. ,   66. ,    1.1],
       [  19. ,  100. ,    0.8],
       [ -53. ,   89. ,    1. ],
       [ -97. ,   31. ,    1.2],
       [ -96. ,  -33. ,    1.1],
       [ -47. ,  -90. ,    0.6],
       [  13. , -101. ,    1.5],
       [  75. ,  -61. ,    1. ]])
        basis0 = BSplineBasis(4, np.array([-1. ,  0. ,  0. ,  0. ,  0.8,  2.2,  3.4,  4.3,  5.1,  6. ,  7. ,  7. ,  7. ,  7.8]),0)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 6)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p23_rational_C0_periodic(self):
        controlpoints = np.array([[  58. ,    3. ,    1.1],
       [   3. ,   59. ,    1.4],
       [ -61. ,    0. ,    0.6],
       [  -2. ,  -64. ,    1.3],
       [  66. ,   -5. ,    1.4],
       [  -4. ,   66. ,    0.7],
       [ -72. ,   -2. ,    1.3],
       [  -2. ,  -73. ,    0.7],
       [  77. ,    4. ,    0.7],
       [  -1. ,   76. ,    0.6],
       [ -79. ,    4. ,    1. ],
       [  -5. ,  -80. ,    1. ],
       [  87. ,   -2. ,    0.9],
       [   2. ,   91. ,    0.7],
       [ -89. ,    3. ,    1.2],
       [   2. ,  -87. ,    1.4],
       [ 100. ,   -5. ,    0.7],
       [   0. ,  103. ,    0.6],
       [-102. ,    0. ,    0.8],
       [   0. , -102. ,    1.1]])
        basis0 = BSplineBasis(2, np.array([-1.1,  0. ,  1.2,  2.2,  2.9,  4. ,  5.2]),0)
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.1,  1.8,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 2)
        self.assertEqual(surf2.order(direction=0), 5)
        self.assertEqual(surf2.order(direction=1), 5)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p4_rational_C1_periodic(self):
        controlpoints = np.array([[  96. ,    1. ,    0.6],
       [  78. ,   63. ,    1. ],
       [  31. ,   95. ,    1.4],
       [ -32. ,   98. ,    1.3],
       [ -78. ,   63. ,    0.7],
       [-101. ,   -1. ,    0.8],
       [ -79. ,  -59. ,    1. ],
       [ -27. ,  -96. ,    1.4],
       [  27. ,  -91. ,    1.1],
       [  85. ,  -62. ,    1. ]])
        basis0 = BSplineBasis(4, np.array([ -1.9,  -1.2,   0. ,   0. ,   0.9,   2.4,   3.2,   3.7,   4.7,   5.9,   7.1,   7.8,   9. ,
         9. ,   9.9,  11.4]),1)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 7)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p42_rational_C1_periodic(self):
        controlpoints = np.array([[  64. ,    0. ,    0.8],       [  48. ,   33. ,    1.4],       [  17. ,   56. ,    1. ],       [ -23. ,   61. ,    1.4],       [ -50. ,   36. ,    1.4],       [ -57. ,   -4. ,    0.6],       [ -46. ,  -38. ,    0.9],       [ -23. ,  -57. ,    1. ],       [  22. ,  -54. ,    1.5],       [  45. ,  -34. ,    1.3],       [  72. ,    1. ,    1. ],       [  59. ,   38. ,    0.5],       [  21. ,   61. ,    0.9],       [ -22. ,   68. ,    1.3],       [ -58. ,   37. ,    1. ],       [ -69. ,   -5. ,    1.5],       [ -54. ,  -36. ,    0.9],       [ -21. ,  -65. ,    0.7],       [  16. ,  -65. ,    0.8],       [  53. ,  -41. ,    1.5],       [  76. ,   -4. ,    0.8],       [  64. ,   49. ,    0.6],       [  22. ,   68. ,    0.8],       [ -26. ,   75. ,    0.7],       [ -65. ,   44. ,    1.5],       [ -75. ,    2. ,    0.5],       [ -65. ,  -41. ,    1.3],       [ -25. ,  -76. ,    0.5],       [  20. ,  -74. ,    0.8],       [  65. ,  -47. ,    0.8],       [  80. ,    0. ,    0.6],       [  64. ,   52. ,    0.9],       [  30. ,   74. ,    1.4],       [ -29. ,   79. ,    0.6],       [ -73. ,   52. ,    1. ],       [ -81. ,   -1. ,    0.8],       [ -69. ,  -53. ,    1.4],       [ -30. ,  -81. ,    1.2],       [  29. ,  -81. ,    0.8],       [  66. ,  -52. ,    1.4],       [  95. ,    4. ,    0.7],       [  72. ,   54. ,    1. ],       [  26. ,   88. ,    1.3],       [ -33. ,   86. ,    0.8],       [ -77. ,   54. ,    1. ],       [ -89. ,   -1. ,    0.5],       [ -76. ,  -51. ,    1.3],       [ -32. ,  -90. ,    1.1],       [  31. ,  -89. ,    0.8],       [  72. ,  -57. ,    1.3],       [ 101. ,   -4. ,    0.6],       [  82. ,   61. ,    1.1],       [  27. ,   97. ,    1.1],       [ -28. ,   93. ,    0.9],       [ -85. ,   59. ,    1.4],       [ -98. ,    4. ,    1.3],       [ -84. ,  -62. ,    1. ],       [ -34. ,  -91. ,    1. ],       [  34. ,  -95. ,    1. ],       [  81. ,  -58. ,    1.4]])
        basis0 = BSplineBasis(4, np.array([ -2.2,  -1.2,   0. ,   0. ,   0.8,   2.1,   2.7,   3.9,   5.4,   6.1,   6.8,   7.8,   9. ,
         9. ,   9.8,  11.1]),1)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.7,  1.6,  3. ,  4. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 3)
        self.assertEqual(surf2.order(direction=0), 7)
        self.assertEqual(surf2.order(direction=1), 5)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p3(self):
        controlpoints = np.array([[  0.,  -5.],
       [ 29.,   1.],
       [ 53.,  -1.],
       [ 75.,   1.],
       [ 96.,   3.]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.1,  1.7,  3. ,  3. ,  3. ]))
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 5)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p33(self):
        controlpoints = np.array([[   2.,   -2.],       [  20.,    4.],       [  52.,   -1.],       [  74.,   -4.],       [ 104.,   -4.],       [  -3.,   18.],       [  24.,   15.],       [  52.,   11.],       [  73.,   11.],       [  97.,   14.],       [   4.,   32.],       [  22.,   31.],       [  54.,   23.],       [  75.,   24.],       [  98.,   32.],       [   4.,   46.],       [  29.,   42.],       [  46.,   39.],       [  74.,   47.],       [ 103.,   46.],       [  -3.,   54.],       [  25.,   54.],       [  47.,   54.],       [  73.,   56.],       [ 103.,   55.],       [  -4.,   66.],       [  24.,   71.],       [  53.,   66.],       [  73.,   70.],       [  97.,   71.],       [   1.,   83.],       [  28.,   80.],       [  48.,   85.],       [  77.,   84.],       [  95.,   81.],       [   4.,   96.],       [  29.,  100.],       [  50.,  100.],       [  72.,  104.],       [ 101.,  100.]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.1,  2. ,  3. ,  3. ,  3. ]))
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.3,  2.4,  2.8,  4.2,  5.4,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 2)
        self.assertEqual(surf2.order(direction=0), 5)
        self.assertEqual(surf2.order(direction=1), 5)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p3_C0_periodic(self):
        controlpoints = np.array([[ 102.,    4.],
       [  59.,   80.],
       [ -26.,   97.],
       [ -88.,   42.],
       [ -90.,  -48.],
       [ -20.,  -95.],
       [  64.,  -74.]])
        basis0 = BSplineBasis(3, np.array([-1.1,  0. ,  0. ,  0.8,  2.1,  3.2,  4.4,  4.9,  6. ,  6. ,  6.8]),0)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 4)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p23_C0_periodic(self):
        controlpoints = np.array([[  55.,   -1.],
       [  -3.,   57.],
       [ -63.,    2.],
       [   2.,  -65.],
       [  62.,    0.],
       [   0.,   67.],
       [ -63.,    2.],
       [  -4.,  -69.],
       [  76.,    2.],
       [   2.,   73.],
       [ -70.,    1.],
       [   0.,  -79.],
       [  80.,    4.],
       [   0.,   77.],
       [ -79.,   -4.],
       [   0.,  -79.],
       [  82.,   -5.],
       [  -1.,   82.],
       [ -88.,   -1.],
       [  -2.,  -90.],
       [  95.,    2.],
       [   0.,   96.],
       [ -99.,    0.],
       [  -2.,  -97.],
       [  98.,    3.],
       [   3.,  101.],
       [ -98.,    0.],
       [  -4., -103.]])
        basis0 = BSplineBasis(2, np.array([-0.7,  0. ,  1.2,  2. ,  3.3,  4. ,  5.2]),0)
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.1,  1.6,  2.8,  4.1,  5. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 2)
        self.assertEqual(surf2.order(direction=0), 4)
        self.assertEqual(surf2.order(direction=1), 5)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p3_C1_periodic(self):
        controlpoints = np.array([[  96.,    2.],
       [  69.,   75.],
       [  -3.,  100.],
       [ -66.,   70.],
       [-101.,    3.],
       [ -74.,  -76.],
       [   0.,  -99.],
       [  67.,  -73.]])
        basis0 = BSplineBasis(3, np.array([ -1.9,  -0.7,   0. ,   1.4,   2.1,   2.8,   3.6,   5. ,   6.1,   7.3,   8. ,   9.4,  10.1]),1)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 6)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p32_C1_periodic(self):
        controlpoints = np.array([[  62.,   -4.],       [  44.,   38.],       [  -5.,   59.],       [ -46.,   46.],       [ -60.,   -5.],       [ -41.,  -43.],       [  -4.,  -59.],       [  40.,  -43.],       [  64.,    2.],       [  45.,   46.],       [   2.,   71.],       [ -46.,   46.],       [ -71.,    1.],       [ -46.,  -48.],       [   4.,  -66.],       [  43.,  -47.],       [  73.,    2.],       [  57.,   51.],       [  -2.,   79.],       [ -56.,   51.],       [ -73.,   -1.],       [ -52.,  -54.],       [  -3.,  -75.],       [  51.,  -57.],       [  81.,   -2.],       [  62.,   60.],       [  -3.,   87.],       [ -63.,   55.],       [ -83.,   -1.],       [ -65.,  -64.],       [  -1.,  -80.],       [  62.,  -64.],       [  96.,   -1.],       [  61.,   69.],       [  -5.,   91.],       [ -62.,   64.],       [ -94.,   -2.],       [ -67.,  -61.],       [  -3.,  -89.],       [  69.,  -70.],       [  98.,   -5.],       [  69.,   74.],       [  -1.,  104.],       [ -76.,   70.],       [-105.,   -3.],       [ -71.,  -70.],       [   1., -100.],       [  72.,  -68.]])
        basis0 = BSplineBasis(3, np.array([ -1.7,  -1.2,   0. ,   1. ,   2.3,   3. ,   3.7,   4.7,   6.3,   6.8,   8. ,   9. ,  10.3]),1)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.6,  1.6,  2.6,  4.4,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 1)
        self.assertEqual(surf2.order(direction=0), 5)
        self.assertEqual(surf2.order(direction=1), 3)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p3_rational(self):
        controlpoints = np.array([[   2. ,    3. ,    1. ,    1.2],
       [  38. ,    4. ,   -5. ,    0.6],
       [  62. ,    0. ,   -5. ,    0.9],
       [ 102. ,   -1. ,    3. ,    0.6]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  1.2,  2. ,  2. ,  2. ]))
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 6)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p42_rational(self):
        controlpoints = np.array([[   0. ,    2. ,    3. ,    1.1],       [  17. ,    0. ,   -1. ,    1.4],       [  31. ,    1. ,    0. ,    1.3],       [  45. ,    0. ,    4. ,    1.1],       [  58. ,   -2. ,    3. ,    0.6],       [  69. ,   -1. ,   -4. ,    0.9],       [  87. ,    3. ,   -3. ,    0.8],       [  97. ,    1. ,    2. ,    0.8],       [  -2. ,   20. ,    1. ,    1.2],       [  18. ,   23. ,   -2. ,    0.7],       [  30. ,   16. ,   -3. ,    0.5],       [  46. ,   22. ,   -1. ,    1.5],       [  54. ,   16. ,    4. ,    1.1],       [  75. ,   20. ,    2. ,    1.4],       [  89. ,   19. ,   -3. ,    0.7],       [  98. ,   22. ,   -3. ,    1.4],       [  -1. ,   41. ,   -4. ,    1. ],       [  14. ,   39. ,    4. ,    1.3],       [  23. ,   38. ,    3. ,    1.2],       [  47. ,   43. ,    3. ,    1.4],       [  52. ,   40. ,   -1. ,    1.2],       [  73. ,   44. ,    0. ,    1. ],       [  86. ,   35. ,   -3. ,    0.6],       [ 102. ,   43. ,    2. ,    0.7],       [  -1. ,   59. ,   -4. ,    0.6],       [  10. ,   59. ,    1. ,    1.5],       [  25. ,   61. ,   -5. ,    0.7],       [  37. ,   58. ,    0. ,    0.6],       [  59. ,   62. ,   -4. ,    1.5],       [  71. ,   58. ,    0. ,    1. ],       [  90. ,   60. ,   -5. ,    0.9],       [  96. ,   58. ,   -1. ,    0.6],       [   2. ,   75. ,    2. ,    0.6],       [  12. ,   77. ,   -1. ,    1. ],       [  24. ,   83. ,   -3. ,    1.1],       [  41. ,   80. ,   -5. ,    0.9],       [  58. ,   79. ,   -3. ,    1.4],       [  69. ,   76. ,   -5. ,    1.2],       [  82. ,   81. ,   -3. ,    1.4],       [  99. ,   83. ,   -4. ,    1.4],       [   0. ,   99. ,   -1. ,    0.8],       [  14. ,   99. ,    4. ,    0.8],       [  25. ,   97. ,    3. ,    1.4],       [  41. ,  102. ,   -3. ,    0.9],       [  53. ,   99. ,    2. ,    1.1],       [  73. ,   96. ,   -5. ,    1. ],       [  87. ,  102. ,   -4. ,    1.2],       [  98. ,   97. ,   -2. ,    1. ]])
        basis0 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.1,  1.7,  3.2,  3.6,  5. ,  5. ,  5. ,  5. ]))
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.9,  1.8,  3.2,  3.6,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 1)
        self.assertEqual(surf2.order(direction=0), 7)
        self.assertEqual(surf2.order(direction=1), 3)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p444_rational(self):
        controlpoints = np.array([[  -3. ,   -5. ,    3. ,    0.8],       [  15. ,    0. ,   -5. ,    1.1],       [  42. ,    3. ,    4. ,    1.1],       [  61. ,   -1. ,    4. ,    0.9],       [  83. ,   -2. ,    3. ,    1. ],       [  95. ,    2. ,   -2. ,    0.6],       [  -3. ,   26. ,    4. ,    1.1],       [  19. ,   29. ,   -2. ,    1.5],       [  41. ,   22. ,   -4. ,    0.7],       [  56. ,   20. ,   -4. ,    0.8],       [  76. ,   26. ,   -3. ,    0.7],       [ 100. ,   27. ,    4. ,    1.4],       [   1. ,   54. ,   -4. ,    1.4],       [  23. ,   53. ,    4. ,    0.9],       [  37. ,   48. ,   -2. ,    1.2],       [  64. ,   46. ,   -5. ,    0.6],       [  78. ,   45. ,   -2. ,    1. ],       [ 104. ,   48. ,    2. ,    1.1],       [  -2. ,   70. ,   -3. ,    0.8],       [  22. ,   74. ,    0. ,    0.9],       [  37. ,   74. ,   -5. ,    1. ],       [  63. ,   79. ,   -4. ,    1.2],       [  75. ,   78. ,   -4. ,    1.1],       [ 104. ,   70. ,    0. ,    1.4],       [  -4. ,  101. ,    4. ,    1. ],       [  22. ,  100. ,   -5. ,    1. ],       [  43. ,   97. ,   -4. ,    0.9],       [  61. ,   96. ,    3. ,    1.5],       [  84. ,  102. ,   -2. ,    1. ],       [ 100. ,   97. ,   -3. ,    1. ],       [  -1. ,    1. ,   18. ,    1.3],       [  22. ,   -4. ,   18. ,    1.4],       [  42. ,   -5. ,   15. ,    0.8],       [  55. ,    1. ,   12. ,    0.7],       [  79. ,   -5. ,   14. ,    1.4],       [ 103. ,    1. ,   13. ,    1. ],       [  -5. ,   27. ,   21. ,    1.1],       [  19. ,   28. ,   12. ,    1.4],       [  37. ,   22. ,   15. ,    0.8],       [  64. ,   20. ,   12. ,    1.1],       [  83. ,   23. ,   15. ,    1.4],       [ 104. ,   21. ,   19. ,    1.3],       [   4. ,   45. ,   20. ,    1.5],       [  16. ,   49. ,   13. ,    1. ],       [  41. ,   50. ,   14. ,    0.6],       [  63. ,   45. ,   20. ,    1.3],       [  75. ,   46. ,   14. ,    0.6],       [ 100. ,   48. ,   20. ,    0.8],       [  -3. ,   77. ,   13. ,    1. ],       [  22. ,   73. ,   15. ,    1.5],       [  38. ,   75. ,   17. ,    0.9],       [  63. ,   72. ,   17. ,    0.8],       [  76. ,   71. ,   12. ,    1.3],       [ 103. ,   71. ,   19. ,    0.6],       [   2. ,  102. ,   18. ,    0.7],       [  24. ,  104. ,   12. ,    1.3],       [  35. ,   99. ,   20. ,    0.9],       [  61. ,  100. ,   12. ,    0.5],       [  84. ,   98. ,   17. ,    0.8],       [ 102. ,   99. ,   14. ,    0.7],       [   0. ,    3. ,   36. ,    0.8],       [  20. ,   -2. ,   35. ,    1.4],       [  39. ,    2. ,   36. ,    0.6],       [  59. ,    1. ,   36. ,    1.1],       [  82. ,    2. ,   36. ,    0.8],       [ 103. ,    3. ,   31. ,    1. ],       [   4. ,   21. ,   36. ,    1.4],       [  17. ,   22. ,   32. ,    1.2],       [  39. ,   20. ,   30. ,    1. ],       [  59. ,   28. ,   30. ,    1.3],       [  82. ,   23. ,   32. ,    1.4],       [  95. ,   22. ,   32. ,    1.4],       [   1. ,   49. ,   28. ,    1.3],       [  15. ,   51. ,   34. ,    0.6],       [  38. ,   51. ,   34. ,    1.3],       [  64. ,   47. ,   30. ,    1.1],       [  75. ,   46. ,   29. ,    0.7],       [  98. ,   46. ,   32. ,    0.6],       [   1. ,   76. ,   37. ,    1.1],       [  18. ,   71. ,   30. ,    1.5],       [  39. ,   75. ,   28. ,    0.5],       [  64. ,   75. ,   32. ,    1.3],       [  79. ,   79. ,   32. ,    1. ],       [  98. ,   79. ,   30. ,    0.6],       [  -3. ,   97. ,   36. ,    0.9],       [  23. ,   96. ,   33. ,    1.4],       [  39. ,  104. ,   33. ,    1.1],       [  55. ,  101. ,   28. ,    1.2],       [  82. ,  104. ,   33. ,    1.4],       [ 100. ,  103. ,   28. ,    1.2],       [  -1. ,    0. ,   49. ,    0.8],       [  24. ,    2. ,   48. ,    1.2],       [  44. ,    4. ,   53. ,    1. ],       [  55. ,    4. ,   50. ,    0.7],       [  82. ,    3. ,   52. ,    0.6],       [  98. ,    0. ,   53. ,    1.2],       [   3. ,   22. ,   49. ,    1. ],       [  18. ,   25. ,   49. ,    0.8],       [  40. ,   29. ,   49. ,    0.9],       [  58. ,   29. ,   50. ,    1.5],       [  78. ,   20. ,   49. ,    1.3],       [  95. ,   28. ,   52. ,    1.3],       [  -2. ,   52. ,   53. ,    0.9],       [  20. ,   48. ,   52. ,    0.9],       [  40. ,   46. ,   53. ,    1. ],       [  64. ,   51. ,   53. ,    1.2],       [  81. ,   50. ,   51. ,    1.1],       [  97. ,   54. ,   49. ,    0.9],       [  -3. ,   74. ,   45. ,    0.5],       [  16. ,   73. ,   49. ,    1.2],       [  36. ,   72. ,   46. ,    1. ],       [  55. ,   70. ,   45. ,    1.2],       [  76. ,   73. ,   53. ,    1. ],       [  99. ,   70. ,   50. ,    1.2],       [   4. ,  102. ,   47. ,    0.8],       [  15. ,  101. ,   51. ,    0.9],       [  39. ,   98. ,   50. ,    1.4],       [  60. ,  102. ,   48. ,    0.7],       [  83. ,   97. ,   51. ,    0.6],       [  98. ,   96. ,   47. ,    1.2],       [  -3. ,    3. ,   69. ,    0.7],       [  22. ,    0. ,   63. ,    0.7],       [  38. ,    1. ,   67. ,    1.2],       [  59. ,    0. ,   63. ,    0.9],       [  79. ,   -5. ,   66. ,    1.5],       [  96. ,   -3. ,   63. ,    0.6],       [   1. ,   21. ,   62. ,    1. ],       [  17. ,   26. ,   71. ,    0.7],       [  42. ,   23. ,   62. ,    0.7],       [  55. ,   24. ,   66. ,    1.4],       [  82. ,   28. ,   67. ,    0.9],       [ 102. ,   25. ,   70. ,    0.6],       [   0. ,   52. ,   64. ,    1.1],       [  21. ,   46. ,   65. ,    0.9],       [  38. ,   53. ,   65. ,    0.8],       [  55. ,   47. ,   64. ,    1. ],       [  76. ,   48. ,   65. ,    1. ],       [  98. ,   51. ,   70. ,    0.6],       [   3. ,   71. ,   71. ,    1.3],       [  15. ,   77. ,   69. ,    0.9],       [  35. ,   71. ,   64. ,    1.4],       [  63. ,   72. ,   65. ,    0.7],       [  76. ,   77. ,   67. ,    0.8],       [ 103. ,   76. ,   62. ,    1.4],       [  -5. ,   99. ,   63. ,    0.9],       [  18. ,   99. ,   69. ,    0.6],       [  42. ,   98. ,   71. ,    0.8],       [  64. ,   99. ,   63. ,    1.1],       [  82. ,   96. ,   65. ,    0.9],       [ 103. ,   99. ,   62. ,    0.9],       [   1. ,   -2. ,   84. ,    0.9],       [  24. ,    4. ,   80. ,    1.1],       [  36. ,   -1. ,   86. ,    0.8],       [  61. ,   -1. ,   86. ,    1.2],       [  83. ,    0. ,   79. ,    1.1],       [  96. ,    0. ,   81. ,    0.6],       [   4. ,   26. ,   87. ,    0.6],       [  20. ,   25. ,   87. ,    0.5],       [  43. ,   24. ,   85. ,    0.9],       [  56. ,   22. ,   85. ,    1.3],       [  77. ,   26. ,   82. ,    0.6],       [  96. ,   26. ,   84. ,    1.3],       [  -4. ,   50. ,   86. ,    1.4],       [  16. ,   46. ,   84. ,    0.7],       [  38. ,   48. ,   78. ,    0.9],       [  62. ,   48. ,   86. ,    0.8],       [  83. ,   48. ,   87. ,    0.6],       [  96. ,   51. ,   81. ,    1. ],       [   1. ,   70. ,   86. ,    1.3],       [  18. ,   72. ,   84. ,    1.2],       [  44. ,   79. ,   83. ,    0.5],       [  62. ,   75. ,   84. ,    1.5],       [  76. ,   77. ,   85. ,    1.3],       [  99. ,   71. ,   78. ,    1.3],       [   4. ,  104. ,   84. ,    0.9],       [  20. ,  103. ,   85. ,    0.6],       [  39. ,  102. ,   82. ,    1.3],       [  58. ,   98. ,   85. ,    0.8],       [  75. ,   96. ,   87. ,    1.1],       [  96. ,   97. ,   85. ,    1.3],       [  -3. ,    1. ,  100. ,    1.3],       [  19. ,    2. ,  104. ,    1.2],       [  41. ,    4. ,   98. ,    1.3],       [  64. ,    2. ,  100. ,    1.1],       [  79. ,   -4. ,   98. ,    0.8],       [ 104. ,    2. ,  104. ,    1. ],       [  -2. ,   23. ,   95. ,    1.1],       [  19. ,   28. ,  103. ,    0.9],       [  42. ,   28. ,   99. ,    1. ],       [  56. ,   22. ,   97. ,    1. ],       [  83. ,   24. ,   97. ,    0.8],       [ 104. ,   23. ,   96. ,    1.3],       [   2. ,   48. ,  100. ,    0.9],       [  17. ,   46. ,  104. ,    1.1],       [  38. ,   46. ,   98. ,    1.4],       [  64. ,   49. ,  100. ,    0.5],       [  75. ,   45. ,  104. ,    1.1],       [ 103. ,   54. ,  104. ,    1. ],       [  -3. ,   79. ,  100. ,    0.7],       [  23. ,   71. ,  102. ,    0.6],       [  44. ,   76. ,  100. ,    0.9],       [  58. ,   77. ,   99. ,    1.4],       [  79. ,   78. ,  104. ,    0.9],       [  98. ,   79. ,  100. ,    1.3],       [   3. ,   99. ,  104. ,    0.9],       [  17. ,   96. ,   98. ,    0.9],       [  42. ,  101. ,   97. ,    0.8],       [  56. ,   97. ,  102. ,    1.2],       [  75. ,  100. ,   97. ,    1. ],       [ 100. ,  101. ,   98. ,    1.2]])
        basis0 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1. ,  2.4,  3. ,  3. ,  3. ,  3. ]))
        basis1 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.1,  2. ,  2. ,  2. ,  2. ]))
        basis2 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  0.6,  1.7,  3.3,  4. ,  4. ,  4. ,  4. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(1, 1, 2)
        self.assertEqual(vol2.order(direction=0), 5)
        self.assertEqual(vol2.order(direction=1), 5)
        self.assertEqual(vol2.order(direction=2), 6)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p3_rational_C0_periodic(self):
        controlpoints = np.array([[ 104. ,    0. ,    2. ,    1.4],
       [  57. ,   75. ,    1. ,    1.3],
       [ -23. ,   99. ,    1. ,    0.6],
       [ -91. ,   42. ,    4. ,    1. ],
       [ -91. ,  -40. ,    0. ,    0.9],
       [ -23. , -101. ,    2. ,    0.6],
       [  59. ,  -76. ,    1. ,    0.9]])
        basis0 = BSplineBasis(3, np.array([-0.7,  0. ,  0. ,  1.4,  2.2,  3.1,  4.1,  5.3,  6. ,  6. ,  7.4]),0)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 5)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p42_rational_C0_periodic(self):
        controlpoints = np.array([[  61. ,   -2. ,    3. ,    1.2],       [  46. ,   42. ,    4. ,    1.2],       [   5. ,   62. ,   -3. ,    1.3],       [ -32. ,   50. ,    3. ,    1. ],       [ -56. ,   22. ,   -2. ,    1. ],       [ -52. ,  -17. ,   -5. ,    1.4],       [ -26. ,  -55. ,    0. ,    1.3],       [   6. ,  -64. ,    4. ,    0.9],       [  44. ,  -38. ,   -1. ,    1.5],       [  66. ,    3. ,   -1. ,    1. ],       [  55. ,   48. ,   -3. ,    1. ],       [  14. ,   67. ,   -3. ,    1.1],       [ -31. ,   53. ,   -4. ,    0.9],       [ -60. ,   24. ,   -4. ,    0.8],       [ -62. ,  -21. ,    2. ,    1. ],       [ -36. ,  -60. ,   -3. ,    1.2],       [   8. ,  -65. ,    1. ,    0.5],       [  55. ,  -42. ,   -4. ,    1. ],       [  78. ,    0. ,   -2. ,    0.9],       [  59. ,   45. ,    1. ,    0.9],       [  15. ,   78. ,    3. ,    0.7],       [ -43. ,   67. ,   -3. ,    0.9],       [ -73. ,   30. ,    2. ,    0.9],       [ -72. ,  -24. ,   -4. ,    0.9],       [ -43. ,  -69. ,   -3. ,    1.4],       [  17. ,  -71. ,   -2. ,    1.1],       [  56. ,  -52. ,   -2. ,    1. ],       [  79. ,   -5. ,   -3. ,    0.9],       [  62. ,   51. ,    4. ,    0.6],       [  12. ,   79. ,    1. ,    0.6],       [ -42. ,   72. ,    0. ,    1.5],       [ -82. ,   24. ,    3. ,    0.8],       [ -77. ,  -27. ,    1. ,    1.4],       [ -47. ,  -76. ,    3. ,    1.2],       [  15. ,  -78. ,   -5. ,    0.9],       [  64. ,  -52. ,    1. ,    1.1],       [  96. ,   -1. ,    3. ,    1.4],       [  75. ,   57. ,   -3. ,    1.2],       [  19. ,   92. ,    4. ,    1. ],       [ -51. ,   80. ,    3. ,    1.5],       [ -89. ,   32. ,    1. ,    1. ],       [ -84. ,  -35. ,   -1. ,    0.8],       [ -46. ,  -76. ,    1. ,    1.4],       [  15. ,  -90. ,   -5. ,    1.2],       [  67. ,  -55. ,    3. ,    0.8],       [  96. ,    0. ,   -1. ,    1.1],       [  79. ,   63. ,    2. ,    1.5],       [  15. ,  102. ,    4. ,    0.8],       [ -49. ,   88. ,   -1. ,    1.3],       [ -95. ,   36. ,    4. ,    1. ],       [ -98. ,  -30. ,    1. ,    1.5],       [ -53. ,  -90. ,    1. ,    0.7],       [  19. , -104. ,   -1. ,    0.7],       [  73. ,  -68. ,   -5. ,    0.9]])
        basis0 = BSplineBasis(4, np.array([-0.8,  0. ,  0. ,  0. ,  0.8,  2. ,  2.8,  4.1,  5.1,  6.2,  7. ,  7. ,  7. ,  7.8]),0)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  1.2,  1.7,  2.9,  4. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(2, 2)
        self.assertEqual(surf2.order(direction=0), 6)
        self.assertEqual(surf2.order(direction=1), 4)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p242_rational_C0_periodic(self):
        controlpoints = np.array([[  53. ,    2. ,    0. ,    1.2],       [   1. ,   45. ,   -5. ,    1.1],       [ -49. ,   -1. ,   -3. ,    1.1],       [  -1. ,  -52. ,    4. ,    0.9],       [  60. ,   -1. ,    1. ,    0.8],       [   4. ,   57. ,   -4. ,    0.7],       [ -61. ,    3. ,   -1. ,    1. ],       [  -2. ,  -56. ,    3. ,    1.2],       [  66. ,    4. ,   -1. ,    0.6],       [   4. ,   63. ,    0. ,    0.7],       [ -60. ,   -4. ,   -3. ,    0.7],       [   3. ,  -65. ,    1. ,    1. ],       [  70. ,   -5. ,   -2. ,    0.8],       [  -4. ,   72. ,   -1. ,    1.4],       [ -66. ,    2. ,    1. ,    0.5],       [  -2. ,  -70. ,   -3. ,    0.7],       [  74. ,    3. ,    1. ,    1.5],       [   3. ,   71. ,    2. ,    1. ],       [ -74. ,    2. ,   -3. ,    1.3],       [  -1. ,  -72. ,   -4. ,    0.7],       [  85. ,    4. ,    0. ,    0.6],       [   1. ,   81. ,    4. ,    0.6],       [ -77. ,   -3. ,    1. ,    0.9],       [   0. ,  -82. ,    3. ,    1.2],       [  90. ,    1. ,    2. ,    1.2],       [  -2. ,   83. ,   -1. ,    0.7],       [ -89. ,    3. ,   -2. ,    0.7],       [  -2. ,  -88. ,   -5. ,    1.2],       [  94. ,   -5. ,    2. ,    0.7],       [  -2. ,   97. ,    4. ,    1. ],       [ -90. ,    2. ,    0. ,    0.9],       [  -2. ,  -98. ,   -1. ,    1.2],       [  99. ,    4. ,   -3. ,    0.8],       [  -5. ,   98. ,    4. ,    0.5],       [ -97. ,   -1. ,   -4. ,    1.3],       [  -2. ,  -97. ,    3. ,    1.5],       [  52. ,   -2. ,   17. ,    1.4],       [   3. ,   49. ,    9. ,    0.8],       [ -49. ,   -3. ,   14. ,    0.6],       [   3. ,  -52. ,   11. ,    0.7],       [  58. ,    3. ,   10. ,    0.7],       [   1. ,   57. ,   10. ,    1.2],       [ -56. ,   -4. ,    8. ,    0.5],       [  -4. ,  -55. ,   10. ,    0.5],       [  63. ,    3. ,   17. ,    0.9],       [  -2. ,   65. ,   14. ,    1.2],       [ -63. ,    1. ,   18. ,    1.4],       [   3. ,  -60. ,   15. ,    0.6],       [  70. ,   -3. ,    8. ,    0.7],       [  -4. ,   72. ,   14. ,    1. ],       [ -69. ,   -1. ,   17. ,    1.3],       [   4. ,  -68. ,   17. ,    0.8],       [  76. ,   -1. ,   13. ,    1. ],       [  -4. ,   72. ,    9. ,    1. ],       [ -72. ,    1. ,   15. ,    0.7],       [   1. ,  -79. ,   13. ,    0.8],       [  78. ,    1. ,   13. ,    1. ],       [  -2. ,   79. ,   15. ,    0.8],       [ -81. ,    3. ,    9. ,    0.9],       [  -2. ,  -83. ,   13. ,    1. ],       [  85. ,   -4. ,   16. ,    0.7],       [   4. ,   84. ,   17. ,    1.1],       [ -92. ,   -3. ,   10. ,    0.7],       [  -2. ,  -91. ,   14. ,    0.9],       [  90. ,    1. ,   15. ,    1. ],       [   2. ,   89. ,   15. ,    1.5],       [ -97. ,   -3. ,   13. ,    1.4],       [   3. ,  -97. ,   14. ,    1.4],       [ 101. ,    3. ,   18. ,    1.1],       [   2. ,  100. ,   13. ,    0.7],       [-100. ,    4. ,   10. ,    1.5],       [  -1. , -100. ,    9. ,    0.7],       [  50. ,   -4. ,   26. ,    1.2],       [   1. ,   54. ,   29. ,    1. ],       [ -53. ,   -1. ,   26. ,    1.5],       [   1. ,  -47. ,   28. ,    1.1],       [  60. ,   -2. ,   22. ,    1.5],       [  -5. ,   59. ,   24. ,    1.2],       [ -59. ,    0. ,   25. ,    1.4],       [  -4. ,  -54. ,   27. ,    1.1],       [  60. ,   -5. ,   28. ,    0.5],       [   0. ,   61. ,   27. ,    0.8],       [ -58. ,   -4. ,   27. ,    1. ],       [   0. ,  -68. ,   24. ,    1. ],       [  70. ,   -4. ,   25. ,    1.3],       [  -5. ,   73. ,   30. ,    0.9],       [ -72. ,    0. ,   24. ,    1.2],       [  -4. ,  -70. ,   31. ,    0.8],       [  72. ,   -2. ,   27. ,    1.1],       [  -1. ,   70. ,   24. ,    1.4],       [ -72. ,    4. ,   27. ,    1.2],       [   3. ,  -73. ,   25. ,    1.3],       [  81. ,   -3. ,   27. ,    0.7],       [   2. ,   81. ,   22. ,    0.6],       [ -84. ,    0. ,   30. ,    0.6],       [  -1. ,  -83. ,   31. ,    0.9],       [  84. ,   -1. ,   25. ,    0.8],       [  -5. ,   89. ,   22. ,    1. ],       [ -84. ,   -5. ,   31. ,    1. ],       [  -2. ,  -87. ,   28. ,    1.3],       [  94. ,   -2. ,   23. ,    1.2],       [  -4. ,   89. ,   30. ,    0.7],       [ -90. ,   -1. ,   27. ,    1.1],       [   2. ,  -93. ,   29. ,    0.6],       [ 104. ,   -1. ,   24. ,    1.3],       [  -3. ,   96. ,   31. ,    0.6],       [ -96. ,   -2. ,   24. ,    1.2],       [   0. ,  -99. ,   27. ,    0.6],       [  51. ,    2. ,   42. ,    1. ],       [  -4. ,   51. ,   41. ,    1.2],       [ -46. ,   -5. ,   41. ,    0.8],       [  -1. ,  -48. ,   37. ,    0.7],       [  53. ,    0. ,   38. ,    1. ],       [  -1. ,   55. ,   44. ,    1.2],       [ -53. ,   -3. ,   42. ,    0.7],       [  -5. ,  -52. ,   44. ,    0.6],       [  63. ,   -1. ,   43. ,    1. ],       [   3. ,   59. ,   37. ,    1.1],       [ -59. ,    3. ,   39. ,    1.1],       [  -5. ,  -67. ,   39. ,    0.5],       [  65. ,   -3. ,   42. ,    1. ],       [   1. ,   72. ,   42. ,    0.8],       [ -70. ,   -3. ,   37. ,    0.9],       [  -5. ,  -72. ,   43. ,    1. ],       [  73. ,    1. ,   43. ,    1. ],       [   2. ,   70. ,   44. ,    0.7],       [ -76. ,    4. ,   37. ,    0.8],       [  -4. ,  -78. ,   41. ,    1.4],       [  76. ,    4. ,   38. ,    1. ],       [  -3. ,   84. ,   40. ,    1.1],       [ -77. ,   -1. ,   35. ,    1. ],       [  -4. ,  -82. ,   38. ,    0.5],       [  91. ,    4. ,   36. ,    0.8],       [   0. ,   90. ,   37. ,    1.1],       [ -87. ,   -4. ,   40. ,    1. ],       [  -1. ,  -88. ,   43. ,    1.3],       [  93. ,   -3. ,   39. ,    1.1],       [   3. ,   89. ,   41. ,    1. ],       [ -98. ,    1. ,   39. ,    1.4],       [  -1. ,  -93. ,   35. ,    1.5],       [ 104. ,   -3. ,   42. ,    0.8],       [  -1. ,  104. ,   39. ,    1.3],       [ -99. ,    1. ,   43. ,    1.1],       [  -4. ,  -99. ,   43. ,    0.7]])
        basis0 = BSplineBasis(2, np.array([-0.7,  0. ,  1. ,  2.1,  3.3,  4. ,  5. ]),0)
        basis1 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.4,  2.1,  2.9,  4.4,  5. ,  6. ,  6. ,  6. ,  6. ]))
        basis2 = BSplineBasis(2, np.array([ 0. ,  0. ,  1. ,  1.8,  3. ,  3. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(3, 2, 1)
        self.assertEqual(vol2.order(direction=0), 5)
        self.assertEqual(vol2.order(direction=1), 6)
        self.assertEqual(vol2.order(direction=2), 3)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p4_rational_C1_periodic(self):
        controlpoints = np.array([[ 101. ,   -1. ,   -5. ,    1.3],
       [  81. ,   53. ,    1. ,    1.3],
       [  30. ,   98. ,   -5. ,    0.6],
       [ -33. ,   95. ,    4. ,    0.5],
       [ -79. ,   60. ,   -1. ,    1.4],
       [ -99. ,    3. ,    2. ,    0.8],
       [ -79. ,  -59. ,    0. ,    0.6],
       [ -28. ,  -96. ,   -1. ,    1. ],
       [  27. ,  -94. ,   -4. ,    1.1],
       [  79. ,  -56. ,    4. ,    1.3]])
        basis0 = BSplineBasis(4, np.array([ -2.2,  -0.8,   0. ,   0. ,   0.9,   1.6,   2.9,   3.7,   4.9,   6.2,   6.8,   8.2,   9. ,
         9. ,   9.9,  10.6]),1)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 7)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p44_rational_C1_periodic(self):
        controlpoints = np.array([[  58. ,   -4. ,    3. ,    0.7],       [  45. ,   37. ,    4. ,    1.5],       [  13. ,   59. ,   -1. ,    1.2],       [ -16. ,   59. ,   -2. ,    1.1],       [ -49. ,   33. ,    3. ,    1.2],       [ -59. ,   -5. ,    3. ,    0.9],       [ -49. ,  -35. ,    1. ,    1.2],       [ -21. ,  -53. ,    2. ,    0.6],       [  15. ,  -55. ,   -2. ,    1.2],       [  50. ,  -36. ,   -4. ,    1. ],       [  66. ,   -4. ,   -4. ,    0.9],       [  55. ,   42. ,    3. ,    1.2],       [  23. ,   66. ,   -2. ,    1.2],       [ -26. ,   62. ,    0. ,    0.9],       [ -55. ,   42. ,    1. ,    1. ],       [ -69. ,    1. ,   -2. ,    0.7],       [ -55. ,  -36. ,   -5. ,    1.4],       [ -20. ,  -64. ,    0. ,    0.6],       [  19. ,  -64. ,   -4. ,    0.8],       [  54. ,  -44. ,   -2. ,    1.3],       [  75. ,    4. ,   -2. ,    1.4],       [  63. ,   47. ,    2. ,    0.9],       [  22. ,   67. ,    2. ,    1.3],       [ -28. ,   68. ,   -2. ,    1.1],       [ -65. ,   49. ,   -1. ,    1.1],       [ -74. ,    0. ,    0. ,    0.9],       [ -67. ,  -46. ,   -3. ,    1.2],       [ -21. ,  -69. ,    2. ,    1.1],       [  28. ,  -71. ,   -2. ,    1.5],       [  62. ,  -46. ,   -1. ,    0.5],       [  88. ,   -3. ,    2. ,    0.9],       [  72. ,   53. ,   -4. ,    1.3],       [  28. ,   80. ,   -2. ,    1.3],       [ -30. ,   84. ,    0. ,    1.5],       [ -65. ,   52. ,    4. ,    0.8],       [ -80. ,   -1. ,    4. ,    0.9],       [ -72. ,  -53. ,    3. ,    0.9],       [ -27. ,  -77. ,   -2. ,    1. ],       [  30. ,  -84. ,    3. ,    1.2],       [  69. ,  -49. ,    0. ,    0.8],       [  88. ,   -3. ,   -3. ,    1.1],       [  69. ,   57. ,    2. ,    1.5],       [  30. ,   89. ,   -5. ,    1. ],       [ -29. ,   90. ,   -5. ,    1.3],       [ -74. ,   57. ,   -1. ,    1.3],       [ -97. ,   -2. ,   -5. ,    1. ],       [ -74. ,  -52. ,   -4. ,    1.4],       [ -29. ,  -87. ,   -3. ,    1.2],       [  32. ,  -93. ,    1. ,    0.7],       [  78. ,  -56. ,   -3. ,    1.2],       [ 104. ,   -2. ,    0. ,    1.5],       [  77. ,   62. ,   -4. ,    0.9],       [  33. ,   94. ,   -4. ,    1. ],       [ -31. ,   92. ,    3. ,    1. ],       [ -84. ,   59. ,   -3. ,    1.4],       [-102. ,    0. ,    2. ,    1. ],       [ -86. ,  -57. ,    4. ,    0.7],       [ -29. ,  -94. ,   -1. ,    1.1],       [  31. , -100. ,   -4. ,    0.9],       [  84. ,  -54. ,    4. ,    0.6]])
        basis0 = BSplineBasis(4, np.array([ -2.2,  -1.3,   0. ,   0. ,   1.3,   1.9,   2.8,   4. ,   5.2,   6.1,   6.8,   7.7,   9. ,
         9. ,  10.3,  10.9]),1)
        basis1 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.3,  2.3,  3. ,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(2, 3)
        self.assertEqual(surf2.order(direction=0), 6)
        self.assertEqual(surf2.order(direction=1), 7)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p42_rational_C2_periodic(self):
        controlpoints = np.array([[  57. ,    3. ,   -5. ,    1.2],       [  56. ,   25. ,    2. ,    0.6],       [  27. ,   48. ,   -4. ,    1.5],       [   2. ,   60. ,    3. ,    0.9],       [ -33. ,   48. ,   -5. ,    0.9],       [ -54. ,   29. ,    0. ,    0.9],       [ -61. ,    4. ,   -3. ,    1.3],       [ -52. ,  -28. ,   -1. ,    1.1],       [ -32. ,  -50. ,    0. ,    0.9],       [   3. ,  -59. ,    4. ,    0.7],       [  27. ,  -53. ,    0. ,    1.1],       [  50. ,  -31. ,   -1. ,    0.9],       [  69. ,    4. ,    4. ,    1.3],       [  57. ,   39. ,   -4. ,    0.8],       [  38. ,   61. ,    2. ,    1.4],       [  -1. ,   69. ,    2. ,    0.9],       [ -39. ,   56. ,    3. ,    1.4],       [ -66. ,   34. ,   -1. ,    1.2],       [ -70. ,    4. ,   -2. ,    1. ],       [ -65. ,  -40. ,    3. ,    1. ],       [ -32. ,  -56. ,    3. ,    1. ],       [  -4. ,  -68. ,   -3. ,    1.4],       [  33. ,  -64. ,    0. ,    0.6],       [  59. ,  -36. ,    1. ,    1.3],       [  82. ,    2. ,   -5. ,    1. ],       [  69. ,   40. ,    0. ,    0.8],       [  40. ,   66. ,   -2. ,    0.6],       [  -5. ,   78. ,    3. ,    0.8],       [ -40. ,   68. ,   -1. ,    1.2],       [ -66. ,   35. ,    3. ,    1.2],       [ -76. ,    0. ,    4. ,    1.2],       [ -71. ,  -42. ,   -3. ,    1.2],       [ -40. ,  -69. ,    0. ,    1. ],       [  -1. ,  -85. ,   -5. ,    1.3],       [  36. ,  -65. ,   -2. ,    0.7],       [  66. ,  -40. ,    0. ,    0.8],       [  90. ,   -5. ,   -5. ,    1. ],       [  80. ,   43. ,   -4. ,    0.9],       [  42. ,   81. ,   -1. ,    1.4],       [  -3. ,   93. ,    4. ,    1.1],       [ -46. ,   79. ,    3. ,    1. ],       [ -74. ,   45. ,    0. ,    0.6],       [ -95. ,   -2. ,   -4. ,    1.1],       [ -78. ,  -44. ,    0. ,    1.5],       [ -41. ,  -78. ,   -3. ,    0.9],       [   2. ,  -89. ,   -2. ,    0.5],       [  46. ,  -81. ,    4. ,    1. ],       [  80. ,  -48. ,   -2. ,    1.2],       [  99. ,    4. ,    1. ,    1.2],       [  85. ,   45. ,   -4. ,    0.8],       [  47. ,   86. ,   -1. ,    1.2],       [   3. ,  103. ,   -4. ,    1.5],       [ -46. ,   85. ,    0. ,    0.6],       [ -87. ,   50. ,    1. ,    1.3],       [-105. ,   -4. ,    4. ,    0.7],       [ -91. ,  -47. ,    1. ,    1.2],       [ -47. ,  -92. ,   -3. ,    0.8],       [   1. ,  -97. ,    4. ,    0.7],       [  48. ,  -83. ,   -2. ,    0.8],       [  90. ,  -49. ,    3. ,    0.8]])
        basis0 = BSplineBasis(4, np.array([ -3. ,  -1.6,  -0.9,   0. ,   1.4,   1.6,   3. ,   3.9,   4.9,   5.7,   7.1,   7.8,   9. ,
        10.4,  11.1,  12. ,  13.4,  13.6,  15. ]),2)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  1.3,  2. ,  2.8,  4. ,  4. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(2, 2)
        self.assertEqual(surf2.order(direction=0), 6)
        self.assertEqual(surf2.order(direction=1), 4)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p3(self):
        controlpoints = np.array([[  0.,  -4.,   4.],
       [ 25.,   0.,   1.],
       [ 46.,   1.,  -5.],
       [ 76.,   2.,  -5.],
       [ 98.,  -1.,   1.]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  0.8,  2.2,  3. ,  3. ,  3. ]))
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 4)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p43(self):
        controlpoints = np.array([[  -4.,   -3.,   -2.],       [  15.,    3.,   -4.],       [  38.,    0.,    4.],       [  54.,   -4.,   -2.],       [  69.,   -4.,   -3.],       [  82.,   -3.,   -2.],       [  97.,   -3.,   -4.],       [   2.,   12.,   -3.],       [  15.,   13.,    0.],       [  34.,   11.,    4.],       [  49.,   17.,   -3.],       [  64.,   15.,    4.],       [  85.,   13.,   -2.],       [  97.,   12.,   -1.],       [  -2.,   27.,    2.],       [  20.,   28.,   -2.],       [  36.,   30.,   -2.],       [  54.,   24.,    4.],       [  65.,   30.,    0.],       [  87.,   30.,    0.],       [  95.,   26.,    1.],       [  -2.,   39.,   -5.],       [  12.,   43.,   -5.],       [  33.,   47.,    2.],       [  51.,   46.,    1.],       [  64.,   45.,    2.],       [  81.,   44.,    3.],       [  97.,   45.,    2.],       [   0.,   55.,    4.],       [  18.,   57.,    0.],       [  34.,   59.,   -1.],       [  48.,   58.,   -5.],       [  71.,   53.,   -4.],       [  79.,   54.,    3.],       [ 101.,   60.,    2.],       [   3.,   74.,    2.],       [  13.,   73.,    3.],       [  34.,   70.,    3.],       [  53.,   68.,   -3.],       [  67.,   69.,   -2.],       [  83.,   66.,   -3.],       [ 101.,   73.,    2.],       [  -2.,   81.,   -1.],       [  14.,   82.,   -1.],       [  31.,   84.,   -5.],       [  48.,   81.,   -3.],       [  66.,   81.,   -2.],       [  83.,   82.,    2.],       [ 104.,   82.,    1.],       [  -5.,   96.,    4.],       [  19.,  104.,    1.],       [  36.,   97.,   -4.],       [  51.,  101.,    2.],       [  69.,  102.,    0.],       [  78.,  103.,    0.],       [  97.,   99.,   -4.]])
        basis0 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.1,  2. ,  2.9,  4. ,  4. ,  4. ,  4. ]))
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  0.8,  1.8,  2.8,  4.2,  4.7,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(1, 3)
        self.assertEqual(surf2.order(direction=0), 5)
        self.assertEqual(surf2.order(direction=1), 6)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p344(self):
        controlpoints = np.array([[  -2.,    4.,   -2.],       [  30.,    0.,   -1.],       [  68.,   -1.,   -5.],       [  96.,   -2.,   -2.],       [  -4.,   20.,   -1.],       [  28.,   16.,    4.],       [  68.,   13.,    2.],       [  97.,   20.,   -5.],       [   3.,   29.,    0.],       [  33.,   34.,    0.],       [  63.,   33.,   -5.],       [ 103.,   33.,    2.],       [   2.,   49.,   -1.],       [  29.,   48.,   -2.],       [  67.,   50.,    0.],       [ 100.,   47.,   -5.],       [  -1.,   62.,    3.],       [  37.,   65.,   -3.],       [  64.,   64.,   -5.],       [ 103.,   68.,   -5.],       [  -5.,   79.,   -1.],       [  36.,   82.,    1.],       [  70.,   85.,    4.],       [ 104.,   85.,   -5.],       [   4.,   97.,    0.],       [  31.,   97.,   -4.],       [  62.,  100.,   -5.],       [ 103.,   95.,   -4.],       [  -1.,    2.,   20.],       [  32.,   -3.,   22.],       [  66.,    2.,   25.],       [ 104.,   -3.,   26.],       [   1.,   13.,   23.],       [  30.,   14.,   25.],       [  62.,   19.,   22.],       [ 101.,   19.,   27.],       [  -1.,   32.,   24.],       [  36.,   35.,   20.],       [  62.,   31.,   28.],       [  98.,   37.,   22.],       [  -5.,   53.,   28.],       [  31.,   52.,   20.],       [  70.,   54.,   24.],       [ 100.,   52.,   27.],       [  -2.,   67.,   26.],       [  37.,   63.,   25.],       [  66.,   65.,   28.],       [ 103.,   69.,   24.],       [   3.,   84.,   21.],       [  31.,   80.,   24.],       [  66.,   87.,   24.],       [ 101.,   81.,   28.],       [   4.,  100.,   29.],       [  30.,   97.,   27.],       [  63.,  101.,   27.],       [ 104.,   95.,   25.],       [  -1.,    3.,   54.],       [  32.,    0.,   49.],       [  70.,    3.,   46.],       [ 103.,    0.,   48.],       [  -1.,   12.,   45.],       [  34.,   21.,   54.],       [  64.,   17.,   54.],       [  99.,   18.,   45.],       [   4.,   36.,   48.],       [  36.,   37.,   51.],       [  62.,   31.,   48.],       [  97.,   28.,   47.],       [   1.,   53.,   51.],       [  29.,   47.,   51.],       [  64.,   48.,   48.],       [  97.,   48.,   51.],       [  -2.,   71.,   53.],       [  35.,   69.,   45.],       [  67.,   62.,   53.],       [  99.,   68.,   50.],       [   3.,   83.,   51.],       [  34.,   79.,   48.],       [  63.,   85.,   47.],       [ 104.,   80.,   47.],       [   0.,   97.,   54.],       [  33.,  103.,   46.],       [  70.,   96.,   50.],       [  95.,   95.,   50.],       [   4.,    1.,   74.],       [  35.,   -2.,   78.],       [  67.,    2.,   76.],       [  96.,    4.,   71.],       [   1.,   17.,   79.],       [  32.,   20.,   79.],       [  67.,   15.,   71.],       [ 102.,   20.,   77.],       [  -3.,   29.,   75.],       [  35.,   33.,   74.],       [  65.,   28.,   70.],       [ 104.,   28.,   74.],       [  -2.,   51.,   70.],       [  33.,   54.,   78.],       [  71.,   54.,   76.],       [  99.,   54.,   73.],       [   3.,   71.,   73.],       [  30.,   70.,   70.],       [  67.,   62.,   75.],       [ 104.,   62.,   78.],       [  -4.,   82.,   75.],       [  36.,   81.,   74.],       [  62.,   84.,   71.],       [  96.,   87.,   74.],       [   0.,  102.,   79.],       [  31.,   96.,   77.],       [  62.,  101.,   73.],       [  95.,  104.,   78.],       [  -2.,   -1.,  101.],       [  28.,   -3.,  104.],       [  70.,   -1.,  100.],       [ 100.,    0.,   95.],       [   2.,   11.,  103.],       [  38.,   16.,  104.],       [  69.,   14.,   97.],       [  98.,   11.,   96.],       [   4.,   37.,   95.],       [  30.,   38.,  104.],       [  63.,   36.,   95.],       [ 101.,   34.,  104.],       [  -2.,   45.,  102.],       [  29.,   48.,  102.],       [  70.,   48.,   97.],       [  95.,   52.,  104.],       [   2.,   70.,  100.],       [  36.,   66.,   99.],       [  68.,   70.,  104.],       [  97.,   62.,  101.],       [  -4.,   84.,   99.],       [  32.,   86.,  104.],       [  62.,   81.,   97.],       [ 101.,   86.,   98.],       [   1.,   96.,   97.],       [  37.,   95.,   97.],       [  67.,  101.,  100.],       [  95.,   96.,  101.]])
        basis0 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  0.8,  2. ,  2. ,  2. ]))
        basis1 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  0.9,  1.8,  3.2,  4. ,  4. ,  4. ,  4. ]))
        basis2 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  1.1,  2. ,  2. ,  2. ,  2. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 3, 2)
        self.assertEqual(vol2.order(direction=0), 4)
        self.assertEqual(vol2.order(direction=1), 7)
        self.assertEqual(vol2.order(direction=2), 6)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p2_C0_periodic(self):
        controlpoints = np.array([[ 95.,   4.,   3.],
       [ 58.,  82.,   3.],
       [-28.,  93.,  -4.],
       [-94.,  40.,   0.],
       [-95., -48.,  -5.],
       [-23., -95.,  -2.],
       [ 59., -76.,  -1.]])
        basis0 = BSplineBasis(2, np.array([-0.8,  0. ,  0.8,  1.8,  2.9,  3.7,  4.8,  6.2,  7. ,  7.8]),0)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 3)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p33_C0_periodic(self):
        controlpoints = np.array([[  64.,   -1.,   -1.],
       [  28.,   47.,    2.],
       [ -31.,   55.,    0.],
       [ -62.,    1.,    0.],
       [ -29.,  -55.,   -3.],
       [  28.,  -53.,    1.],
       [  73.,    2.,    3.],
       [  32.,   65.,    4.],
       [ -31.,   62.,    4.],
       [ -72.,    4.,   -5.],
       [ -31.,  -59.,   -3.],
       [  39.,  -58.,   -4.],
       [  76.,    2.,   -4.],
       [  39.,   67.,   -5.],
       [ -38.,   70.,   -3.],
       [ -83.,    1.,   -2.],
       [ -42.,  -73.,    4.],
       [  40.,  -72.,   -3.],
       [  86.,    2.,   -1.],
       [  47.,   73.,   -2.],
       [ -50.,   77.,    2.],
       [ -93.,    3.,    2.],
       [ -45.,  -82.,   -2.],
       [  47.,  -79.,   -2.],
       [ 101.,    4.,    1.],
       [  48.,   88.,    2.],
       [ -55.,   82.,    3.],
       [-103.,    2.,   -2.],
       [ -50.,  -91.,    2.],
       [  47.,  -83.,   -3.]])
        basis0 = BSplineBasis(3, np.array([-0.7,  0. ,  0. ,  0.6,  1.6,  3.2,  4.3,  5. ,  5. ,  5.6]),0)
        basis1 = BSplineBasis(3, np.array([ 0. ,  0. ,  0. ,  0.9,  2.2,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 3)
        self.assertEqual(surf2.order(direction=0), 5)
        self.assertEqual(surf2.order(direction=1), 6)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p222_C0_periodic(self):
        controlpoints = np.array([[  49.,   -2.,    4.],       [  30.,   42.,    0.],       [ -13.,   46.,    4.],       [ -48.,   19.,    2.],       [ -42.,  -19.,   -1.],       [ -12.,  -53.,    0.],       [  35.,  -37.,    0.],       [  58.,    0.,   -3.],       [  33.,   48.,   -5.],       [ -12.,   58.,   -5.],       [ -51.,   28.,   -5.],       [ -52.,  -27.,    1.],       [ -17.,  -58.,    0.],       [  40.,  -51.,   -4.],       [  70.,   -5.,    3.],       [  42.,   48.,    0.],       [ -18.,   61.,   -1.],       [ -57.,   29.,    0.],       [ -61.,  -27.,    2.],       [ -20.,  -67.,   -4.],       [  41.,  -51.,    1.],       [  78.,    0.,    2.],       [  45.,   63.,   -4.],       [ -17.,   71.,    4.],       [ -66.,   36.,   -1.],       [ -66.,  -31.,    1.],       [ -17.,  -76.,    0.],       [  43.,  -57.,   -2.],       [  86.,    2.,   -3.],       [  55.,   63.,    3.],       [ -16.,   80.,    3.],       [ -80.,   31.,    3.],       [ -76.,  -32.,   -2.],       [ -22.,  -86.,    0.],       [  53.,  -64.,   -3.],       [  93.,   -1.,    0.],       [  59.,   66.,   -5.],       [ -24.,   92.,   -4.],       [ -84.,   34.,    0.],       [ -82.,  -40.,   -5.],       [ -20.,  -90.,    2.],       [  52.,  -76.,    0.],       [  98.,   -4.,    0.],       [  64.,   79.,   -4.],       [ -18.,   95.,    0.],       [ -92.,   45.,   -1.],       [ -89.,  -41.,   -5.],       [ -19., -101.,   -1.],       [  64.,  -82.,   -4.],       [  50.,    3.,   13.],       [  27.,   42.,    9.],       [ -15.,   45.,    9.],       [ -45.,   22.,   15.],       [ -42.,  -18.,   14.],       [ -15.,  -45.,    9.],       [  32.,  -43.,   11.],       [  53.,    1.,    9.],       [  37.,   47.,   15.],       [ -16.,   58.,   16.],       [ -55.,   24.,   10.],       [ -57.,  -26.,    9.],       [ -14.,  -59.,    9.],       [  38.,  -48.,   14.],       [  65.,   -5.,    9.],       [  44.,   50.,   12.],       [ -14.,   61.,   10.],       [ -59.,   31.,    9.],       [ -62.,  -27.,   11.],       [ -15.,  -68.,   16.],       [  41.,  -55.,   13.],       [  72.,   -3.,   12.],       [  50.,   60.,   17.],       [ -13.,   74.,   18.],       [ -71.,   30.,    9.],       [ -65.,  -31.,    8.],       [ -13.,  -70.,    9.],       [  47.,  -59.,    9.],       [  85.,    3.,   12.],       [  51.,   64.,   12.],       [ -23.,   79.,   12.],       [ -72.,   39.,   10.],       [ -75.,  -39.,   17.],       [ -20.,  -80.,   12.],       [  55.,  -61.,   17.],       [  94.,    0.,   10.],       [  61.,   68.,   15.],       [ -20.,   91.,   15.],       [ -88.,   36.,   15.],       [ -83.,  -41.,   16.],       [ -26.,  -88.,   16.],       [  58.,  -75.,   12.],       [ 104.,   -1.,   12.],       [  62.,   81.,    9.],       [ -27.,  100.,   18.],       [ -95.,   47.,    8.],       [ -86.,  -42.,   17.],       [ -21., -101.,   14.],       [  58.,  -81.,   11.],       [  52.,    2.,   21.],       [  27.,   37.,   23.],       [ -13.,   52.,   28.],       [ -49.,   23.,   28.],       [ -50.,  -26.,   29.],       [  -9.,  -46.,   24.],       [  34.,  -39.,   23.],       [  57.,   -2.,   27.],       [  34.,   50.,   21.],       [ -10.,   52.,   28.],       [ -52.,   23.,   26.],       [ -57.,  -22.,   22.],       [ -11.,  -60.,   27.],       [  32.,  -43.,   23.],       [  62.,   -1.,   24.],       [  42.,   50.,   22.],       [ -11.,   65.,   28.],       [ -64.,   32.,   23.],       [ -62.,  -34.,   26.],       [ -14.,  -66.,   31.],       [  45.,  -48.,   22.],       [  76.,    2.,   27.],       [  44.,   55.,   25.],       [ -19.,   76.,   22.],       [ -71.,   35.,   24.],       [ -68.,  -31.,   26.],       [ -13.,  -77.,   22.],       [  50.,  -57.,   25.],       [  82.,   -3.,   31.],       [  49.,   60.,   30.],       [ -23.,   84.,   30.],       [ -74.,   34.,   25.],       [ -75.,  -35.,   29.],       [ -14.,  -77.,   22.],       [  53.,  -69.,   29.],       [  96.,    3.,   22.],       [  59.,   70.,   22.],       [ -20.,   93.,   25.],       [ -82.,   38.,   31.],       [ -87.,  -41.,   27.],       [ -21.,  -92.,   26.],       [  53.,  -76.,   24.],       [ 100.,    2.,   24.],       [  58.,   81.,   25.],       [ -26.,   97.,   30.],       [ -93.,   40.,   29.],       [ -94.,  -43.,   26.],       [ -25., -102.,   24.],       [  64.,  -81.,   23.],       [  47.,   -5.,   41.],       [  29.,   35.,   37.],       [  -7.,   46.,   41.],       [ -48.,   23.,   41.],       [ -42.,  -26.,   39.],       [  -9.,  -53.,   43.],       [  30.,  -36.,   43.],       [  54.,    1.,   35.],       [  32.,   48.,   36.],       [ -16.,   54.,   39.],       [ -49.,   23.,   44.],       [ -50.,  -22.,   44.],       [ -10.,  -54.,   43.],       [  39.,  -42.,   38.],       [  62.,   -5.,   43.],       [  46.,   48.,   36.],       [ -18.,   66.,   40.],       [ -65.,   32.,   44.],       [ -56.,  -29.,   40.],       [ -12.,  -65.,   39.],       [  42.,  -51.,   40.],       [  71.,   -5.,   41.],       [  44.,   62.,   38.],       [ -12.,   76.,   38.],       [ -70.,   29.,   43.],       [ -65.,  -30.,   42.],       [ -19.,  -77.,   44.],       [  44.,  -60.,   41.],       [  86.,   -1.,   37.],       [  50.,   65.,   41.],       [ -19.,   79.,   35.],       [ -74.,   35.,   40.],       [ -79.,  -40.,   36.],       [ -19.,  -77.,   38.],       [  51.,  -66.,   41.],       [  88.,    2.,   43.],       [  54.,   76.,   41.],       [ -24.,   92.,   36.],       [ -80.,   43.,   40.],       [ -85.,  -45.,   39.],       [ -18.,  -90.,   41.],       [  57.,  -76.,   35.],       [  95.,   -3.,   40.],       [  62.,   82.,   35.],       [ -27.,   93.,   41.],       [ -91.,   45.,   44.],       [ -92.,  -46.,   44.],       [ -24.,  -96.,   36.],       [  58.,  -78.,   36.]])
        basis0 = BSplineBasis(2, np.array([-0.9,  0. ,  0.9,  2.1,  3. ,  4.1,  4.7,  6.1,  7. ,  7.9]),0)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.9,  1.9,  2.7,  4.3,  4.8,  6. ,  6. ]))
        basis2 = BSplineBasis(2, np.array([ 0. ,  0. ,  1.3,  2. ,  3. ,  3. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(2, 1, 3)
        self.assertEqual(vol2.order(direction=0), 4)
        self.assertEqual(vol2.order(direction=1), 3)
        self.assertEqual(vol2.order(direction=2), 5)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p3_C1_periodic(self):
        controlpoints = np.array([[  98.,   -1.,    1.],
       [  71.,   66.,    1.],
       [   1.,  101.,    1.],
       [ -71.,   74.,    2.],
       [-100.,   -4.,    2.],
       [ -68.,  -73.,    1.],
       [   3.,  -96.,    4.],
       [  74.,  -70.,   -5.]])
        basis0 = BSplineBasis(3, np.array([ -1.9,  -0.6,   0. ,   1. ,   2.1,   2.6,   4. ,   4.9,   6.1,   7.4,   8. ,   9. ,  10.1]),1)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 4)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p322_C1_periodic(self):
        controlpoints = np.array([[  50.,   -1.,    4.],       [  36.,   38.,   -4.],       [  -2.,   47.,   -3.],       [ -33.,   38.,   -5.],       [ -53.,   -4.,   -3.],       [ -32.,  -39.,    0.],       [   4.,  -54.,   -1.],       [  39.,  -32.,   -4.],       [  64.,   -2.,   -4.],       [  44.,   40.,    2.],       [  -1.,   60.,   -5.],       [ -48.,   40.,   -2.],       [ -63.,   -5.,    2.],       [ -42.,  -41.,    2.],       [  -2.,  -66.,    4.],       [  40.,  -46.,   -5.],       [  78.,    0.,   -3.],       [  54.,   49.,    0.],       [  -4.,   74.,   -4.],       [ -53.,   54.,   -2.],       [ -71.,    3.,   -5.],       [ -55.,  -54.,   -3.],       [  -5.,  -73.,    0.],       [  55.,  -54.,    4.],       [  82.,    0.,   -5.],       [  65.,   64.,    1.],       [  -3.,   86.,    3.],       [ -60.,   59.,    1.],       [ -89.,   -4.,    2.],       [ -63.,  -63.,   -5.],       [  -3.,  -85.,   -4.],       [  62.,  -66.,    3.],       [  98.,   -1.,   -3.],       [  67.,   72.,    4.],       [   0.,   99.,    0.],       [ -75.,   71.,   -2.],       [ -97.,   -2.,   -2.],       [ -75.,  -74.,   -4.],       [  -1., -104.,   -1.],       [  70.,  -72.,   -3.],       [  47.,    2.,   18.],       [  34.,   36.,   16.],       [  -2.,   48.,   15.],       [ -31.,   36.,   18.],       [ -46.,   -5.,   24.],       [ -37.,  -41.,   21.],       [  -1.,  -50.,   22.],       [  38.,  -36.,   18.],       [  67.,   -1.,   17.],       [  39.,   43.,   24.],       [   0.,   64.,   16.],       [ -48.,   47.,   22.],       [ -60.,   -4.,   15.],       [ -47.,  -48.,   20.],       [   3.,  -62.,   18.],       [  42.,  -43.,   24.],       [  76.,   -5.,   17.],       [  48.,   54.,   22.],       [   1.,   78.,   18.],       [ -58.,   48.,   24.],       [ -72.,    3.,   22.],       [ -52.,  -53.,   19.],       [  -2.,  -79.,   23.],       [  48.,  -55.,   19.],       [  88.,   -5.,   15.],       [  63.,   57.,   15.],       [  -1.,   82.,   15.],       [ -60.,   63.,   22.],       [ -88.,   -2.,   24.],       [ -58.,  -59.,   23.],       [  -5.,  -90.,   16.],       [  65.,  -67.,   15.],       [ 104.,   -4.,   16.],       [  72.,   69.,   24.],       [   4.,   96.,   19.],       [ -69.,   65.,   24.],       [ -97.,    0.,   22.],       [ -70.,  -68.,   23.],       [  -5.,  -98.,   18.],       [  72.,  -75.,   16.],       [  46.,    0.,   35.],       [  35.,   38.,   38.],       [   1.,   50.,   36.],       [ -35.,   36.,   40.],       [ -47.,   -5.,   44.],       [ -35.,  -31.,   35.],       [   2.,  -49.,   40.],       [  30.,  -35.,   42.],       [  65.,   -1.,   39.],       [  43.,   43.,   35.],       [   0.,   62.,   44.],       [ -47.,   39.,   39.],       [ -64.,   -3.,   36.],       [ -42.,  -44.,   41.],       [  -2.,  -64.,   42.],       [  48.,  -46.,   42.],       [  71.,   -4.,   39.],       [  52.,   49.,   37.],       [   1.,   78.,   44.],       [ -53.,   56.,   40.],       [ -73.,   -3.,   43.],       [ -49.,  -50.,   37.],       [  -3.,  -78.,   41.],       [  56.,  -54.,   38.],       [  83.,    4.,   40.],       [  59.,   63.,   42.],       [   4.,   87.,   44.],       [ -67.,   61.,   35.],       [ -91.,   -5.,   44.],       [ -58.,  -63.,   38.],       [  -2.,  -83.,   43.],       [  59.,  -61.,   44.],       [ 101.,   -4.,   42.],       [  71.,   68.,   43.],       [  -3.,   98.,   44.],       [ -67.,   66.,   36.],       [-102.,    0.,   44.],       [ -75.,  -75.,   35.],       [   2.,  -99.,   44.],       [  67.,  -68.,   36.]])
        basis0 = BSplineBasis(3, np.array([ -1.6,  -1.3,   0. ,   0.8,   2.1,   3.4,   3.9,   5.3,   6.4,   6.7,   8. ,   8.8,  10.1]),1)
        basis1 = BSplineBasis(2, np.array([ 0. ,  0. ,  1. ,  2.3,  2.9,  4. ,  4. ]))
        basis2 = BSplineBasis(2, np.array([ 0. ,  0. ,  0.7,  2. ,  2. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 2, 1)
        self.assertEqual(vol2.order(direction=0), 4)
        self.assertEqual(vol2.order(direction=1), 4)
        self.assertEqual(vol2.order(direction=2), 3)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p442_C2_periodic(self):
        controlpoints = np.array([[  52.,   -2.,    0.],       [  38.,   20.,   -1.],       [  25.,   46.,    4.],       [   3.,   48.,    1.],       [ -29.,   43.,   -1.],       [ -45.,   23.,   -4.],       [ -50.,    3.,    3.],       [ -40.,  -23.,   -4.],       [ -23.,  -43.,   -5.],       [  -4.,  -54.,   -2.],       [  24.,  -39.,   -5.],       [  46.,  -30.,   -1.],       [  60.,    2.,   -3.],       [  53.,   26.,    4.],       [  32.,   52.,   -5.],       [  -4.,   56.,   -4.],       [ -26.,   51.,   -2.],       [ -55.,   29.,   -3.],       [ -55.,   -5.,    2.],       [ -54.,  -29.,   -1.],       [ -26.,  -50.,   -1.],       [  -1.,  -59.,    3.],       [  25.,  -54.,    4.],       [  48.,  -30.,   -3.],       [  65.,    3.,   -3.],       [  56.,   37.,    3.],       [  35.,   56.,   -3.],       [   3.,   70.,    3.],       [ -34.,   56.,   -2.],       [ -56.,   32.,   -1.],       [ -69.,    1.,    3.],       [ -62.,  -36.,    4.],       [ -30.,  -54.,   -2.],       [  -5.,  -63.,    2.],       [  38.,  -54.,    3.],       [  54.,  -30.,    3.],       [  77.,    4.,   -1.],       [  66.,   33.,   -1.],       [  37.,   65.,    3.],       [   2.,   77.,    4.],       [ -41.,   67.,    4.],       [ -66.,   35.,    1.],       [ -71.,   -5.,    4.],       [ -67.,  -33.,    2.],       [ -38.,  -62.,   -5.],       [   3.,  -76.,    0.],       [  39.,  -67.,    3.],       [  69.,  -36.,   -4.],       [  78.,   -1.,    1.],       [  76.,   45.,    4.],       [  40.,   76.,   -4.],       [   1.,   81.,   -1.],       [ -39.,   77.,   -3.],       [ -74.,   42.,    2.],       [ -83.,    4.,   -4.],       [ -73.,  -39.,   -2.],       [ -47.,  -71.,    3.],       [   4.,  -86.,   -4.],       [  41.,  -76.,    3.],       [  67.,  -41.,   -1.],       [  92.,    4.,    1.],       [  82.,   43.,   -3.],       [  49.,   79.,   -1.],       [   4.,   89.,   -3.],       [ -51.,   74.,    3.],       [ -77.,   49.,   -5.],       [ -88.,   -3.,   -5.],       [ -83.,  -42.,   -5.],       [ -43.,  -77.,   -1.],       [   3.,  -94.,    2.],       [  41.,  -84.,   -2.],       [  76.,  -48.,    3.],       [ 100.,    2.,   -5.],       [  88.,   52.,    3.],       [  53.,   89.,   -5.],       [   0.,   99.,   -2.],       [ -52.,   84.,    1.],       [ -89.,   52.,    2.],       [ -98.,    2.,    0.],       [ -91.,  -50.,    4.],       [ -46.,  -86.,   -1.],       [   3.,  -96.,   -4.],       [  45.,  -87.,   -3.],       [  82.,  -51.,    2.],       [  51.,   -2.,    5.],       [  40.,   20.,    9.],       [  29.,   45.,    6.],       [   1.,   54.,    9.],       [ -26.,   42.,    8.],       [ -43.,   20.,    3.],       [ -51.,   -2.,   11.],       [ -41.,  -30.,    9.],       [ -22.,  -45.,    5.],       [  -4.,  -52.,    9.],       [  24.,  -42.,    7.],       [  41.,  -21.,    9.],       [  53.,    2.,    4.],       [  52.,   25.,    8.],       [  27.,   52.,    3.],       [  -5.,   61.,    4.],       [ -27.,   47.,    6.],       [ -47.,   25.,    9.],       [ -61.,    1.,   10.],       [ -53.,  -28.,    9.],       [ -25.,  -49.,    3.],       [  -4.,  -61.,    3.],       [  28.,  -51.,    6.],       [  47.,  -25.,    5.],       [  62.,   -1.,    3.],       [  59.,   31.,    2.],       [  34.,   56.,    4.],       [   4.,   63.,    8.],       [ -33.,   52.,    5.],       [ -58.,   32.,    3.],       [ -67.,    4.,    9.],       [ -56.,  -38.,    3.],       [ -39.,  -61.,    4.],       [  -1.,  -65.,    4.],       [  30.,  -61.,    2.],       [  54.,  -36.,    6.],       [  79.,    3.,    8.],       [  66.,   39.,    7.],       [  35.,   62.,    8.],       [  -2.,   74.,    6.],       [ -41.,   69.,    3.],       [ -60.,   40.,    5.],       [ -77.,    4.,    2.],       [ -64.,  -40.,    5.],       [ -37.,  -61.,    4.],       [  -4.,  -72.,    9.],       [  38.,  -69.,   11.],       [  62.,  -38.,    2.],       [  81.,    2.,    6.],       [  67.,   39.,    9.],       [  37.,   69.,    5.],       [  -1.,   83.,    2.],       [ -39.,   71.,   11.],       [ -68.,   39.,   10.],       [ -83.,   -1.,    9.],       [ -71.,  -41.,    2.],       [ -46.,  -72.,    9.],       [  -4.,  -85.,    4.],       [  39.,  -70.,   10.],       [  68.,  -38.,    8.],       [  89.,    1.,   10.],       [  76.,   43.,    4.],       [  43.,   77.,    5.],       [   3.,   96.,    6.],       [ -46.,   81.,   10.],       [ -84.,   43.,    2.],       [ -90.,   -1.,    8.],       [ -82.,  -44.,    1.],       [ -44.,  -85.,   11.],       [  -4.,  -92.,    9.],       [  43.,  -78.,    5.],       [  80.,  -45.,    5.],       [  96.,    1.,    7.],       [  88.,   45.,    3.],       [  45.,   89.,    6.],       [   2.,  101.,    3.],       [ -51.,   90.,    7.],       [ -87.,   54.,    9.],       [-104.,   -5.,    7.],       [ -90.,  -54.,    7.],       [ -53.,  -85.,    9.],       [   4., -103.,   11.],       [  49.,  -85.,   11.],       [  90.,  -49.,   10.],       [  47.,    3.,    9.],       [  47.,   27.,   14.],       [  21.,   39.,   13.],       [  -4.,   52.,    9.],       [ -28.,   43.,   17.],       [ -43.,   27.,   18.],       [ -49.,   -1.,   12.],       [ -42.,  -29.,   12.],       [ -23.,  -41.,   11.],       [  -5.,  -47.,   16.],       [  29.,  -41.,   12.],       [  46.,  -28.,   17.],       [  59.,   -2.,   13.],       [  49.,   33.,   18.],       [  29.,   47.,   16.],       [  -1.,   55.,   11.],       [ -34.,   52.,   12.],       [ -48.,   33.,    9.],       [ -63.,   -5.,   16.],       [ -51.,  -30.,   16.],       [ -25.,  -46.,   10.],       [  -1.,  -58.,   14.],       [  31.,  -54.,   12.],       [  51.,  -31.,   14.],       [  66.,   -3.,   14.],       [  59.,   32.,   14.],       [  34.,   55.,   13.],       [   0.,   64.,   17.],       [ -38.,   60.,    8.],       [ -55.,   36.,   12.],       [ -68.,   -5.,   10.],       [ -63.,  -37.,   12.],       [ -36.,  -61.,   10.],       [   3.,  -67.,   17.],       [  28.,  -62.,   17.],       [  61.,  -36.,   14.],       [  78.,   -3.,   15.],       [  69.,   39.,   16.],       [  39.,   69.,   15.],       [   2.,   78.,   13.],       [ -34.,   68.,   11.],       [ -68.,   33.,   10.],       [ -78.,    0.,   17.],       [ -67.,  -40.,   15.],       [ -39.,  -61.,   10.],       [  -3.,  -78.,    9.],       [  33.,  -64.,    9.],       [  64.,  -39.,   13.],       [  81.,    0.,   11.],       [  76.,   37.,   17.],       [  45.,   73.,   10.],       [   0.,   81.,   14.],       [ -47.,   67.,   14.],       [ -69.,   44.,   15.],       [ -79.,    3.,    9.],       [ -76.,  -44.,   15.],       [ -44.,  -73.,   14.],       [  -5.,  -85.,   16.],       [  42.,  -76.,    9.],       [  69.,  -38.,   16.],       [  94.,   -5.,   13.],       [  82.,   41.,   17.],       [  48.,   82.,   11.],       [  -1.,   89.,   14.],       [ -43.,   80.,    9.],       [ -80.,   45.,   12.],       [ -89.,   -5.,   10.],       [ -75.,  -44.,   10.],       [ -50.,  -76.,    9.],       [  -2.,  -93.,   17.],       [  43.,  -75.,   14.],       [  79.,  -47.,   16.],       [  97.,    3.,   14.],       [  90.,   54.,   16.],       [  53.,   85.,   13.],       [   2.,  101.,   17.],       [ -47.,   86.,   11.],       [ -87.,   47.,    9.],       [ -97.,    0.,    8.],       [ -88.,  -52.,    8.],       [ -55.,  -90.,    9.],       [  -1.,  -99.,   11.],       [  48.,  -88.,   12.],       [  88.,  -50.,   10.],       [  48.,   -1.,   20.],       [  44.,   27.,   22.],       [  29.,   41.,   18.],       [  -3.,   47.,   15.],       [ -23.,   42.,   20.],       [ -45.,   26.,   17.],       [ -51.,    1.,   21.],       [ -43.,  -27.,   24.],       [ -24.,  -43.,   20.],       [   1.,  -55.,   22.],       [  23.,  -46.,   19.],       [  38.,  -29.,   21.],       [  60.,    0.,   18.],       [  50.,   30.,   17.],       [  24.,   47.,   22.],       [  -3.,   60.,   24.],       [ -32.,   45.,   19.],       [ -56.,   34.,   20.],       [ -64.,    1.,   15.],       [ -49.,  -34.,   17.],       [ -31.,  -53.,   16.],       [   2.,  -59.,   18.],       [  24.,  -50.,   23.],       [  49.,  -27.,   20.],       [  71.,   -5.,   17.],       [  53.,   37.,   23.],       [  33.,   52.,   15.],       [  -3.,   65.,   22.],       [ -29.,   61.,   15.],       [ -57.,   36.,   17.],       [ -66.,    1.,   21.],       [ -58.,  -31.,   24.],       [ -33.,  -57.,   16.],       [   4.,  -66.,   22.],       [  33.,  -53.,   15.],       [  53.,  -31.,   18.],       [  77.,    4.,   22.],       [  66.,   40.,   21.],       [  37.,   63.,   23.],       [   3.,   70.,   20.],       [ -39.,   63.,   16.],       [ -70.,   38.,   18.],       [ -80.,    4.,   17.],       [ -67.,  -42.,   16.],       [ -38.,  -70.,   21.],       [  -2.,  -75.,   24.],       [  32.,  -68.,   17.],       [  64.,  -35.,   17.],       [  81.,    3.,   15.],       [  71.,   45.,   22.],       [  44.,   75.,   21.],       [  -4.,   86.,   22.],       [ -46.,   72.,   23.],       [ -75.,   46.,   22.],       [ -80.,    0.,   17.],       [ -75.,  -46.,   20.],       [ -45.,  -74.,   22.],       [  -4.,  -82.,   22.],       [  38.,  -68.,   19.],       [  69.,  -39.,   15.],       [  95.,    4.,   15.],       [  78.,   50.,   18.],       [  43.,   80.,   17.],       [   1.,   88.,   21.],       [ -46.,   82.,   24.],       [ -79.,   43.,   21.],       [ -90.,   -4.,   17.],       [ -81.,  -50.,   18.],       [ -42.,  -81.,   18.],       [   2.,  -89.,   22.],       [  48.,  -84.,   20.],       [  74.,  -41.,   16.],       [  99.,   -5.,   22.],       [  82.,   49.,   18.],       [  51.,   84.,   19.],       [  -3.,  103.,   17.],       [ -48.,   89.,   23.],       [ -89.,   45.,   21.],       [-102.,    0.,   19.],       [ -87.,  -47.,   23.],       [ -47.,  -84.,   20.],       [  -3.,  -99.,   24.],       [  51.,  -86.,   18.],       [  88.,  -54.,   15.],       [  52.,   -2.,   31.],       [  39.,   25.,   24.],       [  25.,   40.,   24.],       [  -4.,   45.,   28.],       [ -23.,   44.,   24.],       [ -42.,   29.,   26.],       [ -51.,   -1.,   21.],       [ -44.,  -22.,   23.],       [ -29.,  -40.,   22.],       [   3.,  -46.,   28.],       [  24.,  -43.,   29.],       [  43.,  -21.,   26.],       [  58.,    2.,   30.],       [  47.,   27.,   23.],       [  33.,   53.,   29.],       [  -5.,   57.,   29.],       [ -25.,   47.,   23.],       [ -54.,   26.,   28.],       [ -62.,    4.,   25.],       [ -49.,  -33.,   23.],       [ -32.,  -48.,   22.],       [   3.,  -63.,   31.],       [  32.,  -55.,   27.],       [  52.,  -34.,   30.],       [  66.,    3.,   26.],       [  52.,   29.,   22.],       [  33.,   59.,   30.],       [   1.,   70.,   26.],       [ -29.,   60.,   22.],       [ -57.,   31.,   27.],       [ -71.,    2.,   27.],       [ -59.,  -38.,   24.],       [ -33.,  -57.,   22.],       [  -2.,  -63.,   26.],       [  32.,  -57.,   23.],       [  57.,  -31.,   22.],       [  77.,    0.,   26.],       [  68.,   38.,   22.],       [  35.,   62.,   28.],       [  -5.,   77.,   28.],       [ -39.,   69.,   31.],       [ -66.,   32.,   30.],       [ -72.,   -5.,   28.],       [ -68.,  -39.,   26.],       [ -36.,  -67.,   27.],       [  -3.,  -76.,   27.],       [  38.,  -69.,   27.],       [  61.,  -43.,   29.],       [  87.,   -2.,   23.],       [  73.,   46.,   29.],       [  41.,   68.,   22.],       [   3.,   80.,   28.],       [ -40.,   67.,   25.],       [ -75.,   38.,   30.],       [ -81.,    3.,   26.],       [ -76.,  -39.,   30.],       [ -39.,  -72.,   25.],       [   0.,  -79.,   28.],       [  43.,  -74.,   31.],       [  76.,  -47.,   25.],       [  92.,   -5.,   25.],       [  82.,   41.,   22.],       [  41.,   75.,   27.],       [   3.,   94.,   30.],       [ -51.,   77.,   30.],       [ -81.,   45.,   22.],       [ -90.,   -1.,   22.],       [ -81.,  -49.,   24.],       [ -45.,  -76.,   23.],       [   1.,  -91.,   26.],       [  41.,  -81.,   31.],       [  76.,  -48.,   26.],       [ 104.,   -1.,   31.],       [  82.,   52.,   29.],       [  48.,   89.,   23.],       [  -3.,  100.,   27.],       [ -54.,   84.,   28.],       [ -88.,   51.,   31.],       [ -99.,   -1.,   26.],       [ -92.,  -52.,   29.],       [ -49.,  -92.,   26.],       [  -5., -102.,   21.],       [  45.,  -92.,   27.],       [  90.,  -52.,   26.],       [  47.,    4.,   32.],       [  39.,   25.,   37.],       [  27.,   42.,   37.],       [  -3.,   51.,   30.],       [ -26.,   41.,   30.],       [ -40.,   23.,   35.],       [ -51.,   -3.,   31.],       [ -44.,  -21.,   35.],       [ -27.,  -40.,   36.],       [  -2.,  -49.,   38.],       [  28.,  -43.,   37.],       [  44.,  -27.,   31.],       [  56.,    3.,   35.],       [  52.,   33.,   33.],       [  31.,   47.,   33.],       [  -3.,   54.,   29.],       [ -29.,   52.,   37.],       [ -50.,   30.,   28.],       [ -55.,    0.,   28.],       [ -54.,  -26.,   31.],       [ -27.,  -52.,   33.],       [   3.,  -61.,   36.],       [  25.,  -53.,   30.],       [  46.,  -30.,   37.],       [  69.,    2.,   32.],       [  55.,   34.,   32.],       [  33.,   62.,   32.],       [  -5.,   69.,   32.],       [ -34.,   56.,   38.],       [ -53.,   28.,   35.],       [ -69.,    1.,   29.],       [ -53.,  -36.,   33.],       [ -37.,  -56.,   30.],       [   2.,  -63.,   37.],       [  35.,  -58.,   29.],       [  59.,  -35.,   35.],       [  72.,    1.,   32.],       [  68.,   33.,   32.],       [  32.,   69.,   30.],       [  -1.,   77.,   38.],       [ -38.,   62.,   30.],       [ -63.,   36.,   37.],       [ -79.,   -3.,   30.],       [ -69.,  -34.,   28.],       [ -36.,  -62.,   36.],       [  -1.,  -78.,   35.],       [  40.,  -70.,   34.],       [  63.,  -37.,   30.],       [  78.,    4.,   28.],       [  72.,   37.,   35.],       [  41.,   75.,   36.],       [   3.,   85.,   38.],       [ -37.,   69.,   33.],       [ -77.,   37.,   37.],       [ -80.,    4.,   35.],       [ -72.,  -39.,   33.],       [ -41.,  -69.,   29.],       [   4.,  -79.,   28.],       [  38.,  -73.,   30.],       [  67.,  -46.,   33.],       [  90.,    2.,   34.],       [  74.,   49.,   30.],       [  50.,   80.,   36.],       [   4.,   94.,   36.],       [ -45.,   80.,   31.],       [ -79.,   49.,   34.],       [ -88.,    0.,   36.],       [ -80.,  -45.,   33.],       [ -50.,  -84.,   29.],       [  -3.,  -97.,   31.],       [  46.,  -75.,   29.],       [  79.,  -45.,   30.],       [ 100.,    4.,   29.],       [  83.,   47.,   35.],       [  49.,   86.,   28.],       [  -5.,  103.,   36.],       [ -53.,   87.,   36.],       [ -90.,   54.,   29.],       [-104.,    4.,   28.],       [ -90.,  -52.,   32.],       [ -50.,  -88.,   30.],       [   2., -103.,   31.],       [  49.,  -89.,   28.],       [  88.,  -53.,   34.],       [  50.,   -2.,   40.],       [  39.,   29.,   44.],       [  20.,   41.,   39.],       [  -3.,   53.,   38.],       [ -28.,   41.,   38.],       [ -46.,   26.,   36.],       [ -53.,   -5.,   41.],       [ -40.,  -26.,   40.],       [ -28.,  -47.,   40.],       [  -4.,  -53.,   37.],       [  24.,  -49.,   41.],       [  46.,  -22.,   39.],       [  59.,    4.,   43.],       [  52.,   24.,   38.],       [  27.,   51.,   39.],       [  -2.,   56.,   36.],       [ -30.,   52.,   38.],       [ -47.,   30.,   35.],       [ -57.,   -2.,   43.],       [ -54.,  -28.,   41.],       [ -29.,  -52.,   42.],       [   4.,  -54.,   39.],       [  30.,  -53.,   35.],       [  51.,  -30.,   41.],       [  70.,   -5.,   43.],       [  53.,   30.,   37.],       [  30.,   54.,   36.],       [  -2.,   63.,   44.],       [ -36.,   54.,   42.],       [ -60.,   33.,   39.],       [ -63.,   -4.,   36.],       [ -58.,  -30.,   38.],       [ -35.,  -56.,   37.],       [  -4.,  -70.,   40.],       [  35.,  -54.,   38.],       [  56.,  -37.,   41.],       [  72.,   -3.,   42.],       [  66.,   37.,   36.],       [  38.,   67.,   42.],       [   3.,   73.,   35.],       [ -42.,   68.,   41.],       [ -61.,   32.,   42.],       [ -73.,   -3.,   44.],       [ -63.,  -43.,   39.],       [ -35.,  -67.,   42.],       [  -3.,  -74.,   35.],       [  40.,  -62.,   38.],       [  62.,  -35.,   39.],       [  79.,    0.,   40.],       [  75.,   43.,   43.],       [  45.,   71.,   39.],       [   4.,   86.,   43.],       [ -37.,   75.,   40.],       [ -71.,   38.,   42.],       [ -87.,   -4.,   44.],       [ -68.,  -45.,   39.],       [ -44.,  -73.,   38.],       [  -4.,  -79.,   43.],       [  37.,  -72.,   44.],       [  73.,  -39.,   38.],       [  92.,    3.,   37.],       [  78.,   43.,   39.],       [  45.,   82.,   36.],       [   2.,   95.,   41.],       [ -42.,   80.,   43.],       [ -84.,   45.,   41.],       [ -97.,   -2.,   44.],       [ -83.,  -45.,   37.],       [ -43.,  -78.,   36.],       [   4.,  -88.,   36.],       [  43.,  -76.,   35.],       [  79.,  -49.,   43.],       [  95.,   -5.,   41.],       [  84.,   50.,   42.],       [  50.,   88.,   37.],       [   2.,   99.,   35.],       [ -47.,   83.,   40.],       [ -89.,   50.,   38.],       [ -97.,   -2.,   44.],       [ -89.,  -51.,   38.],       [ -47.,  -83.,   36.],       [   3., -101.,   37.],       [  48.,  -88.,   36.],       [  85.,  -53.,   40.]])
        basis0 = BSplineBasis(4, np.array([ -3.2,  -2.3,  -0.9,   0. ,   1.2,   1.7,   2.9,   4.2,   5.2,   5.7,   7.3,   8.3,   8.8,
         9.7,  11.1,  12. ,  13.2,  13.7,  14.9]),2)
        basis1 = BSplineBasis(4, np.array([ 0. ,  0. ,  0. ,  0. ,  0.7,  2.2,  2.9,  4. ,  4. ,  4. ,  4. ]))
        basis2 = BSplineBasis(2, np.array([ 0. ,  0. ,  1.2,  1.8,  3.4,  4.2,  5.1,  6. ,  6. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 3, 2)
        self.assertEqual(vol2.order(direction=0), 5)
        self.assertEqual(vol2.order(direction=1), 7)
        self.assertEqual(vol2.order(direction=2), 4)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p7_rational(self):
        controlpoints = np.array([[  1. ,   3. ,   0.8],
       [ 13. ,   3. ,   1.5],
       [ 13. ,  -1. ,   1.4],
       [ 22. ,  -4. ,   0.7],
       [ 37. ,   4. ,   0.7],
       [ 41. ,   3. ,   0.9],
       [ 49. ,  -1. ,   0.6],
       [ 61. ,  -2. ,   1.4],
       [ 68. ,  -2. ,   0.8],
       [ 82. ,  -4. ,   0.8],
       [ 91. ,  -4. ,   0.5],
       [ 97. ,  -5. ,   0.5]])
        basis0 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  1.9,  2.9,  4.1,  5.3,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 8)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p67_rational(self):
        controlpoints = np.array([[   2. ,    4. ,    1.4],       [  14. ,    4. ,    0.6],       [  20. ,    0. ,    0.5],       [  31. ,    2. ,    0.7],       [  36. ,    4. ,    0.8],       [  49. ,    2. ,    0.9],       [  64. ,   -1. ,    0.8],       [  71. ,    2. ,    0.6],       [  78. ,    2. ,    1.3],       [  93. ,   -2. ,    1.2],       [ 100. ,    4. ,    0.6],       [   3. ,    7. ,    0.6],       [  13. ,   13. ,    0.5],       [  20. ,    5. ,    1. ],       [  31. ,   11. ,    0.8],       [  42. ,    7. ,    0.7],       [  45. ,   12. ,    1.4],       [  56. ,   11. ,    1.2],       [  70. ,   11. ,    1.2],       [  77. ,    6. ,    0.8],       [  86. ,    6. ,    0.6],       [ 102. ,   11. ,    0.8],       [  -3. ,   22. ,    0.6],       [   7. ,   20. ,    1.4],       [  22. ,   22. ,    0.7],       [  28. ,   22. ,    1.2],       [  40. ,   14. ,    1.1],       [  46. ,   23. ,    1.1],       [  62. ,   16. ,    1.2],       [  72. ,   16. ,    1.1],       [  77. ,   14. ,    1.3],       [  92. ,   21. ,    0.7],       [ 101. ,   18. ,    0.8],       [   3. ,   29. ,    1.3],       [   6. ,   30. ,    1.1],       [  22. ,   28. ,    1.2],       [  29. ,   30. ,    1.2],       [  38. ,   24. ,    0.5],       [  52. ,   26. ,    0.7],       [  63. ,   23. ,    0.7],       [  74. ,   23. ,    1.4],       [  75. ,   23. ,    1. ],       [  87. ,   29. ,    1. ],       [  97. ,   28. ,    1.3],       [   0. ,   40. ,    0.9],       [   9. ,   37. ,    0.8],       [  19. ,   32. ,    1.4],       [  33. ,   39. ,    0.5],       [  36. ,   31. ,    1.3],       [  50. ,   33. ,    0.8],       [  56. ,   40. ,    1.3],       [  72. ,   36. ,    1.3],       [  83. ,   35. ,    0.7],       [  94. ,   34. ,    1.2],       [  96. ,   37. ,    0.8],       [   3. ,   41. ,    0.6],       [  12. ,   41. ,    0.9],       [  18. ,   45. ,    1.3],       [  34. ,   46. ,    1.3],       [  37. ,   49. ,    1.2],       [  49. ,   50. ,    1.1],       [  55. ,   46. ,    0.6],       [  68. ,   46. ,    1.3],       [  84. ,   48. ,    0.7],       [  87. ,   45. ,    1.3],       [  96. ,   45. ,    1.5],       [   1. ,   54. ,    0.6],       [  10. ,   55. ,    0.8],       [  22. ,   57. ,    1.3],       [  31. ,   54. ,    1.2],       [  40. ,   55. ,    0.6],       [  52. ,   50. ,    0.7],       [  60. ,   54. ,    0.8],       [  68. ,   58. ,    1. ],       [  82. ,   57. ,    1.2],       [  90. ,   56. ,    0.6],       [  96. ,   56. ,    1.3],       [  -5. ,   66. ,    0.9],       [  10. ,   59. ,    0.9],       [  17. ,   60. ,    1.3],       [  25. ,   68. ,    1.1],       [  44. ,   60. ,    0.5],       [  45. ,   63. ,    1.2],       [  57. ,   64. ,    1. ],       [  74. ,   65. ,    0.8],       [  77. ,   58. ,    0.5],       [  90. ,   58. ,    1.5],       [ 102. ,   62. ,    1.4],       [  -2. ,   71. ,    1.4],       [  13. ,   68. ,    1.3],       [  17. ,   70. ,    1.2],       [  32. ,   70. ,    1.3],       [  38. ,   73. ,    1.3],       [  51. ,   72. ,    1.5],       [  58. ,   70. ,    0.8],       [  65. ,   75. ,    1.3],       [  82. ,   73. ,    1. ],       [  90. ,   67. ,    1.1],       [  98. ,   77. ,    1.3],       [  -3. ,   79. ,    0.9],       [  14. ,   82. ,    0.5],       [  16. ,   79. ,    0.7],       [  26. ,   86. ,    0.7],       [  44. ,   81. ,    0.5],       [  53. ,   84. ,    1.2],       [  61. ,   85. ,    0.6],       [  73. ,   82. ,    0.9],       [  83. ,   82. ,    1.2],       [  91. ,   82. ,    0.9],       [ 100. ,   79. ,    1.4],       [   4. ,   94. ,    0.9],       [   5. ,   95. ,    1.3],       [  23. ,   93. ,    1.2],       [  27. ,   87. ,    1.4],       [  35. ,   92. ,    1.2],       [  51. ,   88. ,    1.1],       [  58. ,   95. ,    0.6],       [  73. ,   88. ,    1.2],       [  84. ,   94. ,    1. ],       [  86. ,   90. ,    0.9],       [ 101. ,   86. ,    1.1],       [  -2. ,   98. ,    0.9],       [  12. ,  104. ,    1. ],       [  23. ,  100. ,    1.2],       [  28. ,   97. ,    1.3],       [  35. ,   98. ,    0.9],       [  48. ,   99. ,    0.6],       [  56. ,   97. ,    1. ],       [  66. ,   96. ,    1.2],       [  76. ,   95. ,    0.5],       [  87. ,  102. ,    0.9],       [ 101. ,  103. ,    0.5]])
        basis0 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  1.7,  2.6,  3.7,  5.4,  6. ,  6. ,  6. ,  6. ,
        6. ,  6. ]))
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  2.1,  3. ,  4. ,  4.9,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(2, 1)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p7_rational_C0_periodic(self):
        controlpoints = np.array([[ 103. ,   -4. ,    1.3],
       [  87. ,   38. ,    0.5],
       [  58. ,   79. ,    1.1],
       [  26. ,   98. ,    1. ],
       [ -18. ,   94. ,    1.1],
       [ -66. ,   82. ,    0.8],
       [ -94. ,   47. ,    0.9],
       [ -97. ,   -3. ,    1.2],
       [ -87. ,  -49. ,    0.5],
       [ -67. ,  -82. ,    1.2],
       [ -18. , -101. ,    1.3],
       [  19. , -103. ,    1.1],
       [  58. ,  -82. ,    1.1],
       [  92. ,  -49. ,    1.2]])
        basis0 = BSplineBasis(7, np.array([ -0.7,   0. ,   0. ,   0. ,   0. ,   0. ,   0. ,   1. ,   2. ,   2.7,   4.4,   5.2,   5.6,
         6.8,   8.3,   9. ,   9. ,   9. ,   9. ,   9. ,   9. ,  10. ]),0)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 10)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p55_rational_C0_periodic(self):
        controlpoints = np.array([[  56. ,    1. ,    0.8],       [  45. ,   39. ,    1.4],       [  13. ,   61. ,    1.3],       [ -23. ,   52. ,    1.4],       [ -52. ,   38. ,    1.2],       [ -62. ,   -4. ,    0.9],       [ -49. ,  -37. ,    1.3],       [ -20. ,  -54. ,    1.4],       [  14. ,  -58. ,    0.8],       [  45. ,  -35. ,    0.9],       [  61. ,   -1. ,    1. ],       [  52. ,   33. ,    0.7],       [  17. ,   62. ,    1. ],       [ -21. ,   58. ,    1.2],       [ -50. ,   40. ,    1.5],       [ -67. ,    0. ,    0.8],       [ -52. ,  -36. ,    0.9],       [ -21. ,  -62. ,    1.5],       [  15. ,  -60. ,    0.7],       [  55. ,  -35. ,    1.3],       [  71. ,   -1. ,    1.3],       [  53. ,   42. ,    1. ],       [  16. ,   61. ,    1.1],       [ -26. ,   61. ,    1.3],       [ -55. ,   37. ,    1.4],       [ -73. ,   -1. ,    1.3],       [ -54. ,  -37. ,    1.2],       [ -21. ,  -62. ,    0.8],       [  19. ,  -65. ,    1.2],       [  51. ,  -45. ,    0.6],       [  77. ,    0. ,    0.8],       [  64. ,   47. ,    1.1],       [  23. ,   70. ,    1.2],       [ -22. ,   70. ,    1.3],       [ -58. ,   40. ,    1.1],       [ -70. ,    3. ,    1.1],       [ -60. ,  -40. ,    1.1],       [ -19. ,  -70. ,    1.5],       [  22. ,  -73. ,    1. ],       [  55. ,  -48. ,    1.3],       [  73. ,   -4. ,    1.1],       [  61. ,   45. ,    0.7],       [  25. ,   72. ,    0.6],       [ -22. ,   72. ,    0.8],       [ -62. ,   46. ,    0.8],       [ -82. ,    0. ,    1.5],       [ -62. ,  -44. ,    0.9],       [ -29. ,  -70. ,    0.7],       [  21. ,  -73. ,    1.2],       [  64. ,  -49. ,    1.3],       [  78. ,   -1. ,    1.4],       [  63. ,   50. ,    0.6],       [  28. ,   79. ,    1.4],       [ -25. ,   75. ,    0.6],       [ -63. ,   44. ,    0.7],       [ -79. ,   -4. ,    0.7],       [ -70. ,  -54. ,    1. ],       [ -22. ,  -74. ,    1.2],       [  21. ,  -74. ,    1.1],       [  69. ,  -53. ,    0.6],       [  85. ,   -1. ,    1.3],       [  71. ,   49. ,    1.3],       [  22. ,   84. ,    0.6],       [ -30. ,   80. ,    1.2],       [ -72. ,   49. ,    1.3],       [ -85. ,    3. ,    0.7],       [ -73. ,  -56. ,    0.7],       [ -29. ,  -81. ,    0.5],       [  24. ,  -81. ,    0.9],       [  70. ,  -55. ,    0.8],       [  90. ,    2. ,    1.3],       [  77. ,   54. ,    1.3],       [  29. ,   87. ,    0.6],       [ -27. ,   89. ,    0.8],       [ -75. ,   58. ,    1.2],       [ -91. ,    1. ,    0.8],       [ -77. ,  -52. ,    1. ],       [ -31. ,  -83. ,    0.7],       [  29. ,  -90. ,    1.5],       [  77. ,  -58. ,    0.5],       [  99. ,    3. ,    1. ],       [  77. ,   51. ,    1.4],       [  30. ,   86. ,    1.2],       [ -33. ,   90. ,    1.2],       [ -76. ,   60. ,    1.4],       [ -96. ,   -5. ,    0.6],       [ -82. ,  -54. ,    0.8],       [ -34. ,  -94. ,    0.8],       [  27. ,  -88. ,    1.1],       [  74. ,  -55. ,    1. ],       [  99. ,    0. ,    0.6],       [  82. ,   54. ,    0.9],       [  32. ,   93. ,    0.5],       [ -28. ,   95. ,    0.9],       [ -77. ,   63. ,    0.5],       [-101. ,    2. ,    1.2],       [ -86. ,  -63. ,    1. ],       [ -36. ,  -97. ,    1.3],       [  33. ,  -96. ,    1.2],       [  82. ,  -61. ,    1.2]])
        basis0 = BSplineBasis(5, np.array([-0.8,  0. ,  0. ,  0. ,  0. ,  1.2,  2.1,  2.9,  3.7,  4.8,  6.2,  7. ,  7. ,  7. ,  7. ,
        8.2]),0)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  1.9,  2.7,  3.9,  5.1,  6. ,  6. ,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 2)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 7)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p7_rational_C1_periodic(self):
        controlpoints = np.array([[  95. ,    4. ,    0.9],
       [  88. ,   42. ,    1.2],
       [  72. ,   68. ,    0.9],
       [  37. ,   91. ,    1.4],
       [  -4. ,   97. ,    1.3],
       [ -43. ,   89. ,    0.7],
       [ -74. ,   67. ,    1.2],
       [ -98. ,   38. ,    1.4],
       [-102. ,    3. ,    1. ],
       [ -97. ,  -36. ,    0.8],
       [ -72. ,  -70. ,    1.2],
       [ -36. ,  -95. ,    1. ],
       [   0. , -102. ,    1.1],
       [  37. ,  -88. ,    1.2],
       [  70. ,  -74. ,    1. ],
       [  96. ,  -36. ,    1.2]])
        basis0 = BSplineBasis(7, np.array([ -2.1,  -0.7,   0. ,   0. ,   0. ,   0. ,   0. ,   0.6,   2.3,   3. ,   4.3,   5.2,   5.6,
         7. ,   8.3,   8.8,   9.9,  11.3,  12. ,  12. ,  12. ,  12. ,  12. ,  12.6,  14.3]),1)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p65_rational_C1_periodic(self):
        controlpoints = np.array([[  57. ,   -5. ,    0.5],       [  54. ,   30. ,    1. ],       [  35. ,   45. ,    1.3],       [  17. ,   55. ,    1.3],       [ -11. ,   53. ,    0.9],       [ -37. ,   43. ,    0.8],       [ -58. ,   29. ,    1.1],       [ -60. ,   -4. ,    1.2],       [ -55. ,  -22. ,    0.6],       [ -35. ,  -44. ,    1.3],       [ -17. ,  -56. ,    0.8],       [  13. ,  -58. ,    0.7],       [  36. ,  -47. ,    1.5],       [  57. ,  -24. ,    1.3],       [  61. ,    3. ,    0.9],       [  54. ,   31. ,    1.1],       [  36. ,   54. ,    1. ],       [  10. ,   60. ,    0.8],       [ -16. ,   59. ,    0.6],       [ -44. ,   48. ,    0.8],       [ -63. ,   32. ,    0.6],       [ -64. ,   -5. ,    0.8],       [ -58. ,  -27. ,    0.6],       [ -41. ,  -47. ,    1. ],       [ -11. ,  -59. ,    0.5],       [  10. ,  -67. ,    0.5],       [  37. ,  -56. ,    1.2],       [  57. ,  -25. ,    1.4],       [  67. ,   -3. ,    1.2],       [  64. ,   31. ,    1.4],       [  45. ,   57. ,    0.8],       [  19. ,   68. ,    1.1],       [ -20. ,   66. ,    0.9],       [ -42. ,   55. ,    1.5],       [ -61. ,   25. ,    0.5],       [ -73. ,    4. ,    0.7],       [ -61. ,  -28. ,    0.9],       [ -40. ,  -50. ,    1.3],       [ -19. ,  -68. ,    1.4],       [  15. ,  -70. ,    0.8],       [  39. ,  -53. ,    1.4],       [  66. ,  -29. ,    0.7],       [  77. ,    1. ,    1.4],       [  63. ,   27. ,    0.8],       [  42. ,   61. ,    1.3],       [  11. ,   69. ,    0.8],       [ -19. ,   73. ,    0.7],       [ -46. ,   55. ,    1.1],       [ -63. ,   34. ,    1.5],       [ -77. ,   -4. ,    1.1],       [ -64. ,  -31. ,    1.2],       [ -42. ,  -56. ,    1.4],       [ -17. ,  -69. ,    1. ],       [  15. ,  -75. ,    0.6],       [  44. ,  -57. ,    0.8],       [  69. ,  -30. ,    1. ],       [  78. ,    3. ,    0.6],       [  73. ,   37. ,    0.7],       [  45. ,   56. ,    1.1],       [  17. ,   80. ,    0.7],       [ -18. ,   72. ,    0.5],       [ -49. ,   57. ,    0.7],       [ -72. ,   33. ,    0.9],       [ -77. ,   -4. ,    1.3],       [ -71. ,  -29. ,    0.9],       [ -49. ,  -61. ,    1. ],       [ -14. ,  -75. ,    1.5],       [  15. ,  -76. ,    1. ],       [  52. ,  -59. ,    1.3],       [  68. ,  -32. ,    0.8],       [  83. ,    1. ,    1. ],       [  76. ,   40. ,    0.6],       [  51. ,   64. ,    0.7],       [  19. ,   82. ,    1. ],       [ -20. ,   82. ,    1.5],       [ -51. ,   67. ,    1.5],       [ -80. ,   34. ,    0.8],       [ -80. ,    0. ,    0.7],       [ -70. ,  -34. ,    1.1],       [ -50. ,  -61. ,    0.7],       [ -22. ,  -78. ,    1.3],       [  19. ,  -84. ,    0.9],       [  53. ,  -66. ,    1.2],       [  73. ,  -32. ,    1.4],       [  87. ,   -4. ,    0.7],       [  77. ,   35. ,    0.8],       [  58. ,   66. ,    0.6],       [  18. ,   88. ,    1.2],       [ -19. ,   89. ,    0.5],       [ -54. ,   71. ,    0.6],       [ -78. ,   36. ,    0.8],       [ -91. ,   -1. ,    1.4],       [ -75. ,  -40. ,    1.1],       [ -54. ,  -73. ,    0.7],       [ -21. ,  -80. ,    0.7],       [  15. ,  -82. ,    1. ],       [  56. ,  -69. ,    0.5],       [  82. ,  -34. ,    0.9],       [  91. ,   -4. ,    1.4],       [  77. ,   43. ,    1.4],       [  53. ,   68. ,    1. ],       [  18. ,   87. ,    1.5],       [ -23. ,   90. ,    0.7],       [ -55. ,   71. ,    1.5],       [ -86. ,   38. ,    0.6],       [ -90. ,    2. ,    1.2],       [ -79. ,  -38. ,    1.5],       [ -57. ,  -73. ,    1.3],       [ -20. ,  -85. ,    0.9],       [  15. ,  -86. ,    1.3],       [  56. ,  -68. ,    0.7],       [  77. ,  -38. ,    1.2],       [  96. ,   -1. ,    1.4],       [  85. ,   45. ,    1.3],       [  56. ,   73. ,    0.7],       [  20. ,   90. ,    1. ],       [ -20. ,   95. ,    1.2],       [ -62. ,   70. ,    0.7],       [ -88. ,   40. ,    0.8],       [ -93. ,    4. ,    1.3],       [ -91. ,  -42. ,    1.3],       [ -64. ,  -78. ,    0.8],       [ -18. ,  -95. ,    1.4],       [  22. ,  -96. ,    0.5],       [  63. ,  -78. ,    0.6],       [  86. ,  -42. ,    0.7],       [  96. ,   -4. ,    1.2],       [  85. ,   44. ,    1.4],       [  58. ,   82. ,    1.4],       [  21. ,   93. ,    1.3],       [ -18. ,   94. ,    0.9],       [ -59. ,   80. ,    1. ],       [ -95. ,   46. ,    1.5],       [-105. ,    2. ,    0.6],       [ -86. ,  -42. ,    1.4],       [ -65. ,  -75. ,    1.5],       [ -28. ,  -99. ,    1.3],       [  26. ,  -99. ,    0.8],       [  66. ,  -77. ,    0.5],       [  86. ,  -39. ,    1.4]])
        basis0 = BSplineBasis(6, np.array([ -2.1,  -1.1,   0. ,   0. ,   0. ,   0. ,   1.1,   1.8,   2.9,   4. ,   5.3,   6. ,   6.9,
         7.8,   8.9,   9.9,  11. ,  11. ,  11. ,  11. ,  12.1,  12.8]),1)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  2. ,  2.8,  3.8,  4.6,  6. ,  6. ,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 1)
        self.assertEqual(surf2.order(direction=0), 9)
        self.assertEqual(surf2.order(direction=1), 6)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p7_rational_C2_periodic(self):
        controlpoints = np.array([[ 103. ,   -1. ,    1.2],
       [  92. ,   36. ,    0.8],
       [  74. ,   60. ,    1.2],
       [  50. ,   87. ,    0.8],
       [  18. ,   94. ,    0.7],
       [ -15. ,   93. ,    1. ],
       [ -54. ,   83. ,    1.5],
       [ -75. ,   59. ,    0.9],
       [ -92. ,   29. ,    0.8],
       [ -96. ,   -1. ,    1.4],
       [ -91. ,  -33. ,    1. ],
       [ -80. ,  -67. ,    0.9],
       [ -52. ,  -89. ,    0.5],
       [ -14. , -103. ,    1.3],
       [  17. ,  -99. ,    1.2],
       [  45. ,  -83. ,    1.5],
       [  81. ,  -68. ,    0.8],
       [  91. ,  -39. ,    0.6]])
        basis0 = BSplineBasis(7, np.array([ -2.6,  -1.8,  -0.9,   0. ,   0. ,   0. ,   0. ,   1.4,   1.7,   3.3,   4.3,   5. ,   6. ,
         6.6,   7.7,   9.3,  10.1,  10.6,  12.4,  13.2,  14.1,  15. ,  15. ,  15. ,  15. ,  16.4,
        16.7,  18.3]),2)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 8)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p55_rational_C2_periodic(self):
        controlpoints = np.array([[  60. ,   -2. ,    0.6],       [  49. ,   28. ,    1.4],       [  41. ,   47. ,    0.7],       [  11. ,   60. ,    0.8],       [ -16. ,   59. ,    0.9],       [ -41. ,   42. ,    1. ],       [ -51. ,   26. ,    1.5],       [ -57. ,    1. ,    1.3],       [ -58. ,  -31. ,    1.4],       [ -40. ,  -49. ,    1. ],       [ -17. ,  -58. ,    1.2],       [  14. ,  -60. ,    1. ],       [  40. ,  -43. ,    0.9],       [  51. ,  -29. ,    1.4],       [  69. ,    0. ,    1.3],       [  64. ,   26. ,    1.1],       [  44. ,   54. ,    0.8],       [  14. ,   65. ,    1.4],       [ -17. ,   69. ,    1.3],       [ -41. ,   48. ,    1.2],       [ -64. ,   26. ,    1.5],       [ -70. ,   -4. ,    1.3],       [ -62. ,  -27. ,    1.5],       [ -47. ,  -54. ,    0.8],       [ -18. ,  -69. ,    0.7],       [  17. ,  -62. ,    1.1],       [  40. ,  -56. ,    0.6],       [  56. ,  -30. ,    1.1],       [  77. ,   -5. ,    0.6],       [  70. ,   27. ,    1.1],       [  44. ,   54. ,    1.5],       [  19. ,   76. ,    0.6],       [ -18. ,   68. ,    1.4],       [ -45. ,   54. ,    1. ],       [ -67. ,   34. ,    0.6],       [ -72. ,   -2. ,    0.7],       [ -63. ,  -33. ,    1.1],       [ -43. ,  -60. ,    1.3],       [ -15. ,  -75. ,    1.4],       [  16. ,  -69. ,    1.3],       [  44. ,  -54. ,    0.8],       [  70. ,  -30. ,    1.1],       [  76. ,   -3. ,    0.8],       [  74. ,   39. ,    1.2],       [  50. ,   64. ,    1.5],       [  22. ,   80. ,    0.5],       [ -19. ,   79. ,    0.7],       [ -50. ,   67. ,    0.9],       [ -72. ,   35. ,    0.6],       [ -82. ,   -3. ,    0.7],       [ -76. ,  -37. ,    0.8],       [ -49. ,  -62. ,    1.2],       [ -19. ,  -83. ,    0.8],       [  22. ,  -80. ,    1.3],       [  53. ,  -59. ,    1.3],       [  69. ,  -40. ,    0.8],       [  90. ,   -2. ,    1. ],       [  80. ,   36. ,    0.7],       [  54. ,   64. ,    0.8],       [  21. ,   87. ,    1.5],       [ -18. ,   84. ,    1. ],       [ -52. ,   71. ,    1.2],       [ -79. ,   41. ,    1.4],       [ -87. ,    1. ,    0.9],       [ -81. ,  -34. ,    1.1],       [ -54. ,  -68. ,    1.4],       [ -22. ,  -87. ,    1.3],       [  23. ,  -86. ,    1. ],       [  58. ,  -70. ,    1. ],       [  76. ,  -40. ,    0.9],       [  92. ,   -4. ,    1.5],       [  81. ,   36. ,    0.9],       [  59. ,   73. ,    0.6],       [  18. ,   95. ,    1.3],       [ -18. ,   95. ,    0.9],       [ -60. ,   73. ,    0.7],       [ -85. ,   37. ,    1.3],       [ -89. ,   -1. ,    0.6],       [ -89. ,  -45. ,    1. ],       [ -64. ,  -72. ,    0.5],       [ -23. ,  -95. ,    0.7],       [  22. ,  -95. ,    1.3],       [  59. ,  -70. ,    1. ],       [  79. ,  -36. ,    0.9],       [ 100. ,   -5. ,    0.9],       [  86. ,   41. ,    1. ],       [  61. ,   73. ,    1.3],       [  21. ,   96. ,    1.5],       [ -22. ,   96. ,    0.5],       [ -58. ,   76. ,    1.1],       [ -92. ,   47. ,    0.8],       [-101. ,   -3. ,    1. ],       [ -92. ,  -46. ,    1.3],       [ -63. ,  -76. ,    0.9],       [ -21. ,  -95. ,    0.8],       [  22. ,  -97. ,    1.4],       [  59. ,  -79. ,    1.2],       [  86. ,  -42. ,    1.1]])
        basis0 = BSplineBasis(5, np.array([ -2.6,  -2. ,  -0.7,   0. ,   0. ,   1. ,   1.8,   3.4,   4. ,   4.8,   5.6,   7. ,   7.7,
         8.8,  10.4,  11. ,  12.3,  13. ,  13. ,  14. ,  14.8,  16.4]),2)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.8,  1.9,  3. ,  3. ,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(1, 3)
        self.assertEqual(surf2.order(direction=0), 6)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p5(self):
        controlpoints = np.array([[   1.,   -3.],
       [  10.,   -4.],
       [  28.,   -3.],
       [  38.,    3.],
       [  60.,   -5.],
       [  66.,   -1.],
       [  82.,    3.],
       [ 101.,   -1.]])
        basis0 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.6,  1.7,  3.3,  4. ,  4. ,  4. ,  4. ,  4. ]))
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 7)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p67(self):
        controlpoints = np.array([[   0.,    0.],       [  14.,   -1.],       [  25.,    2.],       [  32.,   -4.],       [  49.,    1.],       [  53.,    2.],       [  71.,   -5.],       [  74.,    2.],       [  90.,    3.],       [ 101.,   -4.],       [  -3.,   12.],       [   6.,    7.],       [  26.,    9.],       [  31.,   10.],       [  47.,    9.],       [  57.,   12.],       [  70.,   16.],       [  82.,    9.],       [  92.,   16.],       [  96.,   10.],       [   2.,   26.],       [  14.,   24.],       [  19.,   21.],       [  36.,   20.],       [  48.,   22.],       [  58.,   22.],       [  62.,   26.],       [  73.,   20.],       [  85.,   20.],       [ 103.,   28.],       [   1.,   33.],       [  12.,   42.],       [  23.,   41.],       [  37.,   40.],       [  43.,   35.],       [  60.,   41.],       [  70.,   36.],       [  77.,   32.],       [  85.,   34.],       [ 101.,   40.],       [   0.,   47.],       [   8.,   45.],       [  18.,   46.],       [  30.,   51.],       [  40.,   47.],       [  53.,   49.],       [  71.,   53.],       [  80.,   48.],       [  88.,   50.],       [ 101.,   53.],       [   2.,   61.],       [  15.,   58.],       [  22.,   59.],       [  32.,   62.],       [  45.,   61.],       [  53.,   59.],       [  65.,   61.],       [  76.,   64.],       [  85.,   66.],       [ 101.,   63.],       [  -4.,   72.],       [  12.,   72.],       [  26.,   75.],       [  35.,   79.],       [  45.,   74.],       [  59.,   78.],       [  67.,   71.],       [  74.,   78.],       [  85.,   77.],       [ 102.,   79.],       [   2.,   85.],       [   9.,   84.],       [  20.,   83.],       [  28.,   90.],       [  40.,   92.],       [  58.,   83.],       [  69.,   91.],       [  74.,   86.],       [  84.,   84.],       [  97.,   84.],       [  -1.,   99.],       [  14.,   97.],       [  24.,   95.],       [  32.,  100.],       [  44.,   97.],       [  57.,  102.],       [  63.,   97.],       [  73.,   95.],       [  92.,   97.],       [ 102.,   99.]])
        basis0 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.8,  2.2,  3.2,  4.3,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  2.3,  3. ,  3. ,  3. ,  3. ,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 3)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 10)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p6_C0_periodic(self):
        controlpoints = np.array([[  95.,    4.],
       [  83.,   50.],
       [  51.,   83.],
       [  -4.,  100.],
       [ -48.,   86.],
       [ -89.,   54.],
       [-101.,    1.],
       [ -90.,  -55.],
       [ -55.,  -83.],
       [   3.,  -98.],
       [  54.,  -89.],
       [  85.,  -46.]])
        basis0 = BSplineBasis(6, np.array([-0.8,  0. ,  0. ,  0. ,  0. ,  0. ,  1.4,  2. ,  3.4,  3.8,  5.3,  6.3,  7.2,  8. ,  8. ,
        8. ,  8. ,  8. ,  9.4]),0)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 7)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p66_C0_periodic(self):
        controlpoints = np.array([[  63.,   -5.],       [  48.,   25.],       [  34.,   56.],       [  -1.,   55.],       [ -27.,   50.],       [ -55.,   34.],       [ -58.,    3.],       [ -50.,  -28.],       [ -30.,  -55.],       [   2.,  -60.],       [  34.,  -56.],       [  56.,  -31.],       [  62.,   -5.],       [  58.,   30.],       [  36.,   55.],       [  -2.,   60.],       [ -34.,   54.],       [ -54.,   31.],       [ -70.,   -2.],       [ -60.,  -30.],       [ -29.,  -53.],       [  -5.,  -68.],       [  34.,  -60.],       [  59.,  -30.],       [  68.,   -4.],       [  56.,   38.],       [  37.,   56.],       [   3.,   64.],       [ -31.,   55.],       [ -58.,   35.],       [ -74.,   -4.],       [ -60.,  -38.],       [ -32.,  -56.],       [   4.,  -69.],       [  34.,  -62.],       [  64.,  -33.],       [  74.,   -1.],       [  58.,   37.],       [  34.,   64.],       [  -3.,   71.],       [ -38.,   58.],       [ -65.,   40.],       [ -72.,   -5.],       [ -63.,  -39.],       [ -34.,  -66.],       [   0.,  -74.],       [  40.,  -68.],       [  60.,  -41.],       [  82.,    2.],       [  62.,   39.],       [  37.,   66.],       [   1.,   81.],       [ -39.,   63.],       [ -65.,   39.],       [ -81.,   -5.],       [ -63.,  -43.],       [ -37.,  -71.],       [  -3.,  -78.],       [  35.,  -67.],       [  62.,  -44.],       [  83.,   -1.],       [  74.,   42.],       [  36.,   69.],       [   0.,   84.],       [ -41.,   69.],       [ -68.,   43.],       [ -85.,    0.],       [ -74.,  -42.],       [ -42.,  -70.],       [   4.,  -81.],       [  39.,  -74.],       [  68.,  -39.],       [  84.,    1.],       [  74.,   38.],       [  45.,   76.],       [   3.,   88.],       [ -44.,   70.],       [ -76.,   44.],       [ -91.,   -4.],       [ -77.,  -45.],       [ -46.,  -76.],       [   0.,  -87.],       [  46.,  -74.],       [  78.,  -42.],       [  95.,   -2.],       [  77.,   41.],       [  43.,   83.],       [  -1.,   92.],       [ -50.,   74.],       [ -79.,   40.],       [ -96.,   -5.],       [ -81.,  -42.],       [ -42.,  -84.],       [  -3.,  -93.],       [  48.,  -77.],       [  74.,  -42.],       [  94.,    4.],       [  78.,   45.],       [  49.,   80.],       [  -1.,   94.],       [ -52.,   86.],       [ -81.,   48.],       [ -95.,    0.],       [ -80.,  -51.],       [ -44.,  -81.],       [  -3.,  -95.],       [  46.,  -82.],       [  80.,  -52.],       [ 103.,   -3.],       [  88.,   49.],       [  46.,   86.],       [   0.,  104.],       [ -51.,   90.],       [ -90.,   48.],       [ -99.,   -3.],       [ -85.,  -46.],       [ -47.,  -84.],       [  -5.,  -98.],       [  48.,  -85.],       [  88.,  -48.]])
        basis0 = BSplineBasis(6, np.array([-1.2,  0. ,  0. ,  0. ,  0. ,  0. ,  0.7,  1.8,  3.2,  4.2,  4.9,  5.9,  6.8,  8. ,  8. ,
        8. ,  8. ,  8. ,  8.7]),0)
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  2. ,  3.3,  3.7,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 1)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 7)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p7_C1_periodic(self):
        controlpoints = np.array([[ 103.,    2.],
       [  92.,   43.],
       [  66.,   75.],
       [  42.,   91.],
       [  -1.,   97.],
       [ -38.,   90.],
       [ -67.,   69.],
       [ -91.,   35.],
       [-102.,    1.],
       [ -94.,  -38.],
       [ -76.,  -76.],
       [ -34.,  -96.],
       [  -3., -103.],
       [  36.,  -90.],
       [  73.,  -72.],
       [  91.,  -35.]])
        basis0 = BSplineBasis(7, np.array([ -2.2,  -0.9,   0. ,   0. ,   0. ,   0. ,   0. ,   1.2,   1.8,   3. ,   4.1,   5.3,   5.9,
         7.1,   7.6,   8.8,   9.8,  11.1,  12. ,  12. ,  12. ,  12. ,  12. ,  13.2,  13.8]),1)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p67_C1_periodic(self):
        controlpoints = np.array([[  55.,    2.],       [  56.,   30.],       [  37.,   43.],       [  15.,   61.],       [ -13.,   55.],       [ -39.,   45.],       [ -56.,   26.],       [ -62.,    2.],       [ -59.,  -23.],       [ -38.,  -49.],       [ -15.,  -59.],       [  13.,  -58.],       [  32.,  -44.],       [  53.,  -22.],       [  69.,    2.],       [  63.,   27.],       [  38.,   54.],       [  10.,   63.],       [ -20.,   66.],       [ -45.,   48.],       [ -64.,   27.],       [ -71.,    1.],       [ -60.,  -33.],       [ -44.,  -52.],       [ -16.,  -69.],       [  10.,  -64.],       [  40.,  -54.],       [  57.,  -27.],       [  68.,   -1.],       [  60.,   34.],       [  41.,   53.],       [  18.,   65.],       [ -21.,   71.],       [ -43.,   51.],       [ -64.,   31.],       [ -75.,    2.],       [ -67.,  -33.],       [ -50.,  -56.],       [ -15.,  -73.],       [  20.,  -74.],       [  41.,  -56.],       [  66.,  -35.],       [  81.,    0.],       [  71.,   38.],       [  43.,   56.],       [  12.,   79.],       [ -14.,   76.],       [ -46.,   56.],       [ -73.,   29.],       [ -79.,   -2.],       [ -67.,  -29.],       [ -48.,  -57.],       [ -18.,  -79.],       [  13.,  -71.],       [  51.,  -60.],       [  72.,  -38.],       [  83.,   -2.],       [  76.,   32.],       [  49.,   61.],       [  20.,   76.],       [ -22.,   80.],       [ -55.,   65.],       [ -79.,   34.],       [ -84.,   -1.],       [ -75.,  -39.],       [ -53.,  -63.],       [ -21.,  -81.],       [  20.,  -78.],       [  55.,  -67.],       [  77.,  -39.],       [  93.,   -1.],       [  80.,   35.],       [  56.,   68.],       [  20.,   84.],       [ -18.,   82.],       [ -51.,   65.],       [ -84.,   39.],       [ -88.,    4.],       [ -76.,  -39.],       [ -56.,  -68.],       [ -20.,  -84.],       [  17.,  -83.],       [  58.,  -74.],       [  77.,  -37.],       [  92.,    2.],       [  85.,   41.],       [  59.,   76.],       [  20.,   93.],       [ -25.,   95.],       [ -55.,   68.],       [ -88.,   45.],       [ -94.,    0.],       [ -85.,  -39.],       [ -57.,  -70.],       [ -17.,  -97.],       [  21.,  -95.],       [  62.,  -73.],       [  86.,  -38.],       [  99.,    1.],       [  87.,   45.],       [  66.,   76.],       [  22.,   98.],       [ -23.,   97.],       [ -62.,   76.],       [ -90.,   39.],       [ -97.,    2.],       [ -92.,  -43.],       [ -59.,  -75.],       [ -23., -102.],       [  23.,  -96.],       [  60.,  -81.],       [  91.,  -40.]])
        basis0 = BSplineBasis(6, np.array([ -1.6,  -0.7,   0. ,   0. ,   0. ,   0. ,   1.1,   1.7,   3.3,   4.2,   5. ,   5.8,   6.8,
         7.9,   9.4,  10.3,  11. ,  11. ,  11. ,  11. ,  12.1,  12.7]),1)
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  2. ,  2. ,  2. ,  2. ,  2. ,  2. ,  2. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(2, 2)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 9)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_2D_p6_C2_periodic(self):
        controlpoints = np.array([[ 101.,    3.],
       [  88.,   38.],
       [  67.,   73.],
       [  35.,   93.],
       [  -5.,  101.],
       [ -41.,   96.],
       [ -71.,   69.],
       [ -91.,   38.],
       [-100.,   -4.],
       [ -90.,  -38.],
       [ -72.,  -75.],
       [ -41.,  -88.],
       [   3.,  -97.],
       [  39.,  -96.],
       [  69.,  -72.],
       [  91.,  -40.]])
        basis0 = BSplineBasis(6, np.array([ -3.4,  -2.1,  -1.2,   0. ,   0. ,   0. ,   1. ,   2.1,   3.2,   4.1,   5.3,   5.8,   7.3,
         8.2,   9.1,  10. ,  10.6,  11.9,  12.8,  14. ,  14. ,  14. ,  15. ,  16.1,  17.2]),2)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 8)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_2D_p67_C2_periodic(self):
        controlpoints = np.array([[  60.,   -2.],       [  59.,   27.],       [  43.,   41.],       [  19.,   51.],       [   2.,   62.],       [ -23.,   57.],       [ -47.,   46.],       [ -51.,   23.],       [ -56.,   -3.],       [ -54.,  -24.],       [ -42.,  -47.],       [ -24.,  -51.],       [  -4.,  -65.],       [  22.,  -56.],       [  47.,  -43.],       [  54.,  -27.],       [  61.,   -5.],       [  61.,   22.],       [  42.,   40.],       [  27.,   54.],       [   2.,   68.],       [ -30.,   61.],       [ -42.,   45.],       [ -64.,   28.],       [ -66.,    4.],       [ -60.,  -26.],       [ -45.,  -47.],       [ -21.,  -56.],       [  -1.,  -68.],       [  28.,  -57.],       [  40.,  -43.],       [  61.,  -24.],       [  64.,   -5.],       [  66.,   24.],       [  45.,   46.],       [  21.,   67.],       [  -1.,   66.],       [ -29.,   58.],       [ -54.,   44.],       [ -60.,   25.],       [ -69.,    4.],       [ -63.,  -26.],       [ -48.,  -46.],       [ -29.,  -59.],       [  -2.,  -64.],       [  22.,  -60.],       [  46.,  -48.],       [  66.,  -29.],       [  75.,   -4.],       [  64.,   27.],       [  52.,   55.],       [  31.,   71.],       [  -5.,   68.],       [ -24.,   66.],       [ -54.,   47.],       [ -70.,   31.],       [ -77.,   -2.],       [ -69.,  -24.],       [ -49.,  -49.],       [ -30.,  -69.],       [   1.,  -76.],       [  27.,  -63.],       [  53.,  -47.],       [  64.,  -29.],       [  75.,   -5.],       [  73.,   28.],       [  49.,   51.],       [  33.,   72.],       [  -3.,   72.],       [ -26.,   74.],       [ -54.,   48.],       [ -70.,   32.],       [ -72.,   -4.],       [ -72.,  -29.],       [ -53.,  -57.],       [ -29.,  -70.],       [  -1.,  -74.],       [  32.,  -67.],       [  50.,  -56.],       [  73.,  -32.],       [  76.,    0.],       [  78.,   26.],       [  57.,   55.],       [  29.,   78.],       [  -5.,   82.],       [ -26.,   77.],       [ -57.,   60.],       [ -74.,   30.],       [ -77.,   -3.],       [ -70.,  -33.],       [ -53.,  -58.],       [ -33.,  -74.],       [  -2.,  -85.],       [  31.,  -71.],       [  54.,  -56.],       [  78.,  -31.],       [  84.,    4.],       [  79.,   30.],       [  55.,   55.],       [  30.,   76.],       [   4.,   82.],       [ -30.,   76.],       [ -55.,   57.],       [ -79.,   28.],       [ -82.,    2.],       [ -80.,  -28.],       [ -56.,  -61.],       [ -35.,  -83.],       [  -1.,  -80.],       [  27.,  -74.],       [  58.,  -59.],       [  74.,  -35.],       [  85.,   -1.],       [  82.,   32.],       [  64.,   58.],       [  38.,   78.],       [  -3.,   83.],       [ -32.,   80.],       [ -60.,   64.],       [ -81.,   30.],       [ -91.,   -1.],       [ -83.,  -32.],       [ -61.,  -63.],       [ -38.,  -86.],       [   4.,  -85.],       [  35.,  -81.],       [  59.,  -63.],       [  78.,  -39.],       [  92.,    0.],       [  81.,   30.],       [  61.,   68.],       [  34.,   82.],       [   2.,   90.],       [ -31.,   82.],       [ -70.,   60.],       [ -89.,   34.],       [ -88.,    1.],       [ -86.,  -39.],       [ -67.,  -65.],       [ -37.,  -89.],       [  -3.,  -94.],       [  33.,  -88.],       [  65.,  -62.],       [  84.,  -32.],       [ 100.,   -3.],       [  91.,   32.],       [  63.,   69.],       [  37.,   86.],       [  -3.,   97.],       [ -34.,   93.],       [ -70.,   71.],       [ -85.,   39.],       [-100.,   -4.],       [ -94.,  -35.],       [ -67.,  -67.],       [ -37.,  -94.],       [  -4.,  -92.],       [  33.,  -89.],       [  71.,  -70.],       [  91.,  -36.],       [ 104.,    2.],       [  89.,   38.],       [  66.,   71.],       [  39.,   94.],       [  -5.,  100.],       [ -44.,   94.],       [ -69.,   73.],       [ -95.,   39.],       [ -97.,   -5.],       [ -89.,  -39.],       [ -70.,  -68.],       [ -37.,  -91.],       [   0., -102.],       [  36.,  -91.],       [  73.,  -72.],       [  97.,  -38.]])
        basis0 = BSplineBasis(6, np.array([ -2.7,  -2.3,  -0.7,   0. ,   0. ,   0. ,   1.1,   2.1,   2.9,   3.9,   4.9,   5.8,   6.9,
         8.1,   8.8,  10.3,  11.3,  11.7,  13.3,  14. ,  14. ,  14. ,  15.1,  16.1,  16.9]),2)
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.7,  1.8,  2.7,  3.9,  5. ,  5. ,  5. ,  5. ,
        5. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(1, 1)
        self.assertEqual(surf2.order(direction=0), 7)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p7_rational(self):
        controlpoints = np.array([[  -2. ,    1. ,    2. ,    1.1],
       [   8. ,    2. ,    0. ,    0.6],
       [  21. ,    4. ,    2. ,    0.6],
       [  23. ,    3. ,   -4. ,    1. ],
       [  34. ,   -2. ,   -2. ,    1.4],
       [  47. ,   -3. ,   -5. ,    1.4],
       [  58. ,   -2. ,   -2. ,    1.5],
       [  63. ,   -2. ,    4. ,    1.4],
       [  72. ,   -5. ,   -1. ,    1.3],
       [  80. ,    2. ,    2. ,    0.7],
       [  93. ,    2. ,    1. ,    1.3],
       [ 101. ,   -2. ,   -4. ,    1.1]])
        basis0 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  2.2,  3.2,  3.9,  5.1,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p67_rational(self):
        controlpoints = np.array([[   0. ,    1. ,    1. ,    0.8],       [  12. ,   -4. ,   -1. ,    0.5],       [  20. ,   -3. ,   -2. ,    1.2],       [  28. ,   -2. ,    4. ,    0.8],       [  47. ,   -2. ,   -1. ,    0.7],       [  58. ,   -2. ,   -4. ,    0.7],       [  67. ,   -5. ,   -5. ,    1.1],       [  82. ,    4. ,   -4. ,    0.7],       [  91. ,    1. ,    1. ,    1.4],       [  98. ,   -2. ,    2. ,    1.3],       [   1. ,   14. ,    3. ,    0.8],       [  10. ,   13. ,   -5. ,    1.4],       [  21. ,   13. ,   -3. ,    0.7],       [  35. ,   13. ,   -2. ,    0.6],       [  48. ,    7. ,   -1. ,    1.2],       [  55. ,   14. ,   -4. ,    1.4],       [  68. ,   15. ,   -5. ,    0.8],       [  81. ,    6. ,    1. ,    0.7],       [  92. ,   12. ,   -2. ,    0.9],       [  98. ,    6. ,   -3. ,    1.4],       [  -2. ,   25. ,    3. ,    1.1],       [   9. ,   21. ,    2. ,    0.7],       [  21. ,   21. ,    2. ,    1.1],       [  37. ,   17. ,    4. ,    1. ],       [  45. ,   22. ,   -5. ,    1.3],       [  56. ,   21. ,   -2. ,    0.7],       [  68. ,   26. ,    2. ,    0.9],       [  79. ,   21. ,   -1. ,    1.4],       [  84. ,   22. ,   -5. ,    0.6],       [ 101. ,   21. ,   -2. ,    0.8],       [   0. ,   28. ,    4. ,    0.6],       [   9. ,   30. ,   -1. ,    0.5],       [  21. ,   33. ,    1. ,    1.1],       [  37. ,   34. ,    0. ,    1.5],       [  43. ,   31. ,    0. ,    0.6],       [  56. ,   29. ,    4. ,    1.3],       [  69. ,   35. ,   -5. ,    1.1],       [  77. ,   37. ,    1. ,    1.1],       [  90. ,   32. ,    2. ,    0.9],       [  99. ,   36. ,    1. ,    1.3],       [  -2. ,   47. ,   -5. ,    0.7],       [  11. ,   41. ,   -4. ,    0.7],       [  20. ,   49. ,   -3. ,    0.6],       [  32. ,   41. ,   -2. ,    1.1],       [  44. ,   42. ,   -1. ,    0.8],       [  56. ,   41. ,    0. ,    0.5],       [  70. ,   40. ,   -1. ,    1.3],       [  81. ,   40. ,   -4. ,    1.1],       [  91. ,   41. ,    3. ,    1.2],       [ 100. ,   40. ,   -4. ,    0.6],       [   0. ,   52. ,   -1. ,    1.5],       [   9. ,   55. ,    3. ,    1.4],       [  19. ,   57. ,    0. ,    1. ],       [  37. ,   55. ,    1. ,    1.4],       [  47. ,   52. ,   -5. ,    1. ],       [  56. ,   58. ,   -1. ,    1.3],       [  66. ,   52. ,   -4. ,    0.8],       [  73. ,   58. ,   -4. ,    1.2],       [  90. ,   59. ,   -3. ,    0.7],       [ 104. ,   59. ,   -5. ,    0.8],       [   1. ,   67. ,    4. ,    1.3],       [  10. ,   62. ,    2. ,    1.1],       [  26. ,   66. ,    0. ,    1.1],       [  32. ,   69. ,   -5. ,    0.5],       [  49. ,   61. ,    2. ,    1.3],       [  57. ,   69. ,    0. ,    1.3],       [  71. ,   66. ,    0. ,    0.6],       [  73. ,   69. ,    3. ,    1.3],       [  93. ,   70. ,    3. ,    0.9],       [  99. ,   68. ,   -4. ,    0.9],       [   4. ,   74. ,   -3. ,    1. ],       [   8. ,   81. ,    3. ,    0.6],       [  24. ,   77. ,    4. ,    0.5],       [  35. ,   79. ,    3. ,    0.5],       [  45. ,   79. ,    0. ,    0.5],       [  51. ,   77. ,   -4. ,    1.1],       [  69. ,   81. ,    3. ,    1.4],       [  81. ,   77. ,    4. ,    1.2],       [  91. ,   80. ,   -2. ,    1.3],       [  98. ,   75. ,   -3. ,    0.9],       [  -2. ,   91. ,   -1. ,    0.5],       [  10. ,   86. ,    1. ,    1.4],       [  26. ,   91. ,   -1. ,    0.6],       [  33. ,   93. ,   -4. ,    1.1],       [  48. ,   85. ,    1. ,    1.1],       [  56. ,   84. ,    0. ,    1.3],       [  63. ,   91. ,    4. ,    1.2],       [  79. ,   91. ,   -5. ,    1.3],       [  92. ,   92. ,    2. ,    0.6],       [ 103. ,   84. ,   -3. ,    1.3],       [   3. ,  103. ,   -3. ,    1.1],       [  14. ,  104. ,    0. ,    0.6],       [  23. ,   96. ,   -2. ,    1.2],       [  34. ,   95. ,    4. ,    0.8],       [  47. ,   96. ,    3. ,    1.1],       [  51. ,  101. ,   -3. ,    1.5],       [  70. ,   95. ,    2. ,    1.3],       [  81. ,   95. ,    1. ,    1.5],       [  89. ,   99. ,    0. ,    0.6],       [ 103. ,   97. ,    4. ,    0.8]])
        basis0 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  1.8,  3.4,  4.3,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  1.9,  2.9,  4. ,  4. ,  4. ,  4. ,  4. ,
        4. ,  4. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(1, 2)
        self.assertEqual(surf2.order(direction=0), 7)
        self.assertEqual(surf2.order(direction=1), 9)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p667_rational(self):
        controlpoints = np.array([[   4. ,   -5. ,   -1. ,    0.5],       [  10. ,    2. ,   -4. ,    1.3],       [  21. ,    3. ,   -1. ,    0.8],       [  27. ,    4. ,   -4. ,    1.2],       [  41. ,    3. ,   -3. ,    1. ],       [  45. ,   -2. ,   -1. ,    0.6],       [  57. ,    4. ,    3. ,    1.1],       [  70. ,   -2. ,   -3. ,    0.7],       [  75. ,   -3. ,    2. ,    1.1],       [  87. ,    1. ,    3. ,    1. ],       [ 100. ,   -4. ,   -4. ,    0.6],       [  -5. ,    7. ,   -1. ,    1. ],       [   9. ,   12. ,   -2. ,    0.7],       [  15. ,   13. ,   -5. ,    1.1],       [  26. ,    9. ,    3. ,    1.4],       [  35. ,   14. ,   -2. ,    1.3],       [  51. ,   10. ,    2. ,    0.5],       [  55. ,   10. ,    2. ,    0.5],       [  65. ,   17. ,   -4. ,    0.5],       [  80. ,    7. ,   -5. ,    0.6],       [  85. ,   14. ,    3. ,    1.3],       [ 102. ,   10. ,   -3. ,    1.4],       [  -1. ,   25. ,    3. ,    0.7],       [  11. ,   20. ,   -3. ,    0.9],       [  24. ,   24. ,   -2. ,    1.5],       [  33. ,   29. ,    2. ,    1.5],       [  44. ,   29. ,   -3. ,    1.2],       [  47. ,   20. ,   -1. ,    1. ],       [  62. ,   24. ,   -5. ,    0.6],       [  67. ,   28. ,   -2. ,    1.3],       [  75. ,   22. ,    4. ,    1.4],       [  90. ,   27. ,   -4. ,    1.3],       [ 100. ,   23. ,   -4. ,    1.3],       [  -1. ,   39. ,    1. ,    1. ],       [  11. ,   42. ,   -1. ,    0.5],       [  17. ,   36. ,   -5. ,    1.3],       [  25. ,   34. ,   -2. ,    1. ],       [  39. ,   35. ,   -3. ,    1.1],       [  54. ,   39. ,   -2. ,    0.6],       [  56. ,   36. ,    0. ,    0.8],       [  69. ,   39. ,   -1. ,    1.2],       [  79. ,   34. ,    0. ,    0.8],       [  90. ,   32. ,   -5. ,    1.1],       [ 100. ,   40. ,    2. ,    1.4],       [  -1. ,   52. ,   -5. ,    0.6],       [  11. ,   49. ,   -3. ,    0.8],       [  18. ,   51. ,   -3. ,    0.7],       [  33. ,   50. ,   -2. ,    0.9],       [  35. ,   49. ,   -4. ,    1.5],       [  47. ,   46. ,   -3. ,    1. ],       [  58. ,   49. ,    1. ,    1.1],       [  73. ,   48. ,   -4. ,    1.4],       [  80. ,   46. ,    1. ,    0.8],       [  89. ,   52. ,   -1. ,    0.7],       [  99. ,   45. ,   -1. ,    1.4],       [   4. ,   62. ,    2. ,    1.1],       [   7. ,   62. ,    4. ,    0.9],       [  21. ,   65. ,   -5. ,    0.6],       [  27. ,   65. ,   -2. ,    1. ],       [  38. ,   65. ,   -5. ,    1.3],       [  51. ,   59. ,    1. ,    0.6],       [  59. ,   58. ,   -5. ,    0.7],       [  65. ,   65. ,   -4. ,    1.3],       [  82. ,   61. ,    4. ,    0.7],       [  86. ,   60. ,    0. ,    1. ],       [ 100. ,   62. ,   -3. ,    0.6],       [   4. ,   72. ,    2. ,    0.9],       [   7. ,   77. ,   -5. ,    1.3],       [  20. ,   75. ,    1. ,    0.6],       [  26. ,   77. ,    2. ,    0.7],       [  41. ,   78. ,   -3. ,    0.8],       [  51. ,   74. ,    0. ,    1.3],       [  59. ,   71. ,    4. ,    1.2],       [  68. ,   70. ,   -4. ,    0.8],       [  78. ,   76. ,   -4. ,    0.8],       [  93. ,   75. ,    4. ,    1.1],       [ 102. ,   75. ,    4. ,    1. ],       [  -1. ,   87. ,   -4. ,    1.1],       [  14. ,   90. ,   -4. ,    1. ],       [  19. ,   85. ,   -3. ,    1.4],       [  34. ,   91. ,    3. ,    1. ],       [  37. ,   82. ,   -1. ,    1.2],       [  50. ,   82. ,   -4. ,    0.9],       [  63. ,   86. ,   -1. ,    1.4],       [  67. ,   89. ,   -2. ,    1.1],       [  80. ,   90. ,   -1. ,    0.7],       [  92. ,   87. ,   -4. ,    0.5],       [  96. ,   85. ,    1. ,    1.3],       [  -4. ,   98. ,   -5. ,    0.9],       [  10. ,  103. ,    3. ,    0.7],       [  24. ,  100. ,   -1. ,    0.5],       [  27. ,  100. ,    2. ,    0.9],       [  40. ,  100. ,   -3. ,    0.7],       [  50. ,  104. ,   -2. ,    0.9],       [  57. ,   95. ,   -1. ,    1.1],       [  71. ,  104. ,    1. ,    1.4],       [  78. ,   98. ,    4. ,    0.6],       [  93. ,   98. ,    3. ,    0.7],       [  95. ,   95. ,   -2. ,    1.2],       [   1. ,    4. ,   12. ,    1.2],       [  14. ,   -5. ,    7. ,    0.8],       [  16. ,   -5. ,    9. ,    1.5],       [  29. ,    1. ,   13. ,    1.2],       [  41. ,   -4. ,    4. ,    0.6],       [  45. ,   -2. ,    8. ,    1.2],       [  63. ,    4. ,    4. ,    0.7],       [  66. ,    1. ,    4. ,    1.4],       [  77. ,    0. ,   13. ,    1.4],       [  86. ,    0. ,   13. ,    0.5],       [  96. ,   -2. ,    6. ,    1.4],       [  -2. ,   12. ,    9. ,    0.5],       [  14. ,   16. ,    5. ,    1. ],       [  22. ,    8. ,    8. ,    1.4],       [  29. ,   15. ,    5. ,    1.2],       [  36. ,   14. ,    4. ,    1.2],       [  52. ,   16. ,   11. ,    1.4],       [  64. ,    8. ,    4. ,    1.1],       [  72. ,   15. ,   12. ,    1.2],       [  80. ,   11. ,    8. ,    1.5],       [  86. ,    8. ,   13. ,    0.7],       [ 101. ,   16. ,   13. ,    0.7],       [  -3. ,   28. ,    7. ,    1.4],       [   6. ,   29. ,   10. ,    1.4],       [  24. ,   28. ,   13. ,    0.9],       [  32. ,   24. ,    8. ,    0.6],       [  44. ,   22. ,   12. ,    1. ],       [  52. ,   26. ,    4. ,    1.4],       [  64. ,   29. ,    5. ,    1.1],       [  69. ,   27. ,    7. ,    1.3],       [  84. ,   29. ,   11. ,    1.1],       [  85. ,   23. ,    5. ,    0.7],       [ 104. ,   25. ,    9. ,    1.2],       [   4. ,   34. ,   10. ,    1.4],       [   5. ,   38. ,    8. ,    0.8],       [  24. ,   38. ,    5. ,    0.7],       [  33. ,   37. ,   13. ,    0.7],       [  44. ,   34. ,   12. ,    1.5],       [  48. ,   32. ,    6. ,    1.5],       [  58. ,   37. ,    4. ,    0.5],       [  73. ,   40. ,   12. ,    1.3],       [  77. ,   34. ,    9. ,    0.8],       [  93. ,   41. ,    9. ,    0.8],       [  97. ,   39. ,   11. ,    1.2],       [   4. ,   46. ,   13. ,    1.2],       [  14. ,   52. ,    5. ,    1. ],       [  18. ,   54. ,    7. ,    1. ],       [  34. ,   54. ,   11. ,    1. ],       [  37. ,   48. ,   11. ,    1.1],       [  48. ,   47. ,   13. ,    0.5],       [  57. ,   49. ,   11. ,    0.7],       [  72. ,   53. ,    7. ,    0.6],       [  78. ,   53. ,    4. ,    1.1],       [  87. ,   54. ,   12. ,    0.7],       [  99. ,   45. ,   10. ,    0.8],       [   3. ,   57. ,   11. ,    0.7],       [   8. ,   64. ,   11. ,    1.3],       [  15. ,   66. ,    4. ,    0.5],       [  32. ,   65. ,   11. ,    1.1],       [  42. ,   64. ,   12. ,    1.2],       [  52. ,   67. ,    6. ,    0.5],       [  55. ,   58. ,   10. ,    1.3],       [  69. ,   59. ,   11. ,    0.8],       [  75. ,   63. ,    6. ,    1.3],       [  94. ,   67. ,    6. ,    1. ],       [  98. ,   64. ,   13. ,    0.9],       [  -4. ,   75. ,    4. ,    1.5],       [  14. ,   77. ,   12. ,    0.5],       [  20. ,   77. ,    5. ,    1.4],       [  30. ,   74. ,    9. ,    0.5],       [  43. ,   79. ,   11. ,    1.1],       [  54. ,   72. ,    4. ,    1.2],       [  63. ,   70. ,   10. ,    1.2],       [  70. ,   71. ,    7. ,    0.8],       [  77. ,   75. ,    7. ,    1. ],       [  92. ,   71. ,    7. ,    1.1],       [  97. ,   76. ,   11. ,    1.5],       [  -3. ,   87. ,    6. ,    1. ],       [  10. ,   87. ,    4. ,    1.4],       [  20. ,   85. ,    6. ,    1.3],       [  27. ,   84. ,    5. ,    1.3],       [  37. ,   85. ,    4. ,    1. ],       [  51. ,   89. ,   13. ,    1.4],       [  59. ,   87. ,    9. ,    0.8],       [  73. ,   83. ,    7. ,    0.8],       [  76. ,   91. ,    6. ,    1.2],       [  87. ,   88. ,   11. ,    1.2],       [  96. ,   90. ,   13. ,    1.2],       [   3. ,  104. ,    6. ,    0.8],       [  12. ,   99. ,   11. ,    0.8],       [  21. ,  103. ,    6. ,    1. ],       [  31. ,   99. ,   14. ,    0.6],       [  39. ,  102. ,   12. ,    0.9],       [  52. ,  103. ,    4. ,    0.9],       [  57. ,   96. ,    8. ,    0.8],       [  74. ,  103. ,    8. ,    1.4],       [  79. ,   96. ,   10. ,    0.9],       [  94. ,  103. ,    9. ,    1.2],       [ 104. ,  103. ,   10. ,    0.8],       [  -4. ,   -1. ,   23. ,    1. ],       [  13. ,   -1. ,   19. ,    1. ],       [  16. ,    2. ,   20. ,    1.1],       [  34. ,   -4. ,   18. ,    1.1],       [  36. ,    3. ,   18. ,    1.1],       [  53. ,    2. ,   19. ,    0.8],       [  61. ,   -1. ,   14. ,    1. ],       [  66. ,   -3. ,   14. ,    1.1],       [  78. ,   -3. ,   13. ,    1.2],       [  92. ,   -3. ,   17. ,    1.5],       [ 103. ,   -2. ,   22. ,    0.7],       [  -4. ,   11. ,   20. ,    0.8],       [  11. ,   13. ,   20. ,    1.1],       [  18. ,   16. ,   15. ,    0.6],       [  25. ,   12. ,   22. ,    1.1],       [  40. ,    8. ,   13. ,    0.8],       [  50. ,   15. ,   14. ,    0.5],       [  62. ,   13. ,   22. ,    0.7],       [  72. ,    9. ,   22. ,    0.9],       [  82. ,   10. ,   19. ,    1.4],       [  92. ,   14. ,   13. ,    0.6],       [  99. ,    9. ,   22. ,    0.7],       [   0. ,   20. ,   22. ,    1.3],       [   8. ,   25. ,   22. ,    1.4],       [  20. ,   20. ,   18. ,    0.6],       [  31. ,   27. ,   20. ,    0.7],       [  42. ,   28. ,   17. ,    0.8],       [  51. ,   26. ,   15. ,    1.4],       [  56. ,   28. ,   14. ,    1.4],       [  74. ,   27. ,   13. ,    0.5],       [  79. ,   22. ,   19. ,    0.8],       [  94. ,   24. ,   20. ,    1.1],       [ 103. ,   22. ,   17. ,    1.4],       [  -3. ,   38. ,   21. ,    1.1],       [   6. ,   41. ,   14. ,    1.5],       [  18. ,   36. ,   22. ,    1.3],       [  29. ,   35. ,   18. ,    1.2],       [  39. ,   34. ,   20. ,    1.3],       [  51. ,   37. ,   13. ,    1.2],       [  62. ,   42. ,   13. ,    1. ],       [  73. ,   32. ,   18. ,    0.9],       [  82. ,   38. ,   15. ,    1.1],       [  91. ,   41. ,   21. ,    1. ],       [  95. ,   33. ,   18. ,    0.8],       [  -2. ,   49. ,   22. ,    1.5],       [   6. ,   52. ,   17. ,    0.7],       [  18. ,   50. ,   16. ,    0.9],       [  29. ,   48. ,   20. ,    0.6],       [  37. ,   51. ,   20. ,    0.8],       [  51. ,   49. ,   22. ,    0.8],       [  60. ,   48. ,   21. ,    0.5],       [  72. ,   45. ,   22. ,    1. ],       [  76. ,   45. ,   17. ,    1.2],       [  91. ,   47. ,   16. ,    0.6],       [  97. ,   52. ,   16. ,    1. ],       [  -2. ,   66. ,   13. ,    1. ],       [   8. ,   65. ,   17. ,    0.5],       [  24. ,   61. ,   22. ,    1.3],       [  33. ,   66. ,   21. ,    0.7],       [  37. ,   59. ,   13. ,    0.7],       [  50. ,   60. ,   13. ,    1.3],       [  57. ,   65. ,   13. ,    1.4],       [  66. ,   62. ,   23. ,    1.3],       [  79. ,   59. ,   19. ,    0.8],       [  91. ,   67. ,   17. ,    0.9],       [  96. ,   58. ,   18. ,    1.4],       [   0. ,   72. ,   21. ,    1.4],       [   6. ,   79. ,   18. ,    0.7],       [  17. ,   79. ,   20. ,    0.9],       [  25. ,   74. ,   17. ,    1. ],       [  42. ,   75. ,   13. ,    0.6],       [  48. ,   76. ,   19. ,    1.2],       [  63. ,   77. ,   20. ,    0.6],       [  74. ,   70. ,   15. ,    1.3],       [  76. ,   72. ,   16. ,    0.7],       [  91. ,   75. ,   15. ,    0.8],       [  99. ,   72. ,   21. ,    1.5],       [   0. ,   86. ,   22. ,    0.8],       [   8. ,   85. ,   19. ,    1. ],       [  19. ,   83. ,   16. ,    1. ],       [  31. ,   86. ,   15. ,    0.5],       [  41. ,   84. ,   15. ,    0.7],       [  51. ,   89. ,   13. ,    1.2],       [  55. ,   83. ,   17. ,    0.9],       [  67. ,   88. ,   19. ,    0.8],       [  83. ,   82. ,   20. ,    1. ],       [  92. ,   87. ,   15. ,    1.2],       [ 100. ,   83. ,   17. ,    0.6],       [  -4. ,   95. ,   15. ,    1.4],       [   9. ,   98. ,   15. ,    0.7],       [  21. ,   96. ,   20. ,    0.7],       [  29. ,  103. ,   21. ,    0.5],       [  43. ,   96. ,   16. ,    1.2],       [  47. ,  100. ,   14. ,    1.3],       [  63. ,   96. ,   17. ,    1.2],       [  69. ,   95. ,   17. ,    0.6],       [  81. ,   99. ,   20. ,    1.2],       [  88. ,   96. ,   17. ,    0.9],       [ 101. ,   99. ,   21. ,    1.3],       [   4. ,   -3. ,   26. ,    0.5],       [  10. ,   -1. ,   25. ,    1.5],       [  19. ,   -2. ,   25. ,    0.7],       [  31. ,    4. ,   24. ,    1.2],       [  35. ,   -3. ,   29. ,    1.2],       [  51. ,    1. ,   31. ,    1.2],       [  58. ,    2. ,   24. ,    1. ],       [  69. ,   -2. ,   25. ,    0.9],       [  82. ,   -1. ,   31. ,    0.9],       [  91. ,   -1. ,   31. ,    1.5],       [  99. ,    2. ,   31. ,    0.6],       [   1. ,    8. ,   28. ,    0.8],       [   6. ,   14. ,   25. ,    1.2],       [  21. ,   16. ,   31. ,    1.1],       [  30. ,   15. ,   26. ,    1.1],       [  41. ,   12. ,   28. ,    0.8],       [  52. ,    8. ,   26. ,    0.5],       [  60. ,   10. ,   27. ,    0.9],       [  67. ,    8. ,   28. ,    0.5],       [  81. ,   13. ,   22. ,    1. ],       [  89. ,    8. ,   30. ,    1. ],       [  97. ,   11. ,   32. ,    0.5],       [  -1. ,   29. ,   28. ,    1.5],       [  14. ,   21. ,   31. ,    1.4],       [  22. ,   28. ,   27. ,    1.2],       [  33. ,   25. ,   23. ,    0.9],       [  40. ,   20. ,   26. ,    1.2],       [  49. ,   21. ,   26. ,    0.8],       [  56. ,   29. ,   30. ,    1.5],       [  65. ,   27. ,   28. ,    0.7],       [  82. ,   22. ,   22. ,    0.9],       [  93. ,   29. ,   31. ,    0.7],       [ 101. ,   28. ,   24. ,    0.9],       [  -4. ,   41. ,   23. ,    1. ],       [   8. ,   40. ,   28. ,    0.6],       [  24. ,   34. ,   27. ,    1.4],       [  34. ,   35. ,   30. ,    0.5],       [  42. ,   35. ,   31. ,    1.1],       [  50. ,   40. ,   24. ,    1.3],       [  56. ,   35. ,   30. ,    1.3],       [  70. ,   34. ,   27. ,    1.3],       [  82. ,   38. ,   27. ,    0.8],       [  88. ,   35. ,   28. ,    0.9],       [ 101. ,   36. ,   22. ,    0.9],       [  -4. ,   51. ,   26. ,    0.8],       [  14. ,   51. ,   25. ,    0.9],       [  18. ,   54. ,   30. ,    1.1],       [  30. ,   53. ,   31. ,    1.2],       [  41. ,   47. ,   28. ,    1. ],       [  50. ,   49. ,   23. ,    1.2],       [  57. ,   53. ,   22. ,    1.3],       [  65. ,   53. ,   25. ,    0.8],       [  80. ,   47. ,   22. ,    1.5],       [  90. ,   53. ,   31. ,    1.4],       [  96. ,   49. ,   25. ,    1. ],       [  -3. ,   66. ,   23. ,    0.9],       [   8. ,   59. ,   26. ,    0.9],       [  18. ,   59. ,   27. ,    1. ],       [  25. ,   58. ,   24. ,    1. ],       [  39. ,   60. ,   31. ,    0.9],       [  45. ,   67. ,   28. ,    0.7],       [  56. ,   63. ,   23. ,    0.9],       [  69. ,   65. ,   31. ,    1.3],       [  78. ,   67. ,   29. ,    1.3],       [  94. ,   60. ,   24. ,    1.5],       [  97. ,   62. ,   30. ,    0.6],       [   0. ,   78. ,   24. ,    1.2],       [  11. ,   73. ,   26. ,    1. ],       [  24. ,   77. ,   31. ,    0.7],       [  28. ,   76. ,   23. ,    0.9],       [  44. ,   78. ,   26. ,    0.7],       [  50. ,   74. ,   32. ,    1.2],       [  57. ,   70. ,   29. ,    1.1],       [  66. ,   71. ,   29. ,    1.1],       [  79. ,   73. ,   31. ,    1.1],       [  90. ,   71. ,   22. ,    0.6],       [  95. ,   79. ,   29. ,    1.1],       [  -1. ,   89. ,   25. ,    1.3],       [   5. ,   83. ,   28. ,    0.9],       [  23. ,   84. ,   28. ,    1.4],       [  29. ,   88. ,   31. ,    0.9],       [  36. ,   88. ,   22. ,    0.8],       [  54. ,   91. ,   28. ,    0.9],       [  56. ,   91. ,   29. ,    0.7],       [  69. ,   91. ,   26. ,    0.8],       [  84. ,   88. ,   22. ,    0.6],       [  94. ,   85. ,   30. ,    1.2],       [  98. ,   88. ,   22. ,    0.9],       [   3. ,   99. ,   23. ,    1.1],       [   7. ,  103. ,   32. ,    1.1],       [  17. ,  103. ,   29. ,    1.3],       [  34. ,  103. ,   24. ,    1.4],       [  39. ,  102. ,   25. ,    1.4],       [  49. ,  103. ,   31. ,    1.4],       [  58. ,  102. ,   23. ,    0.9],       [  74. ,  100. ,   30. ,    0.7],       [  83. ,   95. ,   26. ,    0.8],       [  94. ,   98. ,   22. ,    0.7],       [ 103. ,   95. ,   28. ,    0.6],       [   3. ,   -1. ,   34. ,    0.7],       [  13. ,    3. ,   31. ,    1.2],       [  19. ,    3. ,   40. ,    0.9],       [  34. ,    3. ,   34. ,    0.8],       [  38. ,   -2. ,   32. ,    0.6],       [  45. ,    3. ,   34. ,    1.1],       [  55. ,   -5. ,   35. ,    0.5],       [  67. ,    2. ,   40. ,    1.4],       [  80. ,    0. ,   34. ,    1.4],       [  85. ,   -4. ,   33. ,    0.9],       [  96. ,   -3. ,   32. ,    1. ],       [   2. ,   15. ,   39. ,    0.6],       [   6. ,    7. ,   33. ,    1. ],       [  17. ,   11. ,   40. ,    1.2],       [  31. ,   13. ,   38. ,    0.8],       [  35. ,    7. ,   41. ,    1.4],       [  52. ,   10. ,   32. ,    1.1],       [  59. ,   10. ,   39. ,    0.9],       [  69. ,   16. ,   40. ,    1.4],       [  75. ,   16. ,   40. ,    1. ],       [  94. ,   17. ,   31. ,    0.7],       [  99. ,   14. ,   34. ,    1. ],       [  -4. ,   26. ,   31. ,    0.7],       [   9. ,   22. ,   35. ,    1.1],       [  19. ,   24. ,   35. ,    1.1],       [  25. ,   24. ,   31. ,    1.3],       [  41. ,   25. ,   37. ,    0.9],       [  47. ,   21. ,   34. ,    0.8],       [  56. ,   21. ,   39. ,    0.9],       [  71. ,   29. ,   38. ,    1. ],       [  83. ,   24. ,   32. ,    1. ],       [  88. ,   29. ,   33. ,    0.8],       [ 101. ,   28. ,   33. ,    0.8],       [  -5. ,   40. ,   36. ,    0.9],       [   5. ,   40. ,   39. ,    1.4],       [  16. ,   41. ,   31. ,    0.6],       [  31. ,   37. ,   38. ,    0.7],       [  43. ,   34. ,   41. ,    1.5],       [  51. ,   35. ,   39. ,    0.8],       [  63. ,   35. ,   38. ,    0.6],       [  72. ,   37. ,   39. ,    0.8],       [  84. ,   40. ,   34. ,    0.8],       [  85. ,   35. ,   37. ,    0.6],       [ 100. ,   41. ,   33. ,    1.2],       [   2. ,   54. ,   36. ,    1.1],       [  11. ,   50. ,   33. ,    0.7],       [  19. ,   51. ,   38. ,    0.9],       [  25. ,   51. ,   34. ,    0.7],       [  40. ,   47. ,   35. ,    1.2],       [  45. ,   50. ,   31. ,    1.5],       [  56. ,   45. ,   38. ,    1.3],       [  74. ,   48. ,   34. ,    1.1],       [  82. ,   54. ,   39. ,    0.6],       [  90. ,   54. ,   34. ,    0.6],       [ 101. ,   48. ,   40. ,    1.2],       [  -1. ,   57. ,   34. ,    0.6],       [  13. ,   63. ,   33. ,    0.6],       [  24. ,   59. ,   39. ,    1.2],       [  31. ,   65. ,   35. ,    1.2],       [  38. ,   58. ,   33. ,    0.5],       [  45. ,   61. ,   33. ,    1.4],       [  56. ,   58. ,   34. ,    0.6],       [  70. ,   61. ,   39. ,    1.4],       [  83. ,   57. ,   36. ,    0.9],       [  85. ,   62. ,   35. ,    1.1],       [  98. ,   62. ,   35. ,    1.1],       [   1. ,   79. ,   39. ,    1. ],       [  13. ,   73. ,   33. ,    1.3],       [  16. ,   75. ,   36. ,    1. ],       [  33. ,   76. ,   40. ,    1.1],       [  38. ,   71. ,   35. ,    1.2],       [  53. ,   77. ,   38. ,    0.8],       [  56. ,   74. ,   33. ,    1.2],       [  68. ,   78. ,   36. ,    0.6],       [  83. ,   77. ,   39. ,    0.7],       [  87. ,   79. ,   39. ,    0.8],       [  99. ,   75. ,   40. ,    1.2],       [  -3. ,   87. ,   40. ,    0.8],       [  14. ,   89. ,   39. ,    1.3],       [  16. ,   91. ,   34. ,    0.6],       [  27. ,   84. ,   36. ,    1. ],       [  40. ,   86. ,   38. ,    0.9],       [  54. ,   83. ,   36. ,    0.6],       [  57. ,   88. ,   37. ,    0.9],       [  73. ,   91. ,   37. ,    0.9],       [  77. ,   90. ,   31. ,    0.6],       [  86. ,   84. ,   39. ,    1.4],       [ 101. ,   83. ,   39. ,    0.6],       [   3. ,  100. ,   41. ,    0.7],       [  13. ,   97. ,   37. ,    1. ],       [  18. ,   98. ,   36. ,    1.1],       [  26. ,   98. ,   32. ,    0.8],       [  38. ,   98. ,   31. ,    1.1],       [  47. ,   98. ,   32. ,    1.1],       [  61. ,   97. ,   40. ,    1.3],       [  66. ,  101. ,   34. ,    0.9],       [  78. ,  100. ,   37. ,    1.3],       [  90. ,  102. ,   32. ,    1. ],       [  96. ,  100. ,   36. ,    1.2],       [   2. ,    2. ,   42. ,    0.7],       [   9. ,   -1. ,   44. ,    1.2],       [  15. ,    4. ,   46. ,    0.6],       [  26. ,    2. ,   50. ,    0.9],       [  35. ,   -3. ,   43. ,    0.6],       [  47. ,   -5. ,   48. ,    1.1],       [  58. ,   -4. ,   43. ,    0.9],       [  71. ,    2. ,   49. ,    0.9],       [  84. ,   -5. ,   48. ,    0.8],       [  90. ,   -4. ,   44. ,    0.9],       [  95. ,   -5. ,   49. ,    1.3],       [   3. ,   15. ,   46. ,    0.7],       [  10. ,    9. ,   46. ,    1.3],       [  21. ,    7. ,   49. ,    0.8],       [  32. ,   16. ,   47. ,    0.7],       [  40. ,   14. ,   42. ,    1.3],       [  46. ,   10. ,   49. ,    1.3],       [  64. ,   16. ,   46. ,    0.8],       [  71. ,    9. ,   45. ,    1.3],       [  77. ,   13. ,   42. ,    0.6],       [  88. ,   10. ,   47. ,    1.4],       [ 100. ,   17. ,   43. ,    0.8],       [  -2. ,   28. ,   43. ,    0.8],       [   9. ,   21. ,   49. ,    0.6],       [  24. ,   25. ,   43. ,    1.3],       [  26. ,   23. ,   50. ,    1.5],       [  37. ,   20. ,   48. ,    1.4],       [  54. ,   23. ,   47. ,    1.3],       [  57. ,   28. ,   40. ,    0.7],       [  69. ,   27. ,   45. ,    1.1],       [  76. ,   23. ,   46. ,    0.7],       [  86. ,   21. ,   48. ,    1.4],       [ 104. ,   27. ,   47. ,    0.8],       [   3. ,   38. ,   48. ,    0.6],       [   9. ,   37. ,   44. ,    1.2],       [  24. ,   39. ,   40. ,    0.7],       [  26. ,   39. ,   40. ,    0.7],       [  39. ,   34. ,   50. ,    0.6],       [  54. ,   34. ,   42. ,    0.8],       [  62. ,   34. ,   45. ,    1.3],       [  67. ,   33. ,   43. ,    0.7],       [  75. ,   37. ,   49. ,    1.3],       [  86. ,   35. ,   47. ,    0.6],       [ 102. ,   34. ,   41. ,    0.9],       [   4. ,   54. ,   49. ,    1.3],       [  11. ,   50. ,   49. ,    0.8],       [  22. ,   51. ,   49. ,    1.2],       [  32. ,   51. ,   41. ,    1.3],       [  36. ,   45. ,   41. ,    1.1],       [  46. ,   52. ,   44. ,    1. ],       [  63. ,   51. ,   44. ,    1.3],       [  67. ,   51. ,   42. ,    1. ],       [  84. ,   53. ,   44. ,    1.4],       [  94. ,   51. ,   48. ,    0.9],       [ 104. ,   47. ,   48. ,    0.7],       [  -3. ,   66. ,   48. ,    1.2],       [  14. ,   66. ,   47. ,    1. ],       [  19. ,   66. ,   49. ,    0.7],       [  30. ,   60. ,   45. ,    1.1],       [  44. ,   67. ,   47. ,    0.7],       [  47. ,   59. ,   44. ,    0.6],       [  62. ,   57. ,   42. ,    0.8],       [  74. ,   61. ,   42. ,    1.2],       [  79. ,   63. ,   42. ,    1.2],       [  88. ,   65. ,   46. ,    1. ],       [ 101. ,   60. ,   41. ,    0.7],       [   3. ,   74. ,   42. ,    1. ],       [   9. ,   71. ,   48. ,    1.4],       [  23. ,   70. ,   49. ,    1.3],       [  26. ,   75. ,   46. ,    1.1],       [  37. ,   75. ,   48. ,    0.5],       [  50. ,   74. ,   50. ,    0.7],       [  57. ,   79. ,   41. ,    1.4],       [  66. ,   71. ,   46. ,    1.3],       [  83. ,   78. ,   41. ,    1.1],       [  92. ,   79. ,   46. ,    1.4],       [ 104. ,   75. ,   49. ,    0.8],       [  -5. ,   86. ,   47. ,    1.2],       [  11. ,   90. ,   47. ,    0.7],       [  21. ,   83. ,   47. ,    0.6],       [  29. ,   90. ,   41. ,    1.2],       [  43. ,   87. ,   40. ,    0.7],       [  50. ,   83. ,   41. ,    0.6],       [  63. ,   84. ,   42. ,    0.8],       [  71. ,   83. ,   45. ,    1.3],       [  84. ,   89. ,   44. ,    0.7],       [  87. ,   89. ,   41. ,    0.7],       [ 103. ,   83. ,   42. ,    0.7],       [  -3. ,   97. ,   40. ,    1.1],       [   7. ,   95. ,   44. ,    1.2],       [  21. ,  101. ,   43. ,    0.8],       [  26. ,   98. ,   42. ,    0.6],       [  38. ,  101. ,   46. ,    1.4],       [  47. ,  104. ,   49. ,    1.5],       [  57. ,  100. ,   43. ,    0.8],       [  74. ,  104. ,   44. ,    0.8],       [  80. ,   96. ,   46. ,    1.5],       [  89. ,   99. ,   49. ,    1.2],       [  99. ,  104. ,   41. ,    0.9],       [   4. ,    1. ,   55. ,    0.6],       [  13. ,   -2. ,   57. ,    1.2],       [  18. ,   -2. ,   56. ,    0.8],       [  34. ,   -4. ,   56. ,    0.9],       [  43. ,    3. ,   56. ,    1.4],       [  50. ,   -1. ,   53. ,    1.3],       [  58. ,   -4. ,   57. ,    1.2],       [  71. ,   -4. ,   54. ,    1.1],       [  77. ,    0. ,   53. ,    1.2],       [  88. ,    1. ,   58. ,    0.7],       [ 101. ,   -2. ,   56. ,    0.9],       [   4. ,   14. ,   50. ,    1. ],       [   7. ,   10. ,   55. ,    0.8],       [  22. ,   14. ,   55. ,    1.3],       [  32. ,   12. ,   52. ,    1.2],       [  37. ,   17. ,   53. ,    0.8],       [  52. ,   11. ,   59. ,    1. ],       [  61. ,   16. ,   49. ,    1. ],       [  69. ,   14. ,   53. ,    0.8],       [  83. ,    9. ,   51. ,    0.7],       [  91. ,   10. ,   50. ,    1. ],       [ 100. ,   11. ,   58. ,    0.6],       [  -5. ,   25. ,   53. ,    1.1],       [  14. ,   26. ,   55. ,    1.4],       [  19. ,   28. ,   51. ,    1.1],       [  28. ,   27. ,   53. ,    1.2],       [  39. ,   21. ,   54. ,    1.1],       [  50. ,   20. ,   52. ,    1.3],       [  55. ,   24. ,   58. ,    1.2],       [  71. ,   28. ,   55. ,    1.1],       [  76. ,   20. ,   51. ,    1.2],       [  91. ,   22. ,   57. ,    1.4],       [  99. ,   24. ,   56. ,    1.5],       [   1. ,   37. ,   59. ,    1. ],       [   8. ,   41. ,   53. ,    1.2],       [  23. ,   36. ,   57. ,    0.6],       [  34. ,   40. ,   50. ,    0.9],       [  40. ,   34. ,   55. ,    1.4],       [  49. ,   33. ,   58. ,    0.6],       [  55. ,   38. ,   54. ,    0.5],       [  70. ,   37. ,   52. ,    1. ],       [  78. ,   35. ,   53. ,    1.1],       [  94. ,   34. ,   53. ,    0.8],       [ 103. ,   34. ,   54. ,    1.3],       [   2. ,   51. ,   54. ,    0.5],       [  14. ,   50. ,   51. ,    0.7],       [  21. ,   51. ,   57. ,    0.9],       [  29. ,   50. ,   54. ,    0.8],       [  40. ,   46. ,   56. ,    1.4],       [  45. ,   51. ,   55. ,    1.1],       [  60. ,   51. ,   53. ,    0.8],       [  66. ,   47. ,   56. ,    1.1],       [  83. ,   54. ,   52. ,    0.7],       [  92. ,   52. ,   51. ,    1.3],       [  99. ,   54. ,   55. ,    0.9],       [   3. ,   65. ,   53. ,    0.8],       [  14. ,   65. ,   55. ,    0.8],       [  16. ,   64. ,   58. ,    0.9],       [  31. ,   63. ,   53. ,    1. ],       [  40. ,   65. ,   51. ,    1.1],       [  53. ,   67. ,   52. ,    0.8],       [  63. ,   65. ,   59. ,    1.1],       [  67. ,   67. ,   52. ,    0.9],       [  77. ,   66. ,   54. ,    1.1],       [  87. ,   62. ,   56. ,    0.6],       [ 101. ,   66. ,   49. ,    0.6],       [   1. ,   78. ,   57. ,    1.2],       [  14. ,   73. ,   53. ,    1.1],       [  19. ,   73. ,   53. ,    1. ],       [  27. ,   78. ,   59. ,    1. ],       [  43. ,   79. ,   54. ,    1.2],       [  49. ,   74. ,   52. ,    1.1],       [  55. ,   76. ,   54. ,    1.2],       [  66. ,   75. ,   53. ,    1.2],       [  82. ,   70. ,   51. ,    0.9],       [  86. ,   79. ,   50. ,    1.2],       [ 102. ,   73. ,   52. ,    1.3],       [   2. ,   86. ,   54. ,    0.9],       [  12. ,   83. ,   55. ,    0.9],       [  19. ,   88. ,   54. ,    1.3],       [  32. ,   88. ,   49. ,    1.4],       [  38. ,   91. ,   54. ,    0.9],       [  53. ,   91. ,   51. ,    1. ],       [  59. ,   91. ,   56. ,    0.9],       [  66. ,   83. ,   50. ,    0.5],       [  81. ,   87. ,   58. ,    1.1],       [  92. ,   90. ,   50. ,    1.4],       [ 101. ,   83. ,   57. ,    1.1],       [   1. ,  101. ,   57. ,    1.1],       [  10. ,   97. ,   54. ,    0.6],       [  17. ,   99. ,   55. ,    0.8],       [  26. ,  103. ,   59. ,    0.8],       [  41. ,   98. ,   57. ,    1. ],       [  48. ,   98. ,   53. ,    1.4],       [  55. ,  101. ,   51. ,    1.2],       [  69. ,  102. ,   55. ,    0.6],       [  81. ,  100. ,   58. ,    1.1],       [  94. ,  100. ,   57. ,    0.6],       [  97. ,   98. ,   55. ,    1. ],       [  -2. ,    1. ,   63. ,    0.5],       [  10. ,    0. ,   60. ,    1. ],       [  17. ,   -1. ,   68. ,    1. ],       [  27. ,    2. ,   60. ,    1.2],       [  37. ,   -5. ,   62. ,    0.8],       [  49. ,    2. ,   59. ,    0.6],       [  58. ,   -5. ,   59. ,    1.1],       [  69. ,   -3. ,   61. ,    0.8],       [  83. ,   -3. ,   60. ,    0.8],       [  92. ,    0. ,   65. ,    0.6],       [  96. ,    3. ,   61. ,    0.8],       [  -5. ,   11. ,   60. ,    0.6],       [  14. ,    7. ,   59. ,    1. ],       [  20. ,   10. ,   65. ,    0.9],       [  27. ,    7. ,   67. ,    1.4],       [  35. ,   12. ,   64. ,    1.3],       [  51. ,   15. ,   66. ,    1.3],       [  56. ,   12. ,   67. ,    1.4],       [  69. ,   11. ,   62. ,    1.1],       [  76. ,    8. ,   62. ,    1.4],       [  85. ,   13. ,   60. ,    1.3],       [  96. ,   17. ,   66. ,    1. ],       [   3. ,   21. ,   66. ,    1.5],       [  14. ,   29. ,   67. ,    0.8],       [  17. ,   22. ,   62. ,    1.3],       [  34. ,   20. ,   58. ,    1.2],       [  41. ,   22. ,   66. ,    0.8],       [  54. ,   27. ,   66. ,    0.6],       [  61. ,   21. ,   63. ,    0.8],       [  66. ,   22. ,   65. ,    0.6],       [  78. ,   27. ,   60. ,    0.8],       [  89. ,   20. ,   66. ,    0.8],       [ 104. ,   21. ,   61. ,    1.3],       [   0. ,   37. ,   66. ,    1.2],       [  14. ,   36. ,   61. ,    0.5],       [  23. ,   38. ,   59. ,    1.1],       [  32. ,   33. ,   64. ,    1.4],       [  42. ,   40. ,   64. ,    1. ],       [  45. ,   33. ,   64. ,    1.1],       [  59. ,   39. ,   66. ,    0.6],       [  68. ,   40. ,   61. ,    1.2],       [  81. ,   34. ,   66. ,    0.7],       [  87. ,   38. ,   64. ,    0.8],       [  97. ,   35. ,   59. ,    0.7],       [  -5. ,   52. ,   66. ,    1.4],       [  13. ,   48. ,   60. ,    0.7],       [  18. ,   48. ,   67. ,    0.6],       [  33. ,   49. ,   59. ,    1.1],       [  38. ,   51. ,   64. ,    1.3],       [  53. ,   51. ,   63. ,    1. ],       [  62. ,   47. ,   63. ,    0.8],       [  71. ,   54. ,   59. ,    0.5],       [  79. ,   53. ,   63. ,    0.7],       [  91. ,   50. ,   66. ,    1. ],       [ 104. ,   51. ,   63. ,    1.5],       [   0. ,   63. ,   68. ,    1.3],       [   6. ,   67. ,   65. ,    0.5],       [  22. ,   61. ,   62. ,    0.6],       [  26. ,   61. ,   67. ,    0.5],       [  42. ,   63. ,   59. ,    0.7],       [  47. ,   63. ,   67. ,    0.6],       [  56. ,   60. ,   59. ,    0.6],       [  65. ,   60. ,   68. ,    1.1],       [  83. ,   64. ,   62. ,    0.6],       [  92. ,   58. ,   61. ,    1.1],       [ 103. ,   58. ,   67. ,    0.5],       [  -2. ,   76. ,   64. ,    1.3],       [  10. ,   78. ,   67. ,    1. ],       [  17. ,   70. ,   60. ,    0.7],       [  29. ,   76. ,   59. ,    1.4],       [  38. ,   71. ,   66. ,    1.2],       [  47. ,   70. ,   65. ,    0.5],       [  61. ,   73. ,   63. ,    1. ],       [  66. ,   73. ,   64. ,    0.7],       [  78. ,   74. ,   62. ,    1.2],       [  88. ,   72. ,   62. ,    0.7],       [ 101. ,   73. ,   63. ,    0.6],       [  -5. ,   88. ,   66. ,    0.8],       [   9. ,   91. ,   67. ,    1.2],       [  20. ,   85. ,   64. ,    1.1],       [  30. ,   85. ,   64. ,    1. ],       [  39. ,   85. ,   67. ,    0.7],       [  48. ,   88. ,   62. ,    1.2],       [  56. ,   87. ,   60. ,    0.8],       [  70. ,   82. ,   64. ,    0.7],       [  82. ,   84. ,   67. ,    1. ],       [  91. ,   85. ,   63. ,    0.9],       [ 101. ,   92. ,   64. ,    0.8],       [   0. ,   99. ,   61. ,    1.2],       [   7. ,  100. ,   68. ,    0.7],       [  22. ,  101. ,   62. ,    0.7],       [  29. ,  101. ,   61. ,    1.1],       [  39. ,  102. ,   65. ,    0.8],       [  54. ,   99. ,   63. ,    0.8],       [  55. ,   99. ,   68. ,    0.7],       [  68. ,  102. ,   66. ,    1. ],       [  75. ,   99. ,   60. ,    1.3],       [  89. ,   95. ,   59. ,    1.4],       [  95. ,  101. ,   58. ,    1. ],       [   1. ,    1. ,   69. ,    1.3],       [  12. ,   -1. ,   69. ,    1. ],       [  24. ,    4. ,   68. ,    1. ],       [  25. ,    1. ,   72. ,    0.6],       [  43. ,    0. ,   77. ,    0.8],       [  51. ,    0. ,   74. ,    1.3],       [  58. ,    4. ,   68. ,    1.3],       [  66. ,    4. ,   76. ,    1.3],       [  75. ,   -1. ,   76. ,    0.9],       [  89. ,    4. ,   75. ,    1.5],       [ 103. ,   -3. ,   70. ,    1.4],       [   0. ,   17. ,   69. ,    1.1],       [   9. ,   12. ,   68. ,    0.8],       [  17. ,    8. ,   72. ,    1.3],       [  33. ,   17. ,   75. ,    1.3],       [  38. ,   14. ,   68. ,    0.8],       [  48. ,    9. ,   74. ,    1.1],       [  62. ,   16. ,   74. ,    0.9],       [  70. ,   10. ,   75. ,    0.7],       [  76. ,    8. ,   76. ,    1.2],       [  92. ,   14. ,   68. ,    1.3],       [ 102. ,   10. ,   76. ,    1.1],       [  -3. ,   20. ,   73. ,    0.8],       [  10. ,   29. ,   70. ,    0.8],       [  24. ,   26. ,   74. ,    1.5],       [  30. ,   24. ,   68. ,    1.4],       [  35. ,   26. ,   72. ,    1.4],       [  54. ,   21. ,   72. ,    0.7],       [  61. ,   29. ,   75. ,    1.4],       [  66. ,   24. ,   73. ,    1.2],       [  77. ,   22. ,   68. ,    0.5],       [  87. ,   21. ,   75. ,    1.4],       [ 104. ,   23. ,   76. ,    0.9],       [  -1. ,   39. ,   73. ,    1.4],       [  11. ,   40. ,   68. ,    1.5],       [  15. ,   33. ,   73. ,    1.2],       [  27. ,   42. ,   75. ,    0.7],       [  37. ,   34. ,   71. ,    1.1],       [  51. ,   41. ,   73. ,    0.9],       [  56. ,   41. ,   75. ,    0.9],       [  70. ,   32. ,   70. ,    1.3],       [  81. ,   32. ,   75. ,    1.3],       [  93. ,   41. ,   72. ,    1.5],       [ 100. ,   41. ,   69. ,    0.7],       [   3. ,   53. ,   68. ,    1.3],       [   9. ,   50. ,   74. ,    0.7],       [  24. ,   54. ,   70. ,    0.6],       [  29. ,   49. ,   68. ,    1.2],       [  40. ,   49. ,   67. ,    1.3],       [  50. ,   48. ,   71. ,    1.4],       [  55. ,   53. ,   74. ,    0.7],       [  69. ,   47. ,   68. ,    1.2],       [  78. ,   50. ,   68. ,    1.1],       [  94. ,   50. ,   74. ,    0.9],       [  97. ,   51. ,   75. ,    1. ],       [   2. ,   60. ,   76. ,    0.9],       [   5. ,   65. ,   70. ,    1.3],       [  16. ,   59. ,   74. ,    0.7],       [  31. ,   61. ,   73. ,    0.6],       [  36. ,   64. ,   75. ,    1.1],       [  45. ,   57. ,   75. ,    1.3],       [  63. ,   61. ,   75. ,    0.9],       [  66. ,   57. ,   74. ,    0.8],       [  82. ,   58. ,   73. ,    0.9],       [  86. ,   66. ,   69. ,    1.3],       [ 100. ,   63. ,   77. ,    0.6],       [  -2. ,   78. ,   70. ,    0.5],       [  12. ,   76. ,   72. ,    1.4],       [  19. ,   73. ,   73. ,    0.7],       [  30. ,   76. ,   70. ,    1.3],       [  44. ,   71. ,   75. ,    1.3],       [  48. ,   79. ,   68. ,    1.2],       [  55. ,   77. ,   74. ,    1.3],       [  74. ,   75. ,   73. ,    0.9],       [  80. ,   74. ,   75. ,    1.3],       [  88. ,   72. ,   72. ,    1.4],       [  95. ,   74. ,   77. ,    0.7],       [   1. ,   86. ,   74. ,    1.1],       [   7. ,   85. ,   76. ,    0.5],       [  18. ,   89. ,   75. ,    0.8],       [  32. ,   89. ,   74. ,    1.2],       [  39. ,   88. ,   75. ,    0.9],       [  54. ,   86. ,   70. ,    1.3],       [  58. ,   88. ,   76. ,    0.8],       [  73. ,   84. ,   72. ,    0.9],       [  82. ,   88. ,   73. ,    1. ],       [  93. ,   82. ,   71. ,    0.9],       [ 101. ,   87. ,   68. ,    1. ],       [   0. ,   95. ,   76. ,    0.7],       [   9. ,   98. ,   70. ,    1.3],       [  18. ,   99. ,   70. ,    1.4],       [  32. ,   99. ,   72. ,    1.4],       [  38. ,  101. ,   68. ,    1.1],       [  48. ,   98. ,   76. ,    1.1],       [  64. ,   95. ,   74. ,    0.8],       [  67. ,  103. ,   68. ,    1.2],       [  80. ,  100. ,   77. ,    1.2],       [  87. ,  104. ,   77. ,    1.2],       [  97. ,  103. ,   73. ,    1.3],       [   2. ,    2. ,   84. ,    0.8],       [   7. ,   -3. ,   78. ,    0.8],       [  24. ,    0. ,   83. ,    1.1],       [  28. ,    0. ,   82. ,    0.6],       [  44. ,   -2. ,   84. ,    1.4],       [  48. ,    0. ,   86. ,    0.8],       [  62. ,   -5. ,   86. ,    0.8],       [  66. ,    4. ,   80. ,    1.2],       [  83. ,    2. ,   81. ,    0.5],       [  88. ,    3. ,   80. ,    1.2],       [ 104. ,   -3. ,   86. ,    0.6],       [   2. ,   15. ,   81. ,    0.7],       [  13. ,   10. ,   83. ,    0.7],       [  22. ,   16. ,   84. ,    0.7],       [  27. ,   14. ,   78. ,    0.6],       [  37. ,   14. ,   78. ,    1. ],       [  50. ,   12. ,   85. ,    0.7],       [  61. ,    7. ,   79. ,    0.7],       [  70. ,   16. ,   81. ,    0.6],       [  80. ,    8. ,   78. ,    0.7],       [  88. ,    9. ,   83. ,    1.1],       [  95. ,    8. ,   80. ,    1.1],       [  -1. ,   27. ,   80. ,    0.8],       [   9. ,   22. ,   80. ,    0.7],       [  17. ,   20. ,   85. ,    0.9],       [  33. ,   23. ,   82. ,    1.4],       [  42. ,   22. ,   82. ,    1.4],       [  51. ,   22. ,   80. ,    0.8],       [  55. ,   27. ,   85. ,    1.2],       [  65. ,   21. ,   85. ,    0.9],       [  83. ,   28. ,   77. ,    0.6],       [  90. ,   25. ,   81. ,    1.4],       [ 100. ,   20. ,   80. ,    1.3],       [   2. ,   33. ,   86. ,    0.6],       [  12. ,   42. ,   85. ,    0.7],       [  20. ,   41. ,   79. ,    1.2],       [  25. ,   36. ,   79. ,    1.4],       [  39. ,   42. ,   80. ,    1.3],       [  49. ,   42. ,   86. ,    0.8],       [  64. ,   36. ,   77. ,    0.6],       [  65. ,   36. ,   78. ,    0.8],       [  83. ,   34. ,   83. ,    0.5],       [  94. ,   38. ,   80. ,    0.7],       [ 101. ,   38. ,   77. ,    1.1],       [  -1. ,   54. ,   79. ,    1.3],       [  14. ,   51. ,   80. ,    1.4],       [  16. ,   46. ,   78. ,    1. ],       [  32. ,   52. ,   77. ,    1.5],       [  44. ,   48. ,   83. ,    1. ],       [  51. ,   46. ,   82. ,    0.6],       [  57. ,   52. ,   81. ,    0.6],       [  65. ,   51. ,   83. ,    1. ],       [  76. ,   52. ,   82. ,    1.2],       [  88. ,   52. ,   81. ,    0.6],       [ 103. ,   53. ,   84. ,    1. ],       [  -1. ,   64. ,   77. ,    0.7],       [   6. ,   61. ,   82. ,    0.9],       [  20. ,   64. ,   79. ,    1.1],       [  27. ,   65. ,   83. ,    1.3],       [  36. ,   65. ,   77. ,    1. ],       [  47. ,   63. ,   77. ,    0.6],       [  61. ,   63. ,   77. ,    1.5],       [  68. ,   66. ,   86. ,    1.1],       [  82. ,   58. ,   79. ,    1.3],       [  94. ,   67. ,   85. ,    0.8],       [ 101. ,   67. ,   81. ,    0.6],       [  -1. ,   75. ,   84. ,    1.4],       [  11. ,   76. ,   83. ,    1.3],       [  18. ,   79. ,   86. ,    1. ],       [  32. ,   79. ,   83. ,    1. ],       [  36. ,   70. ,   82. ,    0.8],       [  47. ,   78. ,   83. ,    1. ],       [  57. ,   77. ,   82. ,    0.8],       [  69. ,   79. ,   85. ,    1. ],       [  78. ,   78. ,   86. ,    1.2],       [  92. ,   72. ,   80. ,    0.9],       [ 103. ,   75. ,   84. ,    1.1],       [   1. ,   90. ,   78. ,    0.8],       [  10. ,   88. ,   79. ,    1.2],       [  19. ,   90. ,   80. ,    1.2],       [  28. ,   89. ,   81. ,    1.1],       [  35. ,   89. ,   80. ,    1.4],       [  50. ,   87. ,   84. ,    0.9],       [  58. ,   82. ,   78. ,    1.1],       [  73. ,   86. ,   85. ,    0.9],       [  84. ,   88. ,   84. ,    1. ],       [  92. ,   87. ,   85. ,    1.4],       [  97. ,   82. ,   83. ,    1.5],       [   3. ,  103. ,   86. ,    1.2],       [   6. ,   97. ,   78. ,    1.5],       [  21. ,  103. ,   78. ,    0.8],       [  26. ,  104. ,   77. ,    0.8],       [  41. ,   95. ,   77. ,    1. ],       [  48. ,   95. ,   83. ,    0.7],       [  60. ,  102. ,   81. ,    0.6],       [  72. ,   96. ,   80. ,    0.7],       [  79. ,  104. ,   85. ,    1.1],       [  86. ,   95. ,   77. ,    0.5],       [ 102. ,   96. ,   86. ,    1.3],       [  -5. ,    4. ,   93. ,    0.8],       [   7. ,    3. ,   89. ,    1. ],       [  17. ,   -3. ,   90. ,    1.3],       [  28. ,   -4. ,   91. ,    1. ],       [  43. ,   -3. ,   89. ,    0.6],       [  50. ,    2. ,   90. ,    1.5],       [  58. ,   -1. ,   91. ,    0.8],       [  70. ,    3. ,   90. ,    0.9],       [  84. ,   -5. ,   93. ,    0.9],       [  89. ,   -2. ,   87. ,    1.4],       [ 101. ,    3. ,   86. ,    0.8],       [  -5. ,   14. ,   88. ,    0.9],       [  12. ,   11. ,   91. ,    1.3],       [  21. ,    9. ,   87. ,    1.3],       [  30. ,   14. ,   92. ,    1.2],       [  36. ,   13. ,   93. ,    1. ],       [  47. ,    9. ,   94. ,    0.8],       [  58. ,    8. ,   94. ,    0.5],       [  69. ,   14. ,   89. ,    1.1],       [  79. ,    9. ,   94. ,    1.1],       [  90. ,    9. ,   90. ,    1.4],       [  97. ,   16. ,   95. ,    0.7],       [   4. ,   26. ,   89. ,    0.7],       [   9. ,   24. ,   89. ,    0.8],       [  16. ,   25. ,   95. ,    0.8],       [  34. ,   28. ,   94. ,    1.3],       [  35. ,   27. ,   87. ,    1.3],       [  54. ,   26. ,   86. ,    0.9],       [  58. ,   21. ,   86. ,    0.7],       [  74. ,   21. ,   89. ,    1.4],       [  75. ,   24. ,   89. ,    0.9],       [  89. ,   23. ,   89. ,    0.7],       [  95. ,   29. ,   89. ,    1.3],       [   4. ,   36. ,   88. ,    1.4],       [  13. ,   34. ,   92. ,    1.5],       [  19. ,   35. ,   90. ,    1.2],       [  32. ,   34. ,   90. ,    0.6],       [  37. ,   37. ,   88. ,    0.7],       [  46. ,   35. ,   86. ,    0.7],       [  57. ,   38. ,   94. ,    1.4],       [  70. ,   36. ,   94. ,    0.9],       [  79. ,   41. ,   93. ,    1. ],       [  92. ,   36. ,   89. ,    1.2],       [ 101. ,   38. ,   90. ,    1.5],       [   1. ,   53. ,   89. ,    0.7],       [  10. ,   51. ,   95. ,    1.1],       [  15. ,   49. ,   94. ,    0.9],       [  28. ,   51. ,   93. ,    0.9],       [  40. ,   51. ,   89. ,    1.4],       [  48. ,   48. ,   88. ,    1.2],       [  57. ,   51. ,   89. ,    1. ],       [  69. ,   45. ,   92. ,    0.7],       [  81. ,   54. ,   86. ,    0.7],       [  93. ,   45. ,   92. ,    1.4],       [ 100. ,   52. ,   95. ,    0.6],       [   2. ,   63. ,   91. ,    1.1],       [   9. ,   58. ,   94. ,    0.7],       [  19. ,   65. ,   95. ,    1. ],       [  33. ,   64. ,   87. ,    0.6],       [  37. ,   62. ,   91. ,    1.4],       [  48. ,   66. ,   86. ,    1.4],       [  55. ,   58. ,   91. ,    1.4],       [  66. ,   58. ,   95. ,    1.4],       [  78. ,   65. ,   95. ,    1.2],       [  88. ,   61. ,   95. ,    1. ],       [  98. ,   59. ,   90. ,    0.5],       [  -2. ,   79. ,   91. ,    0.9],       [   5. ,   74. ,   87. ,    1.1],       [  22. ,   75. ,   91. ,    0.7],       [  28. ,   70. ,   95. ,    0.8],       [  43. ,   70. ,   86. ,    1.1],       [  53. ,   79. ,   89. ,    0.5],       [  64. ,   75. ,   87. ,    0.6],       [  69. ,   72. ,   91. ,    1.5],       [  76. ,   75. ,   93. ,    1.1],       [  86. ,   74. ,   92. ,    0.8],       [ 103. ,   73. ,   90. ,    0.8],       [  -1. ,   83. ,   91. ,    1. ],       [   6. ,   90. ,   86. ,    0.7],       [  22. ,   89. ,   86. ,    1. ],       [  28. ,   84. ,   87. ,    1.3],       [  38. ,   85. ,   88. ,    0.9],       [  47. ,   90. ,   93. ,    0.7],       [  63. ,   86. ,   88. ,    0.9],       [  72. ,   88. ,   94. ,    1.1],       [  83. ,   88. ,   90. ,    0.6],       [  93. ,   83. ,   90. ,    0.6],       [  98. ,   91. ,   87. ,    1.2],       [   3. ,  104. ,   88. ,    1. ],       [   5. ,   97. ,   93. ,    0.7],       [  17. ,   99. ,   94. ,    0.8],       [  34. ,  101. ,   89. ,    0.7],       [  40. ,  101. ,   86. ,    0.6],       [  46. ,  102. ,   93. ,    1.3],       [  58. ,  103. ,   92. ,    1.3],       [  70. ,   96. ,   92. ,    1.1],       [  75. ,   97. ,   86. ,    0.9],       [  90. ,   99. ,   93. ,    0.9],       [  95. ,  103. ,   89. ,    1.4],       [   3. ,   -5. ,   97. ,    0.8],       [  11. ,   -3. ,   96. ,    1. ],       [  24. ,    2. ,   98. ,    1.1],       [  27. ,    2. ,  103. ,    0.6],       [  36. ,   -4. ,   99. ,    1.1],       [  48. ,   -1. ,   99. ,    1.1],       [  60. ,    1. ,   95. ,    0.6],       [  65. ,    2. ,  101. ,    0.6],       [  80. ,   -1. ,   97. ,    1.4],       [  92. ,   -5. ,   99. ,    0.5],       [  97. ,    2. ,   98. ,    0.9],       [  -2. ,   10. ,  103. ,    1.1],       [   9. ,   10. ,   97. ,    0.8],       [  23. ,   15. ,   95. ,    1.4],       [  32. ,   15. ,  104. ,    0.8],       [  36. ,   15. ,  101. ,    1.2],       [  50. ,   15. ,  101. ,    1.2],       [  63. ,   13. ,  103. ,    0.6],       [  66. ,   10. ,  104. ,    0.7],       [  75. ,   13. ,  103. ,    0.9],       [  87. ,   15. ,  102. ,    0.7],       [ 103. ,    8. ,  103. ,    1.1],       [  -3. ,   21. ,  102. ,    1. ],       [  13. ,   26. ,  101. ,    1.2],       [  17. ,   28. ,  103. ,    1. ],       [  25. ,   22. ,   97. ,    1.5],       [  36. ,   25. ,   95. ,    1. ],       [  49. ,   24. ,   97. ,    1.3],       [  61. ,   25. ,   99. ,    1.1],       [  68. ,   29. ,   99. ,    1.2],       [  80. ,   28. ,   99. ,    1.4],       [  90. ,   27. ,  103. ,    0.9],       [  96. ,   21. ,   99. ,    0.5],       [  -5. ,   37. ,  100. ,    0.8],       [   7. ,   42. ,   95. ,    0.5],       [  19. ,   38. ,   98. ,    1.1],       [  29. ,   40. ,   95. ,    0.5],       [  41. ,   39. ,   97. ,    0.9],       [  50. ,   37. ,  100. ,    0.8],       [  55. ,   33. ,   99. ,    1.2],       [  70. ,   38. ,   95. ,    1.1],       [  82. ,   37. ,   95. ,    0.6],       [  87. ,   37. ,   97. ,    1.4],       [ 100. ,   33. ,  101. ,    0.6],       [  -2. ,   51. ,  100. ,    1.5],       [  13. ,   48. ,   98. ,    1. ],       [  19. ,   54. ,  102. ,    1.3],       [  28. ,   50. ,  102. ,    1.3],       [  37. ,   52. ,  101. ,    0.7],       [  45. ,   54. ,  101. ,    1.3],       [  55. ,   53. ,  104. ,    1.2],       [  66. ,   54. ,   96. ,    0.6],       [  82. ,   48. ,  104. ,    0.6],       [  91. ,   51. ,  100. ,    1.3],       [  97. ,   48. ,   97. ,    0.8],       [   0. ,   65. ,   97. ,    1.4],       [  12. ,   57. ,  102. ,    1.1],       [  19. ,   63. ,   95. ,    1.2],       [  29. ,   65. ,   99. ,    0.9],       [  38. ,   57. ,  102. ,    0.6],       [  53. ,   62. ,  104. ,    0.6],       [  60. ,   63. ,   99. ,    0.6],       [  68. ,   62. ,   98. ,    1.1],       [  81. ,   64. ,   99. ,    0.9],       [  86. ,   62. ,  100. ,    1. ],       [ 102. ,   60. ,  101. ,    1.2],       [  -5. ,   74. ,  103. ,    1.4],       [   7. ,   72. ,  101. ,    0.9],       [  24. ,   75. ,   96. ,    0.8],       [  28. ,   75. ,  101. ,    1.4],       [  36. ,   72. ,   96. ,    0.8],       [  46. ,   73. ,  104. ,    1. ],       [  63. ,   71. ,  102. ,    1. ],       [  69. ,   77. ,  102. ,    1.4],       [  77. ,   71. ,   98. ,    0.9],       [  87. ,   70. ,  104. ,    1.1],       [  97. ,   79. ,   97. ,    0.7],       [  -4. ,   88. ,  100. ,    0.8],       [  12. ,   92. ,   97. ,    1.5],       [  24. ,   83. ,   95. ,    0.6],       [  25. ,   90. ,  100. ,    0.9],       [  37. ,   91. ,  104. ,    1.2],       [  48. ,   84. ,  102. ,    0.5],       [  55. ,   87. ,  100. ,    1.1],       [  73. ,   87. ,   99. ,    1.1],       [  76. ,   83. ,   98. ,    0.7],       [  92. ,   84. ,  103. ,    0.8],       [  98. ,   89. ,  102. ,    0.9],       [   0. ,  103. ,   98. ,    1.4],       [  10. ,   97. ,   96. ,    1.5],       [  19. ,  102. ,  103. ,    1. ],       [  34. ,   99. ,   97. ,    1.4],       [  41. ,   99. ,  100. ,    1.1],       [  45. ,   95. ,   97. ,    1.3],       [  56. ,   96. ,  101. ,    0.7],       [  65. ,  101. ,   97. ,    0.5],       [  75. ,  103. ,   97. ,    1.4],       [  87. ,   98. ,   97. ,    0.6],       [ 103. ,  104. ,   97. ,    1.1]])
        basis0 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.4,  1.9,  2.8,  4.3,  4.9,  6. ,  6. ,  6. ,  6. ,
        6. ,  6. ]))
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  1.6,  3.2,  4. ,  4. ,  4. ,  4. ,  4. ,  4. ]))
        basis2 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.8,  2.3,  3.4,  4. ,  5. ,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(3, 3, 1)
        self.assertEqual(vol2.order(direction=0), 9)
        self.assertEqual(vol2.order(direction=1), 9)
        self.assertEqual(vol2.order(direction=2), 8)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p5_rational_C0_periodic(self):
        controlpoints = np.array([[  99. ,   -4. ,    4. ,    1.4],
       [  81. ,   58. ,   -4. ,    1.4],
       [  29. ,   98. ,   -4. ,    0.6],
       [ -29. ,   95. ,   -1. ,    1.2],
       [ -83. ,   60. ,    4. ,    0.5],
       [-105. ,    3. ,   -4. ,    1. ],
       [ -86. ,  -58. ,    4. ,    1.3],
       [ -31. ,  -91. ,   -2. ,    1.5],
       [  35. , -100. ,   -5. ,    1.4],
       [  76. ,  -63. ,   -5. ,    0.8]])
        basis0 = BSplineBasis(5, np.array([-1.3,  0. ,  0. ,  0. ,  0. ,  1.1,  1.7,  2.9,  3.9,  4.7,  5.7,  7. ,  7. ,  7. ,  7. ,
        8.1]),0)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 6)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p55_rational_C0_periodic(self):
        controlpoints = np.array([[  60. ,    1. ,    1. ,    1.2],       [  51. ,   36. ,    2. ,    0.8],       [  20. ,   57. ,   -5. ,    0.7],       [ -14. ,   57. ,   -4. ,    1.4],       [ -52. ,   37. ,    0. ,    0.9],       [ -62. ,    4. ,   -1. ,    1.2],       [ -52. ,  -38. ,   -5. ,    0.6],       [ -17. ,  -53. ,   -5. ,    1.5],       [  20. ,  -54. ,    2. ,    0.9],       [  51. ,  -33. ,    0. ,    1.4],       [  70. ,    3. ,   -1. ,    0.9],       [  49. ,   41. ,    4. ,    0.5],       [  24. ,   63. ,   -4. ,    0.7],       [ -16. ,   68. ,   -4. ,    0.8],       [ -51. ,   34. ,    0. ,    0.9],       [ -67. ,   -5. ,   -5. ,    0.8],       [ -52. ,  -35. ,   -2. ,    0.8],       [ -23. ,  -67. ,    3. ,    0.5],       [  18. ,  -68. ,   -4. ,    1.3],       [  52. ,  -41. ,   -4. ,    1.5],       [  75. ,   -1. ,    0. ,    1.4],       [  63. ,   41. ,    0. ,    1.4],       [  25. ,   66. ,    2. ,    0.9],       [ -27. ,   74. ,   -5. ,    0.7],       [ -60. ,   45. ,   -4. ,    1.5],       [ -69. ,    1. ,   -1. ,    1. ],       [ -62. ,  -40. ,   -5. ,    1.2],       [ -25. ,  -71. ,    1. ,    1.2],       [  20. ,  -75. ,    4. ,    1.3],       [  55. ,  -42. ,   -2. ,    0.6],       [  83. ,    2. ,   -5. ,    1.4],       [  63. ,   51. ,    3. ,    1. ],       [  20. ,   76. ,   -1. ,    0.8],       [ -21. ,   77. ,    0. ,    0.8],       [ -61. ,   51. ,   -5. ,    0.8],       [ -80. ,    2. ,    4. ,    0.7],       [ -69. ,  -46. ,   -5. ,    0.8],       [ -23. ,  -76. ,   -5. ,    1.5],       [  27. ,  -78. ,   -5. ,    0.6],       [  69. ,  -49. ,   -4. ,    1.5],       [  83. ,   -1. ,    3. ,    1.1],       [  71. ,   52. ,   -3. ,    1.4],       [  24. ,   86. ,    3. ,    0.6],       [ -23. ,   78. ,    0. ,    0.7],       [ -72. ,   46. ,   -1. ,    1.2],       [ -87. ,   -1. ,    1. ,    1.3],       [ -69. ,  -48. ,    4. ,    0.8],       [ -28. ,  -79. ,   -1. ,    1.4],       [  28. ,  -84. ,   -1. ,    1.3],       [  67. ,  -56. ,   -2. ,    1.3],       [  91. ,    0. ,    0. ,    0.9],       [  72. ,   50. ,    2. ,    1.1],       [  26. ,   85. ,    2. ,    0.8],       [ -28. ,   88. ,   -3. ,    0.8],       [ -74. ,   55. ,    1. ,    1.3],       [ -93. ,    2. ,   -3. ,    0.9],       [ -76. ,  -51. ,    3. ,    1.2],       [ -30. ,  -89. ,   -2. ,    1.1],       [  28. ,  -85. ,   -4. ,    0.5],       [  78. ,  -50. ,   -4. ,    0.7],       [ 100. ,    0. ,    3. ,    0.7],       [  76. ,   63. ,    0. ,    0.8],       [  28. ,   93. ,    3. ,    1.1],       [ -36. ,   95. ,    1. ,    0.5],       [ -80. ,   57. ,    1. ,    1.1],       [-100. ,   -5. ,   -5. ,    0.8],       [ -80. ,  -58. ,   -3. ,    0.8],       [ -30. ,  -91. ,    2. ,    0.6],       [  30. ,  -98. ,   -2. ,    1.4],       [  84. ,  -63. ,   -2. ,    1.5]])
        basis0 = BSplineBasis(5, np.array([-1.2,  0. ,  0. ,  0. ,  0. ,  0.9,  2.3,  3.2,  3.9,  5. ,  5.8,  7. ,  7. ,  7. ,  7. ,
        7.9]),0)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  2. ,  3. ,  3. ,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 2)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 7)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p775_rational_C0_periodic(self):
        controlpoints = np.array([[  48. ,    3. ,   -2. ,    1.2],       [  43. ,   18. ,   -5. ,    0.9],       [  27. ,   35. ,    0. ,    0.8],       [   8. ,   47. ,   -2. ,    1. ],       [ -16. ,   48. ,   -4. ,    1.5],       [ -33. ,   35. ,    0. ,    1. ],       [ -41. ,   23. ,    3. ,    1.4],       [ -49. ,    3. ,    0. ,    1.4],       [ -49. ,  -24. ,    1. ,    0.5],       [ -30. ,  -36. ,   -3. ,    1.2],       [  -8. ,  -47. ,   -4. ,    1.5],       [  12. ,  -48. ,   -5. ,    0.5],       [  35. ,  -37. ,   -4. ,    0.5],       [  43. ,  -20. ,    4. ,    1.3],       [  58. ,    3. ,   -5. ,    0.6],       [  45. ,   21. ,   -5. ,    1.1],       [  33. ,   45. ,   -2. ,    0.5],       [   9. ,   50. ,   -1. ,    0.6],       [ -13. ,   57. ,   -1. ,    0.7],       [ -31. ,   39. ,    2. ,    1.5],       [ -48. ,   23. ,    0. ,    0.7],       [ -60. ,    4. ,    2. ,    1. ],       [ -55. ,  -26. ,   -1. ,    1.3],       [ -36. ,  -42. ,   -3. ,    1.1],       [ -10. ,  -56. ,    1. ,    1.2],       [  10. ,  -53. ,   -4. ,    1.2],       [  36. ,  -46. ,   -1. ,    0.9],       [  46. ,  -22. ,    1. ,    0.5],       [  60. ,    2. ,    1. ,    0.8],       [  52. ,   22. ,    1. ,    0.9],       [  38. ,   43. ,   -3. ,    1.4],       [  14. ,   55. ,    0. ,    1.5],       [ -12. ,   62. ,    2. ,    0.9],       [ -39. ,   42. ,    1. ,    0.7],       [ -59. ,   29. ,   -2. ,    0.8],       [ -65. ,    0. ,    0. ,    0.8],       [ -50. ,  -22. ,   -4. ,    0.7],       [ -36. ,  -48. ,    4. ,    1.1],       [ -17. ,  -54. ,    4. ,    1.4],       [  15. ,  -59. ,    3. ,    0.9],       [  32. ,  -50. ,   -5. ,    0.7],       [  55. ,  -23. ,    2. ,    1.2],       [  62. ,   -2. ,   -1. ,    0.8],       [  53. ,   23. ,    0. ,    0.8],       [  38. ,   49. ,    3. ,    1. ],       [  16. ,   61. ,   -4. ,    1.1],       [ -20. ,   61. ,   -1. ,    1.2],       [ -41. ,   46. ,    3. ,    0.6],       [ -64. ,   28. ,    2. ,    1.4],       [ -69. ,    3. ,   -3. ,    0.6],       [ -63. ,  -31. ,   -5. ,    0.6],       [ -43. ,  -50. ,   -4. ,    1.1],       [ -14. ,  -69. ,    3. ,    0.6],       [  13. ,  -60. ,   -2. ,    1.3],       [  44. ,  -56. ,    3. ,    1.2],       [  55. ,  -26. ,   -2. ,    1.1],       [  67. ,   -2. ,   -5. ,    1. ],       [  64. ,   32. ,   -5. ,    1.4],       [  41. ,   55. ,    3. ,    0.7],       [  17. ,   71. ,    4. ,    1.5],       [ -14. ,   71. ,    2. ,    1.2],       [ -40. ,   55. ,   -5. ,    0.8],       [ -62. ,   33. ,    0. ,    1. ],       [ -69. ,   -1. ,    3. ,    0.8],       [ -63. ,  -29. ,    3. ,    0.5],       [ -48. ,  -53. ,   -1. ,    1.3],       [ -21. ,  -70. ,    4. ,    1.2],       [  13. ,  -71. ,    2. ,    0.7],       [  43. ,  -57. ,    1. ,    1.5],       [  66. ,  -31. ,    2. ,    1.3],       [  71. ,   -1. ,    2. ,    0.9],       [  70. ,   34. ,   -2. ,    0.9],       [  46. ,   62. ,    1. ,    1. ],       [  19. ,   71. ,   -1. ,    1. ],       [ -12. ,   77. ,   -1. ,    1.2],       [ -51. ,   59. ,   -2. ,    1.3],       [ -73. ,   32. ,   -5. ,    0.6],       [ -71. ,    0. ,    3. ,    1.2],       [ -67. ,  -34. ,    2. ,    0.8],       [ -49. ,  -61. ,   -1. ,    1.1],       [ -17. ,  -70. ,   -1. ,    1.2],       [  14. ,  -69. ,   -4. ,    0.8],       [  51. ,  -62. ,   -2. ,    1.5],       [  69. ,  -30. ,    3. ,    1.3],       [  84. ,   -4. ,    0. ,    1. ],       [  67. ,   31. ,   -1. ,    1. ],       [  47. ,   59. ,   -2. ,    1. ],       [  17. ,   81. ,    1. ,    1.3],       [ -18. ,   74. ,    0. ,    0.7],       [ -51. ,   59. ,    2. ,    1.4],       [ -69. ,   39. ,    0. ,    1. ],       [ -84. ,    3. ,   -2. ,    1.3],       [ -73. ,  -35. ,    1. ,    0.6],       [ -49. ,  -66. ,    1. ,    1.4],       [ -18. ,  -83. ,   -4. ,    0.9],       [  14. ,  -78. ,    2. ,    1.3],       [  49. ,  -61. ,    3. ,    1.3],       [  75. ,  -38. ,   -5. ,    1.4],       [  89. ,    3. ,    4. ,    1.5],       [  72. ,   39. ,    4. ,    0.9],       [  48. ,   64. ,    0. ,    1.4],       [  21. ,   85. ,   -4. ,    0.5],       [ -17. ,   84. ,    4. ,    0.7],       [ -53. ,   62. ,    4. ,    1.1],       [ -73. ,   37. ,   -3. ,    0.7],       [ -82. ,    1. ,    0. ,    1. ],       [ -79. ,  -40. ,   -5. ,    0.7],       [ -49. ,  -70. ,    3. ,    1.4],       [ -24. ,  -83. ,    2. ,    1.3],       [  17. ,  -82. ,    3. ,    1.5],       [  53. ,  -64. ,    0. ,    1.3],       [  72. ,  -33. ,    3. ,    1.4],       [  85. ,   -1. ,   -1. ,    1.1],       [  82. ,   37. ,   -2. ,    0.9],       [  57. ,   67. ,    2. ,    1.5],       [  15. ,   84. ,    1. ,    1.4],       [ -19. ,   84. ,   -5. ,    0.7],       [ -58. ,   72. ,   -3. ,    0.8],       [ -86. ,   37. ,    2. ,    1.3],       [ -91. ,    3. ,   -5. ,    1.5],       [ -86. ,  -40. ,   -1. ,    1.3],       [ -61. ,  -70. ,    2. ,    1.2],       [ -19. ,  -91. ,    4. ,    1.2],       [  21. ,  -85. ,    2. ,    1.2],       [  60. ,  -69. ,   -4. ,    1.4],       [  79. ,  -36. ,    3. ,    0.9],       [  92. ,   -3. ,   -4. ,    0.9],       [  81. ,   44. ,   -4. ,    1. ],       [  63. ,   76. ,    2. ,    0.8],       [  24. ,   91. ,   -1. ,    1. ],       [ -20. ,   90. ,   -5. ,    1.5],       [ -58. ,   72. ,    2. ,    1.3],       [ -83. ,   37. ,    1. ,    1.3],       [ -91. ,    3. ,   -3. ,    1. ],       [ -87. ,  -40. ,    2. ,    1.2],       [ -63. ,  -78. ,    2. ,    0.9],       [ -21. ,  -97. ,   -2. ,    1.3],       [  24. ,  -97. ,   -2. ,    0.9],       [  61. ,  -74. ,    1. ,    0.5],       [  82. ,  -40. ,   -4. ,    1. ],       [ 104. ,    0. ,    1. ,    1.3],       [  85. ,   46. ,    2. ,    1.3],       [  62. ,   81. ,   -2. ,    1.4],       [  19. ,   94. ,    1. ,    1.4],       [ -27. ,   99. ,    3. ,    1.2],       [ -60. ,   76. ,    3. ,    0.6],       [ -89. ,   43. ,   -2. ,    1.3],       [-105. ,    0. ,   -1. ,    1. ],       [ -87. ,  -42. ,   -3. ,    0.6],       [ -65. ,  -83. ,    2. ,    1.3],       [ -19. ,  -97. ,   -4. ,    0.8],       [  26. ,  -97. ,    3. ,    1.4],       [  66. ,  -74. ,    4. ,    0.9],       [  93. ,  -47. ,   -3. ,    1.4],       [  54. ,    4. ,    7. ,    1.4],       [  42. ,   19. ,    4. ,    0.9],       [  28. ,   40. ,    8. ,    0.6],       [  12. ,   53. ,    5. ,    1.1],       [ -12. ,   47. ,   12. ,    1.3],       [ -34. ,   41. ,    5. ,    0.9],       [ -50. ,   24. ,    6. ,    1.1],       [ -53. ,   -4. ,    6. ,    1.1],       [ -48. ,  -24. ,    8. ,    1.2],       [ -33. ,  -44. ,    6. ,    0.9],       [ -13. ,  -52. ,   10. ,    0.5],       [  12. ,  -46. ,    7. ,    0.9],       [  27. ,  -43. ,    8. ,    0.7],       [  41. ,  -23. ,    3. ,    0.9],       [  51. ,   -4. ,    5. ,    0.6],       [  46. ,   24. ,   10. ,    1.5],       [  29. ,   47. ,    9. ,    1.3],       [  12. ,   55. ,    3. ,    1.4],       [  -9. ,   52. ,   10. ,    1.5],       [ -38. ,   46. ,    8. ,    1.2],       [ -51. ,   28. ,    6. ,    0.7],       [ -54. ,    0. ,    8. ,    0.8],       [ -51. ,  -27. ,    4. ,    0.7],       [ -39. ,  -46. ,    8. ,    1.4],       [ -10. ,  -59. ,    3. ,    1.1],       [  13. ,  -56. ,    5. ,    1.1],       [  35. ,  -40. ,    3. ,    1.1],       [  47. ,  -20. ,    9. ,    1.1],       [  59. ,   -4. ,    7. ,    1.4],       [  54. ,   25. ,    7. ,    0.8],       [  40. ,   49. ,    7. ,    1.2],       [  14. ,   54. ,    4. ,    1.3],       [ -11. ,   54. ,    6. ,    0.8],       [ -34. ,   45. ,   11. ,    1.4],       [ -52. ,   24. ,    4. ,    1.5],       [ -63. ,    3. ,    7. ,    1.2],       [ -58. ,  -23. ,   11. ,    1.2],       [ -42. ,  -50. ,    9. ,    0.5],       [  -9. ,  -63. ,   10. ,    1.1],       [   9. ,  -55. ,    4. ,    1. ],       [  37. ,  -42. ,    6. ,    1.1],       [  51. ,  -24. ,    4. ,    1.4],       [  67. ,   -5. ,    4. ,    0.9],       [  62. ,   24. ,    6. ,    1.1],       [  41. ,   47. ,   11. ,    0.6],       [  16. ,   61. ,    7. ,    0.9],       [ -13. ,   68. ,    7. ,    1.4],       [ -36. ,   49. ,   10. ,    1.2],       [ -57. ,   26. ,    6. ,    1.2],       [ -62. ,   -5. ,   10. ,    0.9],       [ -61. ,  -27. ,    6. ,    0.7],       [ -36. ,  -49. ,    9. ,    0.7],       [ -16. ,  -64. ,   11. ,    0.8],       [  11. ,  -66. ,   12. ,    1.3],       [  37. ,  -50. ,   12. ,    1.1],       [  54. ,  -30. ,   12. ,    0.8],       [  72. ,   -5. ,    4. ,    0.6],       [  66. ,   25. ,   12. ,    0.6],       [  47. ,   53. ,    9. ,    1. ],       [  12. ,   68. ,    6. ,    0.9],       [ -18. ,   70. ,    9. ,    0.5],       [ -43. ,   54. ,    3. ,    0.8],       [ -65. ,   34. ,    9. ,    0.7],       [ -66. ,   -5. ,   12. ,    0.9],       [ -66. ,  -32. ,    5. ,    0.5],       [ -45. ,  -60. ,    3. ,    0.5],       [ -21. ,  -71. ,    5. ,    0.9],       [  13. ,  -70. ,    7. ,    0.6],       [  42. ,  -52. ,   10. ,    0.9],       [  60. ,  -26. ,   11. ,    1.4],       [  72. ,    3. ,    7. ,    0.7],       [  64. ,   30. ,    4. ,    1.2],       [  51. ,   60. ,   11. ,    1.1],       [  11. ,   69. ,    5. ,    1.4],       [ -22. ,   69. ,    9. ,    1.5],       [ -46. ,   59. ,    8. ,    0.7],       [ -72. ,   30. ,   11. ,    1.2],       [ -79. ,    2. ,    5. ,    1. ],       [ -71. ,  -37. ,   10. ,    0.6],       [ -50. ,  -58. ,   10. ,    0.8],       [ -15. ,  -69. ,   11. ,    0.9],       [  16. ,  -73. ,    9. ,    0.9],       [  49. ,  -54. ,    7. ,    1.1],       [  71. ,  -36. ,    3. ,    0.7],       [  75. ,   -4. ,    4. ,    1.4],       [  68. ,   36. ,    7. ,    1.2],       [  47. ,   57. ,    6. ,    0.7],       [  17. ,   74. ,   12. ,    0.5],       [ -19. ,   77. ,    3. ,    1.1],       [ -46. ,   67. ,    5. ,    1. ],       [ -71. ,   33. ,    6. ,    0.8],       [ -79. ,    2. ,    6. ,    1.3],       [ -69. ,  -34. ,   11. ,    1.3],       [ -47. ,  -59. ,   11. ,    1.5],       [ -17. ,  -78. ,    5. ,    1.1],       [  14. ,  -78. ,    3. ,    1.4],       [  45. ,  -63. ,   10. ,    0.7],       [  67. ,  -37. ,    5. ,    0.5],       [  82. ,   -4. ,    4. ,    0.9],       [  76. ,   38. ,   10. ,    0.6],       [  57. ,   62. ,    7. ,    0.9],       [  17. ,   78. ,    4. ,    1.5],       [ -23. ,   85. ,    6. ,    1.3],       [ -57. ,   68. ,   10. ,    0.9],       [ -79. ,   39. ,    8. ,    1.3],       [ -89. ,    2. ,    9. ,    0.6],       [ -72. ,  -35. ,    8. ,    0.8],       [ -56. ,  -65. ,    7. ,    0.6],       [ -16. ,  -79. ,    7. ,    1.5],       [  15. ,  -84. ,    8. ,    1.2],       [  57. ,  -71. ,   10. ,    0.7],       [  74. ,  -42. ,    9. ,    0.9],       [  85. ,   -4. ,    4. ,    1.2],       [  78. ,   41. ,   10. ,    1.1],       [  57. ,   72. ,    6. ,    1. ],       [  19. ,   90. ,    6. ,    1.1],       [ -22. ,   86. ,    3. ,    0.5],       [ -58. ,   72. ,   11. ,    0.7],       [ -80. ,   40. ,    8. ,    1.1],       [ -94. ,    4. ,   10. ,    0.5],       [ -85. ,  -38. ,    5. ,    0.7],       [ -57. ,  -67. ,   11. ,    1.4],       [ -24. ,  -86. ,   11. ,    1.3],       [  19. ,  -85. ,    5. ,    1.2],       [  52. ,  -75. ,    7. ,    1.5],       [  83. ,  -41. ,    6. ,    1.2],       [  99. ,   -5. ,    9. ,    1. ],       [  87. ,   40. ,   12. ,    1.1],       [  63. ,   72. ,    7. ,    0.9],       [  25. ,   94. ,   10. ,    0.7],       [ -26. ,   91. ,    3. ,    1.2],       [ -58. ,   76. ,   12. ,    0.6],       [ -89. ,   42. ,   11. ,    0.8],       [ -97. ,   -2. ,   11. ,    1.1],       [ -83. ,  -38. ,    9. ,    1.1],       [ -56. ,  -74. ,   11. ,    0.7],       [ -24. ,  -94. ,    5. ,    0.7],       [  21. ,  -94. ,    7. ,    0.7],       [  60. ,  -71. ,    9. ,    0.7],       [  85. ,  -41. ,    8. ,    1.5],       [  96. ,   -1. ,   10. ,    0.7],       [  93. ,   46. ,    5. ,    0.8],       [  66. ,   74. ,    9. ,    0.7],       [  18. ,   94. ,    5. ,    1. ],       [ -23. ,   93. ,    9. ,    1.4],       [ -67. ,   82. ,    7. ,    1.4],       [ -91. ,   47. ,    9. ,    0.5],       [-103. ,    1. ,    6. ,    1.1],       [ -94. ,  -44. ,   10. ,    0.9],       [ -62. ,  -74. ,    7. ,    1.3],       [ -25. ,  -98. ,   10. ,    1.4],       [  22. ,  -99. ,    3. ,    1.1],       [  60. ,  -79. ,   12. ,    1.3],       [  93. ,  -44. ,    6. ,    0.9],       [  46. ,    2. ,   16. ,    0.9],       [  43. ,   19. ,   13. ,    0.9],       [  30. ,   41. ,   19. ,    1. ],       [  11. ,   49. ,   18. ,    0.5],       [ -14. ,   49. ,   12. ,    0.5],       [ -36. ,   43. ,   18. ,    1.1],       [ -43. ,   18. ,   11. ,    1.1],       [ -52. ,    0. ,   16. ,    0.6],       [ -49. ,  -25. ,   20. ,    1.3],       [ -32. ,  -35. ,   16. ,    0.9],       [ -15. ,  -47. ,   15. ,    1.1],       [  11. ,  -52. ,   16. ,    1.3],       [  34. ,  -40. ,   12. ,    0.8],       [  45. ,  -19. ,   16. ,    0.6],       [  51. ,    3. ,   14. ,    0.7],       [  51. ,   27. ,   15. ,    0.8],       [  31. ,   43. ,   13. ,    1.2],       [  14. ,   54. ,   17. ,    1.3],       [ -11. ,   55. ,   17. ,    0.8],       [ -34. ,   46. ,   12. ,    0.8],       [ -47. ,   28. ,   18. ,    1.5],       [ -58. ,   -1. ,   14. ,    1.3],       [ -52. ,  -29. ,   13. ,    1. ],       [ -33. ,  -39. ,   18. ,    0.8],       [ -10. ,  -51. ,   13. ,    0.8],       [  11. ,  -54. ,   12. ,    1.3],       [  29. ,  -41. ,   14. ,    0.7],       [  50. ,  -22. ,   12. ,    0.7],       [  57. ,    1. ,   13. ,    1.2],       [  49. ,   24. ,   19. ,    1.3],       [  39. ,   49. ,   11. ,    1.5],       [  13. ,   56. ,   19. ,    1.5],       [ -18. ,   55. ,   20. ,    0.9],       [ -39. ,   49. ,   17. ,    0.8],       [ -56. ,   22. ,   20. ,    1.4],       [ -60. ,    3. ,   19. ,    1.3],       [ -53. ,  -28. ,   18. ,    0.6],       [ -35. ,  -48. ,   19. ,    1.1],       [ -16. ,  -58. ,   18. ,    1. ],       [  14. ,  -63. ,   13. ,    1.5],       [  32. ,  -44. ,   14. ,    1.3],       [  56. ,  -29. ,   15. ,    0.7],       [  60. ,   -2. ,   12. ,    0.9],       [  60. ,   26. ,   13. ,    1. ],       [  43. ,   51. ,   17. ,    0.6],       [  13. ,   61. ,   14. ,    0.6],       [ -14. ,   67. ,   18. ,    1. ],       [ -40. ,   55. ,   14. ,    1.4],       [ -60. ,   29. ,   15. ,    1.3],       [ -64. ,   -5. ,   12. ,    0.9],       [ -54. ,  -25. ,   11. ,    0.8],       [ -45. ,  -52. ,   17. ,    1.1],       [ -17. ,  -64. ,   12. ,    0.6],       [  13. ,  -64. ,   15. ,    0.6],       [  37. ,  -53. ,   14. ,    0.7],       [  61. ,  -30. ,   16. ,    1. ],       [  66. ,    1. ,   14. ,    1. ],       [  59. ,   33. ,   20. ,    1.4],       [  47. ,   52. ,   12. ,    1.2],       [  14. ,   64. ,   14. ,    1.2],       [ -17. ,   69. ,   15. ,    1.2],       [ -42. ,   50. ,   17. ,    0.8],       [ -62. ,   32. ,   12. ,    0.7],       [ -75. ,    1. ,   14. ,    1.5],       [ -61. ,  -28. ,   17. ,    1.1],       [ -39. ,  -57. ,   17. ,    0.7],       [ -16. ,  -69. ,   19. ,    1. ],       [  13. ,  -70. ,   14. ,    1.2],       [  43. ,  -52. ,   18. ,    1.1],       [  67. ,  -35. ,   16. ,    1.1],       [  77. ,   -3. ,   18. ,    0.8],       [  70. ,   32. ,   19. ,    1. ],       [  47. ,   62. ,   20. ,    0.6],       [  15. ,   73. ,   11. ,    0.7],       [ -21. ,   76. ,   15. ,    1.5],       [ -48. ,   60. ,   14. ,    1. ],       [ -64. ,   27. ,   16. ,    0.6],       [ -79. ,   -5. ,   17. ,    1.1],       [ -65. ,  -31. ,   12. ,    1.5],       [ -50. ,  -57. ,   12. ,    1.2],       [ -18. ,  -73. ,   17. ,    1.3],       [  20. ,  -70. ,   14. ,    1.5],       [  46. ,  -55. ,   13. ,    1. ],       [  67. ,  -31. ,   16. ,    1.4],       [  76. ,    2. ,   11. ,    1.4],       [  75. ,   30. ,   13. ,    1.2],       [  45. ,   61. ,   12. ,    0.5],       [  16. ,   73. ,   17. ,    1.1],       [ -14. ,   75. ,   16. ,    1.1],       [ -53. ,   66. ,   17. ,    1. ],       [ -73. ,   30. ,   17. ,    0.9],       [ -80. ,    1. ,   15. ,    1. ],       [ -69. ,  -36. ,   17. ,    0.9],       [ -46. ,  -65. ,   14. ,    0.9],       [ -21. ,  -78. ,   18. ,    0.6],       [  13. ,  -81. ,   18. ,    0.8],       [  49. ,  -63. ,   15. ,    1.3],       [  71. ,  -38. ,   17. ,    1.4],       [  86. ,    1. ,   17. ,    0.6],       [  73. ,   32. ,   20. ,    0.9],       [  49. ,   70. ,   12. ,    1. ],       [  20. ,   78. ,   12. ,    0.9],       [ -20. ,   82. ,   11. ,    1.1],       [ -49. ,   68. ,   20. ,    1.3],       [ -75. ,   34. ,   20. ,    0.9],       [ -82. ,   -3. ,   17. ,    0.6],       [ -74. ,  -39. ,   19. ,    0.8],       [ -49. ,  -62. ,   11. ,    0.8],       [ -17. ,  -82. ,   12. ,    0.8],       [  15. ,  -83. ,   11. ,    0.7],       [  48. ,  -70. ,   15. ,    0.6],       [  72. ,  -38. ,   16. ,    1.3],       [  85. ,    1. ,   12. ,    1.2],       [  77. ,   37. ,   18. ,    1.3],       [  53. ,   75. ,   20. ,    0.6],       [  17. ,   84. ,   11. ,    1.1],       [ -22. ,   85. ,   13. ,    0.5],       [ -55. ,   70. ,   11. ,    1.1],       [ -86. ,   43. ,   14. ,    0.7],       [ -88. ,   -3. ,   11. ,    0.6],       [ -82. ,  -36. ,   18. ,    1.4],       [ -58. ,  -69. ,   15. ,    1.3],       [ -25. ,  -90. ,   17. ,    1.4],       [  23. ,  -89. ,   11. ,    0.8],       [  57. ,  -68. ,   12. ,    0.7],       [  76. ,  -39. ,   15. ,    0.5],       [  97. ,   -1. ,   12. ,    1. ],       [  84. ,   38. ,   15. ,    1.2],       [  55. ,   75. ,   16. ,    0.9],       [  20. ,   95. ,   17. ,    1.3],       [ -18. ,   90. ,   18. ,    1.1],       [ -56. ,   70. ,   14. ,    0.8],       [ -91. ,   38. ,   13. ,    1.3],       [ -99. ,    0. ,   13. ,    0.8],       [ -87. ,  -45. ,   16. ,    1.2],       [ -58. ,  -72. ,   18. ,    1.3],       [ -25. ,  -95. ,   15. ,    1. ],       [  23. ,  -93. ,   11. ,    0.9],       [  59. ,  -74. ,   18. ,    1.2],       [  88. ,  -46. ,   18. ,    1. ],       [  99. ,   -4. ,   18. ,    0.8],       [  93. ,   42. ,   20. ,    1.2],       [  61. ,   78. ,   15. ,    1.2],       [  24. ,  101. ,   11. ,    1.2],       [ -25. ,  100. ,   12. ,    1. ],       [ -60. ,   76. ,   15. ,    1.1],       [ -92. ,   41. ,   16. ,    1.1],       [-103. ,    2. ,   20. ,    1.3],       [ -86. ,  -42. ,   12. ,    0.7],       [ -67. ,  -77. ,   19. ,    0.8],       [ -22. , -102. ,   12. ,    0.8],       [  19. ,  -96. ,   11. ,    0.5],       [  57. ,  -80. ,   12. ,    1. ],       [  85. ,  -41. ,   13. ,    1. ],       [  50. ,   -5. ,   20. ,    1.2],       [  40. ,   26. ,   28. ,    0.6],       [  34. ,   36. ,   22. ,    1.3],       [  15. ,   52. ,   25. ,    1.4],       [ -15. ,   53. ,   28. ,    0.6],       [ -30. ,   37. ,   21. ,    0.6],       [ -50. ,   24. ,   23. ,    0.6],       [ -46. ,   -5. ,   21. ,    1. ],       [ -50. ,  -22. ,   27. ,    0.7],       [ -30. ,  -35. ,   28. ,    1.2],       [ -10. ,  -47. ,   20. ,    0.9],       [  15. ,  -50. ,   28. ,    1.2],       [  30. ,  -40. ,   23. ,    0.7],       [  44. ,  -26. ,   23. ,    1.1],       [  56. ,   -4. ,   20. ,    1.2],       [  53. ,   20. ,   28. ,    1.4],       [  38. ,   41. ,   24. ,    0.7],       [  15. ,   58. ,   22. ,    1.1],       [ -10. ,   57. ,   26. ,    0.7],       [ -38. ,   41. ,   20. ,    0.6],       [ -52. ,   25. ,   26. ,    1.5],       [ -55. ,   -2. ,   25. ,    1.1],       [ -46. ,  -23. ,   22. ,    0.8],       [ -34. ,  -47. ,   23. ,    0.7],       [ -17. ,  -53. ,   22. ,    1.1],       [   7. ,  -56. ,   28. ,    1. ],       [  30. ,  -41. ,   26. ,    1.3],       [  53. ,  -28. ,   21. ,    0.8],       [  57. ,   -5. ,   26. ,    1.2],       [  52. ,   22. ,   27. ,    1.3],       [  36. ,   45. ,   23. ,    1.2],       [  11. ,   62. ,   25. ,    1.1],       [ -11. ,   55. ,   20. ,    0.5],       [ -42. ,   48. ,   26. ,    0.7],       [ -58. ,   29. ,   21. ,    0.7],       [ -56. ,    0. ,   26. ,    1. ],       [ -53. ,  -29. ,   24. ,    0.7],       [ -40. ,  -45. ,   28. ,    1.2],       [  -9. ,  -62. ,   24. ,    1.2],       [  15. ,  -57. ,   20. ,    1.1],       [  32. ,  -48. ,   22. ,    0.8],       [  57. ,  -27. ,   26. ,    0.6],       [  63. ,    1. ,   21. ,    0.6],       [  61. ,   25. ,   28. ,    0.8],       [  44. ,   46. ,   28. ,    1. ],       [  18. ,   64. ,   21. ,    0.7],       [ -12. ,   62. ,   26. ,    1.2],       [ -43. ,   50. ,   23. ,    1.4],       [ -63. ,   24. ,   23. ,    0.7],       [ -68. ,   -2. ,   21. ,    1.4],       [ -62. ,  -30. ,   25. ,    1.4],       [ -39. ,  -51. ,   28. ,    0.8],       [ -13. ,  -68. ,   23. ,    0.9],       [  11. ,  -59. ,   27. ,    0.9],       [  36. ,  -54. ,   23. ,    0.9],       [  58. ,  -25. ,   21. ,    1.3],       [  69. ,    0. ,   20. ,    0.9],       [  67. ,   28. ,   28. ,    0.5],       [  45. ,   52. ,   19. ,    0.9],       [  10. ,   72. ,   20. ,    1.1],       [ -14. ,   64. ,   25. ,    1.3],       [ -43. ,   58. ,   24. ,    1.1],       [ -65. ,   28. ,   27. ,    1.1],       [ -74. ,    0. ,   21. ,    1.2],       [ -64. ,  -34. ,   26. ,    1. ],       [ -47. ,  -58. ,   22. ,    0.8],       [ -13. ,  -74. ,   28. ,    0.7],       [  16. ,  -70. ,   27. ,    0.6],       [  40. ,  -52. ,   27. ,    1. ],       [  63. ,  -33. ,   22. ,    1.4],       [  70. ,   -1. ,   20. ,    1.5],       [  70. ,   34. ,   28. ,    1.4],       [  50. ,   62. ,   24. ,    0.7],       [  13. ,   73. ,   20. ,    1. ],       [ -16. ,   70. ,   22. ,    0.8],       [ -51. ,   61. ,   20. ,    0.7],       [ -69. ,   27. ,   26. ,    0.5],       [ -80. ,   -5. ,   21. ,    0.8],       [ -69. ,  -35. ,   26. ,    1.4],       [ -44. ,  -64. ,   28. ,    1.3],       [ -22. ,  -73. ,   21. ,    1. ],       [  13. ,  -72. ,   26. ,    0.8],       [  50. ,  -58. ,   26. ,    1.3],       [  71. ,  -30. ,   20. ,    0.7],       [  75. ,   -3. ,   24. ,    1.2],       [  69. ,   37. ,   23. ,    1.2],       [  46. ,   63. ,   20. ,    0.9],       [  21. ,   79. ,   27. ,    1. ],       [ -20. ,   79. ,   24. ,    1.2],       [ -48. ,   64. ,   19. ,    0.8],       [ -77. ,   31. ,   27. ,    0.7],       [ -79. ,   -2. ,   23. ,    0.7],       [ -74. ,  -32. ,   28. ,    1.5],       [ -55. ,  -62. ,   21. ,    0.7],       [ -21. ,  -82. ,   27. ,    1. ],       [  19. ,  -74. ,   25. ,    1.1],       [  47. ,  -64. ,   25. ,    1.3],       [  72. ,  -36. ,   26. ,    0.6],       [  88. ,    2. ,   24. ,    1.3],       [  80. ,   33. ,   22. ,    1.2],       [  52. ,   64. ,   28. ,    1.5],       [  16. ,   85. ,   19. ,    1.2],       [ -20. ,   83. ,   27. ,    0.7],       [ -49. ,   63. ,   21. ,    1.5],       [ -79. ,   37. ,   26. ,    1.2],       [ -83. ,    4. ,   20. ,    1.1],       [ -77. ,  -37. ,   23. ,    1. ],       [ -50. ,  -64. ,   20. ,    1.1],       [ -20. ,  -79. ,   19. ,    1.4],       [  21. ,  -81. ,   28. ,    0.8],       [  55. ,  -63. ,   21. ,    0.8],       [  80. ,  -40. ,   21. ,    0.7],       [  93. ,    3. ,   24. ,    1.3],       [  82. ,   34. ,   20. ,    0.9],       [  55. ,   73. ,   21. ,    0.8],       [  23. ,   90. ,   25. ,    1. ],       [ -25. ,   84. ,   22. ,    1.5],       [ -52. ,   66. ,   20. ,    1.3],       [ -82. ,   42. ,   27. ,    1. ],       [ -91. ,    4. ,   20. ,    1.3],       [ -81. ,  -36. ,   26. ,    1. ],       [ -53. ,  -66. ,   20. ,    1.4],       [ -22. ,  -93. ,   28. ,    1.3],       [  20. ,  -90. ,   22. ,    1.5],       [  54. ,  -70. ,   23. ,    0.8],       [  77. ,  -43. ,   20. ,    1. ],       [  96. ,    3. ,   20. ,    1.3],       [  90. ,   36. ,   26. ,    1.4],       [  62. ,   70. ,   22. ,    0.8],       [  25. ,   91. ,   23. ,    1. ],       [ -23. ,   90. ,   19. ,    1.2],       [ -58. ,   74. ,   27. ,    1.4],       [ -85. ,   43. ,   22. ,    1.2],       [ -91. ,    0. ,   24. ,    1.3],       [ -87. ,  -40. ,   20. ,    1.3],       [ -59. ,  -79. ,   26. ,    0.9],       [ -18. ,  -96. ,   19. ,    0.8],       [  17. ,  -93. ,   20. ,    1.3],       [  57. ,  -77. ,   22. ,    1.2],       [  87. ,  -41. ,   19. ,    0.5],       [  98. ,   -1. ,   23. ,    0.6],       [  92. ,   45. ,   26. ,    1.3],       [  58. ,   73. ,   27. ,    1.1],       [  26. ,   93. ,   19. ,    0.8],       [ -25. ,   97. ,   28. ,    1. ],       [ -64. ,   79. ,   23. ,    0.5],       [ -88. ,   38. ,   26. ,    0.8],       [-104. ,   -3. ,   21. ,    0.7],       [ -89. ,  -48. ,   26. ,    0.7],       [ -65. ,  -83. ,   25. ,    0.8],       [ -20. , -101. ,   21. ,    1.1],       [  23. ,  -99. ,   23. ,    1.4],       [  66. ,  -76. ,   23. ,    0.9],       [  85. ,  -49. ,   27. ,    1.4],       [  46. ,   -4. ,   28. ,    0.7],       [  47. ,   22. ,   34. ,    0.8],       [  33. ,   39. ,   36. ,    1. ],       [  10. ,   50. ,   34. ,    0.8],       [ -10. ,   48. ,   33. ,    1.4],       [ -31. ,   43. ,   29. ,    1.4],       [ -49. ,   19. ,   36. ,    1.1],       [ -53. ,   -4. ,   36. ,    1. ],       [ -48. ,  -23. ,   35. ,    1.5],       [ -32. ,  -43. ,   36. ,    1.3],       [ -13. ,  -51. ,   30. ,    1.4],       [  15. ,  -45. ,   32. ,    0.5],       [  31. ,  -37. ,   27. ,    1.1],       [  45. ,  -26. ,   27. ,    1. ],       [  54. ,    4. ,   36. ,    0.7],       [  46. ,   27. ,   27. ,    0.5],       [  29. ,   38. ,   29. ,    1. ],       [  16. ,   53. ,   36. ,    1.1],       [ -15. ,   50. ,   34. ,    1. ],       [ -38. ,   45. ,   29. ,    1.2],       [ -49. ,   21. ,   30. ,    1.2],       [ -53. ,   -2. ,   31. ,    1.2],       [ -51. ,  -24. ,   34. ,    0.7],       [ -39. ,  -39. ,   31. ,    1.5],       [  -9. ,  -52. ,   36. ,    0.6],       [  11. ,  -57. ,   35. ,    0.6],       [  32. ,  -45. ,   35. ,    1.5],       [  46. ,  -26. ,   33. ,    0.9],       [  56. ,   -4. ,   29. ,    0.6],       [  49. ,   23. ,   27. ,    1.2],       [  37. ,   48. ,   29. ,    1.5],       [   9. ,   63. ,   36. ,    0.7],       [ -16. ,   54. ,   30. ,    1.5],       [ -40. ,   46. ,   32. ,    1. ],       [ -51. ,   30. ,   35. ,    1.3],       [ -58. ,    1. ,   30. ,    0.9],       [ -57. ,  -26. ,   32. ,    0.8],       [ -42. ,  -52. ,   34. ,    0.7],       [ -12. ,  -57. ,   33. ,    0.9],       [   8. ,  -57. ,   35. ,    1.1],       [  41. ,  -50. ,   33. ,    1. ],       [  55. ,  -25. ,   36. ,    1.2],       [  66. ,   -2. ,   32. ,    0.6],       [  53. ,   26. ,   36. ,    0.7],       [  43. ,   55. ,   33. ,    0.9],       [  10. ,   61. ,   35. ,    0.7],       [ -16. ,   63. ,   28. ,    1.4],       [ -43. ,   50. ,   36. ,    0.8],       [ -55. ,   25. ,   34. ,    0.7],       [ -68. ,   -4. ,   27. ,    1.3],       [ -55. ,  -29. ,   31. ,    1.3],       [ -36. ,  -51. ,   34. ,    0.7],       [ -20. ,  -63. ,   32. ,    0.9],       [  14. ,  -63. ,   30. ,    1.1],       [  39. ,  -51. ,   32. ,    1.5],       [  59. ,  -27. ,   27. ,    0.5],       [  71. ,    3. ,   35. ,    1.2],       [  67. ,   31. ,   36. ,    1.3],       [  47. ,   55. ,   30. ,    1.2],       [  19. ,   68. ,   29. ,    0.5],       [ -12. ,   67. ,   35. ,    0.9],       [ -45. ,   54. ,   28. ,    1.3],       [ -68. ,   26. ,   34. ,    1. ],       [ -71. ,   -3. ,   36. ,    1.4],       [ -62. ,  -32. ,   29. ,    1.1],       [ -43. ,  -56. ,   32. ,    0.9],       [ -21. ,  -72. ,   32. ,    1.3],       [  15. ,  -72. ,   35. ,    1.1],       [  47. ,  -56. ,   31. ,    0.8],       [  59. ,  -30. ,   27. ,    0.8],       [  76. ,    1. ,   28. ,    1. ],       [  65. ,   30. ,   36. ,    1. ],       [  50. ,   56. ,   34. ,    0.8],       [  18. ,   73. ,   29. ,    0.7],       [ -17. ,   68. ,   30. ,    0.5],       [ -44. ,   59. ,   33. ,    0.8],       [ -71. ,   35. ,   27. ,    1.3],       [ -75. ,    3. ,   28. ,    1.5],       [ -71. ,  -31. ,   32. ,    0.9],       [ -44. ,  -58. ,   28. ,    0.9],       [ -18. ,  -69. ,   31. ,    1.1],       [  21. ,  -72. ,   32. ,    0.7],       [  43. ,  -62. ,   29. ,    1.2],       [  63. ,  -32. ,   28. ,    0.5],       [  82. ,   -4. ,   28. ,    0.8],       [  75. ,   31. ,   28. ,    1.4],       [  47. ,   58. ,   29. ,    0.7],       [  15. ,   76. ,   36. ,    1.2],       [ -16. ,   80. ,   31. ,    1.2],       [ -48. ,   64. ,   33. ,    0.9],       [ -72. ,   31. ,   31. ,    1.2],       [ -82. ,   -4. ,   34. ,    1.3],       [ -77. ,  -36. ,   35. ,    0.9],       [ -54. ,  -61. ,   29. ,    1.2],       [ -19. ,  -78. ,   32. ,    1. ],       [  16. ,  -78. ,   27. ,    1.1],       [  54. ,  -58. ,   34. ,    1. ],       [  77. ,  -37. ,   34. ,    0.9],       [  82. ,   -4. ,   32. ,    1. ],       [  76. ,   34. ,   34. ,    0.7],       [  50. ,   62. ,   33. ,    1.1],       [  15. ,   85. ,   34. ,    0.9],       [ -24. ,   82. ,   36. ,    0.8],       [ -56. ,   70. ,   33. ,    0.9],       [ -75. ,   38. ,   35. ,    0.9],       [ -89. ,   -1. ,   33. ,    0.9],       [ -81. ,  -35. ,   27. ,    1. ],       [ -51. ,  -64. ,   36. ,    1.2],       [ -22. ,  -84. ,   30. ,    1.3],       [  18. ,  -88. ,   35. ,    1.1],       [  50. ,  -70. ,   35. ,    0.9],       [  71. ,  -34. ,   33. ,    1.1],       [  90. ,   -2. ,   28. ,    0.5],       [  77. ,   38. ,   35. ,    0.9],       [  52. ,   68. ,   34. ,    0.6],       [  18. ,   83. ,   35. ,    1.4],       [ -22. ,   87. ,   30. ,    1.5],       [ -55. ,   75. ,   29. ,    0.8],       [ -84. ,   39. ,   35. ,    0.8],       [ -88. ,    3. ,   31. ,    1.1],       [ -78. ,  -35. ,   32. ,    1.4],       [ -57. ,  -70. ,   29. ,    1. ],       [ -16. ,  -90. ,   35. ,    1.4],       [  17. ,  -86. ,   33. ,    0.7],       [  58. ,  -74. ,   27. ,    0.8],       [  78. ,  -42. ,   29. ,    1.5],       [  96. ,    3. ,   27. ,    1.2],       [  88. ,   40. ,   29. ,    1.3],       [  60. ,   74. ,   31. ,    1.3],       [  18. ,   92. ,   34. ,    0.8],       [ -25. ,   87. ,   36. ,    1.2],       [ -56. ,   75. ,   29. ,    1.2],       [ -91. ,   43. ,   33. ,    0.7],       [ -94. ,   -4. ,   29. ,    1. ],       [ -84. ,  -37. ,   35. ,    0.6],       [ -63. ,  -79. ,   27. ,    0.5],       [ -24. ,  -96. ,   27. ,    1.4],       [  23. ,  -93. ,   31. ,    0.6],       [  61. ,  -76. ,   33. ,    0.9],       [  83. ,  -37. ,   30. ,    0.9],       [  96. ,    1. ,   27. ,    0.8],       [  93. ,   40. ,   29. ,    0.9],       [  62. ,   75. ,   36. ,    1.2],       [  22. ,  102. ,   30. ,    1. ],       [ -19. ,   95. ,   28. ,    1. ],       [ -68. ,   77. ,   35. ,    1. ],       [ -87. ,   39. ,   28. ,    1.3],       [ -99. ,    2. ,   32. ,    1. ],       [ -93. ,  -44. ,   35. ,    1.3],       [ -62. ,  -81. ,   36. ,    1.1],       [ -22. , -102. ,   34. ,    1. ],       [  19. ,  -95. ,   34. ,    0.9],       [  57. ,  -81. ,   28. ,    1.2],       [  91. ,  -43. ,   35. ,    1. ],       [  45. ,    1. ,   43. ,    0.9],       [  45. ,   22. ,   37. ,    0.7],       [  35. ,   36. ,   37. ,    1.3],       [   7. ,   47. ,   42. ,    0.8],       [ -10. ,   51. ,   41. ,    1. ],       [ -35. ,   35. ,   35. ,    1. ],       [ -41. ,   19. ,   44. ,    0.8],       [ -48. ,    3. ,   44. ,    0.6],       [ -42. ,  -26. ,   42. ,    1. ],       [ -35. ,  -43. ,   39. ,    1.2],       [  -8. ,  -46. ,   37. ,    0.6],       [  12. ,  -54. ,   36. ,    1.2],       [  28. ,  -44. ,   39. ,    1. ],       [  48. ,  -26. ,   36. ,    1.3],       [  53. ,    1. ,   39. ,    0.7],       [  53. ,   23. ,   40. ,    1. ],       [  36. ,   38. ,   40. ,    0.8],       [  12. ,   53. ,   37. ,    1.3],       [ -16. ,   55. ,   44. ,    1.1],       [ -39. ,   41. ,   42. ,    1.4],       [ -49. ,   28. ,   39. ,    1.1],       [ -59. ,    4. ,   40. ,    1.4],       [ -47. ,  -28. ,   35. ,    1.5],       [ -36. ,  -39. ,   37. ,    0.7],       [ -16. ,  -58. ,   44. ,    0.7],       [  10. ,  -53. ,   44. ,    1.2],       [  34. ,  -43. ,   36. ,    1.3],       [  44. ,  -28. ,   39. ,    0.9],       [  64. ,   -2. ,   37. ,    0.5],       [  54. ,   29. ,   42. ,    0.9],       [  39. ,   46. ,   42. ,    1.1],       [  12. ,   54. ,   40. ,    1.4],       [ -10. ,   54. ,   40. ,    0.9],       [ -43. ,   51. ,   42. ,    0.9],       [ -59. ,   27. ,   42. ,    0.9],       [ -63. ,    0. ,   40. ,    0.9],       [ -52. ,  -28. ,   39. ,    0.9],       [ -38. ,  -52. ,   35. ,    1.3],       [ -16. ,  -58. ,   43. ,    0.9],       [  18. ,  -60. ,   35. ,    1.4],       [  37. ,  -51. ,   36. ,    1.3],       [  51. ,  -24. ,   39. ,    1.3],       [  62. ,   -1. ,   41. ,    0.6],       [  60. ,   25. ,   36. ,    0.8],       [  45. ,   46. ,   37. ,    1.3],       [  15. ,   68. ,   41. ,    0.9],       [ -12. ,   63. ,   44. ,    0.9],       [ -41. ,   50. ,   38. ,    1.4],       [ -58. ,   26. ,   41. ,    1.3],       [ -66. ,    0. ,   37. ,    1.3],       [ -58. ,  -24. ,   42. ,    0.9],       [ -38. ,  -49. ,   36. ,    0.8],       [ -18. ,  -68. ,   44. ,    1.4],       [  11. ,  -63. ,   43. ,    0.9],       [  43. ,  -52. ,   36. ,    1.4],       [  60. ,  -32. ,   40. ,    1.5],       [  73. ,    3. ,   35. ,    0.6],       [  67. ,   28. ,   35. ,    1.2],       [  40. ,   53. ,   43. ,    1.3],       [  13. ,   64. ,   42. ,    1.3],       [ -19. ,   69. ,   37. ,    1.2],       [ -47. ,   58. ,   36. ,    1.2],       [ -65. ,   29. ,   39. ,    1.4],       [ -70. ,    2. ,   39. ,    0.7],       [ -64. ,  -28. ,   39. ,    0.5],       [ -40. ,  -52. ,   36. ,    0.7],       [ -13. ,  -66. ,   43. ,    0.9],       [  11. ,  -71. ,   39. ,    0.9],       [  43. ,  -57. ,   38. ,    1.4],       [  67. ,  -28. ,   40. ,    1.1],       [  75. ,    2. ,   40. ,    0.6],       [  68. ,   28. ,   38. ,    0.6],       [  46. ,   58. ,   40. ,    0.6],       [  21. ,   73. ,   44. ,    0.8],       [ -14. ,   72. ,   36. ,    0.9],       [ -50. ,   60. ,   42. ,    0.8],       [ -70. ,   35. ,   43. ,    0.7],       [ -75. ,    3. ,   35. ,    0.9],       [ -66. ,  -29. ,   44. ,    0.9],       [ -43. ,  -55. ,   40. ,    1. ],       [ -19. ,  -74. ,   42. ,    0.5],       [  12. ,  -75. ,   35. ,    1.2],       [  42. ,  -59. ,   37. ,    1.1],       [  69. ,  -34. ,   39. ,    1.2],       [  76. ,    4. ,   35. ,    1.1],       [  74. ,   35. ,   35. ,    1. ],       [  53. ,   64. ,   36. ,    1. ],       [  15. ,   79. ,   42. ,    1.2],       [ -17. ,   79. ,   44. ,    0.5],       [ -54. ,   58. ,   37. ,    0.9],       [ -73. ,   30. ,   43. ,    1.3],       [ -83. ,    2. ,   37. ,    1.2],       [ -72. ,  -33. ,   43. ,    1.4],       [ -47. ,  -68. ,   40. ,    0.9],       [ -22. ,  -75. ,   42. ,    1.1],       [  20. ,  -78. ,   41. ,    0.9],       [  52. ,  -58. ,   38. ,    1. ],       [  67. ,  -40. ,   36. ,    1.5],       [  82. ,    4. ,   38. ,    1.4],       [  80. ,   33. ,   35. ,    0.9],       [  53. ,   63. ,   36. ,    0.5],       [  19. ,   80. ,   43. ,    0.9],       [ -20. ,   78. ,   40. ,    1.3],       [ -55. ,   61. ,   38. ,    1.4],       [ -76. ,   32. ,   37. ,    0.7],       [ -81. ,    3. ,   40. ,    1. ],       [ -80. ,  -33. ,   44. ,    0.8],       [ -58. ,  -65. ,   39. ,    1.1],       [ -24. ,  -79. ,   41. ,    0.6],       [  19. ,  -81. ,   37. ,    1.5],       [  50. ,  -69. ,   38. ,    1.4],       [  73. ,  -33. ,   37. ,    0.8],       [  85. ,   -1. ,   40. ,    0.9],       [  82. ,   42. ,   38. ,    0.9],       [  55. ,   71. ,   36. ,    1.4],       [  18. ,   89. ,   43. ,    1.4],       [ -23. ,   87. ,   42. ,    0.6],       [ -57. ,   69. ,   44. ,    1.4],       [ -82. ,   41. ,   41. ,    1.3],       [ -95. ,   -5. ,   42. ,    1. ],       [ -85. ,  -42. ,   39. ,    0.7],       [ -54. ,  -72. ,   38. ,    0.6],       [ -20. ,  -93. ,   36. ,    0.6],       [  22. ,  -90. ,   39. ,    1.5],       [  53. ,  -73. ,   35. ,    1.4],       [  85. ,  -43. ,   38. ,    1.2],       [  94. ,   -2. ,   40. ,    1. ],       [  87. ,   41. ,   41. ,    0.9],       [  57. ,   78. ,   41. ,    1.4],       [  24. ,   90. ,   35. ,    1.5],       [ -18. ,   91. ,   40. ,    1. ],       [ -62. ,   70. ,   42. ,    0.9],       [ -84. ,   45. ,   35. ,    1.1],       [ -96. ,    3. ,   42. ,    0.8],       [ -82. ,  -44. ,   40. ,    0.6],       [ -58. ,  -73. ,   40. ,    1.2],       [ -23. ,  -94. ,   43. ,    0.7],       [  21. ,  -96. ,   35. ,    1.2],       [  58. ,  -79. ,   35. ,    1.3],       [  83. ,  -46. ,   38. ,    1. ],       [  99. ,    3. ,   44. ,    0.6],       [  88. ,   43. ,   39. ,    0.6],       [  65. ,   80. ,   35. ,    0.9],       [  19. ,   99. ,   40. ,    0.6],       [ -22. ,   97. ,   35. ,    1.4],       [ -66. ,   80. ,   38. ,    1.2],       [ -86. ,   45. ,   43. ,    0.5],       [ -97. ,   -5. ,   40. ,    0.5],       [ -92. ,  -41. ,   37. ,    1.3],       [ -63. ,  -83. ,   43. ,    0.5],       [ -20. ,  -99. ,   37. ,    1. ],       [  17. ,  -96. ,   36. ,    1.2],       [  65. ,  -83. ,   44. ,    1.3],       [  90. ,  -47. ,   38. ,    1.2]])
        basis0 = BSplineBasis(7, np.array([ -0.7,   0. ,   0. ,   0. ,   0. ,   0. ,   0. ,   1. ,   1.8,   2.8,   3.7,   5.2,   6.2,
         6.8,   8.3,   9. ,   9. ,   9. ,   9. ,   9. ,   9. ,  10. ]),0)
        basis1 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  2.1,  3.4,  3.6,  5. ,  5. ,  5. ,  5. ,
        5. ,  5. ,  5. ]))
        basis2 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.8,  2. ,  2. ,  2. ,  2. ,  2. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(1, 2, 3)
        self.assertEqual(vol2.order(direction=0), 8)
        self.assertEqual(vol2.order(direction=1), 9)
        self.assertEqual(vol2.order(direction=2), 8)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p7_rational_C1_periodic(self):
        controlpoints = np.array([[  97. ,    1. ,   -2. ,    1.1],
       [  96. ,   37. ,    3. ,    1.1],
       [  75. ,   68. ,    4. ,    1.3],
       [  41. ,   92. ,    1. ,    1.3],
       [   1. ,   96. ,    0. ,    0.8],
       [ -39. ,   94. ,   -3. ,    0.9],
       [ -73. ,   75. ,    0. ,    1.3],
       [ -89. ,   36. ,   -4. ,    0.7],
       [-100. ,    1. ,    4. ,    0.6],
       [ -89. ,  -38. ,    2. ,    1.3],
       [ -71. ,  -70. ,   -2. ,    1. ],
       [ -38. ,  -90. ,   -3. ,    0.8],
       [  -5. , -104. ,   -5. ,    0.9],
       [  35. ,  -94. ,   -4. ,    1.4],
       [  73. ,  -73. ,   -5. ,    1.3],
       [  89. ,  -39. ,    4. ,    1.3]])
        basis0 = BSplineBasis(7, np.array([ -2. ,  -0.9,   0. ,   0. ,   0. ,   0. ,   0. ,   1.1,   2.3,   2.9,   3.7,   5.3,   5.6,
         6.7,   7.6,   8.9,  10. ,  11.1,  12. ,  12. ,  12. ,  12. ,  12. ,  13.1,  14.3]),1)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p76_rational_C1_periodic(self):
        controlpoints = np.array([[  57. ,    2. ,   -3. ,    0.9],       [  58. ,   26. ,    3. ,    1.3],       [  39. ,   43. ,   -1. ,    1.1],       [  20. ,   57. ,   -2. ,    0.6],       [  -5. ,   63. ,   -5. ,    0.8],       [ -24. ,   53. ,    2. ,    1.3],       [ -47. ,   39. ,    2. ,    0.8],       [ -60. ,   27. ,   -1. ,    0.6],       [ -58. ,    3. ,   -1. ,    0.6],       [ -53. ,  -23. ,   -5. ,    0.8],       [ -47. ,  -46. ,    1. ,    1.2],       [ -27. ,  -58. ,   -5. ,    0.6],       [  -3. ,  -64. ,    0. ,    0.8],       [  25. ,  -55. ,   -5. ,    1.1],       [  38. ,  -45. ,   -2. ,    0.9],       [  52. ,  -19. ,   -5. ,    1.4],       [  68. ,    2. ,    0. ,    1. ],       [  54. ,   29. ,   -3. ,    1.2],       [  42. ,   41. ,   -1. ,    1.3],       [  29. ,   60. ,   -3. ,    1.1],       [   0. ,   63. ,   -3. ,    1.5],       [ -26. ,   59. ,   -3. ,    0.7],       [ -45. ,   46. ,    3. ,    0.8],       [ -57. ,   29. ,   -2. ,    0.7],       [ -61. ,   -5. ,   -5. ,    0.6],       [ -55. ,  -28. ,    1. ,    1.3],       [ -46. ,  -42. ,   -3. ,    0.7],       [ -26. ,  -59. ,    2. ,    0.6],       [  -4. ,  -61. ,    1. ,    1.2],       [  24. ,  -62. ,   -2. ,    0.9],       [  47. ,  -47. ,    0. ,    1.4],       [  58. ,  -26. ,    4. ,    0.6],       [  66. ,    1. ,   -1. ,    0.8],       [  65. ,   23. ,    4. ,    1.4],       [  50. ,   50. ,   -2. ,    1.4],       [  26. ,   64. ,   -2. ,    0.6],       [   3. ,   66. ,    4. ,    1.4],       [ -31. ,   59. ,   -4. ,    1.4],       [ -45. ,   46. ,    4. ,    0.8],       [ -60. ,   29. ,    0. ,    0.9],       [ -70. ,   -2. ,    4. ,    0.7],       [ -61. ,  -30. ,   -1. ,    1.3],       [ -46. ,  -45. ,    2. ,    0.9],       [ -24. ,  -69. ,   -2. ,    1.1],       [  -2. ,  -74. ,   -2. ,    0.6],       [  26. ,  -64. ,   -4. ,    1.1],       [  44. ,  -49. ,    3. ,    0.7],       [  64. ,  -30. ,    3. ,    0.6],       [  69. ,    3. ,    0. ,    1.2],       [  70. ,   31. ,    3. ,    1.4],       [  49. ,   46. ,   -1. ,    1.3],       [  31. ,   71. ,    1. ,    0.6],       [  -4. ,   76. ,   -2. ,    1.2],       [ -31. ,   69. ,    1. ,    1.1],       [ -48. ,   50. ,    4. ,    1.2],       [ -65. ,   26. ,    2. ,    0.8],       [ -75. ,    1. ,    4. ,    0.9],       [ -72. ,  -26. ,    1. ,    0.5],       [ -56. ,  -52. ,   -4. ,    1.3],       [ -26. ,  -68. ,    4. ,    0.8],       [   2. ,  -70. ,   -2. ,    1.5],       [  23. ,  -66. ,   -4. ,    1. ],       [  53. ,  -56. ,    4. ,    1.3],       [  63. ,  -29. ,   -4. ,    1.4],       [  77. ,   -5. ,   -2. ,    0.6],       [  76. ,   32. ,   -1. ,    0.9],       [  51. ,   53. ,   -4. ,    1.2],       [  27. ,   75. ,   -5. ,    1.1],       [   0. ,   82. ,    3. ,    0.8],       [ -26. ,   73. ,    2. ,    1.2],       [ -55. ,   50. ,    2. ,    0.7],       [ -70. ,   32. ,    2. ,    1.5],       [ -75. ,   -1. ,    1. ,    0.9],       [ -68. ,  -29. ,   -4. ,    1.1],       [ -59. ,  -59. ,    0. ,    1.2],       [ -29. ,  -74. ,    4. ,    1.4],       [  -4. ,  -82. ,   -1. ,    1.3],       [  24. ,  -72. ,   -1. ,    1.3],       [  56. ,  -54. ,    3. ,    1.1],       [  70. ,  -33. ,    2. ,    0.7],       [  85. ,   -4. ,    1. ,    1. ],       [  77. ,   27. ,   -3. ,    1.2],       [  60. ,   56. ,    3. ,    0.8],       [  28. ,   73. ,   -5. ,    0.5],       [  -5. ,   79. ,    4. ,    1.4],       [ -29. ,   80. ,    1. ,    0.9],       [ -56. ,   55. ,   -4. ,    1.2],       [ -80. ,   35. ,    2. ,    0.7],       [ -86. ,    1. ,   -1. ,    1.3],       [ -76. ,  -31. ,    3. ,    1. ],       [ -62. ,  -56. ,   -1. ,    0.7],       [ -33. ,  -74. ,   -4. ,    0.7],       [  -2. ,  -88. ,   -1. ,    1.1],       [  35. ,  -76. ,   -1. ,    0.8],       [  57. ,  -62. ,   -5. ,    0.5],       [  73. ,  -30. ,   -3. ,    1.2],       [  85. ,   -4. ,   -4. ,    0.8],       [  82. ,   31. ,   -2. ,    1. ],       [  65. ,   61. ,   -2. ,    0.8],       [  36. ,   83. ,    2. ,    0.6],       [   0. ,   83. ,    4. ,    0.6],       [ -34. ,   83. ,    2. ,    1.3],       [ -65. ,   59. ,    4. ,    1.1],       [ -78. ,   30. ,    0. ,    1.1],       [ -91. ,    2. ,    2. ,    0.9],       [ -84. ,  -35. ,    2. ,    0.7],       [ -60. ,  -58. ,   -5. ,    1.3],       [ -34. ,  -80. ,   -4. ,    0.6],       [  -1. ,  -90. ,    2. ,    1.4],       [  28. ,  -82. ,    2. ,    1.4],       [  60. ,  -64. ,   -4. ,    0.8],       [  84. ,  -37. ,   -1. ,    1.1],       [  92. ,   -1. ,   -1. ,    1.4],       [  88. ,   30. ,    2. ,    0.7],       [  64. ,   64. ,    3. ,    1.1],       [  39. ,   81. ,    3. ,    1.3],       [  -3. ,   95. ,   -2. ,    1.1],       [ -39. ,   85. ,    0. ,    1.2],       [ -66. ,   61. ,   -2. ,    0.9],       [ -88. ,   34. ,    3. ,    0.8],       [ -90. ,   -2. ,    4. ,    1.3],       [ -88. ,  -38. ,    1. ,    0.8],       [ -66. ,  -61. ,    4. ,    1.1],       [ -38. ,  -80. ,   -3. ,    0.9],       [   0. ,  -88. ,   -3. ,    0.8],       [  33. ,  -83. ,    2. ,    1.3],       [  61. ,  -66. ,    4. ,    0.8],       [  82. ,  -38. ,   -5. ,    0.6],       [  92. ,   -4. ,    4. ,    0.9],       [  90. ,   37. ,   -2. ,    1.5],       [  70. ,   66. ,   -3. ,    1.4],       [  41. ,   85. ,    2. ,    1.1],       [  -3. ,   93. ,    2. ,    1.2],       [ -37. ,   90. ,   -3. ,    0.8],       [ -65. ,   67. ,    1. ,    1.1],       [ -89. ,   34. ,   -1. ,    0.7],       [ -98. ,   -3. ,    4. ,    0.8],       [ -85. ,  -33. ,    0. ,    0.9],       [ -63. ,  -63. ,    3. ,    1.1],       [ -33. ,  -89. ,   -5. ,    1.4],       [   4. ,  -92. ,   -5. ,    0.7],       [  37. ,  -90. ,    1. ,    0.5],       [  69. ,  -69. ,    2. ,    1.2],       [  90. ,  -41. ,   -4. ,    0.8],       [  96. ,    3. ,    2. ,    0.6],       [  96. ,   40. ,   -3. ,    1.3],       [  66. ,   71. ,    0. ,    1.3],       [  37. ,   90. ,   -2. ,    1.1],       [  -4. ,   98. ,    3. ,    0.7],       [ -42. ,   97. ,    3. ,    0.5],       [ -75. ,   69. ,   -3. ,    1.5],       [ -96. ,   40. ,    1. ,    0.7],       [-102. ,   -5. ,   -2. ,    1.5],       [ -88. ,  -43. ,    0. ,    1.4],       [ -68. ,  -76. ,   -4. ,    0.6],       [ -38. ,  -90. ,    4. ,    1. ],       [  -1. , -100. ,    1. ,    0.8],       [  40. ,  -92. ,    2. ,    0.7],       [  67. ,  -74. ,   -3. ,    1.3],       [  93. ,  -42. ,    3. ,    0.5]])
        basis0 = BSplineBasis(7, np.array([ -1.6,  -0.7,   0. ,   0. ,   0. ,   0. ,   0. ,   1.3,   1.7,   3. ,   3.6,   5.1,   5.7,
         7.1,   8. ,   9.2,  10.4,  11.3,  12. ,  12. ,  12. ,  12. ,  12. ,  13.3,  13.7]),1)
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  1.7,  3. ,  4.4,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(3, 2)
        self.assertEqual(surf2.order(direction=0), 10)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p765_rational_C1_periodic(self):
        controlpoints = np.array([[  52. ,    3. ,    1. ,    1.2],       [  43. ,   17. ,    0. ,    0.9],       [  30. ,   34. ,    1. ,    1. ],       [  17. ,   49. ,    2. ,    1.2],       [  -1. ,   52. ,    0. ,    0.8],       [ -21. ,   41. ,    3. ,    0.9],       [ -34. ,   30. ,    4. ,    1.3],       [ -52. ,   16. ,   -5. ,    1. ],       [ -53. ,    0. ,   -4. ,    1.2],       [ -49. ,  -19. ,   -2. ,    1.4],       [ -39. ,  -40. ,   -5. ,    1.5],       [ -16. ,  -47. ,    0. ,    0.8],       [  -1. ,  -49. ,   -2. ,    1.1],       [  16. ,  -49. ,    1. ,    0.7],       [  38. ,  -38. ,    2. ,    1.4],       [  46. ,  -21. ,   -5. ,    1.4],       [  54. ,   -1. ,    4. ,    0.6],       [  48. ,   22. ,    1. ,    0.6],       [  41. ,   39. ,   -2. ,    1.5],       [  22. ,   56. ,    1. ,    1.3],       [  -4. ,   54. ,    1. ,    1.5],       [ -27. ,   51. ,   -1. ,    1. ],       [ -41. ,   44. ,   -4. ,    0.5],       [ -54. ,   22. ,    0. ,    0.6],       [ -54. ,    3. ,    0. ,    1.5],       [ -53. ,  -23. ,   -2. ,    1.2],       [ -44. ,  -44. ,   -4. ,    1.4],       [ -22. ,  -51. ,   -4. ,    0.8],       [   0. ,  -53. ,    1. ,    1.1],       [  19. ,  -50. ,   -4. ,    1. ],       [  43. ,  -40. ,   -3. ,    1.5],       [  52. ,  -27. ,   -2. ,    1.4],       [  61. ,   -4. ,    3. ,    0.9],       [  61. ,   20. ,   -4. ,    1.4],       [  43. ,   48. ,    0. ,    1.1],       [  29. ,   63. ,    3. ,    1.3],       [   1. ,   61. ,    1. ,    1. ],       [ -21. ,   64. ,   -1. ,    0.6],       [ -41. ,   43. ,   -1. ,    0.8],       [ -64. ,   21. ,   -1. ,    1.3],       [ -67. ,    1. ,   -5. ,    1.5],       [ -55. ,  -23. ,    1. ,    0.8],       [ -46. ,  -42. ,    2. ,    0.9],       [ -25. ,  -56. ,   -2. ,    1.3],       [   4. ,  -67. ,    2. ,    0.6],       [  24. ,  -60. ,    1. ,    1.4],       [  42. ,  -45. ,   -5. ,    1.3],       [  60. ,  -23. ,    3. ,    1.2],       [  75. ,   -1. ,    1. ,    1.1],       [  67. ,   26. ,   -2. ,    1.4],       [  48. ,   52. ,   -5. ,    1.2],       [  23. ,   62. ,   -1. ,    0.6],       [  -5. ,   73. ,    4. ,    1.3],       [ -24. ,   67. ,    4. ,    1.5],       [ -55. ,   50. ,    4. ,    0.6],       [ -62. ,   25. ,    3. ,    1.1],       [ -74. ,   -2. ,    0. ,    0.9],       [ -65. ,  -24. ,   -1. ,    1.4],       [ -55. ,  -53. ,   -5. ,    0.6],       [ -24. ,  -69. ,    1. ,    1.1],       [  -1. ,  -71. ,    1. ,    0.5],       [  31. ,  -67. ,    1. ,    1.5],       [  46. ,  -48. ,    1. ,    1. ],       [  63. ,  -25. ,   -1. ,    1.4],       [  78. ,    1. ,    4. ,    0.8],       [  75. ,   33. ,    4. ,    0.9],       [  52. ,   59. ,   -3. ,    0.8],       [  27. ,   76. ,    3. ,    0.6],       [  -4. ,   78. ,   -2. ,    0.6],       [ -34. ,   67. ,   -1. ,    0.7],       [ -59. ,   52. ,    0. ,    0.6],       [ -70. ,   26. ,   -3. ,    1.4],       [ -76. ,   -4. ,    3. ,    0.9],       [ -71. ,  -28. ,    0. ,    1.2],       [ -56. ,  -56. ,    0. ,    1.2],       [ -30. ,  -71. ,    1. ,    1.2],       [   0. ,  -84. ,    2. ,    1. ],       [  31. ,  -78. ,   -3. ,    0.7],       [  59. ,  -54. ,   -4. ,    1.1],       [  75. ,  -30. ,   -3. ,    0.9],       [  85. ,    1. ,    1. ,    0.7],       [  79. ,   36. ,   -4. ,    1.4],       [  58. ,   61. ,    3. ,    1.3],       [  30. ,   76. ,    1. ,    1.2],       [   2. ,   81. ,   -3. ,    1.1],       [ -35. ,   78. ,    1. ,    1.2],       [ -64. ,   56. ,   -1. ,    1.4],       [ -75. ,   29. ,   -3. ,    1.4],       [ -82. ,   -2. ,   -4. ,    0.8],       [ -83. ,  -36. ,   -3. ,    1.1],       [ -61. ,  -59. ,   -5. ,    1.4],       [ -37. ,  -76. ,   -1. ,    1.2],       [   1. ,  -90. ,    4. ,    0.6],       [  36. ,  -84. ,   -2. ,    1.3],       [  56. ,  -62. ,   -5. ,    0.5],       [  79. ,  -29. ,    0. ,    0.6],       [  89. ,   -4. ,    4. ,    0.5],       [  86. ,   39. ,    1. ,    0.9],       [  67. ,   65. ,    0. ,    0.9],       [  38. ,   81. ,    4. ,    1.2],       [  -3. ,   93. ,    3. ,    0.6],       [ -34. ,   86. ,    3. ,    0.9],       [ -66. ,   62. ,   -5. ,    0.6],       [ -88. ,   32. ,    2. ,    0.6],       [ -88. ,   -4. ,   -2. ,    0.8],       [ -83. ,  -38. ,    1. ,    0.8],       [ -65. ,  -61. ,   -1. ,    1.4],       [ -39. ,  -84. ,   -2. ,    0.7],       [  -1. ,  -92. ,   -4. ,    1.1],       [  31. ,  -88. ,    3. ,    0.8],       [  68. ,  -71. ,    4. ,    0.9],       [  88. ,  -37. ,   -3. ,    0.8],       [  95. ,   -5. ,    2. ,    0.7],       [  89. ,   34. ,    4. ,    0.9],       [  72. ,   67. ,   -2. ,    0.5],       [  38. ,   88. ,   -2. ,    1.1],       [  -2. ,   97. ,    3. ,    1.3],       [ -36. ,   88. ,   -4. ,    1.1],       [ -73. ,   66. ,   -3. ,    1.2],       [ -94. ,   43. ,    3. ,    1.1],       [-103. ,   -5. ,   -1. ,    1.4],       [ -88. ,  -34. ,    3. ,    0.6],       [ -72. ,  -71. ,    0. ,    0.7],       [ -43. ,  -93. ,   -4. ,    0.6],       [  -2. , -104. ,    1. ,    1.3],       [  35. ,  -89. ,   -5. ,    1.2],       [  66. ,  -73. ,    2. ,    0.5],       [  93. ,  -39. ,    4. ,    1.5],       [  53. ,    0. ,   10. ,    1.2],       [  48. ,   20. ,   10. ,    0.5],       [  31. ,   36. ,   11. ,    0.6],       [  20. ,   42. ,    5. ,    0.6],       [   2. ,   49. ,   12. ,    1. ],       [ -18. ,   45. ,    9. ,    1.3],       [ -32. ,   37. ,    5. ,    0.8],       [ -42. ,   23. ,    5. ,    1.1],       [ -48. ,    1. ,   10. ,    1.2],       [ -46. ,  -21. ,    8. ,    1.3],       [ -33. ,  -36. ,    5. ,    1.3],       [ -22. ,  -50. ,    3. ,    0.8],       [  -4. ,  -48. ,    8. ,    1.4],       [  21. ,  -45. ,    8. ,    1.4],       [  36. ,  -33. ,    7. ,    0.8],       [  49. ,  -16. ,    8. ,    1.2],       [  58. ,   -4. ,    5. ,    1. ],       [  50. ,   25. ,    3. ,    1.3],       [  42. ,   42. ,    3. ,    0.7],       [  24. ,   55. ,    8. ,    1.4],       [  -2. ,   56. ,    6. ,    0.7],       [ -20. ,   54. ,    3. ,    0.8],       [ -45. ,   43. ,   11. ,    0.7],       [ -50. ,   22. ,   12. ,    1.5],       [ -61. ,   -2. ,    4. ,    0.8],       [ -50. ,  -26. ,    7. ,    0.6],       [ -45. ,  -37. ,    5. ,    0.6],       [ -23. ,  -58. ,    4. ,    0.7],       [  -2. ,  -54. ,    7. ,    1. ],       [  22. ,  -56. ,    5. ,    1.3],       [  38. ,  -41. ,   10. ,    1. ],       [  55. ,  -23. ,    6. ,    0.9],       [  64. ,   -1. ,    7. ,    1.4],       [  55. ,   27. ,    5. ,    0.5],       [  44. ,   41. ,    4. ,    1. ],       [  27. ,   56. ,    3. ,    0.7],       [   1. ,   60. ,   10. ,    1.2],       [ -23. ,   58. ,    9. ,    0.7],       [ -49. ,   41. ,   11. ,    0.6],       [ -56. ,   28. ,   11. ,    1.1],       [ -68. ,    3. ,    3. ,    1. ],       [ -58. ,  -30. ,   11. ,    0.9],       [ -44. ,  -48. ,    4. ,    0.8],       [ -22. ,  -58. ,   10. ,    1. ],       [  -4. ,  -62. ,   12. ,    1. ],       [  21. ,  -62. ,   12. ,    0.8],       [  43. ,  -42. ,   10. ,    1.2],       [  63. ,  -23. ,    6. ,    0.9],       [  76. ,   -3. ,    6. ,    1.1],       [  65. ,   25. ,    9. ,    1.1],       [  45. ,   45. ,    9. ,    1.2],       [  22. ,   67. ,    5. ,    1.2],       [  -4. ,   76. ,    4. ,    1. ],       [ -32. ,   69. ,   11. ,    0.5],       [ -47. ,   54. ,    9. ,    0.8],       [ -70. ,   26. ,    8. ,    1.2],       [ -69. ,    3. ,    9. ,    1.4],       [ -71. ,  -33. ,   10. ,    1.4],       [ -48. ,  -46. ,    3. ,    0.9],       [ -24. ,  -63. ,    9. ,    1.4],       [   4. ,  -77. ,   10. ,    1. ],       [  25. ,  -63. ,    3. ,    1.3],       [  50. ,  -51. ,   11. ,    0.5],       [  67. ,  -30. ,    5. ,    1.4],       [  74. ,   -2. ,    7. ,    1.1],       [  73. ,   26. ,    9. ,    1. ],       [  54. ,   57. ,    9. ,    1.3],       [  26. ,   76. ,    8. ,    1.4],       [   1. ,   79. ,    7. ,    1.5],       [ -35. ,   76. ,    5. ,    0.9],       [ -59. ,   51. ,    5. ,    1.4],       [ -72. ,   33. ,    5. ,    1.3],       [ -80. ,   -4. ,   11. ,    1. ],       [ -73. ,  -34. ,    8. ,    1.3],       [ -52. ,  -59. ,    4. ,    1. ],       [ -30. ,  -75. ,    6. ,    1. ],       [  -3. ,  -82. ,   12. ,    1.3],       [  26. ,  -73. ,   12. ,    1.1],       [  55. ,  -55. ,    6. ,    0.7],       [  72. ,  -31. ,    9. ,    0.7],       [  82. ,    3. ,    4. ,    1.2],       [  74. ,   29. ,   12. ,    1.4],       [  57. ,   61. ,    4. ,    1. ],       [  33. ,   80. ,    8. ,    1.2],       [   4. ,   81. ,    6. ,    1.4],       [ -35. ,   76. ,    6. ,    0.8],       [ -57. ,   63. ,   10. ,    1.1],       [ -84. ,   30. ,    3. ,    0.7],       [ -90. ,   -5. ,   10. ,    0.7],       [ -76. ,  -31. ,    3. ,    1.3],       [ -65. ,  -56. ,   12. ,    1. ],       [ -30. ,  -83. ,    9. ,    1. ],       [   2. ,  -86. ,    6. ,    1.5],       [  28. ,  -77. ,    7. ,    1.2],       [  60. ,  -66. ,    4. ,    0.8],       [  75. ,  -28. ,   11. ,    1.1],       [  97. ,   -4. ,   10. ,    1.4],       [  83. ,   37. ,    8. ,    0.6],       [  69. ,   70. ,    3. ,    1.2],       [  39. ,   85. ,    9. ,    0.7],       [  -2. ,   97. ,    7. ,    1.2],       [ -37. ,   86. ,   12. ,    0.7],       [ -66. ,   68. ,   12. ,    1.4],       [ -82. ,   33. ,   11. ,    1. ],       [ -89. ,   -3. ,    4. ,    0.7],       [ -91. ,  -34. ,   12. ,    1.4],       [ -67. ,  -68. ,    7. ,    1.2],       [ -33. ,  -88. ,   11. ,    1.2],       [  -5. ,  -89. ,   10. ,    0.6],       [  32. ,  -88. ,    4. ,    1.2],       [  65. ,  -68. ,    5. ,    0.5],       [  89. ,  -33. ,    4. ,    0.7],       [  97. ,   -2. ,   10. ,    1.1],       [  87. ,   37. ,    9. ,    1.4],       [  73. ,   67. ,   12. ,    1.4],       [  39. ,   87. ,    7. ,    0.9],       [  -3. ,   99. ,   10. ,    1.3],       [ -40. ,   95. ,    4. ,    1. ],       [ -68. ,   69. ,    4. ,    0.8],       [ -88. ,   38. ,    5. ,    0.9],       [-102. ,   -2. ,   12. ,    1. ],       [ -96. ,  -37. ,    7. ,    1. ],       [ -73. ,  -72. ,   11. ,    1.3],       [ -40. ,  -92. ,    7. ,    1. ],       [  -2. , -103. ,    8. ,    1.5],       [  37. ,  -89. ,    8. ,    1.2],       [  73. ,  -68. ,   10. ,    0.7],       [  93. ,  -35. ,    3. ,    0.9],       [  46. ,    4. ,   12. ,    0.6],       [  42. ,   14. ,   17. ,    0.6],       [  31. ,   31. ,   15. ,    0.7],       [  15. ,   41. ,   18. ,    0.6],       [  -5. ,   51. ,   15. ,    1.3],       [ -21. ,   48. ,   18. ,    1.5],       [ -34. ,   40. ,   12. ,    0.7],       [ -44. ,   16. ,   16. ,    1.4],       [ -49. ,   -5. ,   15. ,    1.1],       [ -47. ,  -21. ,   14. ,    0.6],       [ -41. ,  -37. ,   12. ,    0.8],       [ -18. ,  -43. ,   15. ,    1.3],       [  -4. ,  -55. ,   15. ,    1.2],       [  20. ,  -45. ,   18. ,    0.6],       [  31. ,  -33. ,   15. ,    0.8],       [  45. ,  -18. ,   16. ,    1.2],       [  55. ,   -4. ,   15. ,    0.8],       [  54. ,   17. ,   13. ,    0.7],       [  36. ,   41. ,   11. ,    1.4],       [  24. ,   53. ,   14. ,    0.6],       [   3. ,   61. ,   20. ,    0.5],       [ -18. ,   50. ,   14. ,    1.2],       [ -42. ,   44. ,   18. ,    1.3],       [ -49. ,   17. ,   11. ,    1.1],       [ -53. ,    3. ,   13. ,    1. ],       [ -55. ,  -27. ,   18. ,    1.4],       [ -36. ,  -43. ,   16. ,    1. ],       [ -21. ,  -54. ,   12. ,    1.1],       [  -3. ,  -55. ,   12. ,    1. ],       [  26. ,  -54. ,   15. ,    1.1],       [  42. ,  -39. ,   14. ,    0.5],       [  49. ,  -25. ,   11. ,    1. ],       [  64. ,    4. ,   14. ,    0.8],       [  55. ,   23. ,   12. ,    1.2],       [  45. ,   41. ,   18. ,    1.2],       [  24. ,   59. ,   15. ,    0.7],       [  -3. ,   64. ,   18. ,    0.8],       [ -26. ,   62. ,   11. ,    0.8],       [ -47. ,   48. ,   12. ,    0.9],       [ -60. ,   20. ,   14. ,    0.6],       [ -68. ,   -4. ,   11. ,    1.4],       [ -65. ,  -29. ,   11. ,    1.1],       [ -49. ,  -49. ,   13. ,    0.9],       [ -23. ,  -55. ,   11. ,    1. ],       [  -3. ,  -66. ,   14. ,    1.5],       [  25. ,  -62. ,   15. ,    0.8],       [  47. ,  -49. ,   18. ,    0.7],       [  57. ,  -28. ,   14. ,    1. ],       [  70. ,    4. ,   12. ,    1.2],       [  62. ,   27. ,   11. ,    1.2],       [  51. ,   53. ,   18. ,    0.8],       [  23. ,   67. ,   19. ,    0.6],       [   4. ,   75. ,   13. ,    1.1],       [ -32. ,   61. ,   14. ,    0.8],       [ -48. ,   54. ,   17. ,    1.4],       [ -69. ,   27. ,   16. ,    1. ],       [ -75. ,   -1. ,   19. ,    1. ],       [ -62. ,  -29. ,   17. ,    0.7],       [ -55. ,  -50. ,   15. ,    1. ],       [ -26. ,  -63. ,   12. ,    0.9],       [   2. ,  -71. ,   13. ,    1. ],       [  22. ,  -67. ,   11. ,    1.1],       [  46. ,  -47. ,   12. ,    1.4],       [  66. ,  -31. ,   16. ,    1.5],       [  80. ,   -2. ,   12. ,    0.6],       [  70. ,   30. ,   12. ,    0.7],       [  57. ,   55. ,   16. ,    1.3],       [  30. ,   69. ,   16. ,    0.6],       [   1. ,   74. ,   16. ,    1.4],       [ -28. ,   72. ,   13. ,    0.5],       [ -52. ,   53. ,   11. ,    0.7],       [ -72. ,   32. ,   18. ,    0.6],       [ -79. ,   -1. ,   14. ,    0.9],       [ -76. ,  -26. ,   14. ,    1.1],       [ -54. ,  -51. ,   16. ,    0.9],       [ -31. ,  -68. ,   11. ,    0.6],       [  -5. ,  -76. ,   13. ,    1.4],       [  31. ,  -68. ,   18. ,    1.2],       [  54. ,  -54. ,   15. ,    1.3],       [  75. ,  -30. ,   12. ,    1.3],       [  83. ,    0. ,   11. ,    0.9],       [  78. ,   29. ,   19. ,    1.4],       [  60. ,   65. ,   11. ,    0.9],       [  32. ,   76. ,   13. ,    0.7],       [  -2. ,   83. ,   14. ,    0.9],       [ -35. ,   81. ,   17. ,    1.2],       [ -61. ,   58. ,   14. ,    0.6],       [ -78. ,   37. ,   18. ,    1.1],       [ -89. ,   -4. ,   11. ,    1.3],       [ -77. ,  -37. ,   18. ,    1.2],       [ -59. ,  -66. ,   20. ,    1. ],       [ -34. ,  -79. ,   15. ,    1. ],       [  -3. ,  -88. ,   20. ,    0.5],       [  37. ,  -80. ,   18. ,    1.3],       [  63. ,  -59. ,   13. ,    0.8],       [  79. ,  -33. ,   11. ,    0.6],       [  93. ,    0. ,   15. ,    1. ],       [  86. ,   36. ,   18. ,    0.7],       [  63. ,   68. ,   19. ,    1.3],       [  31. ,   83. ,   14. ,    1.2],       [  -5. ,   92. ,   11. ,    1.3],       [ -32. ,   84. ,   11. ,    1. ],       [ -70. ,   70. ,   15. ,    1.4],       [ -90. ,   32. ,   19. ,    1.5],       [ -90. ,    3. ,   15. ,    1.4],       [ -82. ,  -39. ,   20. ,    1.4],       [ -68. ,  -69. ,   20. ,    0.5],       [ -40. ,  -89. ,   18. ,    1.2],       [  -1. ,  -96. ,   11. ,    0.6],       [  35. ,  -88. ,   13. ,    1.4],       [  67. ,  -61. ,   17. ,    1. ],       [  82. ,  -32. ,   13. ,    0.6],       [ 100. ,   -3. ,   14. ,    0.6],       [  96. ,   39. ,   19. ,    1.1],       [  67. ,   71. ,   12. ,    1.1],       [  34. ,   95. ,   19. ,    1.3],       [   0. ,  103. ,   11. ,    1.3],       [ -34. ,   91. ,   18. ,    1.3],       [ -72. ,   72. ,   16. ,    1.4],       [ -88. ,   40. ,   12. ,    1.3],       [-103. ,   -2. ,   12. ,    0.7],       [ -91. ,  -43. ,   11. ,    1. ],       [ -73. ,  -70. ,   18. ,    1.4],       [ -34. ,  -96. ,   18. ,    1.3],       [  -3. , -104. ,   11. ,    0.9],       [  40. ,  -88. ,   19. ,    1.4],       [  69. ,  -74. ,   14. ,    1. ],       [  96. ,  -39. ,   14. ,    0.5],       [  53. ,    2. ,   26. ,    1.5],       [  43. ,   20. ,   22. ,    1.2],       [  36. ,   32. ,   20. ,    0.9],       [  21. ,   48. ,   25. ,    1.3],       [   0. ,   51. ,   22. ,    0.7],       [ -24. ,   47. ,   21. ,    0.9],       [ -32. ,   40. ,   25. ,    1. ],       [ -47. ,   16. ,   25. ,    0.6],       [ -52. ,    2. ,   26. ,    1.1],       [ -44. ,  -20. ,   27. ,    0.8],       [ -41. ,  -33. ,   23. ,    0.8],       [ -17. ,  -42. ,   21. ,    0.8],       [   3. ,  -51. ,   19. ,    0.6],       [  14. ,  -43. ,   19. ,    1.2],       [  38. ,  -31. ,   28. ,    0.9],       [  43. ,  -17. ,   28. ,    1.4],       [  59. ,   -5. ,   20. ,    1.4],       [  52. ,   18. ,   22. ,    0.5],       [  36. ,   35. ,   23. ,    1. ],       [  20. ,   56. ,   25. ,    0.6],       [   4. ,   55. ,   21. ,    1. ],       [ -22. ,   51. ,   20. ,    0.6],       [ -37. ,   44. ,   20. ,    1.1],       [ -53. ,   18. ,   20. ,    1.4],       [ -57. ,   -1. ,   19. ,    1.2],       [ -49. ,  -20. ,   22. ,    1. ],       [ -41. ,  -39. ,   26. ,    1. ],       [ -21. ,  -54. ,   26. ,    0.7],       [   3. ,  -60. ,   23. ,    0.6],       [  24. ,  -54. ,   28. ,    1.1],       [  40. ,  -45. ,   26. ,    1.1],       [  50. ,  -27. ,   25. ,    1.4],       [  61. ,    2. ,   28. ,    1.5],       [  62. ,   22. ,   22. ,    0.6],       [  47. ,   42. ,   28. ,    0.7],       [  28. ,   59. ,   20. ,    0.9],       [  -3. ,   68. ,   24. ,    0.9],       [ -24. ,   63. ,   19. ,    0.7],       [ -41. ,   43. ,   25. ,    0.6],       [ -59. ,   28. ,   20. ,    0.8],       [ -66. ,    0. ,   19. ,    0.9],       [ -60. ,  -29. ,   26. ,    1.1],       [ -51. ,  -43. ,   22. ,    1.2],       [ -24. ,  -56. ,   25. ,    0.8],       [  -3. ,  -69. ,   24. ,    1.4],       [  26. ,  -62. ,   23. ,    0.6],       [  50. ,  -50. ,   22. ,    1.2],       [  56. ,  -24. ,   23. ,    1.2],       [  71. ,    1. ,   27. ,    0.8],       [  68. ,   23. ,   22. ,    0.8],       [  46. ,   49. ,   22. ,    1.1],       [  29. ,   67. ,   24. ,    0.8],       [  -2. ,   72. ,   27. ,    0.6],       [ -28. ,   67. ,   22. ,    0.9],       [ -49. ,   45. ,   19. ,    1.3],       [ -64. ,   30. ,   22. ,    1.4],       [ -76. ,    4. ,   22. ,    0.6],       [ -70. ,  -29. ,   23. ,    1. ],       [ -52. ,  -49. ,   20. ,    0.6],       [ -24. ,  -63. ,   24. ,    0.9],       [  -2. ,  -76. ,   25. ,    1.2],       [  31. ,  -62. ,   20. ,    1.4],       [  49. ,  -49. ,   26. ,    1.2],       [  67. ,  -32. ,   20. ,    1. ],       [  78. ,    4. ,   24. ,    0.9],       [  71. ,   32. ,   23. ,    1.1],       [  52. ,   60. ,   22. ,    1.1],       [  30. ,   70. ,   23. ,    1.1],       [  -1. ,   81. ,   24. ,    1.5],       [ -35. ,   73. ,   20. ,    0.8],       [ -56. ,   51. ,   23. ,    1.1],       [ -77. ,   25. ,   23. ,    0.8],       [ -77. ,    3. ,   23. ,    0.9],       [ -74. ,  -35. ,   21. ,    0.8],       [ -54. ,  -55. ,   23. ,    1.1],       [ -31. ,  -70. ,   19. ,    0.6],       [   3. ,  -78. ,   21. ,    1.3],       [  28. ,  -71. ,   20. ,    1.1],       [  56. ,  -56. ,   25. ,    1.3],       [  69. ,  -31. ,   25. ,    0.6],       [  85. ,   -1. ,   23. ,    0.6],       [  83. ,   30. ,   26. ,    1. ],       [  62. ,   59. ,   20. ,    1.3],       [  36. ,   76. ,   28. ,    0.6],       [   2. ,   81. ,   22. ,    1.4],       [ -29. ,   83. ,   28. ,    1.3],       [ -57. ,   60. ,   26. ,    1.2],       [ -82. ,   30. ,   19. ,    1.4],       [ -87. ,    1. ,   28. ,    0.8],       [ -79. ,  -37. ,   25. ,    0.7],       [ -56. ,  -66. ,   23. ,    1.3],       [ -34. ,  -75. ,   22. ,    1.3],       [  -4. ,  -87. ,   19. ,    1.5],       [  36. ,  -84. ,   24. ,    1.4],       [  58. ,  -63. ,   21. ,    0.9],       [  79. ,  -36. ,   22. ,    1.1],       [  93. ,    4. ,   21. ,    1.1],       [  82. ,   38. ,   23. ,    0.9],       [  63. ,   63. ,   22. ,    1.4],       [  31. ,   89. ,   24. ,    1. ],       [   4. ,   94. ,   19. ,    1.4],       [ -40. ,   87. ,   19. ,    1.2],       [ -65. ,   62. ,   23. ,    1.3],       [ -84. ,   32. ,   28. ,    0.9],       [ -97. ,    2. ,   19. ,    0.7],       [ -84. ,  -36. ,   23. ,    0.7],       [ -66. ,  -68. ,   25. ,    0.9],       [ -35. ,  -82. ,   28. ,    1. ],       [  -2. ,  -89. ,   24. ,    1.4],       [  37. ,  -82. ,   20. ,    0.5],       [  60. ,  -62. ,   22. ,    0.9],       [  81. ,  -39. ,   23. ,    1.4],       [ 104. ,   -3. ,   19. ,    1.5],       [  88. ,   38. ,   28. ,    0.9],       [  70. ,   67. ,   19. ,    1.4],       [  41. ,   96. ,   28. ,    1.3],       [   1. ,   99. ,   20. ,    0.5],       [ -35. ,   88. ,   25. ,    0.9],       [ -71. ,   74. ,   19. ,    1.2],       [ -92. ,   34. ,   26. ,    1.2],       [-104. ,   -3. ,   23. ,    1.3],       [ -95. ,  -42. ,   22. ,    0.6],       [ -73. ,  -67. ,   26. ,    0.7],       [ -35. ,  -89. ,   24. ,    1.5],       [  -3. ,  -96. ,   23. ,    1.1],       [  35. ,  -95. ,   25. ,    0.7],       [  71. ,  -66. ,   26. ,    1. ],       [  87. ,  -36. ,   22. ,    0.7],       [  53. ,    1. ,   35. ,    0.7],       [  41. ,   21. ,   34. ,    0.8],       [  34. ,   39. ,   35. ,    0.5],       [  20. ,   42. ,   32. ,    1.1],       [   1. ,   48. ,   36. ,    0.8],       [ -21. ,   44. ,   32. ,    0.7],       [ -39. ,   35. ,   34. ,    0.6],       [ -45. ,   15. ,   31. ,    0.8],       [ -48. ,   -5. ,   27. ,    0.8],       [ -43. ,  -19. ,   33. ,    1. ],       [ -36. ,  -34. ,   33. ,    0.8],       [ -18. ,  -47. ,   30. ,    0.7],       [   1. ,  -49. ,   30. ,    0.9],       [  17. ,  -47. ,   32. ,    1.2],       [  34. ,  -37. ,   27. ,    0.6],       [  50. ,  -22. ,   35. ,    0.9],       [  52. ,   -2. ,   33. ,    1.3],       [  47. ,   23. ,   28. ,    1.2],       [  39. ,   43. ,   32. ,    0.9],       [  18. ,   49. ,   35. ,    0.7],       [   3. ,   56. ,   35. ,    0.8],       [ -23. ,   55. ,   31. ,    0.8],       [ -39. ,   41. ,   31. ,    1.1],       [ -51. ,   22. ,   32. ,    1.4],       [ -57. ,    4. ,   27. ,    1.5],       [ -51. ,  -23. ,   30. ,    0.9],       [ -40. ,  -41. ,   29. ,    1. ],       [ -23. ,  -57. ,   32. ,    1.1],       [  -1. ,  -55. ,   31. ,    1.2],       [  23. ,  -49. ,   34. ,    0.9],       [  35. ,  -38. ,   27. ,    0.6],       [  54. ,  -26. ,   34. ,    1.4],       [  68. ,   -3. ,   33. ,    1.1],       [  63. ,   26. ,   28. ,    0.5],       [  41. ,   48. ,   34. ,    0.9],       [  21. ,   61. ,   29. ,    0.6],       [  -4. ,   62. ,   32. ,    1.1],       [ -23. ,   55. ,   30. ,    0.8],       [ -44. ,   42. ,   36. ,    0.6],       [ -63. ,   20. ,   32. ,    0.7],       [ -64. ,   -5. ,   27. ,    0.9],       [ -64. ,  -22. ,   31. ,    1.1],       [ -44. ,  -45. ,   27. ,    1.3],       [ -24. ,  -63. ,   32. ,    0.7],       [  -2. ,  -61. ,   28. ,    1.3],       [  26. ,  -59. ,   33. ,    0.5],       [  43. ,  -44. ,   30. ,    1. ],       [  56. ,  -22. ,   36. ,    0.7],       [  70. ,    2. ,   27. ,    0.9],       [  69. ,   28. ,   33. ,    0.5],       [  53. ,   50. ,   30. ,    0.9],       [  26. ,   63. ,   34. ,    0.7],       [  -4. ,   74. ,   32. ,    1.4],       [ -25. ,   70. ,   28. ,    1.5],       [ -47. ,   51. ,   35. ,    0.7],       [ -66. ,   30. ,   31. ,    0.6],       [ -73. ,    1. ,   35. ,    1. ],       [ -70. ,  -23. ,   35. ,    1.2],       [ -52. ,  -51. ,   34. ,    0.6],       [ -32. ,  -71. ,   30. ,    1.2],       [  -4. ,  -75. ,   31. ,    1.2],       [  27. ,  -65. ,   32. ,    1.2],       [  53. ,  -51. ,   29. ,    1.1],       [  64. ,  -26. ,   28. ,    0.8],       [  76. ,    3. ,   27. ,    0.8],       [  68. ,   33. ,   29. ,    0.8],       [  52. ,   59. ,   33. ,    1.1],       [  34. ,   67. ,   30. ,    1.5],       [  -2. ,   79. ,   34. ,    0.8],       [ -27. ,   69. ,   34. ,    0.9],       [ -55. ,   52. ,   34. ,    1.3],       [ -70. ,   30. ,   29. ,    0.9],       [ -76. ,   -3. ,   29. ,    1.2],       [ -74. ,  -28. ,   32. ,    1.3],       [ -58. ,  -53. ,   31. ,    1.1],       [ -35. ,  -75. ,   30. ,    0.5],       [   0. ,  -76. ,   36. ,    1.1],       [  28. ,  -69. ,   33. ,    0.8],       [  50. ,  -51. ,   31. ,    0.9],       [  73. ,  -26. ,   36. ,    1.4],       [  80. ,   -2. ,   32. ,    0.9],       [  78. ,   29. ,   32. ,    1.4],       [  63. ,   58. ,   30. ,    0.8],       [  35. ,   83. ,   31. ,    0.8],       [  -3. ,   84. ,   35. ,    0.6],       [ -33. ,   82. ,   32. ,    1. ],       [ -61. ,   61. ,   28. ,    0.9],       [ -80. ,   28. ,   34. ,    1.3],       [ -89. ,    3. ,   34. ,    0.9],       [ -83. ,  -35. ,   29. ,    0.6],       [ -65. ,  -61. ,   34. ,    0.9],       [ -31. ,  -84. ,   33. ,    1.1],       [   0. ,  -89. ,   30. ,    0.9],       [  32. ,  -76. ,   35. ,    1.4],       [  55. ,  -61. ,   29. ,    0.7],       [  83. ,  -37. ,   27. ,    1.3],       [  96. ,    3. ,   30. ,    0.5],       [  83. ,   33. ,   30. ,    1.1],       [  66. ,   68. ,   31. ,    1.2],       [  35. ,   86. ,   30. ,    0.9],       [  -1. ,   97. ,   35. ,    0.5],       [ -31. ,   90. ,   30. ,    1.5],       [ -62. ,   66. ,   34. ,    0.6],       [ -85. ,   30. ,   32. ,    1.2],       [ -90. ,    3. ,   32. ,    0.6],       [ -88. ,  -41. ,   34. ,    0.7],       [ -68. ,  -66. ,   29. ,    1. ],       [ -38. ,  -86. ,   29. ,    0.8],       [   1. ,  -95. ,   28. ,    0.7],       [  38. ,  -82. ,   32. ,    0.8],       [  63. ,  -65. ,   29. ,    0.6],       [  83. ,  -33. ,   28. ,    1.3],       [  98. ,   -1. ,   35. ,    0.5],       [  91. ,   39. ,   32. ,    0.5],       [  74. ,   72. ,   33. ,    0.7],       [  39. ,   94. ,   32. ,    0.8],       [   2. ,  100. ,   28. ,    1.4],       [ -39. ,   89. ,   34. ,    1.3],       [ -72. ,   75. ,   33. ,    1. ],       [ -91. ,   38. ,   34. ,    0.8],       [-105. ,    1. ,   34. ,    0.9],       [ -98. ,  -37. ,   32. ,    0.9],       [ -70. ,  -71. ,   35. ,    1.4],       [ -40. ,  -92. ,   29. ,    0.6],       [   4. , -102. ,   31. ,    0.8],       [  34. ,  -97. ,   36. ,    0.8],       [  66. ,  -70. ,   31. ,    1.4],       [  92. ,  -35. ,   27. ,    1.3],       [  48. ,   -2. ,   42. ,    1.3],       [  45. ,   21. ,   42. ,    0.7],       [  34. ,   34. ,   37. ,    0.7],       [  22. ,   41. ,   39. ,    0.6],       [   3. ,   53. ,   35. ,    1.1],       [ -16. ,   46. ,   36. ,    0.7],       [ -40. ,   36. ,   35. ,    0.7],       [ -51. ,   15. ,   41. ,    1.1],       [ -54. ,   -1. ,   41. ,    1.4],       [ -51. ,  -18. ,   37. ,    0.8],       [ -36. ,  -39. ,   43. ,    1. ],       [ -15. ,  -44. ,   35. ,    0.6],       [   0. ,  -48. ,   44. ,    1.3],       [  17. ,  -48. ,   36. ,    1.1],       [  39. ,  -32. ,   38. ,    0.9],       [  49. ,  -17. ,   40. ,    0.6],       [  55. ,   -4. ,   44. ,    0.9],       [  53. ,   18. ,   39. ,    0.7],       [  38. ,   38. ,   39. ,    1. ],       [  23. ,   55. ,   41. ,    0.9],       [   0. ,   58. ,   39. ,    0.6],       [ -27. ,   55. ,   38. ,    1.2],       [ -42. ,   38. ,   43. ,    1.1],       [ -55. ,   21. ,   35. ,    1. ],       [ -54. ,    1. ,   35. ,    1.3],       [ -55. ,  -20. ,   38. ,    1.1],       [ -44. ,  -37. ,   43. ,    1. ],       [ -21. ,  -54. ,   42. ,    1.1],       [   0. ,  -53. ,   43. ,    1.1],       [  17. ,  -54. ,   39. ,    0.7],       [  44. ,  -40. ,   43. ,    0.7],       [  54. ,  -22. ,   41. ,    1. ],       [  67. ,   -3. ,   42. ,    1. ],       [  64. ,   26. ,   36. ,    1.5],       [  43. ,   41. ,   40. ,    1. ],       [  26. ,   59. ,   35. ,    0.6],       [  -3. ,   68. ,   35. ,    0.9],       [ -22. ,   60. ,   44. ,    0.6],       [ -47. ,   45. ,   39. ,    1.3],       [ -61. ,   22. ,   43. ,    1.4],       [ -66. ,   -5. ,   43. ,    0.7],       [ -61. ,  -28. ,   43. ,    1.3],       [ -44. ,  -47. ,   41. ,    1.1],       [ -24. ,  -55. ,   41. ,    0.7],       [   1. ,  -66. ,   36. ,    1.2],       [  25. ,  -64. ,   35. ,    0.5],       [  46. ,  -42. ,   42. ,    1.3],       [  55. ,  -22. ,   43. ,    0.7],       [  72. ,    1. ,   37. ,    1. ],       [  69. ,   30. ,   37. ,    1.2],       [  52. ,   49. ,   35. ,    1.3],       [  26. ,   67. ,   42. ,    1. ],       [   0. ,   71. ,   43. ,    1.3],       [ -24. ,   62. ,   41. ,    1. ],       [ -52. ,   48. ,   42. ,    0.8],       [ -70. ,   29. ,   39. ,    1.3],       [ -76. ,    1. ,   43. ,    1. ],       [ -69. ,  -24. ,   37. ,    1.2],       [ -52. ,  -51. ,   38. ,    1.4],       [ -29. ,  -66. ,   36. ,    0.7],       [  -2. ,  -73. ,   35. ,    0.8],       [  28. ,  -69. ,   39. ,    1. ],       [  47. ,  -50. ,   36. ,    1. ],       [  70. ,  -23. ,   42. ,    1.2],       [  81. ,   -2. ,   44. ,    0.9],       [  68. ,   35. ,   41. ,    1.4],       [  51. ,   57. ,   44. ,    1.3],       [  28. ,   69. ,   42. ,    0.9],       [  -1. ,   76. ,   40. ,    1.1],       [ -32. ,   69. ,   39. ,    0.7],       [ -59. ,   52. ,   40. ,    1. ],       [ -73. ,   31. ,   35. ,    0.9],       [ -81. ,    1. ,   42. ,    1.1],       [ -69. ,  -26. ,   38. ,    0.9],       [ -59. ,  -59. ,   37. ,    1.2],       [ -26. ,  -71. ,   37. ,    0.9],       [   1. ,  -83. ,   41. ,    0.7],       [  28. ,  -69. ,   39. ,    1.3],       [  57. ,  -61. ,   35. ,    1.1],       [  70. ,  -30. ,   39. ,    1. ],       [  86. ,    1. ,   38. ,    1.1],       [  74. ,   33. ,   36. ,    1.2],       [  58. ,   60. ,   43. ,    1.4],       [  37. ,   83. ,   44. ,    1. ],       [   3. ,   88. ,   40. ,    1. ],       [ -35. ,   75. ,   39. ,    0.7],       [ -61. ,   59. ,   35. ,    1.5],       [ -83. ,   32. ,   42. ,    1.4],       [ -84. ,   -3. ,   43. ,    0.7],       [ -81. ,  -32. ,   36. ,    1. ],       [ -63. ,  -60. ,   38. ,    0.7],       [ -36. ,  -84. ,   39. ,    0.5],       [   2. ,  -90. ,   42. ,    0.8],       [  36. ,  -81. ,   40. ,    0.7],       [  62. ,  -60. ,   42. ,    1.3],       [  82. ,  -32. ,   41. ,    0.8],       [  92. ,    1. ,   38. ,    0.7],       [  88. ,   35. ,   40. ,    0.7],       [  69. ,   67. ,   41. ,    1.3],       [  31. ,   84. ,   36. ,    1.3],       [   3. ,   96. ,   38. ,    1.3],       [ -33. ,   90. ,   39. ,    1.4],       [ -68. ,   67. ,   44. ,    1.4],       [ -89. ,   36. ,   35. ,    1.4],       [ -96. ,   -3. ,   35. ,    0.6],       [ -82. ,  -40. ,   36. ,    0.9],       [ -65. ,  -66. ,   42. ,    1.4],       [ -40. ,  -85. ,   36. ,    1.3],       [  -4. ,  -94. ,   44. ,    0.6],       [  38. ,  -82. ,   41. ,    1.1],       [  64. ,  -64. ,   42. ,    1.2],       [  88. ,  -36. ,   39. ,    1. ],       [  95. ,   -5. ,   37. ,    0.6],       [  90. ,   39. ,   37. ,    1.3],       [  69. ,   73. ,   44. ,    0.8],       [  41. ,   88. ,   39. ,    0.9],       [  -1. ,  100. ,   39. ,    0.8],       [ -38. ,   92. ,   44. ,    1.3],       [ -66. ,   72. ,   40. ,    0.9],       [ -95. ,   41. ,   35. ,    0.5],       [-103. ,    3. ,   35. ,    1.2],       [ -89. ,  -42. ,   39. ,    1.1],       [ -70. ,  -74. ,   38. ,    0.7],       [ -43. ,  -97. ,   37. ,    0.5],       [  -4. , -100. ,   44. ,    0.6],       [  33. ,  -97. ,   38. ,    1.4],       [  67. ,  -75. ,   40. ,    0.9],       [  94. ,  -38. ,   42. ,    1.1]])
        basis0 = BSplineBasis(7, np.array([ -1.8,  -1.2,   0. ,   0. ,   0. ,   0. ,   0. ,   1.2,   2.2,   3.4,   3.9,   5.3,   6.1,
         6.7,   7.9,   8.9,  10.2,  10.8,  12. ,  12. ,  12. ,  12. ,  12. ,  13.2,  14.2]),1)
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  2.1,  3. ,  3. ,  3. ,  3. ,  3. ,  3. ]))
        basis2 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  2. ,  2. ,  2. ,  2. ,  2. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(3, 3, 3)
        self.assertEqual(vol2.order(direction=0), 10)
        self.assertEqual(vol2.order(direction=1), 9)
        self.assertEqual(vol2.order(direction=2), 8)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p7_rational_C2_periodic(self):
        controlpoints = np.array([[  96. ,    4. ,    4. ,    0.6],
       [  98. ,   35. ,   -4. ,    0.5],
       [  81. ,   68. ,   -4. ,    1.1],
       [  47. ,   88. ,   -2. ,    1.2],
       [  17. ,   99. ,    4. ,    1.2],
       [ -22. ,   95. ,   -3. ,    0.7],
       [ -53. ,   84. ,    0. ,    1.4],
       [ -77. ,   64. ,    2. ,    1.2],
       [ -94. ,   34. ,    1. ,    0.8],
       [-100. ,    1. ,   -3. ,    0.7],
       [ -90. ,  -30. ,    2. ,    1.2],
       [ -76. ,  -64. ,   -1. ,    0.8],
       [ -46. ,  -88. ,    3. ,    1. ],
       [ -17. ,  -95. ,   -3. ,    1.1],
       [  15. ,  -99. ,   -5. ,    1.4],
       [  51. ,  -84. ,   -5. ,    1.2],
       [  76. ,  -63. ,   -4. ,    1.3],
       [  92. ,  -31. ,   -1. ,    1.1]])
        basis0 = BSplineBasis(7, np.array([ -3. ,  -1.7,  -1. ,   0. ,   0. ,   0. ,   0. ,   0.9,   2.3,   2.9,   4.3,   4.6,   5.9,
         7.1,   7.8,   9.3,  10. ,  11.3,  12. ,  13.3,  14. ,  15. ,  15. ,  15. ,  15. ,  15.9,
        17.3,  17.9]),2)
        crv  = Curve(basis0, controlpoints,True)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 10)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p56_rational_C2_periodic(self):
        controlpoints = np.array([[  60. ,    0. ,    4. ,    1.2],       [  53. ,   29. ,    2. ,    1.4],       [  40. ,   44. ,   -3. ,    0.8],       [  16. ,   58. ,    1. ,    0.7],       [ -11. ,   60. ,    1. ,    1.2],       [ -38. ,   42. ,   -5. ,    1.4],       [ -53. ,   29. ,    2. ,    0.6],       [ -58. ,   -2. ,    1. ,    1. ],       [ -51. ,  -24. ,   -4. ,    0.6],       [ -38. ,  -44. ,   -5. ,    0.8],       [ -16. ,  -59. ,   -4. ,    1.1],       [   8. ,  -59. ,    1. ,    0.7],       [  36. ,  -44. ,    4. ,    1. ],       [  57. ,  -27. ,   -2. ,    1.4],       [  65. ,   -3. ,   -3. ,    1.1],       [  59. ,   31. ,    0. ,    0.8],       [  36. ,   48. ,    4. ,    1.2],       [  15. ,   65. ,   -3. ,    1. ],       [ -10. ,   64. ,   -3. ,    0.8],       [ -38. ,   48. ,    2. ,    1.2],       [ -54. ,   29. ,    3. ,    0.9],       [ -64. ,    2. ,   -2. ,    0.6],       [ -59. ,  -26. ,   -1. ,    1. ],       [ -40. ,  -52. ,   -4. ,    0.8],       [ -15. ,  -63. ,   -5. ,    0.7],       [  12. ,  -67. ,    0. ,    1. ],       [  43. ,  -47. ,    3. ,    0.9],       [  58. ,  -29. ,   -2. ,    0.6],       [  67. ,   -2. ,    3. ,    1.1],       [  60. ,   32. ,   -2. ,    1.4],       [  40. ,   54. ,    4. ,    0.6],       [  18. ,   66. ,    3. ,    1.4],       [ -17. ,   67. ,   -1. ,    0.8],       [ -42. ,   55. ,   -1. ,    0.5],       [ -65. ,   34. ,   -4. ,    0.6],       [ -74. ,    3. ,    2. ,    0.7],       [ -60. ,  -30. ,   -1. ,    0.6],       [ -47. ,  -55. ,    3. ,    1. ],       [ -14. ,  -67. ,    2. ,    1.1],       [  12. ,  -66. ,   -2. ,    0.6],       [  41. ,  -54. ,    4. ,    1.2],       [  67. ,  -30. ,   -5. ,    0.6],       [  70. ,    0. ,   -1. ,    0.7],       [  68. ,   35. ,   -4. ,    0.7],       [  45. ,   56. ,   -4. ,    1. ],       [  19. ,   72. ,   -2. ,    0.6],       [ -17. ,   70. ,   -3. ,    0.9],       [ -44. ,   60. ,    0. ,    1.4],       [ -68. ,   31. ,   -3. ,    1.3],       [ -77. ,   -1. ,   -5. ,    1.4],       [ -63. ,  -29. ,   -3. ,    1.1],       [ -42. ,  -56. ,    0. ,    0.9],       [ -18. ,  -73. ,    3. ,    1.1],       [  11. ,  -78. ,    0. ,    1.2],       [  50. ,  -62. ,   -2. ,    1.1],       [  63. ,  -38. ,   -5. ,    1.2],       [  83. ,    2. ,    0. ,    1.4],       [  71. ,   30. ,    0. ,    1.4],       [  53. ,   58. ,    1. ,    1.1],       [  19. ,   82. ,   -5. ,    0.6],       [ -19. ,   79. ,    3. ,    1.2],       [ -50. ,   65. ,    1. ,    0.9],       [ -71. ,   33. ,   -1. ,    1.5],       [ -76. ,    4. ,    4. ,    1. ],       [ -71. ,  -38. ,    2. ,    1. ],       [ -49. ,  -64. ,    1. ,    0.7],       [ -22. ,  -76. ,   -4. ,    0.7],       [  21. ,  -80. ,   -4. ,    0.7],       [  45. ,  -66. ,    0. ,    1.5],       [  72. ,  -36. ,    0. ,    0.9],       [  87. ,    1. ,   -4. ,    0.8],       [  76. ,   34. ,    2. ,    0.7],       [  53. ,   68. ,    2. ,    0.7],       [  17. ,   81. ,   -1. ,    0.7],       [ -23. ,   78. ,    3. ,    0.9],       [ -52. ,   68. ,    1. ,    0.9],       [ -82. ,   36. ,   -4. ,    1.2],       [ -81. ,    2. ,   -5. ,    0.7],       [ -77. ,  -41. ,   -5. ,    1.3],       [ -56. ,  -68. ,    3. ,    1.2],       [ -16. ,  -81. ,    4. ,    0.6],       [  20. ,  -79. ,   -1. ,    1.3],       [  53. ,  -69. ,    1. ,    1.1],       [  71. ,  -38. ,   -5. ,    1.4],       [  91. ,   -5. ,   -4. ,    1.3],       [  77. ,   40. ,    2. ,    0.9],       [  53. ,   74. ,   -2. ,    0.9],       [  21. ,   85. ,   -2. ,    1.3],       [ -16. ,   85. ,   -1. ,    0.9],       [ -59. ,   70. ,   -4. ,    0.7],       [ -79. ,   39. ,   -4. ,    0.8],       [ -90. ,   -3. ,   -2. ,    1. ],       [ -78. ,  -39. ,    2. ,    0.8],       [ -60. ,  -73. ,   -4. ,    0.7],       [ -20. ,  -91. ,    3. ,    0.8],       [  23. ,  -92. ,    0. ,    0.9],       [  55. ,  -74. ,   -4. ,    1.5],       [  83. ,  -44. ,   -2. ,    0.9],       [  99. ,   -4. ,    3. ,    1.4],       [  87. ,   40. ,   -2. ,    1.4],       [  61. ,   73. ,    1. ,    1.1],       [  21. ,   96. ,   -1. ,    1.1],       [ -17. ,   91. ,   -1. ,    1.3],       [ -58. ,   76. ,   -3. ,    1.4],       [ -87. ,   44. ,    0. ,    1.1],       [ -98. ,    3. ,    1. ,    0.6],       [ -84. ,  -41. ,   -3. ,    1.2],       [ -64. ,  -76. ,   -1. ,    0.6],       [ -23. ,  -95. ,    2. ,    0.6],       [  16. ,  -95. ,    4. ,    1.3],       [  58. ,  -74. ,    3. ,    0.7],       [  83. ,  -44. ,   -4. ,    1.4],       [ 103. ,    0. ,    3. ,    0.8],       [  93. ,   47. ,    1. ,    1.2],       [  65. ,   81. ,   -5. ,    1.2],       [  21. ,   99. ,    1. ,    0.9],       [ -21. ,   99. ,   -3. ,    0.5],       [ -64. ,   77. ,    2. ,    0.7],       [ -86. ,   44. ,    0. ,    1.4],       [-100. ,   -5. ,   -1. ,    0.6],       [ -90. ,  -47. ,   -4. ,    0.8],       [ -65. ,  -75. ,    1. ,    1.4],       [ -20. , -102. ,   -3. ,    0.8],       [  26. ,  -98. ,    3. ,    0.5],       [  64. ,  -84. ,    2. ,    0.8],       [  93. ,  -46. ,    2. ,    1.3]])
        basis0 = BSplineBasis(5, np.array([ -2.9,  -2.1,  -0.7,   0. ,   0. ,   1.2,   2.1,   2.8,   4.2,   5. ,   6.1,   7. ,   7.9,
         8.6,  10.1,  10.9,  12.3,  13. ,  13. ,  14.2,  15.1,  15.8]),2)
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  1.8,  2.6,  4. ,  4. ,  4. ,  4. ,  4. ,  4. ]))
        surf  = Surface(basis0, basis1, controlpoints,True)
        surf2 = surf.clone()
        surf2.raise_order(1, 2)
        self.assertEqual(surf2.order(direction=0), 6)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p555_rational_C2_periodic(self):
        controlpoints = np.array([[  46. ,   -5. ,    0. ,    1.1],       [  48. ,   18. ,   -5. ,    0.8],       [  27. ,   34. ,   -5. ,    0.6],       [  14. ,   49. ,   -3. ,    0.9],       [ -13. ,   44. ,   -5. ,    1.2],       [ -35. ,   40. ,   -2. ,    1. ],       [ -49. ,   19. ,   -3. ,    1.2],       [ -49. ,   -1. ,    0. ,    0.8],       [ -44. ,  -20. ,   -2. ,    1.5],       [ -33. ,  -35. ,    2. ,    0.8],       [  -7. ,  -49. ,    4. ,    1.3],       [  13. ,  -54. ,   -2. ,    0.7],       [  35. ,  -40. ,   -3. ,    1.3],       [  40. ,  -27. ,   -3. ,    1. ],       [  52. ,    3. ,   -3. ,    1.4],       [  47. ,   22. ,    4. ,    1. ],       [  33. ,   40. ,   -2. ,    1.4],       [   9. ,   53. ,    3. ,    0.9],       [ -14. ,   50. ,    1. ,    0.6],       [ -33. ,   48. ,    0. ,    0.7],       [ -55. ,   28. ,    3. ,    0.9],       [ -56. ,   -2. ,    4. ,    1.1],       [ -50. ,  -29. ,   -2. ,    1.3],       [ -33. ,  -46. ,   -1. ,    1. ],       [ -11. ,  -56. ,    4. ,    1. ],       [  16. ,  -54. ,    4. ,    1.2],       [  32. ,  -45. ,   -4. ,    1.2],       [  53. ,  -29. ,    4. ,    1.1],       [  60. ,   -5. ,    1. ,    1.3],       [  59. ,   31. ,    2. ,    1.3],       [  42. ,   46. ,   -4. ,    1.5],       [  12. ,   60. ,   -1. ,    1.4],       [ -10. ,   62. ,    0. ,    0.6],       [ -43. ,   49. ,    2. ,    1.4],       [ -60. ,   31. ,    3. ,    1.1],       [ -59. ,   -1. ,   -5. ,    0.5],       [ -53. ,  -29. ,   -1. ,    1.3],       [ -43. ,  -54. ,    2. ,    1.3],       [ -19. ,  -60. ,   -3. ,    0.5],       [  14. ,  -65. ,   -4. ,    0.7],       [  40. ,  -51. ,   -5. ,    1.3],       [  51. ,  -25. ,   -3. ,    0.7],       [  66. ,   -1. ,   -2. ,    0.5],       [  58. ,   31. ,   -3. ,    0.8],       [  38. ,   57. ,    1. ,    1. ],       [  19. ,   69. ,   -2. ,    0.6],       [ -12. ,   62. ,   -5. ,    0.6],       [ -40. ,   57. ,    1. ,    0.7],       [ -60. ,   28. ,    1. ,    0.9],       [ -69. ,    1. ,   -4. ,    1.3],       [ -58. ,  -26. ,   -3. ,    0.7],       [ -47. ,  -51. ,    3. ,    1. ],       [ -17. ,  -68. ,   -4. ,    1.2],       [  17. ,  -68. ,    4. ,    1.3],       [  45. ,  -51. ,   -5. ,    0.8],       [  65. ,  -32. ,    4. ,    1.2],       [  78. ,    0. ,    1. ,    1.3],       [  64. ,   37. ,   -3. ,    1. ],       [  47. ,   57. ,    1. ,    1.2],       [  18. ,   71. ,   -4. ,    1.1],       [ -20. ,   70. ,    3. ,    0.6],       [ -47. ,   56. ,    4. ,    1.5],       [ -71. ,   33. ,    4. ,    0.7],       [ -75. ,   -1. ,   -3. ,    0.9],       [ -68. ,  -32. ,    3. ,    1. ],       [ -51. ,  -56. ,   -4. ,    1.1],       [ -16. ,  -74. ,    0. ,    0.5],       [  12. ,  -72. ,   -4. ,    0.8],       [  46. ,  -59. ,   -3. ,    0.7],       [  63. ,  -29. ,   -5. ,    1.4],       [  76. ,   -3. ,    1. ,    0.6],       [  70. ,   36. ,    2. ,    1.3],       [  48. ,   66. ,    4. ,    1. ],       [  16. ,   79. ,   -5. ,    0.9],       [ -14. ,   78. ,    0. ,    1.5],       [ -49. ,   61. ,   -2. ,    1.4],       [ -73. ,   33. ,    2. ,    1. ],       [ -86. ,    4. ,   -3. ,    1.4],       [ -74. ,  -34. ,    2. ,    1.5],       [ -51. ,  -60. ,   -5. ,    0.6],       [ -21. ,  -83. ,   -4. ,    1.4],       [  19. ,  -82. ,    0. ,    1.1],       [  49. ,  -61. ,    1. ,    0.8],       [  71. ,  -32. ,    0. ,    1.3],       [  91. ,    3. ,    4. ,    0.9],       [  78. ,   41. ,    0. ,    0.9],       [  50. ,   66. ,    0. ,    0.7],       [  17. ,   85. ,    1. ,    1.2],       [ -23. ,   84. ,   -2. ,    0.6],       [ -50. ,   71. ,    2. ,    1.3],       [ -76. ,   39. ,    3. ,    0.8],       [ -85. ,    3. ,   -3. ,    1.1],       [ -83. ,  -36. ,    0. ,    0.7],       [ -59. ,  -68. ,   -2. ,    1.5],       [ -23. ,  -91. ,    1. ,    1.1],       [  21. ,  -86. ,    0. ,    0.7],       [  56. ,  -65. ,   -1. ,    1.5],       [  75. ,  -41. ,    2. ,    1.1],       [  90. ,   -3. ,   -3. ,    1.2],       [  88. ,   35. ,   -5. ,    1.5],       [  56. ,   75. ,   -5. ,    0.6],       [  20. ,   93. ,   -5. ,    1.1],       [ -24. ,   91. ,   -1. ,    1. ],       [ -61. ,   68. ,   -4. ,    0.8],       [ -86. ,   43. ,   -5. ,    0.9],       [ -91. ,   -1. ,    2. ,    1.1],       [ -88. ,  -43. ,    1. ,    0.7],       [ -58. ,  -72. ,    4. ,    1.4],       [ -18. ,  -89. ,   -1. ,    0.8],       [  25. ,  -89. ,    2. ,    1.3],       [  62. ,  -69. ,   -1. ,    0.7],       [  87. ,  -38. ,   -4. ,    0.6],       [ 103. ,   -1. ,   -3. ,    1.5],       [  93. ,   45. ,    1. ,    0.9],       [  63. ,   74. ,    3. ,    0.9],       [  26. ,   94. ,    2. ,    1.4],       [ -24. ,   99. ,    1. ,    1.3],       [ -66. ,   77. ,   -5. ,    0.6],       [ -88. ,   43. ,   -4. ,    1.2],       [ -97. ,   -5. ,   -2. ,    0.6],       [ -92. ,  -41. ,    0. ,    1.2],       [ -62. ,  -82. ,    1. ,    0.9],       [ -19. , -102. ,   -4. ,    0.6],       [  26. ,  -95. ,    1. ,    1.1],       [  59. ,  -81. ,    2. ,    1.1],       [  87. ,  -43. ,    0. ,    0.5],       [  49. ,   -1. ,    8. ,    1.2],       [  41. ,   17. ,    3. ,    1.4],       [  33. ,   40. ,    5. ,    0.9],       [   9. ,   48. ,    6. ,    0.6],       [ -10. ,   53. ,    7. ,    1.4],       [ -35. ,   39. ,   11. ,    0.9],       [ -49. ,   26. ,    5. ,    0.7],       [ -47. ,    1. ,    7. ,    1.4],       [ -43. ,  -25. ,    8. ,    0.8],       [ -31. ,  -41. ,    5. ,    1. ],       [ -14. ,  -47. ,    8. ,    1.3],       [  11. ,  -50. ,    9. ,    0.7],       [  29. ,  -37. ,   12. ,    0.9],       [  49. ,  -21. ,    5. ,    1. ],       [  58. ,   -5. ,    7. ,    0.5],       [  49. ,   23. ,    5. ,    1.3],       [  38. ,   47. ,    5. ,    0.9],       [  10. ,   57. ,   12. ,    0.5],       [ -16. ,   52. ,    7. ,    1. ],       [ -38. ,   48. ,    7. ,    0.8],       [ -54. ,   22. ,    5. ,    1.1],       [ -55. ,    0. ,    8. ,    1.1],       [ -50. ,  -30. ,   11. ,    1. ],       [ -33. ,  -46. ,   12. ,    1.5],       [ -10. ,  -53. ,    4. ,    0.6],       [  13. ,  -54. ,    7. ,    1.2],       [  37. ,  -48. ,   10. ,    0.7],       [  54. ,  -26. ,    9. ,    1.3],       [  59. ,    4. ,    3. ,    1.2],       [  52. ,   31. ,    5. ,    1. ],       [  43. ,   46. ,    5. ,    1.2],       [  13. ,   56. ,    7. ,    0.9],       [ -15. ,   62. ,    6. ,    0.9],       [ -39. ,   52. ,   10. ,    1. ],       [ -52. ,   29. ,    6. ,    0.8],       [ -66. ,   -4. ,    4. ,    1.3],       [ -58. ,  -32. ,   12. ,    0.9],       [ -43. ,  -46. ,    3. ,    0.7],       [ -13. ,  -58. ,    6. ,    1.4],       [  14. ,  -66. ,    9. ,    0.6],       [  37. ,  -49. ,    9. ,    0.9],       [  54. ,  -26. ,    9. ,    0.8],       [  71. ,    3. ,    4. ,    1.2],       [  59. ,   29. ,   11. ,    0.6],       [  45. ,   58. ,   10. ,    0.7],       [  15. ,   62. ,    4. ,    1.1],       [ -17. ,   69. ,    5. ,    0.7],       [ -39. ,   49. ,    8. ,    0.5],       [ -66. ,   27. ,   12. ,    0.6],       [ -68. ,   -5. ,   10. ,    0.7],       [ -64. ,  -35. ,   12. ,    1.3],       [ -45. ,  -55. ,   11. ,    1.5],       [ -20. ,  -71. ,   10. ,    1.1],       [  12. ,  -68. ,    6. ,    1.1],       [  39. ,  -59. ,    8. ,    1.1],       [  63. ,  -32. ,    3. ,    1.4],       [  74. ,    0. ,    7. ,    0.8],       [  68. ,   30. ,    6. ,    0.7],       [  46. ,   55. ,   11. ,    1.4],       [  13. ,   77. ,    4. ,    1.3],       [ -21. ,   70. ,    7. ,    1.3],       [ -50. ,   63. ,    7. ,    1.3],       [ -71. ,   33. ,    6. ,    0.6],       [ -72. ,   -5. ,    6. ,    1.2],       [ -63. ,  -34. ,    5. ,    0.9],       [ -50. ,  -58. ,    9. ,    0.7],       [ -15. ,  -73. ,   10. ,    1.2],       [  17. ,  -70. ,   11. ,    1.5],       [  44. ,  -54. ,    3. ,    1.5],       [  68. ,  -34. ,    7. ,    1.3],       [  78. ,   -1. ,   12. ,    1. ],       [  72. ,   31. ,    3. ,    1.3],       [  52. ,   61. ,   10. ,    0.6],       [  16. ,   81. ,    6. ,    0.7],       [ -17. ,   79. ,   10. ,    0.6],       [ -47. ,   61. ,    9. ,    1.4],       [ -72. ,   39. ,    9. ,    0.7],       [ -86. ,   -2. ,   12. ,    1.4],       [ -74. ,  -34. ,    9. ,    0.6],       [ -55. ,  -60. ,    9. ,    1.4],       [ -15. ,  -78. ,    5. ,    1.3],       [  17. ,  -83. ,    9. ,    1.5],       [  52. ,  -64. ,    8. ,    1.1],       [  77. ,  -34. ,    4. ,    0.6],       [  91. ,   -3. ,    4. ,    1.2],       [  76. ,   36. ,    3. ,    0.7],       [  51. ,   70. ,    6. ,    1.3],       [  16. ,   84. ,    7. ,    1.2],       [ -24. ,   87. ,   10. ,    0.9],       [ -58. ,   72. ,    7. ,    1.2],       [ -80. ,   41. ,   12. ,    1.4],       [ -92. ,    3. ,    3. ,    0.6],       [ -80. ,  -35. ,    6. ,    0.6],       [ -58. ,  -64. ,    8. ,    0.6],       [ -15. ,  -88. ,    7. ,    0.7],       [  21. ,  -86. ,   11. ,    0.5],       [  50. ,  -68. ,    6. ,    0.5],       [  74. ,  -40. ,    9. ,    1.4],       [  90. ,   -1. ,    5. ,    0.9],       [  80. ,   45. ,   11. ,    1. ],       [  58. ,   74. ,   11. ,    1.1],       [  17. ,   90. ,    9. ,    1.3],       [ -22. ,   92. ,    5. ,    1.3],       [ -63. ,   73. ,    9. ,    0.6],       [ -89. ,   37. ,    4. ,    1.4],       [ -92. ,   -5. ,    3. ,    1.3],       [ -85. ,  -37. ,    4. ,    1.4],       [ -57. ,  -76. ,    9. ,    1. ],       [ -23. ,  -92. ,    6. ,    0.9],       [  18. ,  -93. ,   10. ,    1. ],       [  55. ,  -75. ,    9. ,    0.7],       [  88. ,  -41. ,    7. ,    1.5],       [  98. ,    3. ,    6. ,    0.7],       [  90. ,   42. ,    9. ,    0.8],       [  65. ,   80. ,    6. ,    1.5],       [  26. ,  100. ,   12. ,    0.9],       [ -25. ,   96. ,    4. ,    1.4],       [ -59. ,   83. ,    3. ,    0.9],       [ -87. ,   44. ,    3. ,    0.6],       [-100. ,    0. ,   10. ,    1.5],       [ -91. ,  -46. ,    6. ,    1.3],       [ -61. ,  -81. ,    5. ,    1.3],       [ -26. ,  -97. ,   10. ,    0.5],       [  20. ,  -96. ,   10. ,    0.7],       [  63. ,  -82. ,    8. ,    0.7],       [  94. ,  -45. ,   12. ,    0.9],       [  49. ,   -5. ,   11. ,    1.4],       [  44. ,   19. ,   19. ,    0.7],       [  33. ,   36. ,   13. ,    0.8],       [   6. ,   49. ,   17. ,    1.4],       [ -12. ,   49. ,   17. ,    0.7],       [ -34. ,   35. ,   13. ,    1.1],       [ -50. ,   25. ,   17. ,    0.6],       [ -50. ,   -3. ,   20. ,    0.5],       [ -46. ,  -20. ,   15. ,    0.5],       [ -35. ,  -44. ,   19. ,    1.5],       [ -14. ,  -49. ,   17. ,    0.7],       [  12. ,  -45. ,   13. ,    1. ],       [  35. ,  -41. ,   17. ,    1.2],       [  45. ,  -18. ,   17. ,    0.5],       [  56. ,    0. ,   18. ,    1.4],       [  52. ,   24. ,   20. ,    1.1],       [  33. ,   47. ,   11. ,    1.2],       [  15. ,   55. ,   15. ,    0.8],       [  -9. ,   56. ,   19. ,    1.3],       [ -38. ,   42. ,   20. ,    0.7],       [ -48. ,   22. ,   11. ,    1. ],       [ -56. ,   -4. ,   17. ,    1.1],       [ -55. ,  -29. ,   17. ,    1.5],       [ -34. ,  -42. ,   13. ,    1.4],       [ -17. ,  -52. ,   20. ,    1. ],       [  14. ,  -58. ,   13. ,    0.9],       [  39. ,  -40. ,   19. ,    1.5],       [  47. ,  -22. ,   17. ,    1.1],       [  65. ,    1. ,   18. ,    0.5],       [  56. ,   27. ,   19. ,    1. ],       [  34. ,   50. ,   14. ,    1.1],       [  12. ,   61. ,   11. ,    1.5],       [ -12. ,   63. ,   16. ,    1.1],       [ -37. ,   45. ,   13. ,    0.6],       [ -52. ,   29. ,   20. ,    1.1],       [ -67. ,   -1. ,   18. ,    0.6],       [ -58. ,  -31. ,   18. ,    0.6],       [ -36. ,  -51. ,   18. ,    1. ],       [ -10. ,  -59. ,   11. ,    0.6],       [  16. ,  -58. ,   18. ,    0.9],       [  40. ,  -46. ,   14. ,    1.1],       [  51. ,  -27. ,   16. ,    0.8],       [  67. ,    1. ,   14. ,    0.6],       [  64. ,   25. ,   11. ,    1. ],       [  38. ,   50. ,   15. ,    1. ],       [  18. ,   69. ,   19. ,    0.7],       [ -20. ,   71. ,   17. ,    0.5],       [ -41. ,   53. ,   12. ,    1.3],       [ -59. ,   32. ,   14. ,    0.7],       [ -71. ,   -2. ,   15. ,    0.8],       [ -59. ,  -33. ,   12. ,    0.7],       [ -45. ,  -52. ,   13. ,    1.3],       [ -15. ,  -68. ,   18. ,    1.3],       [  17. ,  -71. ,   11. ,    1.5],       [  46. ,  -54. ,   13. ,    0.9],       [  64. ,  -29. ,   12. ,    1.2],       [  77. ,   -3. ,   14. ,    1.1],       [  68. ,   35. ,   11. ,    0.7],       [  45. ,   54. ,   16. ,    0.7],       [  15. ,   77. ,   11. ,    0.8],       [ -17. ,   69. ,   17. ,    0.6],       [ -44. ,   59. ,   14. ,    0.9],       [ -70. ,   32. ,   20. ,    0.9],       [ -78. ,    1. ,   20. ,    0.5],       [ -64. ,  -31. ,   18. ,    1. ],       [ -52. ,  -60. ,   19. ,    0.5],       [ -17. ,  -71. ,   17. ,    1.1],       [  17. ,  -74. ,   12. ,    1.3],       [  51. ,  -62. ,   15. ,    1.4],       [  63. ,  -37. ,   11. ,    1.1],       [  82. ,    2. ,   11. ,    0.7],       [  71. ,   32. ,   11. ,    0.8],       [  51. ,   59. ,   15. ,    1.1],       [  13. ,   76. ,   17. ,    1.2],       [ -20. ,   79. ,   11. ,    0.9],       [ -47. ,   65. ,   16. ,    1.2],       [ -70. ,   39. ,   19. ,    1.4],       [ -80. ,    0. ,   16. ,    1.1],       [ -72. ,  -39. ,   17. ,    0.6],       [ -48. ,  -68. ,   20. ,    1.4],       [ -20. ,  -78. ,   11. ,    1.1],       [  13. ,  -83. ,   20. ,    1.3],       [  49. ,  -62. ,   11. ,    1.3],       [  77. ,  -38. ,   16. ,    0.5],       [  92. ,    1. ,   14. ,    0.8],       [  80. ,   36. ,   12. ,    0.6],       [  52. ,   68. ,   11. ,    1. ],       [  15. ,   81. ,   17. ,    0.6],       [ -24. ,   83. ,   18. ,    1.5],       [ -52. ,   66. ,   18. ,    1.1],       [ -82. ,   39. ,   12. ,    1.3],       [ -92. ,    4. ,   15. ,    0.6],       [ -84. ,  -41. ,   12. ,    1.4],       [ -56. ,  -69. ,   19. ,    1.1],       [ -16. ,  -89. ,   16. ,    1.1],       [  23. ,  -83. ,   17. ,    1.3],       [  54. ,  -68. ,   14. ,    1.3],       [  76. ,  -38. ,   20. ,    0.7],       [  92. ,   -3. ,   11. ,    1.2],       [  86. ,   44. ,   16. ,    1. ],       [  59. ,   72. ,   13. ,    1. ],       [  18. ,   93. ,   19. ,    1.1],       [ -19. ,   95. ,   12. ,    0.6],       [ -63. ,   73. ,   17. ,    0.9],       [ -88. ,   41. ,   14. ,    0.9],       [ -94. ,   -5. ,   11. ,    1. ],       [ -86. ,  -38. ,   17. ,    0.8],       [ -58. ,  -72. ,   13. ,    0.6],       [ -22. ,  -93. ,   16. ,    1.5],       [  17. ,  -88. ,   17. ,    1.5],       [  63. ,  -73. ,   18. ,    0.7],       [  88. ,  -41. ,   12. ,    1.1],       [ 100. ,   -4. ,   14. ,    1. ],       [  94. ,   48. ,   17. ,    0.9],       [  63. ,   82. ,   20. ,    1.3],       [  25. ,   99. ,   15. ,    0.8],       [ -19. ,   96. ,   15. ,    0.7],       [ -62. ,   74. ,   16. ,    1.3],       [ -87. ,   47. ,   13. ,    1.4],       [-101. ,    2. ,   15. ,    1.1],       [ -93. ,  -47. ,   18. ,    0.5],       [ -59. ,  -81. ,   11. ,    1.4],       [ -28. ,  -96. ,   11. ,    0.8],       [  18. , -100. ,   11. ,    0.9],       [  59. ,  -78. ,   19. ,    1. ],       [  92. ,  -47. ,   18. ,    0.8],       [  46. ,   -4. ,   24. ,    0.5],       [  49. ,   22. ,   21. ,    0.9],       [  32. ,   41. ,   28. ,    1.2],       [  10. ,   44. ,   20. ,    1.2],       [ -11. ,   47. ,   26. ,    0.9],       [ -33. ,   36. ,   25. ,    1. ],       [ -42. ,   18. ,   27. ,    1.2],       [ -52. ,   -1. ,   19. ,    0.8],       [ -44. ,  -23. ,   25. ,    1.3],       [ -34. ,  -40. ,   28. ,    1.4],       [ -16. ,  -52. ,   20. ,    1.3],       [  13. ,  -45. ,   24. ,    1.3],       [  35. ,  -41. ,   19. ,    0.8],       [  44. ,  -18. ,   26. ,    0.8],       [  52. ,   -4. ,   21. ,    0.9],       [  50. ,   22. ,   23. ,    1.3],       [  30. ,   46. ,   25. ,    1.3],       [   8. ,   51. ,   28. ,    0.6],       [ -10. ,   59. ,   23. ,    1.1],       [ -38. ,   45. ,   22. ,    1.4],       [ -55. ,   28. ,   27. ,    0.5],       [ -52. ,   -1. ,   19. ,    1. ],       [ -56. ,  -28. ,   22. ,    1.4],       [ -40. ,  -39. ,   27. ,    1.4],       [ -15. ,  -52. ,   25. ,    0.8],       [  11. ,  -53. ,   24. ,    1. ],       [  32. ,  -48. ,   28. ,    1.4],       [  55. ,  -23. ,   27. ,    1.1],       [  64. ,    3. ,   25. ,    0.7],       [  56. ,   28. ,   20. ,    1.4],       [  34. ,   45. ,   20. ,    0.8],       [  16. ,   57. ,   28. ,    1.4],       [ -10. ,   58. ,   24. ,    1.2],       [ -41. ,   46. ,   27. ,    0.6],       [ -58. ,   32. ,   27. ,    1.3],       [ -68. ,   -3. ,   24. ,    0.8],       [ -56. ,  -26. ,   23. ,    0.9],       [ -36. ,  -52. ,   25. ,    0.8],       [ -19. ,  -61. ,   20. ,    0.6],       [  18. ,  -57. ,   19. ,    1.1],       [  42. ,  -46. ,   21. ,    0.8],       [  58. ,  -26. ,   20. ,    0.9],       [  65. ,   -4. ,   28. ,    1.1],       [  58. ,   26. ,   23. ,    1.2],       [  45. ,   58. ,   26. ,    1. ],       [  12. ,   65. ,   27. ,    1.3],       [ -19. ,   64. ,   25. ,    1.3],       [ -42. ,   51. ,   20. ,    0.9],       [ -65. ,   31. ,   25. ,    1.1],       [ -74. ,    1. ,   28. ,    1.3],       [ -66. ,  -33. ,   26. ,    1. ],       [ -46. ,  -52. ,   25. ,    0.8],       [ -15. ,  -65. ,   25. ,    1.1],       [  16. ,  -66. ,   28. ,    1. ],       [  47. ,  -50. ,   24. ,    1. ],       [  66. ,  -28. ,   26. ,    0.9],       [  70. ,    2. ,   19. ,    1.4],       [  71. ,   33. ,   19. ,    0.5],       [  46. ,   58. ,   25. ,    0.7],       [  21. ,   74. ,   21. ,    0.9],       [ -20. ,   69. ,   27. ,    1.4],       [ -48. ,   54. ,   22. ,    1.2],       [ -71. ,   36. ,   28. ,    1.5],       [ -78. ,    1. ,   28. ,    1.1],       [ -70. ,  -31. ,   22. ,    1.2],       [ -43. ,  -55. ,   21. ,    0.9],       [ -19. ,  -76. ,   26. ,    1.1],       [  14. ,  -75. ,   25. ,    0.5],       [  46. ,  -64. ,   28. ,    1.3],       [  63. ,  -36. ,   26. ,    1.5],       [  78. ,    2. ,   28. ,    1.2],       [  68. ,   40. ,   21. ,    1.1],       [  51. ,   59. ,   22. ,    1.2],       [  21. ,   83. ,   26. ,    1.5],       [ -22. ,   79. ,   26. ,    1.5],       [ -52. ,   62. ,   24. ,    0.8],       [ -76. ,   36. ,   26. ,    1.4],       [ -81. ,    2. ,   19. ,    0.8],       [ -72. ,  -40. ,   24. ,    1.1],       [ -51. ,  -63. ,   19. ,    0.8],       [ -19. ,  -75. ,   23. ,    1. ],       [  22. ,  -79. ,   22. ,    0.6],       [  47. ,  -66. ,   25. ,    0.6],       [  72. ,  -38. ,   25. ,    1. ],       [  91. ,    3. ,   21. ,    1.4],       [  77. ,   41. ,   19. ,    0.7],       [  59. ,   71. ,   24. ,    1.1],       [  19. ,   87. ,   22. ,    0.6],       [ -20. ,   85. ,   24. ,    1.1],       [ -54. ,   68. ,   27. ,    0.8],       [ -84. ,   35. ,   24. ,    0.7],       [ -85. ,    3. ,   19. ,    0.9],       [ -80. ,  -37. ,   25. ,    0.5],       [ -52. ,  -68. ,   27. ,    1. ],       [ -15. ,  -86. ,   22. ,    0.9],       [  20. ,  -85. ,   19. ,    0.7],       [  52. ,  -68. ,   25. ,    1.2],       [  74. ,  -42. ,   19. ,    1. ],       [  98. ,   -5. ,   22. ,    1.3],       [  84. ,   43. ,   27. ,    1. ],       [  53. ,   71. ,   26. ,    0.7],       [  20. ,   92. ,   23. ,    1.2],       [ -19. ,   89. ,   22. ,    0.7],       [ -60. ,   68. ,   23. ,    1.1],       [ -87. ,   39. ,   25. ,    0.7],       [ -91. ,    4. ,   20. ,    1. ],       [ -87. ,  -45. ,   22. ,    1. ],       [ -62. ,  -75. ,   24. ,    0.7],       [ -23. ,  -89. ,   27. ,    1.5],       [  22. ,  -94. ,   21. ,    0.9],       [  57. ,  -73. ,   21. ,    1.3],       [  85. ,  -42. ,   19. ,    1.2],       [ 102. ,    2. ,   23. ,    0.7],       [  91. ,   41. ,   21. ,    1.1],       [  60. ,   81. ,   27. ,    1.3],       [  19. ,   95. ,   23. ,    1. ],       [ -26. ,   97. ,   22. ,    0.5],       [ -64. ,   78. ,   28. ,    0.7],       [ -91. ,   47. ,   22. ,    0.6],       [ -98. ,   -2. ,   23. ,    0.9],       [ -91. ,  -39. ,   19. ,    0.8],       [ -60. ,  -74. ,   27. ,    1.4],       [ -20. ,  -98. ,   22. ,    0.8],       [  20. ,  -97. ,   19. ,    0.5],       [  59. ,  -84. ,   23. ,    0.9],       [  94. ,  -43. ,   28. ,    1. ],       [  47. ,    0. ,   27. ,    1. ],       [  40. ,   26. ,   31. ,    0.9],       [  34. ,   35. ,   28. ,    0.9],       [   9. ,   47. ,   32. ,    0.5],       [  -8. ,   51. ,   30. ,    0.9],       [ -35. ,   37. ,   30. ,    0.8],       [ -45. ,   18. ,   29. ,    1.2],       [ -47. ,    1. ,   30. ,    0.9],       [ -48. ,  -26. ,   29. ,    1.3],       [ -31. ,  -36. ,   29. ,    0.8],       [ -12. ,  -51. ,   33. ,    0.8],       [  15. ,  -47. ,   33. ,    0.7],       [  33. ,  -37. ,   35. ,    1.4],       [  48. ,  -18. ,   30. ,    0.5],       [  57. ,   -3. ,   34. ,    0.8],       [  45. ,   27. ,   29. ,    0.6],       [  33. ,   48. ,   27. ,    1.2],       [   9. ,   58. ,   36. ,    0.6],       [ -18. ,   56. ,   33. ,    0.6],       [ -38. ,   41. ,   29. ,    0.5],       [ -52. ,   25. ,   31. ,    1.3],       [ -55. ,    3. ,   33. ,    1.1],       [ -49. ,  -30. ,   29. ,    0.7],       [ -35. ,  -48. ,   33. ,    0.9],       [ -10. ,  -56. ,   32. ,    1. ],       [  11. ,  -52. ,   36. ,    0.8],       [  39. ,  -49. ,   33. ,    1.3],       [  50. ,  -29. ,   33. ,    0.7],       [  64. ,    0. ,   30. ,    0.8],       [  57. ,   26. ,   35. ,    1.2],       [  34. ,   46. ,   28. ,    1.3],       [  17. ,   58. ,   36. ,    0.7],       [ -11. ,   58. ,   29. ,    0.7],       [ -41. ,   48. ,   28. ,    0.5],       [ -52. ,   29. ,   36. ,    0.7],       [ -59. ,    1. ,   32. ,    0.8],       [ -56. ,  -26. ,   32. ,    0.6],       [ -41. ,  -49. ,   29. ,    0.8],       [ -19. ,  -61. ,   29. ,    1.4],       [   9. ,  -58. ,   33. ,    1. ],       [  42. ,  -46. ,   28. ,    1. ],       [  54. ,  -27. ,   33. ,    0.8],       [  69. ,   -3. ,   32. ,    0.9],       [  66. ,   34. ,   31. ,    1. ],       [  41. ,   49. ,   29. ,    0.6],       [  13. ,   67. ,   27. ,    0.7],       [ -15. ,   64. ,   30. ,    1.5],       [ -39. ,   51. ,   28. ,    1.1],       [ -64. ,   33. ,   32. ,    1.2],       [ -64. ,    4. ,   33. ,    0.8],       [ -60. ,  -31. ,   36. ,    0.7],       [ -43. ,  -57. ,   31. ,    0.9],       [ -16. ,  -65. ,   29. ,    1.4],       [  17. ,  -70. ,   34. ,    1.3],       [  42. ,  -53. ,   28. ,    1.4],       [  62. ,  -33. ,   30. ,    0.6],       [  73. ,   -3. ,   31. ,    1.2],       [  67. ,   34. ,   35. ,    1.2],       [  48. ,   62. ,   36. ,    1.1],       [  21. ,   68. ,   28. ,    1.5],       [ -17. ,   76. ,   32. ,    1.4],       [ -49. ,   55. ,   34. ,    1.2],       [ -65. ,   35. ,   36. ,    0.7],       [ -72. ,    3. ,   33. ,    1. ],       [ -70. ,  -34. ,   29. ,    1.2],       [ -49. ,  -57. ,   32. ,    1. ],       [ -19. ,  -73. ,   33. ,    1.5],       [  13. ,  -70. ,   30. ,    1. ],       [  47. ,  -55. ,   34. ,    1.1],       [  67. ,  -33. ,   29. ,    0.5],       [  85. ,   -4. ,   33. ,    0.6],       [  73. ,   31. ,   28. ,    1.1],       [  48. ,   62. ,   27. ,    1.1],       [  19. ,   77. ,   36. ,    1.3],       [ -15. ,   82. ,   36. ,    1.2],       [ -47. ,   65. ,   27. ,    0.5],       [ -74. ,   32. ,   29. ,    0.9],       [ -84. ,   -1. ,   30. ,    0.6],       [ -76. ,  -33. ,   28. ,    0.8],       [ -54. ,  -63. ,   34. ,    1.1],       [ -23. ,  -82. ,   34. ,    1.1],       [  16. ,  -78. ,   35. ,    0.8],       [  52. ,  -66. ,   28. ,    1.2],       [  68. ,  -39. ,   34. ,    0.7],       [  87. ,    3. ,   31. ,    1.4],       [  76. ,   35. ,   28. ,    1.1],       [  55. ,   65. ,   28. ,    1.4],       [  23. ,   81. ,   29. ,    1.2],       [ -17. ,   89. ,   32. ,    1.4],       [ -58. ,   64. ,   28. ,    1.3],       [ -83. ,   40. ,   29. ,    1.3],       [ -91. ,    2. ,   31. ,    1.2],       [ -84. ,  -39. ,   33. ,    1.4],       [ -59. ,  -71. ,   32. ,    1. ],       [ -17. ,  -90. ,   31. ,    1.5],       [  18. ,  -89. ,   29. ,    0.6],       [  55. ,  -64. ,   36. ,    1.4],       [  80. ,  -35. ,   27. ,    0.7],       [  91. ,   -5. ,   31. ,    1.5],       [  81. ,   42. ,   28. ,    0.6],       [  54. ,   71. ,   27. ,    1.2],       [  24. ,   93. ,   27. ,    1.2],       [ -21. ,   93. ,   27. ,    1.4],       [ -54. ,   69. ,   29. ,    0.9],       [ -89. ,   45. ,   28. ,    0.7],       [ -92. ,   -4. ,   32. ,    1.5],       [ -85. ,  -40. ,   33. ,    1. ],       [ -61. ,  -76. ,   29. ,    1.5],       [ -25. ,  -89. ,   36. ,    1.2],       [  19. ,  -92. ,   31. ,    1. ],       [  57. ,  -73. ,   34. ,    1.1],       [  87. ,  -43. ,   31. ,    0.8],       [ 100. ,   -3. ,   30. ,    1. ],       [  94. ,   40. ,   32. ,    0.8],       [  61. ,   81. ,   30. ,    0.8],       [  21. ,   98. ,   31. ,    1. ],       [ -21. ,  100. ,   30. ,    0.7],       [ -62. ,   75. ,   27. ,    1.4],       [ -94. ,   39. ,   28. ,    0.5],       [-100. ,   -3. ,   36. ,    0.5],       [ -87. ,  -43. ,   27. ,    1.3],       [ -63. ,  -83. ,   33. ,    0.7],       [ -25. ,  -98. ,   32. ,    0.7],       [  25. ,  -95. ,   28. ,    1.2],       [  60. ,  -76. ,   36. ,    0.6],       [  88. ,  -48. ,   32. ,    1.3],       [  47. ,   -2. ,   38. ,    1. ],       [  48. ,   22. ,   42. ,    1. ],       [  34. ,   37. ,   36. ,    1.1],       [   8. ,   46. ,   44. ,    0.6],       [ -13. ,   45. ,   42. ,    0.5],       [ -35. ,   39. ,   36. ,    0.7],       [ -50. ,   25. ,   42. ,    0.5],       [ -46. ,    4. ,   44. ,    0.6],       [ -47. ,  -18. ,   43. ,    1.2],       [ -29. ,  -40. ,   40. ,    0.7],       [ -13. ,  -51. ,   35. ,    0.7],       [  13. ,  -53. ,   37. ,    1.5],       [  35. ,  -39. ,   44. ,    1.2],       [  45. ,  -25. ,   37. ,    0.6],       [  56. ,    2. ,   38. ,    1.4],       [  49. ,   25. ,   38. ,    1.1],       [  38. ,   44. ,   42. ,    0.6],       [   7. ,   55. ,   36. ,    0.7],       [ -17. ,   58. ,   37. ,    1.3],       [ -36. ,   41. ,   35. ,    1.1],       [ -54. ,   28. ,   44. ,    1.5],       [ -52. ,   -3. ,   36. ,    0.7],       [ -54. ,  -25. ,   42. ,    1.5],       [ -37. ,  -42. ,   42. ,    0.7],       [  -8. ,  -53. ,   38. ,    1.4],       [  12. ,  -56. ,   41. ,    1.3],       [  32. ,  -40. ,   39. ,    0.9],       [  47. ,  -21. ,   35. ,    0.8],       [  59. ,   -1. ,   39. ,    0.6],       [  59. ,   30. ,   36. ,    0.8],       [  40. ,   44. ,   38. ,    0.5],       [  17. ,   65. ,   40. ,    0.6],       [ -19. ,   62. ,   35. ,    1.4],       [ -42. ,   48. ,   41. ,    0.8],       [ -54. ,   30. ,   44. ,    0.7],       [ -61. ,    2. ,   40. ,    0.9],       [ -56. ,  -30. ,   41. ,    1.3],       [ -42. ,  -45. ,   41. ,    0.9],       [ -17. ,  -62. ,   39. ,    0.9],       [  10. ,  -63. ,   40. ,    1.1],       [  39. ,  -45. ,   38. ,    0.7],       [  55. ,  -29. ,   36. ,    1.2],       [  70. ,    0. ,   42. ,    1.5],       [  65. ,   29. ,   38. ,    1.4],       [  46. ,   49. ,   44. ,    0.9],       [  12. ,   71. ,   41. ,    1.4],       [ -16. ,   63. ,   37. ,    0.9],       [ -46. ,   51. ,   39. ,    1. ],       [ -62. ,   31. ,   37. ,    0.8],       [ -68. ,   -4. ,   37. ,    1.5],       [ -61. ,  -27. ,   44. ,    0.9],       [ -43. ,  -55. ,   44. ,    0.8],       [ -14. ,  -72. ,   39. ,    0.6],       [  16. ,  -67. ,   41. ,    1. ],       [  46. ,  -50. ,   41. ,    1.1],       [  57. ,  -34. ,   43. ,    0.8],       [  76. ,   -2. ,   37. ,    0.6],       [  67. ,   37. ,   35. ,    0.8],       [  44. ,   56. ,   39. ,    0.6],       [  20. ,   73. ,   44. ,    0.8],       [ -17. ,   73. ,   42. ,    1.4],       [ -51. ,   60. ,   42. ,    1.3],       [ -63. ,   30. ,   42. ,    1.1],       [ -73. ,   -2. ,   37. ,    1.2],       [ -72. ,  -29. ,   43. ,    1.4],       [ -47. ,  -60. ,   42. ,    1.3],       [ -16. ,  -74. ,   42. ,    1.4],       [  13. ,  -75. ,   40. ,    1.1],       [  49. ,  -58. ,   40. ,    1.3],       [  72. ,  -37. ,   35. ,    1.4],       [  84. ,    3. ,   43. ,    1. ],       [  72. ,   39. ,   36. ,    1. ],       [  49. ,   60. ,   44. ,    1. ],       [  18. ,   74. ,   37. ,    0.8],       [ -20. ,   79. ,   35. ,    1. ],       [ -48. ,   63. ,   39. ,    0.7],       [ -74. ,   36. ,   39. ,    0.8],       [ -78. ,    3. ,   35. ,    1.2],       [ -77. ,  -39. ,   37. ,    0.8],       [ -52. ,  -64. ,   39. ,    1.2],       [ -22. ,  -78. ,   44. ,    0.8],       [  20. ,  -76. ,   40. ,    1.2],       [  52. ,  -68. ,   44. ,    0.9],       [  77. ,  -39. ,   41. ,    1.3],       [  82. ,    3. ,   39. ,    1.5],       [  80. ,   38. ,   37. ,    0.7],       [  51. ,   65. ,   43. ,    0.8],       [  20. ,   87. ,   40. ,    0.7],       [ -22. ,   89. ,   35. ,    0.6],       [ -56. ,   71. ,   35. ,    1.4],       [ -82. ,   36. ,   43. ,    1.3],       [ -92. ,    4. ,   37. ,    0.9],       [ -83. ,  -41. ,   43. ,    0.7],       [ -52. ,  -71. ,   40. ,    0.8],       [ -19. ,  -82. ,   37. ,    1. ],       [  24. ,  -81. ,   35. ,    0.7],       [  55. ,  -66. ,   38. ,    1.3],       [  83. ,  -36. ,   43. ,    1.5],       [  97. ,   -3. ,   38. ,    1.4],       [  81. ,   40. ,   42. ,    1.3],       [  57. ,   74. ,   38. ,    0.7],       [  23. ,   87. ,   43. ,    0.5],       [ -17. ,   88. ,   44. ,    1.4],       [ -59. ,   77. ,   35. ,    0.5],       [ -82. ,   38. ,   44. ,    0.9],       [ -96. ,    4. ,   35. ,    1.2],       [ -81. ,  -44. ,   41. ,    0.8],       [ -63. ,  -70. ,   42. ,    1. ],       [ -22. ,  -96. ,   44. ,    1.3],       [  23. ,  -88. ,   37. ,    1.5],       [  63. ,  -78. ,   42. ,    1.2],       [  84. ,  -44. ,   43. ,    1.2],       [  97. ,   -2. ,   35. ,    1.2],       [  87. ,   43. ,   38. ,    0.6],       [  59. ,   73. ,   39. ,    0.8],       [  22. ,  100. ,   35. ,    0.8],       [ -23. ,   92. ,   41. ,    1.1],       [ -63. ,   80. ,   37. ,    1.5],       [ -90. ,   43. ,   35. ,    1.3],       [-104. ,   -1. ,   36. ,    1.3],       [ -90. ,  -42. ,   35. ,    1.5],       [ -60. ,  -76. ,   42. ,    1.1],       [ -27. , -101. ,   36. ,    1. ],       [  25. ,  -98. ,   39. ,    0.9],       [  64. ,  -78. ,   35. ,    1.1],       [  92. ,  -43. ,   38. ,    1.5]])
        basis0 = BSplineBasis(5, np.array([ -2.8,  -1.6,  -0.9,   0. ,   0. ,   1.1,   1.8,   2.9,   3.8,   4.7,   6.4,   7.1,   7.6,
         9.2,  10.2,  11.4,  12.1,  13. ,  13. ,  14.1,  14.8,  15.9]),2)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  2. ,  2.8,  4.1,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis2 = BSplineBasis(5, np.array([ 0.,  0.,  0.,  0.,  0.,  1.,  2.,  2.,  2.,  2.,  2.]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,True)
        vol2 = vol.clone()
        vol2.raise_order(1, 1, 1)
        self.assertEqual(vol2.order(direction=0), 6)
        self.assertEqual(vol2.order(direction=1), 6)
        self.assertEqual(vol2.order(direction=2), 6)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p7(self):
        controlpoints = np.array([[  2.,  -2.,  -2.],
       [  7.,   2.,  -1.],
       [ 16.,  -2.,   4.],
       [ 31.,   2.,  -5.],
       [ 37.,  -4.,  -3.],
       [ 42.,   3.,   1.],
       [ 56.,  -2.,   3.],
       [ 68.,   2.,   1.],
       [ 77.,  -2.,   0.],
       [ 79.,  -2.,  -1.],
       [ 94.,  -1.,   4.],
       [ 95.,  -4.,  -4.]])
        basis0 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.1,  2. ,  2.6,  3.7,  4.8,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p65(self):
        controlpoints = np.array([[   3.,    1.,    4.],       [  10.,    4.,    4.],       [  24.,   -3.,   -1.],       [  30.,   -4.,    0.],       [  41.,    2.,    0.],       [  54.,   -3.,    3.],       [  63.,   -3.,    0.],       [  80.,   -2.,   -4.],       [  88.,    3.,    1.],       [  98.,   -3.,   -3.],       [  -5.,   12.,   -4.],       [  11.,   16.,    1.],       [  25.,   19.,    3.],       [  28.,   12.,   -5.],       [  44.,   15.,   -5.],       [  51.,   15.,   -3.],       [  67.,   19.,   -1.],       [  74.,   13.,    2.],       [  93.,   20.,   -5.],       [ 104.,   15.,    2.],       [  -5.,   29.,   -3.],       [  13.,   34.,    0.],       [  20.,   35.,    1.],       [  30.,   34.,    4.],       [  46.,   31.,    1.],       [  53.,   29.,    1.],       [  69.,   33.,    1.],       [  80.,   32.,   -5.],       [  92.,   34.,    2.],       [ 100.,   33.,    4.],       [  -2.,   48.,   -5.],       [  11.,   49.,   -3.],       [  19.,   54.,    1.],       [  30.,   45.,   -5.],       [  43.,   45.,    3.],       [  52.,   53.,    4.],       [  66.,   51.,    0.],       [  73.,   48.,   -5.],       [  93.,   46.,    1.],       [  95.,   47.,   -5.],       [   4.,   71.,   -3.],       [   9.,   71.,   -4.],       [  19.,   69.,    3.],       [  29.,   61.,    3.],       [  45.,   69.,    4.],       [  59.,   62.,   -1.],       [  68.,   62.,   -4.],       [  82.,   71.,    1.],       [  84.,   63.,   -1.],       [ 102.,   65.,    3.],       [   4.,   84.,   -3.],       [  16.,   84.,    4.],       [  26.,   84.,   -3.],       [  28.,   81.,   -1.],       [  49.,   86.,   -2.],       [  59.,   83.,   -4.],       [  65.,   79.,    3.],       [  75.,   88.,    0.],       [  90.,   86.,   -2.],       [  99.,   78.,    4.],       [  -3.,   99.,    4.],       [   6.,  100.,   -5.],       [  17.,   98.,    3.],       [  31.,  104.,    4.],       [  46.,  104.,   -4.],       [  56.,   95.,   -3.],       [  63.,  103.,   -4.],       [  77.,   96.,    4.],       [  90.,  101.,    4.],       [  97.,  101.,   -4.]])
        basis0 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.8,  2.1,  2.9,  3.7,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.7,  1.7,  3. ,  3. ,  3. ,  3. ,  3. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(3, 3)
        self.assertEqual(surf2.order(direction=0), 9)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p767(self):
        controlpoints = np.array([[  -5.,    0.,    4.],       [   8.,    4.,   -4.],       [  15.,   -4.,    4.],       [  27.,    4.,   -5.],       [  38.,    0.,    2.],       [  43.,   -2.,   -4.],       [  52.,   -2.,    4.],       [  62.,   -1.,   -4.],       [  68.,   -2.,    0.],       [  84.,    1.,   -5.],       [  94.,   -5.,   -3.],       [ 104.,   -2.,   -3.],       [  -1.,   13.,   -4.],       [   9.,   14.,    1.],       [  14.,   12.,    2.],       [  31.,    6.,    1.],       [  34.,   13.,   -1.],       [  43.,   12.,   -5.],       [  54.,    8.,    4.],       [  68.,   14.,    0.],       [  77.,   14.,   -5.],       [  83.,   10.,   -2.],       [  89.,    9.,   -4.],       [ 103.,    7.,    1.],       [   1.,   18.,    1.],       [   6.,   23.,   -4.],       [  15.,   23.,    1.],       [  24.,   19.,    2.],       [  33.,   25.,   -1.],       [  40.,   22.,   -1.],       [  50.,   18.,    1.],       [  67.,   21.,    4.],       [  71.,   19.,    1.],       [  86.,   25.,    0.],       [  86.,   22.,   -2.],       [ 103.,   17.,   -2.],       [   3.,   31.,   -3.],       [   6.,   35.,    1.],       [  14.,   32.,   -4.],       [  24.,   28.,    0.],       [  33.,   36.,    1.],       [  48.,   32.,   -2.],       [  50.,   33.,    1.],       [  62.,   33.,   -3.],       [  67.,   32.,   -2.],       [  86.,   32.,    0.],       [  93.,   36.,    3.],       [ 100.,   30.,    1.],       [  -3.,   49.,   -2.],       [   5.,   44.,   -5.],       [  19.,   41.,    2.],       [  24.,   46.,   -3.],       [  37.,   44.,    1.],       [  44.,   41.,    4.],       [  55.,   44.,   -2.],       [  67.,   40.,    2.],       [  70.,   42.,   -3.],       [  85.,   46.,   -1.],       [  88.,   44.,   -4.],       [ 103.,   40.,    1.],       [  -5.,   59.,   -3.],       [  10.,   54.,    2.],       [  18.,   59.,   -1.],       [  25.,   56.,   -1.],       [  39.,   59.,   -3.],       [  44.,   58.,   -2.],       [  54.,   51.,    0.],       [  64.,   52.,    4.],       [  70.,   57.,   -2.],       [  79.,   55.,   -1.],       [  86.,   58.,    2.],       [ 102.,   58.,   -4.],       [   1.,   62.,   -4.],       [   5.,   66.,   -4.],       [  14.,   66.,   -1.],       [  28.,   66.,    1.],       [  36.,   69.,    0.],       [  40.,   70.,    0.],       [  55.,   66.,   -2.],       [  63.,   63.,   -3.],       [  74.,   64.,    1.],       [  79.,   65.,    1.],       [  87.,   63.,    0.],       [ 102.,   71.,   -2.],       [  -1.,   73.,   -2.],       [  11.,   78.,    1.],       [  22.,   77.,   -4.],       [  24.,   79.,   -4.],       [  32.,   80.,   -1.],       [  47.,   80.,    3.],       [  59.,   77.,    2.],       [  63.,   76.,    1.],       [  73.,   73.,   -3.],       [  80.,   79.,    0.],       [  92.,   73.,   -2.],       [ 102.,   76.,   -5.],       [  -4.,   85.,    4.],       [  13.,   89.,   -4.],       [  21.,   88.,    3.],       [  24.,   91.,   -5.],       [  40.,   86.,    4.],       [  45.,   86.,   -2.],       [  52.,   83.,   -5.],       [  64.,   93.,   -5.],       [  70.,   88.,   -4.],       [  86.,   87.,   -3.],       [  86.,   93.,    4.],       [  98.,   84.,   -3.],       [  -2.,   99.,   -1.],       [  13.,   99.,    1.],       [  21.,  104.,   -3.],       [  27.,   97.,    1.],       [  40.,   97.,   -3.],       [  43.,   99.,    1.],       [  50.,  104.,   -4.],       [  65.,  100.,    4.],       [  69.,  100.,    4.],       [  76.,  104.,    1.],       [  87.,   95.,   -1.],       [  97.,   97.,   -1.],       [   0.,    1.,   17.],       [  13.,    1.,   11.],       [  17.,    2.,   15.],       [  24.,    3.,   12.],       [  37.,   -1.,    8.],       [  41.,    0.,   13.],       [  56.,   -2.,   17.],       [  67.,   -1.,    8.],       [  75.,   -1.,   17.],       [  81.,   -1.,   13.],       [  88.,    4.,   10.],       [  95.,   -4.,   17.],       [   1.,    7.,    8.],       [   7.,   14.,   10.],       [  23.,   11.,    9.],       [  31.,    9.,    8.],       [  35.,   14.,   14.],       [  41.,    6.,   12.],       [  53.,    6.,    8.],       [  68.,    7.,   15.],       [  67.,    6.,   10.],       [  78.,   11.,    8.],       [  87.,    9.,   12.],       [ 103.,   14.,    7.],       [   0.,   25.,   11.],       [   6.,   20.,   13.],       [  22.,   21.,   15.],       [  30.,   26.,   11.],       [  34.,   24.,   16.],       [  44.,   22.,   10.],       [  54.,   23.,   11.],       [  65.,   18.,   15.],       [  71.,   25.,   10.],       [  83.,   25.,   10.],       [  90.,   23.,    9.],       [ 104.,   26.,   11.],       [   1.,   33.,   10.],       [  13.,   30.,   16.],       [  18.,   34.,   10.],       [  23.,   30.,    8.],       [  33.,   36.,   16.],       [  48.,   34.,   16.],       [  54.,   35.,   15.],       [  62.,   31.,   10.],       [  75.,   32.,   11.],       [  77.,   34.,    8.],       [  86.,   37.,    8.],       [  95.,   35.,   16.],       [  -1.,   42.,   12.],       [   9.,   46.,   10.],       [  23.,   49.,    7.],       [  31.,   42.,   13.],       [  32.,   47.,   13.],       [  49.,   47.,    9.],       [  55.,   46.,   12.],       [  65.,   45.,    8.],       [  70.,   43.,   13.],       [  80.,   43.,   10.],       [  86.,   41.,   16.],       [  96.,   39.,    8.],       [   0.,   52.,    8.],       [   7.,   53.,    8.],       [  15.,   59.,   16.],       [  29.,   57.,   13.],       [  40.,   54.,   13.],       [  50.,   51.,    8.],       [  54.,   57.,   11.],       [  62.,   52.,   11.],       [  72.,   54.,    9.],       [  85.,   55.,   16.],       [  90.,   55.,    9.],       [ 103.,   57.,    7.],       [   3.,   66.,    9.],       [  13.,   63.,   15.],       [  14.,   61.,   11.],       [  30.,   65.,    7.],       [  34.,   68.,   14.],       [  47.,   68.,   16.],       [  51.,   70.,   13.],       [  61.,   62.,   14.],       [  70.,   67.,   14.],       [  86.,   71.,    8.],       [  89.,   65.,   16.],       [ 103.,   61.,   17.],       [   4.,   79.,   12.],       [   7.,   74.,   17.],       [  19.,   76.,   10.],       [  31.,   79.,   17.],       [  33.,   82.,   12.],       [  41.,   79.,    7.],       [  56.,   75.,   17.],       [  59.,   78.,   10.],       [  76.,   77.,   14.],       [  77.,   75.,   13.],       [  88.,   81.,   11.],       [  97.,   73.,   11.],       [   3.,   90.,   12.],       [   8.,   88.,   11.],       [  18.,   88.,    8.],       [  29.,   93.,    7.],       [  35.,   84.,   13.],       [  41.,   86.,   10.],       [  54.,   93.,    7.],       [  66.,   86.,    7.],       [  72.,   90.,    7.],       [  79.,   88.,    8.],       [  92.,   91.,    8.],       [  96.,   85.,   11.],       [  -4.,  102.,    8.],       [  12.,   98.,   15.],       [  14.,   96.,   11.],       [  28.,   96.,    9.],       [  37.,  102.,   15.],       [  43.,  101.,    8.],       [  49.,   96.,   14.],       [  63.,   95.,   16.],       [  73.,   99.,   14.],       [  86.,  102.,   13.],       [  91.,   96.,   10.],       [  99.,   98.,   12.],       [   3.,    3.,   27.],       [   4.,    4.,   24.],       [  13.,   -4.,   23.],       [  32.,    3.,   28.],       [  34.,    3.,   26.],       [  46.,    1.,   24.],       [  51.,    4.,   25.],       [  65.,    4.,   24.],       [  68.,    0.,   20.],       [  85.,    4.,   22.],       [  93.,   -5.,   20.],       [  97.,    4.,   29.],       [  -4.,    6.,   23.],       [   8.,   14.,   26.],       [  17.,    8.,   26.],       [  25.,    9.,   24.],       [  32.,    7.,   24.],       [  49.,    9.,   24.],       [  55.,   10.,   20.],       [  62.,   13.,   25.],       [  69.,   14.,   22.],       [  78.,   12.,   24.],       [  94.,   12.,   25.],       [  98.,   14.,   24.],       [  -3.,   23.,   26.],       [   5.,   20.,   27.],       [  20.,   23.,   21.],       [  31.,   21.,   23.],       [  34.,   21.,   22.],       [  50.,   21.,   21.],       [  52.,   19.,   24.],       [  63.,   21.,   28.],       [  68.,   20.,   24.],       [  86.,   26.,   29.],       [  92.,   19.,   23.],       [ 100.,   22.,   20.],       [   0.,   29.,   28.],       [  13.,   28.,   24.],       [  18.,   30.,   22.],       [  27.,   30.,   26.],       [  32.,   31.,   25.],       [  41.,   37.,   28.],       [  53.,   32.,   25.],       [  60.,   34.,   29.],       [  76.,   32.,   20.],       [  81.,   29.,   27.],       [  90.,   28.,   25.],       [  97.,   28.,   26.],       [   4.,   48.,   23.],       [   9.,   42.,   28.],       [  20.,   47.,   24.],       [  27.,   45.,   24.],       [  33.,   40.,   28.],       [  46.,   41.,   24.],       [  51.,   47.,   27.],       [  65.,   49.,   24.],       [  71.,   43.,   22.],       [  80.,   48.,   25.],       [  93.,   43.,   27.],       [ 102.,   41.,   24.],       [   4.,   59.,   23.],       [  13.,   54.,   25.],       [  13.,   51.,   22.],       [  27.,   53.,   29.],       [  32.,   60.,   29.],       [  48.,   60.,   23.],       [  52.,   53.,   20.],       [  59.,   51.,   29.],       [  76.,   56.,   26.],       [  85.,   57.,   22.],       [  86.,   58.,   23.],       [  99.,   60.,   28.],       [  -2.,   66.,   28.],       [   4.,   67.,   20.],       [  17.,   63.,   20.],       [  29.,   70.,   26.],       [  31.,   62.,   27.],       [  47.,   67.,   20.],       [  51.,   61.,   21.],       [  64.,   61.,   29.],       [  77.,   69.,   26.],       [  84.,   66.,   20.],       [  88.,   65.,   27.],       [ 102.,   70.,   24.],       [  -2.,   80.,   24.],       [   9.,   80.,   20.],       [  21.,   79.,   20.],       [  29.,   76.,   28.],       [  39.,   82.,   23.],       [  42.,   77.,   20.],       [  53.,   76.,   26.],       [  64.,   80.,   28.],       [  77.,   72.,   28.],       [  78.,   79.,   22.],       [  87.,   81.,   22.],       [ 104.,   76.,   24.],       [  -2.,   84.,   21.],       [  10.,   85.,   23.],       [  20.,   91.,   24.],       [  28.,   86.,   28.],       [  34.,   88.,   27.],       [  43.,   88.,   28.],       [  55.,   85.,   29.],       [  65.,   87.,   24.],       [  71.,   86.,   29.],       [  76.,   87.,   23.],       [  92.,   90.,   21.],       [ 104.,   86.,   27.],       [   1.,  101.,   20.],       [  11.,  103.,   25.],       [  16.,   96.,   25.],       [  24.,   96.,   25.],       [  33.,  102.,   22.],       [  40.,   96.,   29.],       [  58.,  104.,   21.],       [  63.,   99.,   29.],       [  75.,  104.,   28.],       [  78.,  104.,   23.],       [  87.,  101.,   25.],       [ 103.,   95.,   26.],       [   3.,    4.,   35.],       [   8.,   -4.,   42.],       [  18.,   -1.,   32.],       [  28.,    4.,   41.],       [  40.,   -1.,   34.],       [  40.,   -1.,   37.],       [  55.,    3.,   35.],       [  61.,    1.,   37.],       [  74.,    2.,   38.],       [  84.,    4.,   41.],       [  94.,   -3.,   39.],       [  97.,    2.,   34.],       [   3.,   10.,   32.],       [   7.,    8.,   36.],       [  20.,   15.,   34.],       [  23.,   12.,   41.],       [  36.,    7.,   38.],       [  40.,    7.,   32.],       [  54.,   13.,   33.],       [  58.,   14.,   41.],       [  74.,    7.,   38.],       [  78.,    6.,   33.],       [  87.,    7.,   33.],       [ 104.,   15.,   42.],       [  -2.,   26.,   33.],       [   6.,   27.,   36.],       [  16.,   18.,   34.],       [  28.,   25.,   38.],       [  31.,   22.,   41.],       [  44.,   24.,   32.],       [  59.,   18.,   37.],       [  62.,   18.,   38.],       [  76.,   23.,   37.],       [  80.,   18.,   39.],       [  95.,   19.,   34.],       [ 101.,   20.,   38.],       [   2.,   35.,   36.],       [  13.,   30.,   35.],       [  17.,   28.,   35.],       [  22.,   34.,   33.],       [  32.,   31.,   41.],       [  44.,   35.,   36.],       [  54.,   37.,   34.],       [  60.,   30.,   33.],       [  74.,   35.,   34.],       [  80.,   34.,   33.],       [  86.,   37.,   38.],       [ 101.,   31.,   37.],       [  -1.,   40.,   33.],       [   7.,   42.,   33.],       [  22.,   41.,   42.],       [  29.,   46.,   40.],       [  31.,   42.,   35.],       [  42.,   43.,   34.],       [  55.,   43.,   36.],       [  61.,   44.,   40.],       [  77.,   42.,   37.],       [  77.,   40.,   32.],       [  88.,   42.,   36.],       [ 102.,   42.,   34.],       [   3.,   54.,   36.],       [  12.,   52.,   38.],       [  18.,   57.,   36.],       [  24.,   58.,   32.],       [  35.,   52.,   40.],       [  47.,   57.,   36.],       [  58.,   55.,   39.],       [  68.,   59.,   38.],       [  76.,   57.,   41.],       [  78.,   59.,   36.],       [  87.,   52.,   33.],       [  99.,   56.,   38.],       [   1.,   65.,   33.],       [   7.,   69.,   41.],       [  20.,   70.,   40.],       [  22.,   68.,   33.],       [  36.,   66.,   36.],       [  48.,   67.,   34.],       [  51.,   66.,   40.],       [  63.,   69.,   41.],       [  69.,   69.,   35.],       [  83.,   62.,   36.],       [  93.,   70.,   40.],       [  97.,   69.,   41.],       [  -5.,   81.,   37.],       [   7.,   80.,   35.],       [  22.,   78.,   34.],       [  25.,   74.,   32.],       [  35.,   76.,   39.],       [  47.,   80.,   42.],       [  51.,   78.,   38.],       [  63.,   79.,   41.],       [  68.,   78.,   32.],       [  82.,   72.,   40.],       [  93.,   77.,   38.],       [  95.,   77.,   35.],       [   4.,   87.,   40.],       [   9.,   86.,   39.],       [  17.,   85.,   35.],       [  31.,   88.,   39.],       [  39.,   88.,   41.],       [  49.,   91.,   37.],       [  56.,   88.,   38.],       [  65.,   85.,   40.],       [  73.,   89.,   38.],       [  78.,   89.,   38.],       [  94.,   87.,   37.],       [  99.,   91.,   34.],       [  -3.,   97.,   34.],       [   9.,  102.,   38.],       [  23.,  100.,   33.],       [  24.,   96.,   36.],       [  40.,  104.,   33.],       [  50.,  101.,   40.],       [  52.,  102.,   35.],       [  61.,   99.,   37.],       [  67.,   97.,   41.],       [  84.,  100.,   34.],       [  94.,   95.,   40.],       [ 104.,   95.,   33.],       [   4.,    4.,   47.],       [   7.,   -1.,   45.],       [  21.,    4.,   54.],       [  24.,    3.,   47.],       [  39.,    0.,   54.],       [  45.,   -1.,   54.],       [  54.,   -2.,   49.],       [  63.,   -5.,   45.],       [  77.,    1.,   45.],       [  81.,   -2.,   46.],       [  95.,    1.,   50.],       [ 102.,   -2.,   45.],       [   1.,   10.,   46.],       [   8.,   12.,   53.],       [  21.,    6.,   54.],       [  23.,   13.,   47.],       [  40.,   12.,   47.],       [  48.,    9.,   49.],       [  50.,    8.,   47.],       [  65.,   13.,   46.],       [  72.,    6.,   45.],       [  83.,   15.,   46.],       [  90.,    8.,   49.],       [ 100.,   13.,   46.],       [  -5.,   26.,   46.],       [   6.,   17.,   54.],       [  20.,   22.,   51.],       [  23.,   17.,   50.],       [  37.,   20.,   51.],       [  42.,   20.,   51.],       [  50.,   24.,   53.],       [  62.,   24.,   52.],       [  73.,   20.,   52.],       [  85.,   18.,   46.],       [  86.,   19.,   54.],       [ 103.,   26.,   48.],       [  -2.,   36.,   45.],       [   9.,   30.,   45.],       [  22.,   30.,   48.],       [  28.,   35.,   51.],       [  32.,   28.,   48.],       [  40.,   36.,   54.],       [  53.,   34.,   50.],       [  67.,   35.,   51.],       [  73.,   37.,   45.],       [  80.,   33.,   45.],       [  89.,   30.,   48.],       [  95.,   37.,   53.],       [   0.,   46.,   49.],       [  10.,   43.,   49.],       [  15.,   40.,   48.],       [  24.,   45.,   53.],       [  38.,   41.,   52.],       [  42.,   47.,   47.],       [  56.,   48.,   46.],       [  61.,   48.,   45.],       [  70.,   46.,   51.],       [  82.,   40.,   46.],       [  86.,   47.,   45.],       [ 104.,   42.,   46.],       [  -2.,   56.,   49.],       [  12.,   56.,   46.],       [  21.,   55.,   45.],       [  28.,   53.,   49.],       [  39.,   52.,   50.],       [  44.,   53.,   54.],       [  57.,   54.,   45.],       [  60.,   50.,   46.],       [  76.,   52.,   46.],       [  80.,   54.,   49.],       [  90.,   58.,   51.],       [  97.,   55.,   51.],       [  -2.,   67.,   53.],       [   8.,   61.,   46.],       [  17.,   67.,   52.],       [  27.,   68.,   46.],       [  32.,   64.,   49.],       [  47.,   61.,   50.],       [  55.,   63.,   48.],       [  67.,   66.,   45.],       [  69.,   70.,   46.],       [  84.,   64.,   46.],       [  95.,   66.,   45.],       [  99.,   62.,   46.],       [   0.,   73.,   53.],       [   9.,   77.,   45.],       [  20.,   74.,   52.],       [  25.,   74.,   47.],       [  33.,   80.,   47.],       [  43.,   77.,   52.],       [  58.,   80.,   52.],       [  61.,   77.,   53.],       [  69.,   76.,   50.],       [  79.,   77.,   51.],       [  89.,   74.,   48.],       [ 100.,   81.,   50.],       [  -1.,   91.,   45.],       [  10.,   85.,   48.],       [  15.,   85.,   48.],       [  27.,   88.,   53.],       [  39.,   91.,   47.],       [  44.,   88.,   45.],       [  51.,   90.,   49.],       [  65.,   91.,   50.],       [  76.,   90.,   48.],       [  82.,   87.,   54.],       [  86.,   86.,   51.],       [  98.,   89.,   51.],       [   3.,   95.,   51.],       [   7.,   97.,   53.],       [  20.,  103.,   54.],       [  30.,  102.,   53.],       [  36.,   95.,   53.],       [  42.,  104.,   48.],       [  51.,  103.,   46.],       [  65.,   96.,   48.],       [  68.,  103.,   48.],       [  82.,  100.,   51.],       [  89.,   98.,   49.],       [ 104.,  100.,   52.],       [  -5.,   -5.,   65.],       [   9.,    2.,   62.],       [  18.,   -1.,   58.],       [  30.,   -5.,   62.],       [  37.,    0.,   57.],       [  48.,    1.,   67.],       [  55.,   -1.,   61.],       [  63.,    2.,   63.],       [  73.,    2.,   64.],       [  85.,    0.,   57.],       [  94.,    4.,   65.],       [ 104.,   -2.,   66.],       [  -3.,   13.,   60.],       [  12.,   14.,   57.],       [  23.,    7.,   61.],       [  27.,    6.,   64.],       [  33.,    6.,   63.],       [  49.,   11.,   62.],       [  50.,   14.,   59.],       [  66.,   11.,   63.],       [  76.,   14.,   65.],       [  78.,    7.,   65.],       [  94.,    7.,   61.],       [  99.,    9.,   67.],       [   2.,   19.,   64.],       [   4.,   23.,   63.],       [  19.,   23.,   65.],       [  25.,   23.,   61.],       [  34.,   24.,   65.],       [  43.,   25.,   65.],       [  49.,   24.,   59.],       [  61.,   24.,   59.],       [  77.,   26.,   66.],       [  79.,   22.,   64.],       [  95.,   21.,   64.],       [  95.,   27.,   65.],       [   1.,   33.,   63.],       [  10.,   33.,   59.],       [  21.,   34.,   57.],       [  23.,   35.,   62.],       [  39.,   37.,   63.],       [  46.,   29.,   60.],       [  55.,   37.,   59.],       [  60.,   33.,   61.],       [  69.,   31.,   64.],       [  84.,   29.,   66.],       [  90.,   31.,   58.],       [ 103.,   28.,   59.],       [   2.,   49.,   64.],       [  10.,   41.,   66.],       [  17.,   47.,   61.],       [  31.,   45.,   63.],       [  35.,   40.,   64.],       [  49.,   45.,   61.],       [  52.,   44.,   67.],       [  66.,   41.,   62.],       [  73.,   48.,   66.],       [  79.,   41.,   65.],       [  89.,   39.,   67.],       [  98.,   43.,   66.],       [   1.,   54.,   59.],       [  11.,   53.,   64.],       [  17.,   50.,   58.],       [  31.,   54.,   62.],       [  34.,   55.,   62.],       [  41.,   59.,   61.],       [  55.,   57.,   66.],       [  64.,   52.,   67.],       [  74.,   52.,   58.],       [  85.,   50.,   60.],       [  86.,   51.,   61.],       [ 100.,   56.,   61.],       [  -4.,   63.,   61.],       [  10.,   62.,   58.],       [  19.,   64.,   60.],       [  23.,   67.,   65.],       [  39.,   69.,   58.],       [  47.,   67.,   61.],       [  51.,   66.,   64.],       [  68.,   70.,   58.],       [  70.,   70.,   60.],       [  78.,   64.,   62.],       [  86.,   65.,   62.],       [ 100.,   67.,   66.],       [   1.,   73.,   62.],       [  10.,   78.,   59.],       [  22.,   79.,   60.],       [  30.,   79.,   66.],       [  34.,   79.,   58.],       [  45.,   81.,   66.],       [  55.,   78.,   66.],       [  66.,   74.,   64.],       [  69.,   76.,   63.],       [  80.,   74.,   66.],       [  91.,   81.,   65.],       [  98.,   81.,   66.],       [  -2.,   87.,   61.],       [  11.,   91.,   67.],       [  17.,   89.,   61.],       [  28.,   93.,   63.],       [  40.,   89.,   66.],       [  48.,   92.,   61.],       [  51.,   88.,   67.],       [  60.,   88.,   66.],       [  71.,   85.,   58.],       [  82.,   84.,   63.],       [  92.,   91.,   61.],       [  97.,   90.,   58.],       [   1.,  101.,   59.],       [   8.,  101.,   63.],       [  18.,  104.,   64.],       [  27.,   97.,   64.],       [  32.,   99.,   62.],       [  43.,   97.,   57.],       [  57.,  103.,   67.],       [  63.,  104.,   62.],       [  73.,   99.,   57.],       [  80.,   99.,   59.],       [  89.,  103.,   66.],       [ 101.,  102.,   59.],       [   3.,    4.,   75.],       [   5.,    3.,   78.],       [  13.,    1.,   76.],       [  31.,   -4.,   78.],       [  35.,   -4.,   78.],       [  48.,   -1.,   77.],       [  52.,    2.,   78.],       [  67.,   -4.,   79.],       [  69.,    3.,   72.],       [  78.,   -2.,   78.],       [  91.,    3.,   77.],       [ 104.,    0.,   76.],       [  -1.,    9.,   79.],       [  13.,   10.,   76.],       [  22.,    8.,   79.],       [  24.,   14.,   75.],       [  40.,   12.,   74.],       [  41.,    7.,   78.],       [  51.,   11.,   77.],       [  58.,   11.,   73.],       [  69.,   10.,   72.],       [  78.,   12.,   77.],       [  88.,   14.,   79.],       [ 104.,   14.,   79.],       [   1.,   25.,   70.],       [  10.,   22.,   74.],       [  17.,   23.,   76.],       [  31.,   24.,   78.],       [  40.,   26.,   72.],       [  49.,   25.,   78.],       [  51.,   22.,   72.],       [  64.,   19.,   73.],       [  70.,   21.,   79.],       [  82.,   18.,   75.],       [  92.,   26.,   76.],       [ 104.,   25.,   76.],       [  -4.,   29.,   78.],       [   7.,   34.,   76.],       [  22.,   34.,   78.],       [  28.,   33.,   79.],       [  32.,   31.,   71.],       [  41.,   34.,   75.],       [  52.,   32.,   70.],       [  64.,   31.,   79.],       [  68.,   31.,   72.],       [  79.,   36.,   70.],       [  93.,   32.,   77.],       [ 100.,   30.,   77.],       [   2.,   45.,   79.],       [   9.,   48.,   75.],       [  13.,   46.,   71.],       [  25.,   47.,   75.],       [  37.,   48.,   78.],       [  49.,   48.,   79.],       [  52.,   45.,   70.],       [  59.,   43.,   70.],       [  70.,   45.,   73.],       [  81.,   47.,   75.],       [  90.,   45.,   70.],       [  99.,   45.,   77.],       [   2.,   60.,   70.],       [   6.,   56.,   72.],       [  18.,   52.,   72.],       [  26.,   58.,   72.],       [  40.,   53.,   74.],       [  45.,   60.,   77.],       [  49.,   56.,   72.],       [  66.,   56.,   70.],       [  72.,   54.,   75.],       [  85.,   60.,   76.],       [  89.,   53.,   76.],       [  95.,   58.,   72.],       [   1.,   68.,   79.],       [  11.,   66.,   72.],       [  13.,   65.,   77.],       [  26.,   65.,   76.],       [  32.,   66.,   70.],       [  42.,   63.,   79.],       [  59.,   64.,   71.],       [  64.,   66.,   70.],       [  72.,   71.,   78.],       [  81.,   67.,   78.],       [  93.,   71.,   70.],       [ 100.,   62.,   79.],       [  -3.,   77.,   71.],       [  13.,   73.,   71.],       [  23.,   73.,   74.],       [  27.,   79.,   72.],       [  35.,   73.,   79.],       [  44.,   74.,   76.],       [  52.,   81.,   76.],       [  59.,   79.,   75.],       [  71.,   73.,   74.],       [  83.,   78.,   76.],       [  88.,   80.,   73.],       [  99.,   81.,   73.],       [  -3.,   90.,   77.],       [   8.,   84.,   74.],       [  15.,   91.,   71.],       [  23.,   89.,   72.],       [  40.,   91.,   77.],       [  45.,   91.,   77.],       [  57.,   89.,   70.],       [  64.,   93.,   70.],       [  69.,   84.,   74.],       [  77.,   87.,   75.],       [  88.,   90.,   75.],       [  97.,   87.,   79.],       [   2.,   96.,   70.],       [   9.,  101.,   78.],       [  15.,  104.,   71.],       [  23.,   96.,   78.],       [  39.,  101.,   78.],       [  50.,  101.,   71.],       [  53.,  100.,   76.],       [  65.,  103.,   76.],       [  69.,  103.,   73.],       [  84.,  104.,   77.],       [  88.,   97.,   78.],       [ 103.,   99.,   76.],       [   3.,    3.,   84.],       [  12.,   -5.,   89.],       [  16.,    3.,   91.],       [  28.,   -2.,   83.],       [  32.,   -4.,   83.],       [  45.,    2.,   88.],       [  55.,    4.,   87.],       [  67.,    0.,   89.],       [  77.,   -5.,   90.],       [  80.,    2.,   87.],       [  94.,   -1.,   84.],       [ 102.,   -3.,   87.],       [  -1.,    7.,   88.],       [  11.,    9.,   90.],       [  20.,   15.,   87.],       [  24.,   13.,   83.],       [  32.,    9.,   92.],       [  44.,   14.,   92.],       [  54.,   11.,   87.],       [  68.,    6.,   83.],       [  77.,   12.,   85.],       [  81.,   11.,   85.],       [  93.,    8.,   89.],       [ 100.,   15.,   91.],       [  -2.,   23.,   83.],       [   9.,   18.,   86.],       [  15.,   21.,   86.],       [  31.,   24.,   89.],       [  38.,   21.,   89.],       [  46.,   20.,   84.],       [  53.,   21.,   89.],       [  61.,   20.,   83.],       [  77.,   18.,   92.],       [  83.,   27.,   85.],       [  89.,   25.,   87.],       [ 101.,   20.,   85.],       [   2.,   31.,   89.],       [   7.,   37.,   82.],       [  18.,   34.,   89.],       [  22.,   32.,   85.],       [  37.,   33.,   83.],       [  48.,   38.,   89.],       [  51.,   34.,   85.],       [  66.,   28.,   87.],       [  69.,   34.,   83.],       [  85.,   32.,   86.],       [  95.,   33.,   83.],       [ 103.,   35.,   84.],       [   3.,   43.,   83.],       [   8.,   48.,   89.],       [  17.,   44.,   91.],       [  31.,   42.,   84.],       [  36.,   43.,   84.],       [  40.,   45.,   86.],       [  58.,   42.,   89.],       [  67.,   46.,   86.],       [  75.,   44.,   86.],       [  81.,   41.,   87.],       [  88.,   43.,   91.],       [ 102.,   47.,   87.],       [   1.,   58.,   90.],       [   8.,   58.,   82.],       [  20.,   56.,   89.],       [  31.,   60.,   92.],       [  41.,   51.,   89.],       [  41.,   52.,   89.],       [  51.,   52.,   84.],       [  66.,   53.,   91.],       [  72.,   58.,   90.],       [  77.,   53.,   91.],       [  86.,   50.,   83.],       [ 103.,   57.,   86.],       [  -4.,   62.,   91.],       [  13.,   65.,   89.],       [  17.,   67.,   86.],       [  28.,   67.,   86.],       [  32.,   63.,   86.],       [  41.,   66.,   90.],       [  57.,   65.,   90.],       [  60.,   66.,   91.],       [  75.,   66.,   86.],       [  83.,   63.,   92.],       [  90.,   64.,   90.],       [ 103.,   65.,   91.],       [  -1.,   73.,   89.],       [   6.,   76.,   82.],       [  13.,   81.,   82.],       [  27.,   78.,   84.],       [  35.,   79.,   86.],       [  47.,   82.,   84.],       [  52.,   78.,   88.],       [  59.,   76.,   92.],       [  76.,   74.,   83.],       [  81.,   74.,   89.],       [  94.,   74.,   85.],       [ 101.,   82.,   83.],       [  -5.,   90.,   86.],       [   6.,   87.,   88.],       [  19.,   87.,   91.],       [  26.,   93.,   89.],       [  40.,   88.,   87.],       [  45.,   84.,   86.],       [  53.,   91.,   86.],       [  66.,   91.,   82.],       [  74.,   83.,   89.],       [  78.,   88.,   83.],       [  91.,   91.,   83.],       [  98.,   92.,   91.],       [  -2.,   96.,   84.],       [   4.,   97.,   90.],       [  18.,  103.,   83.],       [  28.,  101.,   89.],       [  38.,  100.,   85.],       [  47.,   98.,   91.],       [  55.,   95.,   86.],       [  65.,   95.,   91.],       [  76.,   97.,   83.],       [  79.,  103.,   84.],       [  92.,  102.,   86.],       [  99.,   99.,   89.],       [  -3.,   -2.,  104.],       [  13.,   -4.,  103.],       [  17.,   -3.,   97.],       [  23.,    0.,  101.],       [  39.,    4.,  103.],       [  48.,   -1.,  101.],       [  54.,   -3.,  103.],       [  68.,   -1.,  101.],       [  77.,   -4.,  103.],       [  80.,    0.,  100.],       [  87.,   -4.,  102.],       [ 101.,   -3.,  102.],       [  -1.,    7.,  102.],       [   5.,   14.,   96.],       [  17.,   11.,  102.],       [  30.,   14.,   99.],       [  37.,   13.,  103.],       [  48.,    7.,   99.],       [  49.,   11.,  103.],       [  66.,    7.,   98.],       [  75.,   10.,  100.],       [  78.,   12.,  100.],       [  93.,   15.,  104.],       [ 100.,   14.,  100.],       [  -2.,   17.,   96.],       [   4.,   19.,  100.],       [  14.,   20.,  104.],       [  28.,   18.,  103.],       [  36.,   26.,  103.],       [  47.,   17.,   95.],       [  59.,   20.,   95.],       [  60.,   26.,  103.],       [  70.,   18.,  103.],       [  82.,   27.,   97.],       [  92.,   21.,  100.],       [ 104.,   18.,  104.],       [   4.,   28.,   95.],       [  10.,   35.,   96.],       [  16.,   31.,   99.],       [  23.,   30.,  102.],       [  39.,   33.,  102.],       [  45.,   35.,   99.],       [  56.,   31.,   96.],       [  64.,   31.,  104.],       [  74.,   29.,   97.],       [  83.,   33.,  104.],       [  95.,   36.,   98.],       [  99.,   33.,  104.],       [   0.,   45.,   97.],       [  13.,   40.,   98.],       [  19.,   41.,  103.],       [  28.,   39.,   96.],       [  40.,   48.,  104.],       [  40.,   43.,   97.],       [  58.,   48.,   95.],       [  62.,   46.,   97.],       [  76.,   48.,   97.],       [  81.,   45.,   99.],       [  92.,   41.,   95.],       [  98.,   48.,  101.],       [   2.,   53.,  102.],       [   5.,   51.,   98.],       [  21.,   53.,  103.],       [  23.,   53.,  100.],       [  34.,   54.,   99.],       [  44.,   53.,  100.],       [  57.,   58.,  103.],       [  61.,   54.,   98.],       [  70.,   56.,   99.],       [  79.,   60.,  100.],       [  92.,   60.,  103.],       [  99.,   58.,   99.],       [   0.,   71.,   95.],       [   8.,   63.,  102.],       [  19.,   61.,   99.],       [  27.,   65.,  103.],       [  36.,   68.,  103.],       [  42.,   62.,   98.],       [  56.,   66.,  102.],       [  60.,   63.,  100.],       [  72.,   62.,  102.],       [  79.,   66.,  100.],       [  90.,   62.,   95.],       [  99.,   63.,  100.],       [  -5.,   82.,   95.],       [  13.,   79.,   96.],       [  13.,   75.,  102.],       [  26.,   74.,  102.],       [  35.,   82.,   99.],       [  45.,   81.,  100.],       [  52.,   76.,   97.],       [  64.,   80.,  103.],       [  69.,   78.,  102.],       [  84.,   79.,  104.],       [  87.,   75.,  101.],       [  98.,   81.,  100.],       [   0.,   88.,   98.],       [   4.,   93.,  100.],       [  21.,   92.,  102.],       [  30.,   84.,  102.],       [  36.,   90.,  100.],       [  45.,   89.,   95.],       [  51.,   86.,   95.],       [  66.,   90.,  103.],       [  67.,   93.,   98.],       [  83.,   90.,   95.],       [  91.,   88.,   98.],       [  99.,   84.,   96.],       [   3.,  104.,   95.],       [  10.,   99.,   98.],       [  18.,   96.,  102.],       [  23.,   96.,  101.],       [  33.,   98.,  101.],       [  41.,  101.,  101.],       [  53.,  103.,   95.],       [  64.,  102.,   98.],       [  76.,   95.,   99.],       [  77.,  102.,   98.],       [  93.,  100.,  101.],       [ 100.,   96.,  102.]])
        basis0 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.3,  2.1,  3. ,  4. ,  5.3,  6. ,  6. ,  6. ,
        6. ,  6. ,  6. ,  6. ]))
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  2.2,  2.6,  3.7,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis2 = BSplineBasis(7, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  0.6,  1.9,  3. ,  3. ,  3. ,  3. ,  3. ,  3. ,  3. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 3, 3)
        self.assertEqual(vol2.order(direction=0), 8)
        self.assertEqual(vol2.order(direction=1), 9)
        self.assertEqual(vol2.order(direction=2), 10)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p5_C0_periodic(self):
        controlpoints = np.array([[ 103.,   -3.,   -1.],
       [  82.,   56.,    0.],
       [  33.,   98.,   -5.],
       [ -27.,   90.,    1.],
       [ -82.,   60.,   -3.],
       [-104.,    3.,   -5.],
       [ -77.,  -58.,    4.],
       [ -36., -100.,   -2.],
       [  27.,  -99.,    2.],
       [  79.,  -60.,   -1.]])
        basis0 = BSplineBasis(5, np.array([-0.9,  0. ,  0. ,  0. ,  0. ,  0.6,  2.2,  2.9,  4.1,  5.2,  6.1,  7. ,  7. ,  7. ,  7. ,
        7.6]),0)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(1)
        self.assertEqual(crv2.order(direction=0), 6)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p67_C0_periodic(self):
        controlpoints = np.array([[  60.,   -1.,   -1.],       [  47.,   34.,    3.],       [  33.,   54.,    1.],       [   3.,   55.,    2.],       [ -27.,   51.,    2.],       [ -49.,   31.,   -5.],       [ -60.,   -1.,   -4.],       [ -51.,  -35.,   -4.],       [ -34.,  -52.,   -5.],       [  -3.,  -56.,    2.],       [  26.,  -53.,   -3.],       [  55.,  -34.,   -3.],       [  70.,    2.,   -5.],       [  56.,   29.,   -5.],       [  34.,   53.,    2.],       [  -1.,   66.,   -5.],       [ -35.,   56.,    1.],       [ -57.,   31.,    4.],       [ -66.,    0.,   -4.],       [ -61.,  -29.,   -4.],       [ -34.,  -58.,   -4.],       [   3.,  -63.,    4.],       [  31.,  -58.,   -2.],       [  55.,  -34.,    2.],       [  72.,   -5.,    3.],       [  66.,   39.,   -3.],       [  35.,   58.,    3.],       [  -5.,   69.,   -1.],       [ -40.,   66.,   -1.],       [ -59.,   32.,    0.],       [ -72.,    2.,   -1.],       [ -62.,  -32.,    1.],       [ -40.,  -62.,   -3.],       [   2.,  -73.,   -3.],       [  36.,  -62.,   -5.],       [  59.,  -36.,   -2.],       [  77.,    0.,   -5.],       [  64.,   34.,    0.],       [  37.,   68.,    3.],       [  -4.,   79.,   -3.],       [ -39.,   67.,    3.],       [ -63.,   34.,    3.],       [ -78.,    1.,   -4.],       [ -70.,  -42.,    1.],       [ -41.,  -69.,   -1.],       [  -1.,  -76.,   -3.],       [  40.,  -67.,   -5.],       [  65.,  -41.,   -4.],       [  83.,   -5.,    1.],       [  68.,   43.,    2.],       [  45.,   68.,    1.],       [   0.,   85.,    4.],       [ -40.,   72.,   -1.],       [ -73.,   37.,    0.],       [ -83.,    4.,    2.],       [ -73.,  -43.,    0.],       [ -38.,  -77.,   -3.],       [  -1.,  -79.,   -5.],       [  40.,  -77.,   -3.],       [  73.,  -39.,   -4.],       [  88.,    3.,    0.],       [  77.,   47.,    3.],       [  46.,   77.,   -5.],       [   0.,   90.,   -4.],       [ -42.,   80.,    2.],       [ -80.,   43.,   -3.],       [ -87.,    1.,   -2.],       [ -73.,  -48.,   -3.],       [ -41.,  -78.,   -4.],       [   0.,  -86.,   -4.],       [  49.,  -78.,   -2.],       [  74.,  -49.,    2.],       [  92.,   -3.,    3.],       [  78.,   44.,    3.],       [  45.,   78.,   -5.],       [  -2.,   95.,    2.],       [ -44.,   78.,    3.],       [ -78.,   50.,    2.],       [ -96.,   -4.,   -4.],       [ -86.,  -44.,   -3.],       [ -49.,  -80.,   -3.],       [  -4.,  -92.,    2.],       [  48.,  -83.,   -4.],       [  77.,  -47.,    3.],       [ 103.,   -2.,   -2.],       [  81.,   54.,    1.],       [  54.,   87.,   -3.],       [  -1.,  102.,    4.],       [ -47.,   82.,   -4.],       [ -87.,   52.,   -3.],       [ -96.,   -3.,    3.],       [ -88.,  -52.,   -4.],       [ -48.,  -89.,   -4.],       [   4., -102.,    3.],       [  45.,  -90.,   -3.],       [  87.,  -50.,   -2.]])
        basis0 = BSplineBasis(6, np.array([-1.2,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  2.4,  2.9,  4.1,  4.8,  6.2,  6.8,  8. ,  8. ,
        8. ,  8. ,  8. ,  9.2]),0)
        basis1 = BSplineBasis(7, np.array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.,  2.,  2.,  2.,  2.,  2.,  2.,  2.]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(3, 2)
        self.assertEqual(surf2.order(direction=0), 9)
        self.assertEqual(surf2.order(direction=1), 9)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p655_C0_periodic(self):
        controlpoints = np.array([[  51.,    1.,    1.],       [  47.,   24.,    2.],       [  25.,   46.,   -3.],       [   1.,   48.,    3.],       [ -21.,   46.,   -4.],       [ -41.,   26.,    2.],       [ -49.,    1.,   -5.],       [ -40.,  -25.,    4.],       [ -25.,  -40.,    3.],       [  -4.,  -47.,   -3.],       [  25.,  -41.,    4.],       [  40.,  -24.,   -4.],       [  55.,    2.,   -2.],       [  44.,   26.,    3.],       [  31.,   52.,   -3.],       [  -5.,   53.,   -2.],       [ -32.,   45.,   -5.],       [ -54.,   23.,   -4.],       [ -55.,    1.,   -1.],       [ -52.,  -29.,    3.],       [ -25.,  -47.,   -1.],       [   3.,  -59.,   -3.],       [  25.,  -49.,   -3.],       [  45.,  -32.,   -3.],       [  65.,   -2.,    1.],       [  58.,   36.,    4.],       [  33.,   54.,   -1.],       [   2.,   62.,    1.],       [ -28.,   50.,    4.],       [ -55.,   31.,   -2.],       [ -66.,    2.,    1.],       [ -50.,  -28.,    3.],       [ -33.,  -53.,    2.],       [  -2.,  -63.,    0.],       [  30.,  -52.,   -1.],       [  55.,  -27.,    4.],       [  64.,    0.,    1.],       [  56.,   39.,    3.],       [  32.,   58.,   -3.],       [   4.,   63.,   -4.],       [ -36.,   55.,   -3.],       [ -63.,   32.,    2.],       [ -68.,    0.,   -1.],       [ -60.,  -32.,    4.],       [ -37.,  -62.,   -2.],       [   3.,  -72.,   -4.],       [  32.,  -61.,   -1.],       [  60.,  -34.,   -2.],       [  70.,   -3.,   -2.],       [  60.,   34.,   -5.],       [  36.,   62.,   -4.],       [  -2.,   78.,    2.],       [ -33.,   62.,   -2.],       [ -66.,   40.,    1.],       [ -73.,   -5.,   -3.],       [ -63.,  -40.,   -3.],       [ -43.,  -65.,    2.],       [  -3.,  -76.,    3.],       [  40.,  -70.,   -1.],       [  66.,  -40.,   -3.],       [  78.,    1.,   -2.],       [  70.,   44.,    2.],       [  43.,   69.,   -2.],       [   3.,   81.,   -4.],       [ -37.,   74.,   -5.],       [ -69.,   39.,    4.],       [ -83.,   -3.,   -3.],       [ -68.,  -43.,    2.],       [ -44.,  -67.,   -1.],       [  -2.,  -86.,   -2.],       [  36.,  -69.,    4.],       [  71.,  -40.,   -1.],       [  85.,   -5.,   -4.],       [  76.,   40.,    0.],       [  40.,   75.,    3.],       [   4.,   88.,    4.],       [ -45.,   72.,    2.],       [ -79.,   47.,   -5.],       [ -85.,    1.,   -1.],       [ -81.,  -40.,   -1.],       [ -46.,  -76.,    0.],       [   4.,  -84.,   -1.],       [  41.,  -72.,    4.],       [  72.,  -43.,    0.],       [  94.,   -1.,    1.],       [  78.,   47.,   -2.],       [  45.,   78.,   -4.],       [  -3.,   98.,   -2.],       [ -48.,   78.,    3.],       [ -83.,   48.,    0.],       [ -99.,   -1.,    1.],       [ -86.,  -45.,   -3.],       [ -47.,  -85.,    1.],       [  -2.,  -96.,    1.],       [  49.,  -86.,   -2.],       [  82.,  -48.,    0.],       [ 102.,    2.,   -5.],       [  86.,   48.,    3.],       [  51.,   82.,    4.],       [  -1.,  103.,    4.],       [ -51.,   90.,    3.],       [ -85.,   45.,    0.],       [ -98.,    0.,    1.],       [ -85.,  -53.,    3.],       [ -51.,  -89.,   -5.],       [  -5., -101.,   -2.],       [  45.,  -84.,   -1.],       [  91.,  -46.,    1.],       [  46.,    4.,    5.],       [  47.,   29.,    6.],       [  21.,   39.,    8.],       [  -4.,   52.,    2.],       [ -22.,   42.,    8.],       [ -46.,   22.,    9.],       [ -51.,    0.,    2.],       [ -42.,  -22.,    8.],       [ -24.,  -45.,    0.],       [  -2.,  -50.,    7.],       [  21.,  -47.,    7.],       [  43.,  -24.,    8.],       [  51.,    2.,    1.],       [  48.,   31.,    5.],       [  27.,   47.,   -1.],       [   0.,   52.,    5.],       [ -24.,   50.,    0.],       [ -47.,   27.,    2.],       [ -52.,   -1.,    0.],       [ -48.,  -31.,    3.],       [ -30.,  -47.,    7.],       [  -4.,  -59.,    2.],       [  23.,  -45.,    6.],       [  51.,  -33.,    0.],       [  60.,    4.,    6.],       [  50.,   35.,    1.],       [  30.,   52.,   -1.],       [  -4.,   64.,    9.],       [ -32.,   55.,    8.],       [ -51.,   33.,    1.],       [ -61.,   -3.,    9.],       [ -58.,  -36.,    8.],       [ -36.,  -51.,    6.],       [   0.,  -63.,    8.],       [  28.,  -56.,    1.],       [  55.,  -30.,    2.],       [  67.,    2.,   -1.],       [  58.,   38.,    0.],       [  29.,   63.,    2.],       [   3.,   65.,    7.],       [ -39.,   64.,    8.],       [ -62.,   32.,    4.],       [ -72.,    3.,    5.],       [ -56.,  -35.,    2.],       [ -39.,  -63.,    1.],       [   4.,  -70.,    5.],       [  38.,  -64.,   -1.],       [  55.,  -33.,    4.],       [  70.,    1.,    2.],       [  69.,   40.,    4.],       [  39.,   63.,    1.],       [   2.,   71.,    8.],       [ -43.,   65.,    7.],       [ -61.,   32.,    6.],       [ -73.,    3.,    1.],       [ -66.,  -36.,    1.],       [ -38.,  -64.,    6.],       [   3.,  -80.,    5.],       [  40.,  -65.,    2.],       [  66.,  -33.,    6.],       [  80.,    3.,    3.],       [  68.,   38.,    0.],       [  39.,   71.,    0.],       [  -4.,   83.,    4.],       [ -43.,   72.,    5.],       [ -69.,   38.,    6.],       [ -84.,   -2.,    4.],       [ -71.,  -42.,    0.],       [ -42.,  -68.,    5.],       [   0.,  -85.,    1.],       [  36.,  -76.,    5.],       [  69.,  -40.,    8.],       [  84.,    1.,    3.],       [  75.,   41.,    2.],       [  42.,   79.,    5.],       [   0.,   83.,    8.],       [ -49.,   77.,    6.],       [ -78.,   42.,    0.],       [ -85.,    4.,    3.],       [ -81.,  -40.,    3.],       [ -45.,  -78.,    6.],       [  -2.,  -89.,    0.],       [  44.,  -75.,    2.],       [  77.,  -43.,    0.],       [  89.,    2.,    5.],       [  79.,   50.,    9.],       [  43.,   79.,   -1.],       [   4.,   92.,    8.],       [ -51.,   85.,    6.],       [ -77.,   47.,    4.],       [ -95.,   -5.,    4.],       [ -84.,  -47.,    4.],       [ -52.,  -78.,    4.],       [   1.,  -90.,    9.],       [  47.,  -82.,    3.],       [  83.,  -52.,    1.],       [  95.,   -2.,    1.],       [  86.,   49.,    2.],       [  51.,   90.,    2.],       [  -5.,  102.,    2.],       [ -51.,   88.,    0.],       [ -86.,   48.,    3.],       [-102.,    0.,    6.],       [ -92.,  -52.,    4.],       [ -55.,  -92.,    2.],       [  -1.,  -98.,    6.],       [  46.,  -91.,    0.],       [  91.,  -52.,    7.],       [  53.,   -1.,    5.],       [  43.,   27.,   11.],       [  25.,   47.,   12.],       [  -2.,   46.,    6.],       [ -28.,   42.,   12.],       [ -47.,   23.,   10.],       [ -48.,   -3.,   13.],       [ -48.,  -29.,   10.],       [ -25.,  -49.,    8.],       [   0.,  -52.,    6.],       [  23.,  -48.,    7.],       [  44.,  -22.,    4.],       [  57.,   -3.,   12.],       [  49.,   29.,    9.],       [  23.,   44.,    4.],       [   2.,   53.,    6.],       [ -25.,   48.,   12.],       [ -46.,   29.,    6.],       [ -58.,    3.,   13.],       [ -49.,  -31.,   12.],       [ -28.,  -49.,   12.],       [   3.,  -55.,   11.],       [  29.,  -45.,    5.],       [  44.,  -27.,    8.],       [  62.,   -4.,    9.],       [  51.,   34.,    5.],       [  31.,   58.,   11.],       [  -3.,   59.,   11.],       [ -34.,   49.,   13.],       [ -54.,   34.,    7.],       [ -66.,   -5.,   12.],       [ -57.,  -36.,    5.],       [ -37.,  -52.,    9.],       [   2.,  -62.,   10.],       [  31.,  -57.,   10.],       [  58.,  -35.,    5.],       [  70.,   -3.,    9.],       [  59.,   31.,    6.],       [  36.,   63.,   10.],       [  -4.,   73.,   12.],       [ -30.,   60.,   11.],       [ -65.,   38.,    7.],       [ -71.,    2.,    9.],       [ -64.,  -36.,    6.],       [ -30.,  -58.,    5.],       [  -2.,  -69.,    7.],       [  39.,  -65.,   10.],       [  55.,  -33.,   10.],       [  70.,   -3.,    8.],       [  63.,   36.,   12.],       [  41.,   62.,    9.],       [  -2.,   72.,    4.],       [ -43.,   68.,   12.],       [ -63.,   34.,    6.],       [ -71.,   -1.,    4.],       [ -64.,  -34.,    7.],       [ -42.,  -65.,    5.],       [   2.,  -75.,    8.],       [  38.,  -66.,   10.],       [  64.,  -37.,    6.],       [  84.,   -3.,   12.],       [  70.,   42.,    7.],       [  36.,   71.,    6.],       [   4.,   86.,    4.],       [ -44.,   71.,    9.],       [ -73.,   44.,    6.],       [ -78.,   -4.,    7.],       [ -67.,  -42.,    5.],       [ -45.,  -74.,    4.],       [   3.,  -86.,    7.],       [  36.,  -66.,    6.],       [  67.,  -38.,   11.],       [  88.,    0.,   13.],       [  79.,   47.,    5.],       [  42.,   77.,    8.],       [  -4.,   83.,   12.],       [ -44.,   79.,    8.],       [ -73.,   46.,    6.],       [ -86.,   -1.,    9.],       [ -72.,  -46.,    7.],       [ -43.,  -78.,   10.],       [  -4.,  -91.,   12.],       [  47.,  -72.,   12.],       [  77.,  -46.,    9.],       [  89.,   -5.,    5.],       [  85.,   48.,   11.],       [  44.,   79.,   12.],       [  -2.,   96.,   10.],       [ -51.,   84.,    7.],       [ -81.,   50.,    4.],       [ -93.,    3.,    5.],       [ -83.,  -43.,   10.],       [ -49.,  -79.,   10.],       [  -1.,  -90.,   11.],       [  49.,  -86.,   12.],       [  76.,  -49.,   10.],       [  97.,   -2.,    6.],       [  90.,   50.,    6.],       [  50.,   88.,   11.],       [  -2.,   99.,    8.],       [ -55.,   83.,   12.],       [ -91.,   45.,    9.],       [-101.,   -4.,    7.],       [ -90.,  -52.,    7.],       [ -48.,  -89.,    4.],       [   3., -100.,   11.],       [  54.,  -82.,   10.],       [  85.,  -52.,   13.],       [  49.,    0.,    9.],       [  43.,   29.,   16.],       [  20.,   45.,   18.],       [   3.,   45.,   16.],       [ -25.,   46.,   17.],       [ -46.,   26.,   17.],       [ -48.,   -3.,   18.],       [ -42.,  -21.,   10.],       [ -24.,  -46.,   15.],       [   3.,  -54.,   12.],       [  29.,  -40.,   13.],       [  47.,  -28.,   14.],       [  56.,    1.,   14.],       [  47.,   25.,   15.],       [  26.,   44.,   12.],       [   1.,   60.,   13.],       [ -24.,   49.,   14.],       [ -47.,   29.,   10.],       [ -54.,   -1.,   16.],       [ -46.,  -30.,   16.],       [ -32.,  -44.,   15.],       [  -1.,  -53.,   15.],       [  24.,  -50.,   15.],       [  46.,  -33.,   11.],       [  57.,   -3.,    8.],       [  58.,   33.,   17.],       [  33.,   59.,   14.],       [   1.,   59.,   14.],       [ -28.,   56.,   12.],       [ -51.,   33.,   14.],       [ -68.,   -3.,   15.],       [ -57.,  -29.,   15.],       [ -35.,  -50.,   11.],       [  -5.,  -65.,   17.],       [  27.,  -57.,   13.],       [  58.,  -30.,   15.],       [  67.,   -4.,   11.],       [  55.,   35.,    9.],       [  36.,   58.,    9.],       [   1.,   68.,   12.],       [ -33.,   61.,   12.],       [ -57.,   33.,   14.],       [ -66.,    2.,   12.],       [ -65.,  -36.,   12.],       [ -33.,  -58.,   14.],       [  -1.,  -72.,   11.],       [  31.,  -59.,   17.],       [  63.,  -34.,   15.],       [  79.,    1.,   11.],       [  66.,   37.,   11.],       [  38.,   66.,   11.],       [  -1.,   74.,   15.],       [ -37.,   60.,   12.],       [ -61.,   40.,   16.],       [ -80.,   -3.,   12.],       [ -62.,  -36.,   14.],       [ -40.,  -63.,   14.],       [   2.,  -74.,   10.],       [  41.,  -62.,   15.],       [  64.,  -34.,   14.],       [  80.,   -3.,   11.],       [  69.,   39.,   12.],       [  43.,   72.,   17.],       [  -4.,   86.,   11.],       [ -43.,   69.,   12.],       [ -74.,   35.,   11.],       [ -77.,    4.,   15.],       [ -71.,  -45.,   17.],       [ -36.,  -75.,   18.],       [  -3.,  -82.,   11.],       [  44.,  -69.,   15.],       [  73.,  -43.,   14.],       [  86.,    1.,   11.],       [  76.,   40.,   11.],       [  40.,   75.,   16.],       [   3.,   85.,    9.],       [ -45.,   78.,   11.],       [ -72.,   40.,   11.],       [ -88.,   -5.,    9.],       [ -80.,  -48.,   17.],       [ -44.,  -72.,   17.],       [  -5.,  -91.,    8.],       [  46.,  -74.,    8.],       [  76.,  -40.,   16.],       [  92.,    0.,   10.],       [  80.,   44.,    9.],       [  45.,   78.,   15.],       [  -2.,   91.,   11.],       [ -43.,   80.,   11.],       [ -78.,   48.,   15.],       [ -95.,   -4.,   16.],       [ -79.,  -48.,   12.],       [ -48.,  -85.,   14.],       [  -2.,  -95.,   15.],       [  44.,  -82.,   15.],       [  85.,  -43.,   11.],       [  96.,    2.,   11.],       [  84.,   48.,   10.],       [  52.,   90.,   13.],       [   3.,  101.,   15.],       [ -46.,   89.,   11.],       [ -85.,   53.,    8.],       [-102.,    2.,   16.],       [ -83.,  -46.,   17.],       [ -53.,  -85.,   10.],       [   0.,  -96.,   17.],       [  45.,  -83.,   15.],       [  83.,  -48.,   12.],       [  46.,   -1.,   12.],       [  41.,   24.,   20.],       [  25.,   47.,   17.],       [  -1.,   48.,   20.],       [ -28.,   39.,   15.],       [ -43.,   21.,   16.],       [ -54.,    4.,   18.],       [ -45.,  -21.,   13.],       [ -28.,  -42.,   16.],       [  -3.,  -54.,   18.],       [  24.,  -42.,   13.],       [  45.,  -24.,   20.],       [  54.,    3.,   20.],       [  49.,   32.,   22.],       [  26.,   53.,   14.],       [  -2.,   58.,   21.],       [ -32.,   47.,   18.],       [ -52.,   26.,   19.],       [ -57.,    3.,   19.],       [ -48.,  -32.,   14.],       [ -29.,  -54.,   16.],       [  -5.,  -58.,   21.],       [  30.,  -45.,   19.],       [  49.,  -28.,   12.],       [  64.,    1.,   16.],       [  56.,   30.,   18.],       [  33.,   55.,   14.],       [  -4.,   60.,   14.],       [ -37.,   51.,   19.],       [ -56.,   27.,   13.],       [ -66.,   -1.,   17.],       [ -59.,  -30.,   16.],       [ -27.,  -55.,   15.],       [   4.,  -64.,   14.],       [  27.,  -53.,   17.],       [  55.,  -27.,   19.],       [  70.,   -4.,   15.],       [  62.,   38.,   17.],       [  36.,   63.,   18.],       [  -3.,   72.,   15.],       [ -37.,   60.,   22.],       [ -57.,   33.,   15.],       [ -66.,    4.,   15.],       [ -61.,  -31.,   16.],       [ -37.,  -60.,   22.],       [   1.,  -74.,   13.],       [  36.,  -59.,   21.],       [  54.,  -33.,   14.],       [  75.,   -4.,   16.],       [  65.,   34.,   17.],       [  33.,   67.,   20.],       [   3.,   73.,   19.],       [ -42.,   65.,   19.],       [ -64.,   37.,   22.],       [ -71.,    1.,   20.],       [ -70.,  -36.,   20.],       [ -37.,  -64.,   22.],       [  -2.,  -78.,   17.],       [  41.,  -65.,   19.],       [  67.,  -35.,   21.],       [  84.,    2.,   12.],       [  71.,   39.,   14.],       [  39.,   75.,   15.],       [  -2.,   82.,   17.],       [ -41.,   68.,   22.],       [ -68.,   44.,   16.],       [ -85.,    2.,   17.],       [ -72.,  -42.,   21.],       [ -41.,  -68.,   21.],       [  -1.,  -84.,   14.],       [  41.,  -74.,   19.],       [  71.,  -40.,   19.],       [  86.,   -3.,   18.],       [  74.,   48.,   19.],       [  39.,   77.,   21.],       [   3.,   92.,   18.],       [ -44.,   79.,   15.],       [ -79.,   45.,   13.],       [ -92.,    3.,   17.],       [ -77.,  -40.,   17.],       [ -45.,  -76.,   19.],       [  -2.,  -84.,   17.],       [  43.,  -81.,   20.],       [  74.,  -46.,   19.],       [  89.,   -5.,   20.],       [  84.,   42.,   22.],       [  43.,   83.,   16.],       [  -4.,   98.,   16.],       [ -45.,   80.,   15.],       [ -83.,   49.,   16.],       [ -99.,   -1.,   19.],       [ -84.,  -43.,   21.],       [ -50.,  -83.,   18.],       [   1.,  -93.,   18.],       [  42.,  -81.,   13.],       [  78.,  -43.,   17.],       [  99.,   -2.,   19.],       [  91.,   54.,   19.],       [  48.,   81.,   19.],       [  -2.,   99.,   21.],       [ -46.,   81.,   21.],       [ -89.,   48.,   14.],       [-104.,    0.,   12.],       [ -84.,  -54.,   18.],       [ -50.,  -84.,   18.],       [   3., -101.,   21.],       [  52.,  -86.,   16.],       [  88.,  -51.,   19.],       [  52.,    2.,   18.],       [  41.,   29.,   18.],       [  26.,   47.,   25.],       [   3.,   53.,   24.],       [ -21.,   39.,   20.],       [ -45.,   22.,   26.],       [ -53.,   -5.,   25.],       [ -41.,  -23.,   21.],       [ -27.,  -47.,   23.],       [  -3.,  -54.,   17.],       [  27.,  -46.,   18.],       [  38.,  -30.,   20.],       [  56.,   -4.,   24.],       [  49.,   26.,   21.],       [  24.,   52.,   22.],       [  -3.,   58.,   21.],       [ -28.,   44.,   20.],       [ -49.,   23.,   25.],       [ -56.,    1.,   25.],       [ -46.,  -28.,   22.],       [ -29.,  -44.,   17.],       [   4.,  -53.,   25.],       [  31.,  -50.,   23.],       [  47.,  -26.,   21.],       [  59.,    2.,   22.],       [  56.,   35.,   26.],       [  28.,   49.,   22.],       [  -4.,   63.,   18.],       [ -30.,   55.,   18.],       [ -58.,   31.,   27.],       [ -68.,    3.,   20.],       [ -59.,  -32.,   20.],       [ -29.,  -50.,   21.],       [   2.,  -65.,   26.],       [  35.,  -50.,   18.],       [  51.,  -30.,   17.],       [  66.,    3.,   17.],       [  55.,   31.,   23.],       [  30.,   59.,   18.],       [  -4.,   66.,   21.],       [ -35.,   64.,   26.],       [ -59.,   38.,   21.],       [ -70.,    3.,   23.],       [ -56.,  -34.,   24.],       [ -32.,  -63.,   26.],       [   0.,  -72.,   19.],       [  39.,  -64.,   25.],       [  57.,  -34.,   19.],       [  79.,   -4.,   18.],       [  62.,   39.,   23.],       [  35.,   60.,   21.],       [  -5.,   75.,   18.],       [ -34.,   66.,   23.],       [ -68.,   37.,   23.],       [ -73.,    1.,   26.],       [ -69.,  -37.,   26.],       [ -36.,  -68.,   20.],       [   2.,  -79.,   27.],       [  37.,  -64.,   20.],       [  62.,  -43.,   23.],       [  86.,    0.,   25.],       [  68.,   45.,   26.],       [  41.,   74.,   19.],       [  -5.,   76.,   21.],       [ -37.,   72.,   25.],       [ -68.,   36.,   19.],       [ -78.,    0.,   23.],       [ -76.,  -45.,   23.],       [ -42.,  -70.,   23.],       [  -4.,  -78.,   18.],       [  42.,  -74.,   19.],       [  67.,  -37.,   24.],       [  89.,   -1.,   26.],       [  76.,   44.,   22.],       [  48.,   75.,   24.],       [   0.,   86.,   21.],       [ -45.,   77.,   26.],       [ -77.,   41.,   19.],       [ -90.,   -4.,   21.],       [ -78.,  -48.,   19.],       [ -40.,  -72.,   24.],       [  -2.,  -88.,   26.],       [  43.,  -77.,   26.],       [  75.,  -39.,   26.],       [  97.,    2.,   21.],       [  76.,   47.,   22.],       [  47.,   79.,   23.],       [   1.,   95.,   23.],       [ -50.,   83.,   19.],       [ -78.,   42.,   20.],       [ -94.,    2.,   20.],       [ -77.,  -45.,   23.],       [ -50.,  -85.,   20.],       [   4.,  -92.,   24.],       [  43.,  -79.,   23.],       [  79.,  -44.,   24.],       [  99.,   -3.,   23.],       [  90.,   54.,   21.],       [  50.,   89.,   27.],       [  -1.,  101.,   20.],       [ -53.,   85.,   19.],       [ -92.,   48.,   26.],       [-104.,    1.,   20.],       [ -91.,  -53.,   24.],       [ -48.,  -85.,   23.],       [  -3.,  -98.,   26.],       [  53.,  -88.,   24.],       [  84.,  -54.,   18.],       [  46.,    1.,   25.],       [  42.,   21.,   30.],       [  29.,   40.,   30.],       [  -2.,   47.,   28.],       [ -28.,   39.,   23.],       [ -40.,   23.,   22.],       [ -54.,   -5.,   26.],       [ -48.,  -26.,   28.],       [ -24.,  -45.,   22.],       [   0.,  -49.,   21.],       [  22.,  -45.,   30.],       [  40.,  -24.,   26.],       [  58.,   -4.,   22.],       [  48.,   27.,   24.],       [  25.,   53.,   25.],       [   4.,   57.,   30.],       [ -31.,   51.,   28.],       [ -52.,   32.,   25.],       [ -60.,    1.,   25.],       [ -54.,  -24.,   26.],       [ -24.,  -50.,   29.],       [   0.,  -56.,   22.],       [  23.,  -44.,   30.],       [  45.,  -30.,   24.],       [  60.,    3.,   26.],       [  57.,   31.,   27.],       [  32.,   51.,   27.],       [   1.,   65.,   27.],       [ -32.,   58.,   27.],       [ -58.,   30.,   24.],       [ -65.,    1.,   30.],       [ -55.,  -31.,   22.],       [ -36.,  -57.,   30.],       [   2.,  -67.,   25.],       [  29.,  -53.,   24.],       [  52.,  -29.,   30.],       [  69.,    0.,   25.],       [  58.,   33.,   27.],       [  33.,   56.,   30.],       [  -5.,   65.,   28.],       [ -35.,   62.,   31.],       [ -58.,   32.,   27.],       [ -68.,    0.,   25.],       [ -63.,  -36.,   29.],       [ -35.,  -62.,   30.],       [   1.,  -68.,   30.],       [  30.,  -64.,   28.],       [  57.,  -30.,   28.],       [  76.,    4.,   31.],       [  69.,   36.,   29.],       [  39.,   67.,   22.],       [   3.,   74.,   26.],       [ -40.,   66.,   26.],       [ -70.,   39.,   23.],       [ -72.,   -1.,   27.],       [ -67.,  -35.,   27.],       [ -34.,  -67.,   25.],       [  -3.,  -78.,   27.],       [  34.,  -63.,   28.],       [  63.,  -33.,   24.],       [  84.,   -2.,   24.],       [  66.,   40.,   26.],       [  43.,   75.,   25.],       [   1.,   84.,   24.],       [ -40.,   70.,   31.],       [ -75.,   39.,   26.],       [ -78.,    4.,   22.],       [ -71.,  -45.,   23.],       [ -36.,  -70.,   27.],       [  -4.,  -86.,   22.],       [  41.,  -75.,   31.],       [  68.,  -36.,   22.],       [  84.,    3.,   25.],       [  77.,   42.,   28.],       [  48.,   73.,   25.],       [  -5.,   91.,   28.],       [ -46.,   79.,   24.],       [ -71.,   43.,   26.],       [ -85.,   -4.,   23.],       [ -75.,  -47.,   24.],       [ -41.,  -77.,   27.],       [   2.,  -90.,   29.],       [  45.,  -76.,   27.],       [  74.,  -45.,   26.],       [  88.,   -3.,   31.],       [  82.,   47.,   25.],       [  46.,   84.,   27.],       [   3.,   93.,   29.],       [ -47.,   76.,   21.],       [ -80.,   47.,   29.],       [ -96.,    2.,   25.],       [ -82.,  -47.,   22.],       [ -47.,  -86.,   28.],       [  -5.,  -94.,   29.],       [  49.,  -81.,   27.],       [  81.,  -52.,   25.],       [  97.,    0.,   30.],       [  82.,   47.,   23.],       [  54.,   89.,   23.],       [  -1.,   98.,   24.],       [ -49.,   85.,   24.],       [ -90.,   47.,   25.],       [ -96.,   -4.,   25.],       [ -87.,  -47.,   28.],       [ -47.,  -87.,   23.],       [  -3.,  -97.,   25.],       [  47.,  -84.,   31.],       [  91.,  -52.,   31.],       [  48.,    2.,   30.],       [  39.,   21.,   29.],       [  28.,   46.,   35.],       [   3.,   51.,   29.],       [ -23.,   47.,   32.],       [ -43.,   25.,   29.],       [ -54.,   -5.,   33.],       [ -48.,  -30.,   26.],       [ -27.,  -39.,   28.],       [   2.,  -50.,   34.],       [  20.,  -39.,   31.],       [  46.,  -28.,   26.],       [  56.,    0.,   27.],       [  45.,   31.,   35.],       [  25.,   48.,   31.],       [  -4.,   55.,   26.],       [ -25.,   48.,   33.],       [ -44.,   30.,   29.],       [ -52.,    3.,   29.],       [ -51.,  -33.,   28.],       [ -29.,  -51.,   31.],       [   0.,  -53.,   34.],       [  29.,  -45.,   29.],       [  51.,  -24.,   33.],       [  60.,    1.,   29.],       [  52.,   26.,   26.],       [  28.,   52.,   27.],       [   3.,   63.,   26.],       [ -27.,   53.,   30.],       [ -55.,   33.,   33.],       [ -58.,    1.,   35.],       [ -56.,  -32.,   33.],       [ -36.,  -54.,   28.],       [  -4.,  -61.,   28.],       [  27.,  -52.,   28.],       [  56.,  -28.,   33.],       [  70.,   -1.,   27.],       [  61.,   31.,   34.],       [  36.,   56.,   32.],       [  -3.,   71.,   32.],       [ -32.,   57.,   26.],       [ -57.,   32.,   31.],       [ -67.,   -4.,   35.],       [ -65.,  -33.,   28.],       [ -39.,  -62.,   32.],       [  -2.,  -71.,   26.],       [  37.,  -57.,   31.],       [  59.,  -37.,   26.],       [  72.,   -1.,   33.],       [  66.,   33.,   31.],       [  42.,   60.,   30.],       [   4.,   73.,   34.],       [ -39.,   61.,   29.],       [ -65.,   33.,   29.],       [ -80.,    3.,   32.],       [ -62.,  -38.,   30.],       [ -35.,  -70.,   31.],       [   1.,  -77.,   33.],       [  38.,  -66.,   31.],       [  67.,  -38.,   26.],       [  79.,    4.,   28.],       [  71.,   41.,   31.],       [  42.,   67.,   32.],       [  -1.,   84.,   35.],       [ -44.,   73.,   27.],       [ -74.,   39.,   27.],       [ -80.,   -3.,   33.],       [ -73.,  -43.,   30.],       [ -38.,  -69.,   30.],       [   4.,  -78.,   28.],       [  38.,  -69.,   26.],       [  66.,  -39.,   35.],       [  89.,   -4.,   28.],       [  79.,   44.,   27.],       [  42.,   76.,   30.],       [   3.,   86.,   27.],       [ -46.,   73.,   34.],       [ -77.,   45.,   35.],       [ -87.,   -4.,   28.],       [ -73.,  -49.,   27.],       [ -48.,  -78.,   34.],       [   3.,  -92.,   30.],       [  40.,  -79.,   29.],       [  80.,  -43.,   34.],       [  97.,   -4.,   33.],       [  81.,   42.,   26.],       [  51.,   82.,   29.],       [   2.,   89.,   30.],       [ -52.,   77.,   35.],       [ -82.,   49.,   34.],       [ -96.,    1.,   35.],       [ -86.,  -47.,   34.],       [ -43.,  -80.,   28.],       [  -3.,  -93.,   35.],       [  45.,  -86.,   34.],       [  85.,  -46.,   28.],       [ 103.,   -1.,   35.],       [  88.,   52.,   30.],       [  48.,   88.,   34.],       [  -5.,   97.,   32.],       [ -49.,   87.,   29.],       [ -85.,   45.,   31.],       [ -97.,    3.,   34.],       [ -91.,  -52.,   32.],       [ -52.,  -86.,   33.],       [   2., -103.,   30.],       [  46.,  -88.,   28.],       [  89.,  -51.,   26.],       [  50.,    1.,   34.],       [  41.,   26.,   37.],       [  28.,   40.,   31.],       [  -4.,   46.,   31.],       [ -30.,   41.,   37.],       [ -46.,   24.,   33.],       [ -53.,    2.,   38.],       [ -40.,  -30.,   30.],       [ -26.,  -47.,   38.],       [  -4.,  -51.,   36.],       [  26.,  -47.,   33.],       [  40.,  -26.,   31.],       [  57.,    4.,   39.],       [  53.,   27.,   35.],       [  28.,   49.,   31.],       [   4.,   58.,   38.],       [ -30.,   50.,   34.],       [ -51.,   33.,   37.],       [ -57.,   -1.,   38.],       [ -53.,  -33.,   36.],       [ -29.,  -44.,   39.],       [  -4.,  -61.,   35.],       [  30.,  -52.,   33.],       [  50.,  -32.,   32.],       [  60.,   -3.,   37.],       [  53.,   30.,   37.],       [  32.,   53.,   33.],       [   1.,   65.,   39.],       [ -30.,   49.,   38.],       [ -50.,   29.,   31.],       [ -63.,   -5.,   39.],       [ -52.,  -30.,   38.],       [ -27.,  -55.,   38.],       [  -1.,  -59.,   31.],       [  26.,  -58.,   37.],       [  51.,  -33.,   37.],       [  68.,   -5.,   35.],       [  54.,   37.,   36.],       [  30.,   60.,   32.],       [   4.,   67.,   34.],       [ -39.,   63.,   32.],       [ -65.,   31.,   39.],       [ -72.,   -2.,   35.],       [ -62.,  -38.,   37.],       [ -30.,  -59.,   36.],       [  -3.,  -73.,   35.],       [  38.,  -58.,   38.],       [  63.,  -34.,   37.],       [  79.,   -4.,   35.],       [  64.,   39.,   40.],       [  36.,   65.,   32.],       [  -4.,   78.,   35.],       [ -39.,   66.,   39.],       [ -61.,   35.,   39.],       [ -75.,    3.,   33.],       [ -61.,  -37.,   38.],       [ -36.,  -69.,   32.],       [  -5.,  -80.,   32.],       [  36.,  -62.,   35.],       [  67.,  -35.,   30.],       [  81.,   -3.,   38.],       [  68.,   44.,   34.],       [  43.,   69.,   35.],       [   0.,   81.,   34.],       [ -38.,   66.,   38.],       [ -67.,   43.,   36.],       [ -84.,   -5.,   40.],       [ -73.,  -39.,   36.],       [ -42.,  -66.,   36.],       [  -5.,  -78.,   31.],       [  41.,  -75.,   40.],       [  72.,  -45.,   35.],       [  89.,    3.,   39.],       [  74.,   47.,   34.],       [  40.,   78.,   39.],       [   0.,   82.,   33.],       [ -40.,   78.,   32.],       [ -74.,   42.,   35.],       [ -89.,   -5.,   36.],       [ -80.,  -48.,   37.],       [ -45.,  -78.,   32.],       [   2.,  -93.,   33.],       [  38.,  -78.,   32.],       [  71.,  -44.,   36.],       [  98.,    2.,   36.],       [  84.,   49.,   34.],       [  46.,   85.,   32.],       [   2.,   90.,   39.],       [ -44.,   81.,   39.],       [ -81.,   47.,   37.],       [ -95.,   -1.,   31.],       [ -78.,  -42.,   31.],       [ -44.,  -80.,   34.],       [  -2.,  -90.,   33.],       [  51.,  -83.,   35.],       [  80.,  -45.,   30.],       [ 100.,    2.,   34.],       [  89.,   50.,   39.],       [  52.,   84.,   34.],       [   0.,  102.,   36.],       [ -50.,   86.,   37.],       [ -84.,   46.,   37.],       [ -96.,    3.,   37.],       [ -91.,  -48.,   38.],       [ -55.,  -87.,   30.],       [  -2.,  -97.,   36.],       [  51.,  -88.,   33.],       [  87.,  -47.,   39.],       [  46.,    0.,   41.],       [  43.,   22.,   40.],       [  22.,   42.,   41.],       [  -3.,   46.,   38.],       [ -28.,   43.,   41.],       [ -40.,   29.,   40.],       [ -50.,    1.,   36.],       [ -42.,  -26.,   40.],       [ -23.,  -45.,   40.],       [  -3.,  -50.,   41.],       [  27.,  -43.,   39.],       [  41.,  -24.,   42.],       [  52.,    2.,   43.],       [  53.,   26.,   37.],       [  23.,   48.,   41.],       [   0.,   60.,   38.],       [ -30.,   51.,   39.],       [ -45.,   26.,   35.],       [ -57.,    4.,   42.],       [ -45.,  -30.,   44.],       [ -24.,  -50.,   36.],       [   3.,  -56.,   35.],       [  30.,  -51.,   38.],       [  45.,  -31.,   35.],       [  66.,   -4.,   38.],       [  57.,   28.,   44.],       [  30.,   56.,   38.],       [  -1.,   59.,   44.],       [ -32.,   57.,   37.],       [ -59.,   30.,   40.],       [ -61.,    4.,   35.],       [ -57.,  -35.,   37.],       [ -35.,  -58.,   37.],       [   0.,  -65.,   36.],       [  34.,  -52.,   40.],       [  53.,  -29.,   43.],       [  71.,   -5.,   36.],       [  61.,   35.,   42.],       [  37.,   58.,   39.],       [   4.,   66.,   37.],       [ -33.,   59.,   44.],       [ -63.,   30.,   36.],       [ -67.,    3.,   41.],       [ -55.,  -38.,   41.],       [ -30.,  -62.,   40.],       [  -1.,  -64.,   36.],       [  37.,  -56.,   42.],       [  60.,  -32.,   36.],       [  79.,    2.,   37.],       [  68.,   39.,   41.],       [  41.,   62.,   39.],       [  -4.,   75.,   42.],       [ -40.,   60.,   35.],       [ -68.,   41.,   40.],       [ -71.,   -5.,   41.],       [ -66.,  -42.,   42.],       [ -38.,  -63.,   35.],       [  -1.,  -78.,   36.],       [  42.,  -61.,   35.],       [  65.,  -34.,   41.],       [  84.,    2.,   42.],       [  72.,   38.,   40.],       [  45.,   71.,   35.],       [  -3.,   83.,   39.],       [ -44.,   74.,   41.],       [ -76.,   37.,   44.],       [ -79.,   -1.,   40.],       [ -71.,  -39.,   37.],       [ -43.,  -73.,   39.],       [  -5.,  -81.,   43.],       [  41.,  -69.,   41.],       [  72.,  -42.,   40.],       [  89.,   -2.,   35.],       [  75.,   47.,   41.],       [  39.,   75.,   37.],       [  -2.,   90.,   37.],       [ -48.,   76.,   37.],       [ -73.,   39.,   41.],       [ -88.,   -1.,   37.],       [ -77.,  -44.,   36.],       [ -49.,  -76.,   42.],       [   4.,  -91.,   35.],       [  39.,  -81.,   39.],       [  76.,  -45.,   37.],       [  92.,    1.,   36.],       [  85.,   45.,   40.],       [  51.,   82.,   36.],       [   2.,   93.,   36.],       [ -51.,   79.,   43.],       [ -85.,   46.,   44.],       [ -96.,    2.,   38.],       [ -86.,  -52.,   37.],       [ -52.,  -77.,   40.],       [  -3.,  -95.,   40.],       [  50.,  -79.,   36.],       [  78.,  -45.,   40.],       [ 102.,    3.,   35.],       [  88.,   53.,   35.],       [  53.,   82.,   40.],       [   0.,  103.,   41.],       [ -50.,   90.,   44.],       [ -85.,   46.,   38.],       [ -96.,    0.,   36.],       [ -84.,  -53.,   37.],       [ -47.,  -87.,   37.],       [  -5., -105.,   43.],       [  50.,  -84.,   41.],       [  86.,  -46.,   39.]])
        basis0 = BSplineBasis(6, np.array([-1.2,  0. ,  0. ,  0. ,  0. ,  0. ,  1.1,  1.9,  2.7,  4.1,  4.7,  5.6,  6.8,  8. ,  8. ,
        8. ,  8. ,  8. ,  9.1]),0)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  2.1,  2.7,  4.3,  5. ,  5. ,  5. ,  5. ,  5. ]))
        basis2 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0.9,  1.9,  2.9,  3.8,  5.2,  6. ,  6. ,  6. ,  6. ,  6. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 1, 2)
        self.assertEqual(vol2.order(direction=0), 7)
        self.assertEqual(vol2.order(direction=1), 6)
        self.assertEqual(vol2.order(direction=2), 7)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p6_C1_periodic(self):
        controlpoints = np.array([[  96.,    2.,    1.],
       [  92.,   42.,    2.],
       [  59.,   78.,   -5.],
       [  25.,  102.,   -3.],
       [ -21.,   92.,   -2.],
       [ -62.,   78.,   -1.],
       [ -95.,   39.,    2.],
       [-103.,   -5.,   -2.],
       [ -89.,  -46.,   -3.],
       [ -58.,  -81.,    3.],
       [ -22.,  -99.,    2.],
       [  27.,  -96.,    0.],
       [  66.,  -79.,    1.],
       [  93.,  -45.,    4.]])
        basis0 = BSplineBasis(6, np.array([ -1.9,  -1.1,   0. ,   0. ,   0. ,   0. ,   1.2,   1.8,   3.4,   4.2,   5.1,   6. ,   6.9,
         7.8,   9.1,   9.9,  11. ,  11. ,  11. ,  11. ,  12.2,  12.8]),1)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(3)
        self.assertEqual(crv2.order(direction=0), 9)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p55_C1_periodic(self):
        controlpoints = np.array([[  55.,    4.,    1.],       [  52.,   28.,   -2.],       [  30.,   50.,   -1.],       [  -5.,   63.,    1.],       [ -34.,   49.,   -3.],       [ -54.,   32.,   -3.],       [ -63.,   -1.,    0.],       [ -48.,  -27.,    1.],       [ -33.,  -55.,    0.],       [   2.,  -65.,    4.],       [  26.,  -47.,   -1.],       [  49.,  -34.,   -3.],       [  63.,   -2.,    3.],       [  59.,   32.,   -3.],       [  30.,   53.,   -5.],       [   0.,   60.,    3.],       [ -28.,   54.,   -5.],       [ -56.,   28.,   -4.],       [ -65.,   -2.,   -2.],       [ -59.,  -32.,   -4.],       [ -35.,  -60.,    4.],       [   3.,  -62.,   -3.],       [  30.,  -56.,    1.],       [  53.,  -30.,    4.],       [  69.,    1.,   -1.],       [  62.,   30.,    4.],       [  33.,   60.,    1.],       [  -4.,   71.,   -5.],       [ -33.,   60.,   -5.],       [ -57.,   35.,    1.],       [ -66.,   -5.,   -4.],       [ -59.,  -40.,    0.],       [ -34.,  -63.,   -2.],       [   3.,  -67.,    1.],       [  36.,  -62.,   -2.],       [  59.,  -38.,    2.],       [  76.,   -1.,    1.],       [  67.,   41.,    4.],       [  33.,   62.,    4.],       [   0.,   73.,   -1.],       [ -37.,   60.,   -1.],       [ -67.,   39.,    1.],       [ -77.,   -2.,    2.],       [ -61.,  -37.,    4.],       [ -34.,  -60.,    3.],       [   1.,  -69.,   -3.],       [  33.,  -66.,   -3.],       [  64.,  -35.,   -2.],       [  76.,   -4.,    0.],       [  64.,   39.,   -5.],       [  35.,   65.,    2.],       [  -3.,   78.,    3.],       [ -40.,   67.,   -3.],       [ -72.,   42.,   -2.],       [ -79.,    2.,   -1.],       [ -65.,  -41.,    0.],       [ -36.,  -69.,    3.],       [  -1.,  -79.,    0.],       [  33.,  -70.,    0.],       [  64.,  -36.,   -1.],       [  77.,    1.,   -1.],       [  74.,   37.,    2.],       [  44.,   69.,    3.],       [   0.,   78.,   -4.],       [ -43.,   69.,   -5.],       [ -70.,   44.,   -3.],       [ -81.,   -3.,   -2.],       [ -73.,  -44.,    1.],       [ -41.,  -73.,   -1.],       [  -3.,  -84.,    1.],       [  45.,  -69.,    2.],       [  67.,  -45.,   -1.],       [  88.,   -4.,   -1.],       [  77.,   43.,    3.],       [  39.,   76.,   -1.],       [  -4.,   82.,   -3.],       [ -45.,   70.,   -5.],       [ -77.,   47.,   -1.],       [ -89.,    1.,   -2.],       [ -79.,  -42.,   -5.],       [ -40.,  -77.,   -4.],       [  -1.,  -84.,    0.],       [  40.,  -79.,   -4.],       [  76.,  -41.,    4.],       [  93.,   -1.,   -5.],       [  78.,   49.,    0.],       [  40.,   83.,   -4.],       [  -4.,   88.,    3.],       [ -49.,   77.,   -5.],       [ -83.,   48.,   -1.],       [ -92.,   -2.,    3.],       [ -83.,  -43.,    4.],       [ -44.,  -78.,   -4.],       [  -4.,  -87.,   -5.],       [  43.,  -83.,   -1.],       [  74.,  -43.,   -3.],       [  90.,    2.,    2.],       [  83.,   51.,    1.],       [  43.,   80.,   -5.],       [   2.,   95.,    0.],       [ -48.,   78.,   -3.],       [ -84.,   44.,    0.],       [-100.,    4.,    0.],       [ -84.,  -49.,    2.],       [ -47.,  -81.,   -3.],       [  -1.,  -94.,   -3.],       [  45.,  -81.,   -4.],       [  81.,  -47.,    0.],       [  97.,    4.,   -3.],       [  85.,   54.,    2.],       [  54.,   89.,    0.],       [  -5.,   98.,    2.],       [ -52.,   89.,    0.],       [ -88.,   51.,    1.],       [ -96.,    3.,   -4.],       [ -87.,  -54.,    3.],       [ -50.,  -83.,    4.],       [   2.,  -96.,    2.],       [  48.,  -88.,    4.],       [  90.,  -51.,    1.]])
        basis0 = BSplineBasis(5, np.array([ -2.3,  -1.3,   0. ,   0. ,   0. ,   1.1,   1.6,   3.3,   3.8,   4.7,   6.2,   7.3,   7.7,
         8.7,  10. ,  10. ,  10. ,  11.1,  11.6]),1)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  1.8,  3. ,  3.8,  4.8,  6. ,  6. ,  6. ,  6. ,  6. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(3, 3)
        self.assertEqual(surf2.order(direction=0), 8)
        self.assertEqual(surf2.order(direction=1), 8)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p566_C1_periodic(self):
        controlpoints = np.array([[  52.,    2.,   -3.],       [  45.,   23.,   -3.],       [  26.,   43.,    3.],       [   2.,   50.,    4.],       [ -27.,   41.,    4.],       [ -47.,   24.,   -4.],       [ -53.,   -4.,    1.],       [ -48.,  -24.,   -3.],       [ -21.,  -47.,   -3.],       [  -5.,  -47.,   -5.],       [  27.,  -41.,   -4.],       [  41.,  -27.,   -5.],       [  57.,   -1.,   -2.],       [  53.,   26.,    1.],       [  27.,   45.,   -4.],       [   3.,   57.,   -4.],       [ -28.,   52.,    3.],       [ -55.,   28.,   -3.],       [ -63.,   -3.,    3.],       [ -53.,  -28.,    4.],       [ -33.,  -48.,   -2.],       [  -3.,  -62.,    0.],       [  28.,  -47.,   -5.],       [  53.,  -28.,    2.],       [  66.,    2.,   -1.],       [  53.,   31.,    3.],       [  34.,   58.,    3.],       [  -1.,   64.,   -3.],       [ -35.,   53.,   -4.],       [ -57.,   28.,    1.],       [ -64.,    1.,   -1.],       [ -56.,  -34.,    4.],       [ -34.,  -63.,    2.],       [   1.,  -63.,   -2.],       [  32.,  -54.,    0.],       [  59.,  -36.,   -4.],       [  75.,    3.,    4.],       [  66.,   34.,    2.],       [  34.,   68.,   -2.],       [  -2.,   74.,   -4.],       [ -40.,   68.,   -4.],       [ -60.,   36.,    1.],       [ -79.,   -4.,    3.],       [ -68.,  -34.,    3.],       [ -42.,  -67.,    0.],       [  -2.,  -71.,    3.],       [  40.,  -70.,   -2.],       [  69.,  -41.,    2.],       [  78.,   -2.,   -1.],       [  70.,   42.,    1.],       [  38.,   75.,    2.],       [   2.,   83.,   -2.],       [ -45.,   72.,    2.],       [ -71.,   37.,    3.],       [ -83.,    4.,   -5.],       [ -68.,  -45.,    1.],       [ -39.,  -71.,    2.],       [  -2.,  -88.,    3.],       [  40.,  -77.,    4.],       [  74.,  -41.,    1.],       [  91.,   -3.,    0.],       [  80.,   48.,    4.],       [  44.,   81.,   -5.],       [  -5.,   88.,   -2.],       [ -48.,   78.,   -1.],       [ -83.,   45.,    1.],       [ -95.,    2.,    0.],       [ -76.,  -45.,    1.],       [ -42.,  -76.,   -4.],       [  -2.,  -97.,    0.],       [  48.,  -76.,    3.],       [  82.,  -43.,   -5.],       [ 103.,   -3.,   -4.],       [  82.,   45.,    4.],       [  46.,   87.,    3.],       [  -3.,   98.,   -3.],       [ -54.,   85.,   -2.],       [ -83.,   51.,   -1.],       [-104.,    2.,    2.],       [ -91.,  -46.,   -5.],       [ -53.,  -82.,    1.],       [   1., -105.,   -2.],       [  49.,  -85.,    3.],       [  91.,  -50.,   -1.],       [  53.,    3.,    1.],       [  46.,   29.,    9.],       [  26.,   41.,   -1.],       [   2.,   48.,    9.],       [ -25.,   42.,    5.],       [ -47.,   23.,    5.],       [ -48.,   -4.,    3.],       [ -47.,  -26.,    8.],       [ -29.,  -45.,    1.],       [   1.,  -53.,    9.],       [  21.,  -44.,    6.],       [  38.,  -22.,    2.],       [  57.,   -3.,    1.],       [  48.,   31.,    4.],       [  33.,   49.,    0.],       [  -2.,   53.,    2.],       [ -31.,   53.,    3.],       [ -56.,   31.,    8.],       [ -62.,    2.,    4.],       [ -47.,  -32.,    5.],       [ -34.,  -47.,    2.],       [  -2.,  -55.,    1.],       [  33.,  -53.,    5.],       [  49.,  -30.,    0.],       [  63.,    1.,    0.],       [  59.,   29.,    4.],       [  37.,   62.,    2.],       [  -3.,   67.,    2.],       [ -33.,   60.,    6.],       [ -57.,   38.,    0.],       [ -64.,    3.,    6.],       [ -58.,  -34.,    7.],       [ -33.,  -54.,    1.],       [  -1.,  -71.,    5.],       [  35.,  -57.,    1.],       [  56.,  -36.,    2.],       [  79.,   -2.,    8.],       [  66.,   35.,    4.],       [  32.,   65.,    2.],       [  -2.,   74.,    2.],       [ -40.,   60.,    5.],       [ -63.,   33.,    8.],       [ -78.,   -1.,    1.],       [ -70.,  -36.,    2.],       [ -34.,  -64.,    8.],       [   1.,  -72.,    8.],       [  34.,  -68.,    8.],       [  63.,  -41.,    1.],       [  80.,    2.,    3.],       [  71.,   43.,    7.],       [  41.,   68.,    9.],       [   0.,   79.,    3.],       [ -41.,   76.,    8.],       [ -75.,   37.,    4.],       [ -85.,   -3.,    5.],       [ -70.,  -40.,    4.],       [ -42.,  -72.,    9.],       [  -2.,  -83.,    3.],       [  38.,  -75.,    7.],       [  74.,  -44.,    2.],       [  88.,    4.,    5.],       [  79.,   48.,    6.],       [  46.,   76.,    5.],       [  -3.,   94.,   -1.],       [ -50.,   74.,    7.],       [ -84.,   46.,    5.],       [ -94.,   -1.,    7.],       [ -81.,  -47.,    0.],       [ -47.,  -84.,    8.],       [   3.,  -93.,   -1.],       [  41.,  -82.,    0.],       [  79.,  -51.,    4.],       [ 100.,   -4.,    5.],       [  90.,   49.,    9.],       [  48.,   84.,    3.],       [   0.,   97.,    5.],       [ -55.,   89.,    1.],       [ -84.,   50.,    3.],       [ -98.,    1.,    8.],       [ -86.,  -54.,    6.],       [ -53.,  -88.,    4.],       [   0., -102.,   -1.],       [  48.,  -89.,    7.],       [  85.,  -49.,    8.],       [  45.,   -3.,    7.],       [  45.,   23.,    6.],       [  20.,   45.,    7.],       [  -4.,   47.,    5.],       [ -24.,   38.,   13.],       [ -42.,   20.,    6.],       [ -47.,   -5.,   13.],       [ -41.,  -29.,    6.],       [ -30.,  -39.,    9.],       [   0.,  -52.,    9.],       [  20.,  -42.,   13.],       [  38.,  -21.,    7.],       [  61.,   -5.,   11.],       [  55.,   28.,   10.],       [  24.,   53.,    4.],       [  -2.,   54.,   12.],       [ -25.,   46.,    8.],       [ -51.,   31.,    5.],       [ -57.,   -1.,   13.],       [ -50.,  -33.,    7.],       [ -25.,  -46.,    5.],       [  -2.,  -56.,   13.],       [  29.,  -47.,   12.],       [  51.,  -30.,   10.],       [  62.,   -5.,   11.],       [  60.,   35.,    4.],       [  30.,   58.,    6.],       [   4.,   68.,   13.],       [ -35.,   61.,    6.],       [ -55.,   29.,   10.],       [ -68.,    1.,    6.],       [ -63.,  -33.,   10.],       [ -33.,  -60.,    7.],       [  -4.,  -71.,   10.],       [  29.,  -55.,   10.],       [  53.,  -31.,   10.],       [  78.,    2.,   10.],       [  65.,   40.,   11.],       [  39.,   65.,    5.],       [   3.,   79.,   10.],       [ -35.,   60.,    8.],       [ -70.,   36.,    4.],       [ -71.,    1.,   13.],       [ -68.,  -39.,   11.],       [ -40.,  -61.,   12.],       [  -5.,  -76.,    6.],       [  33.,  -66.,    3.],       [  66.,  -36.,   10.],       [  78.,    4.,   13.],       [  70.,   44.,   12.],       [  40.,   72.,   11.],       [   0.,   85.,    6.],       [ -44.,   73.,   12.],       [ -72.,   44.,    6.],       [ -81.,   -1.,   11.],       [ -70.,  -38.,    9.],       [ -40.,  -69.,    9.],       [  -2.,  -87.,    7.],       [  38.,  -75.,    3.],       [  75.,  -47.,    9.],       [  95.,   -5.,   11.],       [  83.,   44.,    3.],       [  44.,   75.,    9.],       [  -5.,   96.,    8.],       [ -49.,   82.,    5.],       [ -82.,   41.,    5.],       [ -87.,   -2.,    5.],       [ -83.,  -45.,    9.],       [ -51.,  -82.,    4.],       [  -1.,  -91.,   12.],       [  50.,  -79.,    5.],       [  77.,  -48.,   12.],       [  97.,    4.,   12.],       [  82.,   48.,    6.],       [  47.,   91.,    5.],       [  -5.,   95.,    5.],       [ -50.,   89.,    7.],       [ -84.,   49.,   12.],       [-101.,    3.,    4.],       [ -85.,  -50.,    9.],       [ -52.,  -87.,    7.],       [  -5., -102.,    5.],       [  49.,  -87.,    7.],       [  90.,  -54.,   11.],       [  48.,   -5.,   17.],       [  42.,   21.,   15.],       [  20.,   40.,    9.],       [  -3.,   50.,   10.],       [ -22.,   47.,    8.],       [ -45.,   26.,   15.],       [ -46.,    4.,    9.],       [ -44.,  -29.,   10.],       [ -29.,  -47.,   11.],       [   3.,  -46.,    8.],       [  25.,  -46.,   15.],       [  41.,  -21.,   13.],       [  61.,    2.,   14.],       [  48.,   28.,   16.],       [  28.,   54.,    9.],       [   2.,   63.,    9.],       [ -25.,   51.,    9.],       [ -53.,   26.,   10.],       [ -62.,   -5.,   11.],       [ -53.,  -29.,   14.],       [ -27.,  -54.,   10.],       [  -2.,  -59.,   13.],       [  25.,  -54.,   14.],       [  54.,  -26.,   15.],       [  67.,    3.,   12.],       [  61.,   33.,   13.],       [  29.,   57.,   16.],       [  -4.,   64.,   13.],       [ -35.,   61.,   11.],       [ -61.,   38.,   14.],       [ -63.,    1.,   10.],       [ -59.,  -35.,   11.],       [ -31.,  -62.,   12.],       [  -5.,  -66.,   10.],       [  32.,  -58.,   13.],       [  62.,  -35.,   10.],       [  70.,    2.,   15.],       [  63.,   35.,    8.],       [  36.,   68.,   12.],       [  -2.,   70.,    9.],       [ -34.,   68.,   15.],       [ -61.,   37.,   11.],       [ -74.,    1.,   10.],       [ -68.,  -39.,   10.],       [ -38.,  -66.,   14.],       [  -1.,  -80.,   12.],       [  37.,  -66.,   12.],       [  67.,  -38.,   15.],       [  78.,    2.,    9.],       [  71.,   41.,   10.],       [  40.,   68.,   15.],       [  -5.,   84.,   13.],       [ -41.,   69.,    9.],       [ -72.,   46.,   16.],       [ -84.,   -1.,   10.],       [ -77.,  -42.,   10.],       [ -44.,  -77.,   10.],       [  -5.,  -84.,   11.],       [  44.,  -69.,    9.],       [  76.,  -37.,   12.],       [  89.,   -1.,   16.],       [  75.,   44.,   11.],       [  44.,   77.,   15.],       [  -5.,   88.,    8.],       [ -47.,   77.,   14.],       [ -79.,   48.,   15.],       [ -90.,   -2.,   12.],       [ -83.,  -47.,   12.],       [ -50.,  -83.,    9.],       [  -3.,  -92.,   16.],       [  48.,  -76.,   15.],       [  74.,  -43.,   13.],       [  96.,   -3.,   11.],       [  82.,   48.,   15.],       [  54.,   81.,   17.],       [  -5.,  102.,   12.],       [ -46.,   82.,   13.],       [ -87.,   53.,   17.],       [-102.,   -2.,   13.],       [ -90.,  -52.,   10.],       [ -55.,  -89.,   15.],       [  -3., -100.,   16.],       [  50.,  -89.,   10.],       [  90.,  -48.,   12.],       [  46.,    4.,   18.],       [  42.,   20.,   17.],       [  28.,   39.,   15.],       [  -4.,   47.,   16.],       [ -22.,   44.,   19.],       [ -47.,   24.,   21.],       [ -46.,   -2.,   16.],       [ -41.,  -22.,   13.],       [ -25.,  -48.,   21.],       [  -4.,  -52.,   20.],       [  21.,  -42.,   17.],       [  38.,  -28.,   16.],       [  62.,    4.,   14.],       [  51.,   30.,   20.],       [  32.,   45.,   17.],       [   2.,   55.,   22.],       [ -34.,   54.,   17.],       [ -50.,   25.,   13.],       [ -63.,    0.,   20.],       [ -47.,  -31.,   16.],       [ -34.,  -48.,   15.],       [   2.,  -54.,   21.],       [  33.,  -48.,   22.],       [  49.,  -29.,   17.],       [  62.,   -4.,   18.],       [  52.,   32.,   19.],       [  33.,   59.,   21.],       [   4.,   70.,   18.],       [ -31.,   59.,   15.],       [ -62.,   28.,   18.],       [ -63.,    1.,   14.],       [ -57.,  -32.,   21.],       [ -34.,  -57.,   19.],       [  -3.,  -66.,   19.],       [  35.,  -57.,   21.],       [  59.,  -38.,   22.],       [  71.,   -4.,   18.],       [  63.,   42.,   19.],       [  41.,   62.,   16.],       [   2.,   77.,   17.],       [ -41.,   68.,   18.],       [ -63.,   40.,   17.],       [ -73.,    2.,   13.],       [ -62.,  -38.,   15.],       [ -40.,  -66.,   21.],       [  -2.,  -72.,   17.],       [  40.,  -66.,   15.],       [  62.,  -37.,   14.],       [  80.,   -4.,   18.],       [  71.,   42.,   19.],       [  44.,   74.,   20.],       [   3.,   83.,   19.],       [ -46.,   70.,   21.],       [ -71.,   44.,   17.],       [ -81.,   -5.,   20.],       [ -71.,  -46.,   19.],       [ -38.,  -71.,   12.],       [   1.,  -86.,   21.],       [  37.,  -74.,   17.],       [  74.,  -40.,   21.],       [  92.,   -1.,   18.],       [  83.,   45.,   18.],       [  48.,   79.,   15.],       [  -5.,   91.,   14.],       [ -46.,   77.,   20.],       [ -77.,   42.,   21.],       [ -90.,    4.,   20.],       [ -83.,  -50.,   19.],       [ -48.,  -80.,   21.],       [   0.,  -95.,   20.],       [  46.,  -75.,   16.],       [  82.,  -47.,   18.],       [  98.,    3.,   13.],       [  84.,   51.,   18.],       [  49.,   89.,   18.],       [   2.,  101.,   16.],       [ -46.,   85.,   17.],       [ -85.,   47.,   20.],       [-102.,    4.,   19.],       [ -90.,  -47.,   15.],       [ -47.,  -84.,   19.],       [  -4., -100.,   16.],       [  45.,  -92.,   13.],       [  84.,  -55.,   20.],       [  49.,    0.,   23.],       [  47.,   21.,   22.],       [  22.,   47.,   22.],       [   3.,   54.,   20.],       [ -29.,   44.,   25.],       [ -42.,   22.,   19.],       [ -55.,    2.,   26.],       [ -42.,  -29.,   23.],       [ -29.,  -44.,   19.],       [  -2.,  -49.,   20.],       [  25.,  -45.,   17.],       [  38.,  -30.,   25.],       [  56.,   -2.,   19.],       [  46.,   31.,   21.],       [  33.,   47.,   18.],       [  -2.,   63.,   24.],       [ -30.,   54.,   23.],       [ -46.,   25.,   26.],       [ -59.,   -5.,   25.],       [ -49.,  -34.,   23.],       [ -31.,  -47.,   26.],       [   0.,  -59.,   18.],       [  27.,  -48.,   18.],       [  51.,  -32.,   21.],       [  69.,    3.,   26.],       [  56.,   37.,   19.],       [  37.,   62.,   26.],       [   1.,   70.,   26.],       [ -38.,   56.,   24.],       [ -60.,   34.,   18.],       [ -70.,   -3.,   23.],       [ -57.,  -31.,   25.],       [ -37.,  -61.,   20.],       [   1.,  -69.,   20.],       [  31.,  -60.,   20.],       [  60.,  -30.,   24.],       [  73.,    0.,   18.],       [  63.,   42.,   18.],       [  37.,   69.,   25.],       [   3.,   71.,   23.],       [ -42.,   67.,   17.],       [ -62.,   40.,   25.],       [ -77.,    0.,   23.],       [ -70.,  -35.,   21.],       [ -37.,  -69.,   19.],       [   2.,  -71.,   25.],       [  40.,  -67.,   25.],       [  63.,  -34.,   22.],       [  85.,    1.,   26.],       [  71.,   45.,   23.],       [  36.,   70.,   18.],       [   2.,   85.,   22.],       [ -39.,   73.,   24.],       [ -77.,   37.,   24.],       [ -84.,   -4.,   22.],       [ -77.,  -38.,   19.],       [ -45.,  -69.,   24.],       [  -1.,  -79.,   23.],       [  38.,  -72.,   17.],       [  68.,  -45.,   18.],       [  89.,   -3.,   18.],       [  79.,   46.,   19.],       [  45.,   78.,   20.],       [   3.,   93.,   21.],       [ -43.,   79.,   20.],       [ -79.,   41.,   22.],       [ -89.,    0.,   24.],       [ -75.,  -44.,   18.],       [ -43.,  -77.,   21.],       [   4.,  -90.,   19.],       [  44.,  -78.,   22.],       [  80.,  -45.,   20.],       [ 104.,    2.,   23.],       [  87.,   46.,   25.],       [  48.,   85.,   26.],       [   4.,  104.,   17.],       [ -52.,   86.,   24.],       [ -88.,   51.,   18.],       [-102.,   -4.,   25.],       [ -86.,  -46.,   22.],       [ -52.,  -92.,   25.],       [  -5.,  -97.,   22.],       [  54.,  -86.,   24.],       [  91.,  -54.,   22.],       [  45.,   -4.,   26.],       [  47.,   28.,   27.],       [  23.,   40.,   27.],       [  -4.,   49.,   21.],       [ -30.,   43.,   24.],       [ -45.,   21.,   26.],       [ -52.,   -2.,   24.],       [ -39.,  -30.,   28.],       [ -30.,  -44.,   30.],       [   2.,  -50.,   31.],       [  27.,  -40.,   31.],       [  39.,  -24.,   24.],       [  54.,   -4.,   27.],       [  48.,   29.,   26.],       [  25.,   55.,   26.],       [  -3.,   55.,   23.],       [ -26.,   54.,   26.],       [ -53.,   33.,   21.],       [ -55.,   -1.,   23.],       [ -46.,  -32.,   30.],       [ -31.,  -53.,   24.],       [  -3.,  -60.,   23.],       [  33.,  -47.,   28.],       [  46.,  -26.,   24.],       [  66.,    3.,   25.],       [  60.,   30.,   24.],       [  34.,   54.,   28.],       [  -1.,   64.,   28.],       [ -34.,   60.,   29.],       [ -55.,   29.,   31.],       [ -69.,   -4.,   29.],       [ -62.,  -37.,   22.],       [ -35.,  -55.,   26.],       [   1.,  -65.,   23.],       [  38.,  -59.,   22.],       [  54.,  -35.,   30.],       [  71.,   -5.,   26.],       [  65.,   37.,   28.],       [  40.,   69.,   28.],       [   1.,   76.,   30.],       [ -36.,   63.,   28.],       [ -62.,   39.,   26.],       [ -75.,    4.,   28.],       [ -65.,  -38.,   29.],       [ -35.,  -61.,   27.],       [  -4.,  -73.,   22.],       [  33.,  -70.,   28.],       [  64.,  -36.,   25.],       [  87.,   -4.,   29.],       [  76.,   37.,   22.],       [  41.,   72.,   22.],       [   2.,   79.,   29.],       [ -38.,   73.,   28.],       [ -77.,   43.,   29.],       [ -87.,    0.,   24.],       [ -73.,  -46.,   26.],       [ -44.,  -76.,   22.],       [  -2.,  -84.,   28.],       [  41.,  -77.,   25.],       [  67.,  -47.,   31.],       [  90.,    0.,   28.],       [  80.,   50.,   28.],       [  45.,   81.,   30.],       [  -4.,   89.,   30.],       [ -49.,   79.,   28.],       [ -76.,   46.,   30.],       [ -94.,    1.,   23.],       [ -82.,  -47.,   22.],       [ -48.,  -85.,   22.],       [  -3.,  -95.,   24.],       [  47.,  -85.,   28.],       [  77.,  -48.,   24.],       [  98.,   -4.,   21.],       [  90.,   45.,   29.],       [  51.,   88.,   29.],       [  -3.,   97.,   27.],       [ -52.,   87.,   28.],       [ -91.,   45.,   22.],       [-104.,   -3.,   29.],       [ -92.,  -47.,   30.],       [ -46.,  -83.,   26.],       [  -3., -102.,   26.],       [  50.,  -90.,   26.],       [  86.,  -49.,   27.],       [  54.,    0.,   35.],       [  42.,   24.,   27.],       [  26.,   44.,   35.],       [  -5.,   47.,   30.],       [ -24.,   47.,   33.],       [ -41.,   29.,   31.],       [ -47.,    2.,   34.],       [ -45.,  -21.,   26.],       [ -29.,  -45.,   26.],       [   0.,  -49.,   34.],       [  27.,  -49.,   28.],       [  44.,  -24.,   28.],       [  56.,    3.,   28.],       [  47.,   31.,   34.],       [  32.,   45.,   28.],       [  -3.,   62.,   30.],       [ -32.,   51.,   30.],       [ -48.,   28.,   27.],       [ -56.,    4.,   34.],       [ -48.,  -32.,   31.],       [ -33.,  -49.,   27.],       [  -3.,  -58.,   26.],       [  33.,  -55.,   33.],       [  54.,  -27.,   31.],       [  70.,    3.,   35.],       [  59.,   34.,   30.],       [  30.,   53.,   31.],       [  -2.,   68.,   32.],       [ -33.,   57.,   34.],       [ -54.,   35.,   28.],       [ -63.,   -3.,   26.],       [ -57.,  -38.,   28.],       [ -37.,  -58.,   28.],       [   0.,  -67.,   27.],       [  37.,  -57.,   26.],       [  54.,  -35.,   31.],       [  70.,   -2.,   36.],       [  67.,   41.,   27.],       [  33.,   61.,   27.],       [   2.,   74.,   32.],       [ -41.,   62.,   29.],       [ -61.,   37.,   33.],       [ -79.,    4.,   31.],       [ -64.,  -33.,   32.],       [ -37.,  -70.,   34.],       [  -3.,  -77.,   33.],       [  39.,  -65.,   31.],       [  62.,  -36.,   34.],       [  84.,   -2.,   33.],       [  71.,   39.,   31.],       [  45.,   71.,   33.],       [   0.,   83.,   33.],       [ -44.,   71.,   32.],       [ -71.,   45.,   35.],       [ -82.,    3.,   35.],       [ -77.,  -46.,   36.],       [ -41.,  -72.,   27.],       [  -1.,  -88.,   32.],       [  45.,  -75.,   30.],       [  74.,  -39.,   32.],       [  95.,   -2.,   27.],       [  83.,   46.,   30.],       [  47.,   75.,   31.],       [  -2.,   87.,   33.],       [ -45.,   78.,   29.],       [ -76.,   47.,   34.],       [ -91.,    1.,   26.],       [ -79.,  -43.,   33.],       [ -50.,  -79.,   27.],       [  -3.,  -92.,   28.],       [  49.,  -81.,   29.],       [  76.,  -51.,   29.],       [  99.,    3.,   33.],       [  81.,   54.,   26.],       [  52.,   83.,   27.],       [  -5.,   95.,   26.],       [ -53.,   91.,   26.],       [ -86.,   48.,   28.],       [ -96.,    1.,   26.],       [ -91.,  -50.,   36.],       [ -51.,  -91.,   32.],       [   4., -104.,   28.],       [  49.,  -85.,   32.],       [  90.,  -52.,   32.],       [  53.,    3.,   37.],       [  42.,   23.,   33.],       [  20.,   42.,   31.],       [   2.,   53.,   37.],       [ -28.,   40.,   39.],       [ -44.,   24.,   37.],       [ -54.,    0.,   35.],       [ -41.,  -28.,   36.],       [ -27.,  -40.,   32.],       [  -3.,  -52.,   36.],       [  29.,  -40.,   31.],       [  47.,  -27.,   31.],       [  56.,    1.,   34.],       [  50.,   26.,   32.],       [  32.,   50.,   33.],       [  -3.,   60.,   40.],       [ -33.,   53.,   37.],       [ -46.,   28.,   39.],       [ -59.,   -1.,   35.],       [ -49.,  -31.,   35.],       [ -27.,  -52.,   40.],       [   4.,  -62.,   39.],       [  33.,  -48.,   36.],       [  45.,  -33.,   35.],       [  65.,   -4.,   31.],       [  58.,   36.,   34.],       [  29.,   61.,   37.],       [   2.,   69.,   33.],       [ -35.,   60.,   34.],       [ -58.,   36.,   33.],       [ -63.,   -2.,   38.],       [ -54.,  -35.,   36.],       [ -32.,  -63.,   33.],       [  -1.,  -65.,   37.],       [  30.,  -57.,   38.],       [  53.,  -39.,   34.],       [  76.,    0.,   39.],       [  60.,   41.,   39.],       [  41.,   61.,   36.],       [   4.,   77.,   33.],       [ -39.,   63.,   32.],       [ -63.,   41.,   36.],       [ -78.,   -5.,   30.],       [ -66.,  -40.,   35.],       [ -43.,  -66.,   36.],       [   1.,  -71.,   32.],       [  35.,  -65.,   36.],       [  66.,  -43.,   34.],       [  83.,    2.,   33.],       [  76.,   42.,   36.],       [  40.,   69.,   35.],       [  -5.,   78.,   37.],       [ -45.,   69.,   38.],       [ -73.,   40.,   34.],       [ -88.,   -2.,   31.],       [ -71.,  -41.,   38.],       [ -46.,  -73.,   35.],       [  -5.,  -85.,   37.],       [  46.,  -70.,   31.],       [  74.,  -44.,   31.],       [  95.,   -1.,   32.],       [  78.,   41.,   35.],       [  49.,   74.,   33.],       [  -2.,   89.,   40.],       [ -44.,   80.,   33.],       [ -82.,   49.,   36.],       [ -92.,    3.,   39.],       [ -82.,  -51.,   34.],       [ -49.,  -81.,   36.],       [   1.,  -94.,   40.],       [  42.,  -84.,   32.],       [  84.,  -48.,   33.],       [  98.,    3.,   34.],       [  85.,   48.,   40.],       [  48.,   84.,   32.],       [   4.,  102.,   31.],       [ -50.,   91.,   32.],       [ -83.,   46.,   40.],       [-100.,    0.,   36.],       [ -87.,  -53.,   34.],       [ -46.,  -92.,   39.],       [  -1., -101.,   33.],       [  45.,  -87.,   33.],       [  84.,  -50.,   34.],       [  45.,   -1.,   38.],       [  43.,   26.,   43.],       [  21.,   44.,   39.],       [   0.,   54.,   43.],       [ -22.,   40.,   40.],       [ -45.,   27.,   43.],       [ -48.,    3.,   40.],       [ -41.,  -25.,   41.],       [ -24.,  -42.,   39.],       [  -1.,  -53.,   44.],       [  28.,  -43.,   39.],       [  42.,  -23.,   38.],       [  56.,   -5.,   40.],       [  50.,   28.,   41.],       [  33.,   46.,   42.],       [   2.,   58.,   37.],       [ -33.,   48.,   36.],       [ -55.,   24.,   40.],       [ -57.,    3.,   37.],       [ -49.,  -34.,   36.],       [ -34.,  -50.,   38.],       [   0.,  -61.,   38.],       [  33.,  -52.,   41.],       [  47.,  -29.,   38.],       [  71.,    2.,   39.],       [  61.,   36.,   37.],       [  28.,   60.,   40.],       [  -1.,   71.,   35.],       [ -38.,   58.,   42.],       [ -55.,   32.,   39.],       [ -64.,   -1.,   42.],       [ -62.,  -31.,   43.],       [ -38.,  -60.,   40.],       [   3.,  -69.,   43.],       [  29.,  -62.,   39.],       [  61.,  -38.,   41.],       [  74.,    3.,   43.],       [  62.,   38.,   42.],       [  40.,   63.,   39.],       [  -3.,   76.,   44.],       [ -37.,   67.,   36.],       [ -68.,   36.,   35.],       [ -73.,    3.,   37.],       [ -68.,  -42.,   42.],       [ -40.,  -69.,   39.],       [  -4.,  -72.,   37.],       [  39.,  -70.,   43.],       [  68.,  -37.,   44.],       [  86.,    3.,   37.],       [  76.,   39.,   37.],       [  40.,   67.,   41.],       [   0.,   81.,   40.],       [ -38.,   73.,   37.],       [ -75.,   40.,   39.],       [ -80.,   -3.,   41.],       [ -75.,  -41.,   40.],       [ -38.,  -70.,   37.],       [   2.,  -83.,   43.],       [  39.,  -71.,   36.],       [  68.,  -42.,   44.],       [  96.,   -4.,   39.],       [  81.,   46.,   37.],       [  50.,   74.,   44.],       [  -3.,   86.,   43.],       [ -50.,   78.,   43.],       [ -82.,   41.,   38.],       [ -93.,   -3.,   41.],       [ -82.,  -44.,   39.],       [ -51.,  -84.,   44.],       [   0.,  -95.,   35.],       [  46.,  -81.,   41.],       [  84.,  -48.,   37.],       [ 100.,   -5.,   42.],       [  91.,   49.,   41.],       [  46.,   86.,   39.],       [  -1.,  102.,   40.],       [ -50.,   87.,   41.],       [ -88.,   51.,   40.],       [-101.,    4.,   41.],       [ -83.,  -50.,   41.],       [ -51.,  -87.,   38.],       [  -5.,  -98.,   36.],       [  45.,  -85.,   35.],       [  86.,  -54.,   41.]])
        basis0 = BSplineBasis(5, np.array([ -1.9,  -0.8,   0. ,   0. ,   0. ,   1.3,   2.3,   2.9,   4.1,   4.8,   6.1,   6.6,   8.1,
         9.2,  10. ,  10. ,  10. ,  11.3,  12.3]),1)
        basis1 = BSplineBasis(6, np.array([ 0.,  0.,  0.,  0.,  0.,  0.,  1.,  2.,  2.,  2.,  2.,  2.,  2.]))
        basis2 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  2.1,  3.3,  3.7,  5. ,  5. ,  5. ,  5. ,  5. ,  5. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 3, 2)
        self.assertEqual(vol2.order(direction=0), 6)
        self.assertEqual(vol2.order(direction=1), 9)
        self.assertEqual(vol2.order(direction=2), 8)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_curve_3D_p6_C2_periodic(self):
        controlpoints = np.array([[ 102.,   -1.,   -2.],
       [  88.,   33.,   -3.],
       [  67.,   73.,    4.],
       [  39.,   95.,   -5.],
       [   0.,   97.,    4.],
       [ -40.,   91.,   -5.],
       [ -69.,   74.,   -1.],
       [ -96.,   36.,    0.],
       [ -98.,   -4.,   -4.],
       [ -91.,  -39.,    1.],
       [ -74.,  -71.,   -3.],
       [ -42.,  -92.,    3.],
       [  -5.,  -96.,   -5.],
       [  38.,  -88.,    4.],
       [  71.,  -72.,    3.],
       [  87.,  -40.,   -5.]])
        basis0 = BSplineBasis(6, np.array([ -2.7,  -2.2,  -1.2,   0. ,   0. ,   0. ,   1.1,   1.6,   2.7,   3.9,   4.8,   6.3,   6.9,
         8.3,   9.2,   9.8,  11.3,  11.8,  12.8,  14. ,  14. ,  14. ,  15.1,  15.6,  16.7]),2)
        crv  = Curve(basis0, controlpoints,False)
        crv2 = crv.clone()
        crv2.raise_order(2)
        self.assertEqual(crv2.order(direction=0), 8)

        u    = np.linspace(crv.start(0), crv.end(0), 13)
        pt   = crv(u)
        pt2  = crv2(u)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_surface_3D_p66_C2_periodic(self):
        controlpoints = np.array([[  57.,    1.,   -5.],       [  50.,   18.,    0.],       [  40.,   41.,    2.],       [  27.,   51.,   -3.],       [  -5.,   60.,    1.],       [ -23.,   52.,   -1.],       [ -38.,   45.,    0.],       [ -57.,   18.,    0.],       [ -60.,   -1.,   -4.],       [ -59.,  -26.,   -5.],       [ -39.,  -38.,    4.],       [ -22.,  -56.,   -1.],       [  -4.,  -56.,    0.],       [  25.,  -60.,   -3.],       [  40.,  -40.,   -2.],       [  56.,  -19.,   -5.],       [  65.,    2.,    4.],       [  60.,   29.,   -3.],       [  48.,   44.,    0.],       [  22.,   58.,   -4.],       [   0.,   68.,    0.],       [ -28.,   56.,   -3.],       [ -51.,   47.,    3.],       [ -62.,   25.,    2.],       [ -67.,   -2.,    4.],       [ -64.,  -25.,   -2.],       [ -43.,  -42.,    0.],       [ -21.,  -58.,    1.],       [  -3.,  -69.,   -2.],       [  29.,  -62.,    0.],       [  41.,  -51.,    0.],       [  63.,  -25.,   -1.],       [  71.,    0.,   -2.],       [  68.,   21.,    3.],       [  47.,   53.,    4.],       [  27.,   64.,    2.],       [  -1.,   69.,    2.],       [ -29.,   59.,    4.],       [ -49.,   51.,   -4.],       [ -70.,   31.,    1.],       [ -66.,    0.,   -4.],       [ -66.,  -32.,   -4.],       [ -46.,  -53.,    1.],       [ -30.,  -62.,    1.],       [   2.,  -68.,   -2.],       [  29.,  -63.,   -3.],       [  52.,  -46.,   -2.],       [  61.,  -26.,   -2.],       [  75.,   -3.,    2.],       [  64.,   24.,    4.],       [  48.,   51.,   -5.],       [  31.,   66.,    0.],       [  -5.,   76.,    4.],       [ -32.,   70.,   -2.],       [ -53.,   52.,   -2.],       [ -74.,   27.,    0.],       [ -75.,   -1.,   -2.],       [ -71.,  -29.,   -5.],       [ -57.,  -49.,   -2.],       [ -31.,  -73.,   -1.],       [   1.,  -72.,    2.],       [  29.,  -74.,   -5.],       [  53.,  -55.,   -3.],       [  66.,  -28.,   -1.],       [  80.,   -2.,    1.],       [  78.,   25.,   -5.],       [  56.,   56.,    1.],       [  29.,   72.,    1.],       [  -1.,   75.,    3.],       [ -28.,   72.,    1.],       [ -54.,   57.,    1.],       [ -76.,   35.,   -5.],       [ -76.,   -5.,    4.],       [ -74.,  -27.,   -5.],       [ -56.,  -62.,   -5.],       [ -32.,  -74.,   -1.],       [  -2.,  -84.,    3.],       [  32.,  -72.,    1.],       [  60.,  -59.,   -4.],       [  73.,  -30.,   -1.],       [  85.,   -2.,    1.],       [  78.,   35.,    1.],       [  61.,   61.,   -3.],       [  33.,   83.,   -1.],       [  -1.,   80.,   -2.],       [ -30.,   82.,   -1.],       [ -61.,   56.,   -1.],       [ -75.,   31.,    3.],       [ -83.,    3.,    0.],       [ -79.,  -34.,    1.],       [ -62.,  -65.,   -4.],       [ -35.,  -83.,   -5.],       [   4.,  -82.,   -1.],       [  34.,  -74.,   -1.],       [  60.,  -60.,   -4.],       [  75.,  -29.,   -5.],       [  86.,    0.,    0.],       [  79.,   30.,    1.],       [  60.,   67.,   -5.],       [  38.,   79.,   -3.],       [   1.,   93.,   -1.],       [ -35.,   86.,    3.],       [ -67.,   59.,   -5.],       [ -83.,   31.,    2.],       [ -90.,   -1.,   -1.],       [ -82.,  -36.,   -2.],       [ -65.,  -65.,    2.],       [ -34.,  -86.,    4.],       [   3.,  -94.,   -1.],       [  36.,  -89.,    3.],       [  65.,  -67.,    0.],       [  87.,  -35.,    4.],       [  99.,   -3.,    0.],       [  92.,   38.,    0.],       [  62.,   64.,    0.],       [  34.,   84.,    0.],       [   4.,   91.,    3.],       [ -41.,   84.,   -3.],       [ -67.,   71.,   -5.],       [ -89.,   38.,   -4.],       [ -99.,   -5.,   -3.],       [ -89.,  -38.,   -1.],       [ -72.,  -71.,    3.],       [ -36.,  -84.,    2.],       [   2., -100.,   -5.],       [  34.,  -86.,   -2.],       [  66.,  -67.,    2.],       [  90.,  -37.,    4.],       [  98.,    4.,    1.],       [  95.,   38.,    1.],       [  74.,   72.,   -4.],       [  37.,   87.,    2.],       [   0.,  102.,   -1.],       [ -39.,   92.,    0.],       [ -68.,   66.,   -2.],       [ -95.,   35.,   -2.],       [-100.,   -2.,   -3.],       [ -98.,  -36.,    3.],       [ -76.,  -69.,   -5.],       [ -40.,  -93.,   -1.],       [   3., -100.,   -5.],       [  40.,  -94.,   -1.],       [  74.,  -70.,   -4.],       [  87.,  -38.,    1.]])
        basis0 = BSplineBasis(6, np.array([ -2.8,  -2. ,  -0.9,   0. ,   0. ,   0. ,   1.2,   2.3,   3.3,   4. ,   4.8,   6.3,   6.9,
         8.1,   8.6,  10. ,  11.2,  12. ,  13.1,  14. ,  14. ,  14. ,  15.2,  16.3,  17.3]),2)
        basis1 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.4,  1.8,  2.8,  4. ,  4. ,  4. ,  4. ,  4. ,  4. ]))
        surf  = Surface(basis0, basis1, controlpoints,False)
        surf2 = surf.clone()
        surf2.raise_order(1, 1)
        self.assertEqual(surf2.order(direction=0), 7)
        self.assertEqual(surf2.order(direction=1), 7)

        u    = np.linspace(surf.start(0), surf.end(0), 9)
        v    = np.linspace(surf.start(1), surf.end(1), 9)
        pt   = surf(u,v)
        pt2  = surf2(u,v)

        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)

    def test_volume_3D_p556_C2_periodic(self):
        controlpoints = np.array([[  51.,    4.,    4.],       [  45.,   20.,    4.],       [  26.,   37.,   -2.],       [  11.,   45.,    2.],       [ -15.,   53.,   -2.],       [ -29.,   37.,    2.],       [ -46.,   19.,   -1.],       [ -49.,   -4.,    1.],       [ -43.,  -22.,    2.],       [ -32.,  -37.,    1.],       [ -16.,  -50.,    4.],       [   6.,  -53.,   -2.],       [  35.,  -39.,   -5.],       [  41.,  -23.,   -4.],       [  55.,   -2.,   -2.],       [  56.,   26.,    3.],       [  33.,   44.,    2.],       [  17.,   58.,    4.],       [ -13.,   57.,    3.],       [ -35.,   44.,    4.],       [ -51.,   25.,    2.],       [ -59.,    3.,    2.],       [ -50.,  -26.,    4.],       [ -38.,  -45.,    1.],       [ -11.,  -54.,    2.],       [  15.,  -58.,   -3.],       [  37.,  -44.,    4.],       [  54.,  -29.,   -5.],       [  63.,   -1.,    2.],       [  61.,   26.,    1.],       [  39.,   53.,   -5.],       [  12.,   60.,   -1.],       [ -12.,   66.,   -4.],       [ -39.,   49.,   -4.],       [ -61.,   28.,    4.],       [ -68.,   -5.,   -4.],       [ -58.,  -30.,   -5.],       [ -42.,  -53.,   -3.],       [ -10.,  -61.,    4.],       [  11.,  -61.,    2.],       [  37.,  -46.,    3.],       [  56.,  -24.,   -5.],       [  74.,   -2.,   -1.],       [  60.,   28.,   -4.],       [  41.,   56.,   -5.],       [  15.,   74.,    4.],       [ -15.,   72.,    1.],       [ -48.,   59.,    1.],       [ -63.,   27.,   -4.],       [ -73.,   -2.,    0.],       [ -68.,  -32.,    1.],       [ -49.,  -53.,    0.],       [ -19.,  -71.,    4.],       [  16.,  -73.,    1.],       [  46.,  -55.,    0.],       [  66.,  -32.,   -5.],       [  77.,    3.,   -1.],       [  75.,   33.,   -5.],       [  48.,   58.,    1.],       [  21.,   77.,   -5.],       [ -14.,   77.,    4.],       [ -51.,   58.,   -4.],       [ -69.,   33.,   -2.],       [ -83.,    2.,   -1.],       [ -73.,  -36.,    1.],       [ -52.,  -64.,   -1.],       [ -23.,  -74.,    1.],       [  20.,  -80.,    0.],       [  49.,  -59.,    0.],       [  75.,  -35.,    2.],       [  83.,   -2.,   -3.],       [  82.,   34.,   -1.],       [  49.,   68.,   -1.],       [  20.,   85.,   -4.],       [ -16.,   88.,   -4.],       [ -54.,   71.,    4.],       [ -76.,   38.,   -5.],       [ -88.,    4.,   -5.],       [ -79.,  -36.,   -2.],       [ -53.,  -65.,   -2.],       [ -20.,  -83.,   -4.],       [  15.,  -88.,   -3.],       [  50.,  -71.,    2.],       [  76.,  -42.,   -5.],       [  93.,    1.,   -2.],       [  83.,   43.,   -2.],       [  61.,   71.,    2.],       [  17.,   94.,    1.],       [ -21.,   91.,    2.],       [ -57.,   70.,    2.],       [ -79.,   36.,   -1.],       [ -95.,    1.,   -1.],       [ -81.,  -40.,   -1.],       [ -61.,  -75.,   -4.],       [ -17.,  -89.,   -5.],       [  16.,  -93.,   -2.],       [  59.,  -71.,   -5.],       [  85.,  -43.,   -1.],       [ 101.,   -3.,    3.],       [  89.,   44.,    1.],       [  67.,   75.,   -3.],       [  27.,  102.,   -1.],       [ -18.,   93.,   -4.],       [ -60.,   73.,   -2.],       [ -86.,   39.,   -4.],       [-105.,    2.,    0.],       [ -91.,  -41.,    4.],       [ -63.,  -79.,    4.],       [ -19., -100.,   -2.],       [  19.,  -98.,    3.],       [  59.,  -81.,    1.],       [  90.,  -42.,    4.],       [  46.,    0.,    7.],       [  46.,   21.,    5.],       [  31.,   38.,    8.],       [   8.,   44.,    0.],       [  -7.,   51.,    8.],       [ -31.,   39.,    6.],       [ -43.,   20.,    6.],       [ -52.,    3.,    1.],       [ -44.,  -22.,    8.],       [ -33.,  -37.,    8.],       [ -10.,  -51.,    7.],       [  15.,  -49.,    9.],       [  29.,  -35.,    5.],       [  48.,  -17.,    5.],       [  59.,    1.,    4.],       [  55.,   26.,    2.],       [  37.,   44.,    3.],       [  13.,   54.,    0.],       [ -13.,   58.,    8.],       [ -31.,   45.,    6.],       [ -56.,   24.,    3.],       [ -63.,   -1.,    9.],       [ -55.,  -21.,    2.],       [ -40.,  -42.,    9.],       [ -16.,  -54.,    2.],       [   9.,  -51.,    5.],       [  31.,  -49.,    5.],       [  51.,  -27.,    3.],       [  60.,   -5.,    9.],       [  60.,   28.,    6.],       [  40.,   50.,    2.],       [  18.,   62.,    6.],       [ -20.,   61.,    1.],       [ -44.,   50.,    1.],       [ -57.,   26.,    2.],       [ -66.,    4.,    1.],       [ -57.,  -27.,    7.],       [ -40.,  -47.,    4.],       [ -13.,  -59.,    2.],       [  14.,  -60.,    0.],       [  39.,  -54.,    3.],       [  58.,  -24.,    9.],       [  70.,   -2.,    3.],       [  62.,   35.,    1.],       [  39.,   57.,    2.],       [  14.,   73.,    1.],       [ -20.,   72.,    5.],       [ -45.,   58.,    2.],       [ -63.,   35.,    6.],       [ -74.,    1.,    9.],       [ -69.,  -28.,    7.],       [ -41.,  -52.,    6.],       [ -20.,  -68.,    1.],       [  15.,  -67.,    7.],       [  40.,  -53.,    6.],       [  64.,  -31.,    3.],       [  74.,   -5.,    3.],       [  68.,   37.,    7.],       [  48.,   61.,    1.],       [  20.,   80.,    2.],       [ -14.,   80.,    2.],       [ -50.,   62.,    8.],       [ -75.,   34.,    8.],       [ -74.,   -4.,    5.],       [ -67.,  -36.,    3.],       [ -51.,  -63.,    3.],       [ -20.,  -81.,    0.],       [  22.,  -82.,    4.],       [  52.,  -61.,    1.],       [  72.,  -34.,    2.],       [  84.,   -2.,    4.],       [  76.,   40.,    3.],       [  50.,   69.,    7.],       [  14.,   83.,    2.],       [ -19.,   78.,    4.],       [ -55.,   63.,    7.],       [ -82.,   33.,    8.],       [ -84.,   -3.,    8.],       [ -81.,  -34.,    1.],       [ -54.,  -69.,    8.],       [ -23.,  -84.,    8.],       [  17.,  -79.,    0.],       [  51.,  -70.,    7.],       [  74.,  -38.,    7.],       [  87.,   -2.,    0.],       [  83.,   41.,    0.],       [  61.,   73.,    0.],       [  19.,   86.,    5.],       [ -22.,   93.,    1.],       [ -60.,   70.,    7.],       [ -83.,   44.,    1.],       [ -89.,    3.,    2.],       [ -85.,  -40.,    4.],       [ -56.,  -71.,    3.],       [ -26.,  -86.,    9.],       [  20.,  -92.,    4.],       [  53.,  -77.,    2.],       [  88.,  -44.,    7.],       [  97.,    3.,    6.],       [  86.,   45.,    8.],       [  65.,   77.,    9.],       [  20.,  101.,    3.],       [ -21.,   97.,    7.],       [ -67.,   74.,    5.],       [ -91.,   47.,    2.],       [-100.,    0.,    7.],       [ -90.,  -45.,    6.],       [ -60.,  -83.,    9.],       [ -18.,  -95.,    1.],       [  19.,  -93.,    2.],       [  64.,  -80.,    3.],       [  86.,  -40.,    4.],       [  48.,   -1.,   10.],       [  47.,   20.,   12.],       [  30.,   39.,    7.],       [  15.,   47.,    9.],       [  -8.,   50.,    7.],       [ -36.,   41.,    6.],       [ -49.,   17.,    9.],       [ -53.,    0.,    8.],       [ -47.,  -18.,    9.],       [ -27.,  -43.,   12.],       [ -13.,  -48.,    5.],       [  11.,  -51.,   14.],       [  27.,  -45.,    5.],       [  47.,  -18.,   13.],       [  54.,   -1.,   10.],       [  55.,   20.,   11.],       [  38.,   48.,   14.],       [  14.,   58.,    8.],       [ -12.,   52.,   10.],       [ -38.,   49.,    8.],       [ -54.,   27.,    5.],       [ -62.,   -5.,   12.],       [ -56.,  -27.,   10.],       [ -41.,  -42.,    6.],       [ -18.,  -61.,   10.],       [  13.,  -56.,    7.],       [  37.,  -41.,   14.],       [  52.,  -29.,    9.],       [  63.,   -2.,   12.],       [  61.,   30.,    8.],       [  36.,   48.,    5.],       [  10.,   61.,    7.],       [ -17.,   61.,   12.],       [ -41.,   53.,   12.],       [ -60.,   31.,   12.],       [ -65.,   -5.,   10.],       [ -59.,  -29.,    5.],       [ -38.,  -50.,   10.],       [ -17.,  -65.,    6.],       [  12.,  -63.,    7.],       [  39.,  -49.,   13.],       [  56.,  -33.,   14.],       [  75.,    2.,   11.],       [  63.,   33.,    8.],       [  45.,   53.,    9.],       [  12.,   65.,   13.],       [ -15.,   66.,   12.],       [ -43.,   50.,    9.],       [ -66.,   35.,    6.],       [ -73.,   -1.,    9.],       [ -67.,  -31.,    8.],       [ -41.,  -52.,   10.],       [ -20.,  -68.,   13.],       [  20.,  -65.,    8.],       [  41.,  -61.,    9.],       [  67.,  -35.,   12.],       [  73.,   -2.,   10.],       [  72.,   34.,    7.],       [  49.,   60.,    9.],       [  14.,   79.,    6.],       [ -22.,   77.,   13.],       [ -46.,   65.,    6.],       [ -72.,   30.,    8.],       [ -74.,    4.,   11.],       [ -74.,  -36.,   10.],       [ -51.,  -58.,   12.],       [ -23.,  -80.,   13.],       [  21.,  -79.,   12.],       [  45.,  -57.,    7.],       [  75.,  -39.,    8.],       [  87.,    4.,   14.],       [  81.,   34.,   11.],       [  58.,   69.,   13.],       [  20.,   79.,   11.],       [ -21.,   82.,    9.],       [ -51.,   68.,   14.],       [ -81.,   38.,    5.],       [ -86.,   -4.,    8.],       [ -82.,  -40.,   13.],       [ -57.,  -70.,   13.],       [ -15.,  -83.,    6.],       [  16.,  -85.,   11.],       [  58.,  -70.,    9.],       [  75.,  -40.,   12.],       [  91.,    2.,   11.],       [  80.,   37.,   13.],       [  60.,   67.,    9.],       [  21.,   87.,   14.],       [ -22.,   92.,    6.],       [ -61.,   71.,    9.],       [ -80.,   44.,    6.],       [ -94.,    1.,    9.],       [ -84.,  -40.,    8.],       [ -61.,  -76.,   12.],       [ -21.,  -93.,   10.],       [  16.,  -88.,    6.],       [  57.,  -71.,   14.],       [  86.,  -40.,    6.],       [  99.,   -2.,    8.],       [  95.,   40.,   12.],       [  63.,   81.,    8.],       [  25.,  100.,    5.],       [ -21.,   94.,    8.],       [ -59.,   76.,    7.],       [ -87.,   40.,   14.],       [-101.,    4.,   12.],       [ -86.,  -44.,   13.],       [ -61.,  -79.,    5.],       [ -20.,  -94.,   12.],       [  19.,  -95.,    9.],       [  60.,  -78.,   11.],       [  94.,  -42.,   14.],       [  49.,   -5.,   11.],       [  41.,   21.,   10.],       [  32.,   35.,   14.],       [   7.,   48.,   15.],       [ -10.,   49.,   12.],       [ -31.,   40.,   18.],       [ -47.,   20.,   19.],       [ -48.,    3.,   16.],       [ -50.,  -24.,   19.],       [ -31.,  -35.,   13.],       [ -12.,  -52.,   13.],       [  10.,  -52.,   19.],       [  29.,  -44.,   13.],       [  42.,  -20.,   17.],       [  53.,   -2.,   15.],       [  50.,   27.,   15.],       [  34.,   46.,   10.],       [  16.,   54.,   13.],       [ -15.,   59.,   19.],       [ -36.,   44.,   14.],       [ -52.,   21.,   16.],       [ -61.,   -2.,   16.],       [ -53.,  -24.,   17.],       [ -35.,  -49.,   19.],       [ -17.,  -56.,   12.],       [  14.,  -58.,   10.],       [  31.,  -46.,   13.],       [  46.,  -30.,   14.],       [  67.,   -2.,   15.],       [  56.,   30.,   11.],       [  40.,   52.,   16.],       [  17.,   59.,   19.],       [ -11.,   65.,   15.],       [ -41.,   45.,   16.],       [ -56.,   32.,   11.],       [ -68.,   -3.,   12.],       [ -57.,  -26.,   18.],       [ -42.,  -48.,   17.],       [ -17.,  -64.,   10.],       [  15.,  -66.,   18.],       [  42.,  -50.,   13.],       [  61.,  -23.,   12.],       [  76.,    1.,   17.],       [  62.,   31.,   15.],       [  43.,   54.,   11.],       [  14.,   70.,   12.],       [ -18.,   73.,   19.],       [ -45.,   55.,   17.],       [ -63.,   30.,   16.],       [ -76.,   -1.,   12.],       [ -63.,  -36.,   11.],       [ -46.,  -59.,   11.],       [ -21.,  -67.,   18.],       [  16.,  -70.,   19.],       [  43.,  -56.,   10.],       [  67.,  -32.,   11.],       [  74.,    4.,   14.],       [  69.,   31.,   13.],       [  49.,   66.,   12.],       [  18.,   78.,   19.],       [ -18.,   81.,   14.],       [ -48.,   65.,   16.],       [ -75.,   35.,   13.],       [ -80.,   -2.,   16.],       [ -73.,  -38.,   13.],       [ -49.,  -57.,   12.],       [ -21.,  -80.,   17.],       [  12.,  -75.,   19.],       [  47.,  -64.,   10.],       [  73.,  -39.,   18.],       [  82.,    2.,   16.],       [  79.,   40.,   10.],       [  52.,   62.,   10.],       [  21.,   83.,   13.],       [ -15.,   82.,   18.],       [ -53.,   70.,   12.],       [ -78.,   37.,   10.],       [ -89.,    1.,   15.],       [ -81.,  -42.,   13.],       [ -58.,  -69.,   18.],       [ -18.,  -86.,   11.],       [  18.,  -85.,   18.],       [  52.,  -66.,   13.],       [  81.,  -41.,   14.],       [  88.,    1.,   12.],       [  88.,   39.,   16.],       [  55.,   69.,   10.],       [  22.,   85.,   13.],       [ -21.,   86.,   11.],       [ -59.,   75.,   14.],       [ -82.,   35.,   14.],       [ -97.,    2.,   11.],       [ -86.,  -41.,   18.],       [ -60.,  -71.,   18.],       [ -21.,  -95.,   16.],       [  20.,  -88.,   10.],       [  53.,  -71.,   11.],       [  85.,  -42.,   16.],       [  96.,   -1.,   11.],       [  89.,   46.,   14.],       [  66.,   79.,   12.],       [  19.,   96.,   11.],       [ -19.,  100.,   10.],       [ -60.,   79.,   10.],       [ -86.,   47.,   16.],       [-103.,    0.,   12.],       [ -95.,  -41.,   14.],       [ -64.,  -84.,   12.],       [ -22.,  -99.,   16.],       [  25., -101.,   17.],       [  60.,  -75.,   18.],       [  92.,  -44.,   13.],       [  48.,   -3.,   15.],       [  41.,   25.,   22.],       [  26.,   37.,   23.],       [  10.,   46.,   22.],       [ -10.,   44.,   18.],       [ -28.,   34.,   23.],       [ -42.,   20.,   21.],       [ -46.,    0.,   20.],       [ -42.,  -22.,   16.],       [ -34.,  -44.,   22.],       [ -11.,  -49.,   21.],       [   6.,  -45.,   20.],       [  30.,  -37.,   24.],       [  46.,  -22.,   20.],       [  61.,    2.,   23.],       [  48.,   23.,   20.],       [  34.,   41.,   17.],       [  12.,   53.,   16.],       [ -10.,   60.,   23.],       [ -36.,   39.,   18.],       [ -51.,   29.,   17.],       [ -62.,    4.,   23.],       [ -54.,  -24.,   23.],       [ -38.,  -46.,   17.],       [ -17.,  -59.,   17.],       [  11.,  -61.,   23.],       [  37.,  -48.,   24.],       [  50.,  -22.,   23.],       [  61.,    4.,   20.],       [  62.,   26.,   16.],       [  37.,   48.,   16.],       [  13.,   66.,   23.],       [ -15.,   60.,   24.],       [ -36.,   52.,   18.],       [ -55.,   25.,   15.],       [ -69.,   -4.,   23.],       [ -59.,  -29.,   20.],       [ -42.,  -47.,   21.],       [ -12.,  -61.,   17.],       [  14.,  -65.,   15.],       [  39.,  -53.,   16.],       [  62.,  -28.,   15.],       [  73.,   -2.,   23.],       [  60.,   33.,   20.],       [  44.,   54.,   16.],       [  16.,   72.,   16.],       [ -19.,   69.,   19.],       [ -45.,   53.,   20.],       [ -61.,   31.,   15.],       [ -71.,   -1.,   17.],       [ -69.,  -30.,   18.],       [ -44.,  -53.,   16.],       [ -20.,  -73.,   16.],       [  14.,  -70.,   18.],       [  44.,  -59.,   16.],       [  69.,  -30.,   22.],       [  73.,   -4.,   19.],       [  73.,   36.,   17.],       [  49.,   64.,   24.],       [  18.,   73.,   23.],       [ -17.,   77.,   17.],       [ -45.,   62.,   17.],       [ -70.,   36.,   16.],       [ -83.,   -5.,   22.],       [ -74.,  -36.,   24.],       [ -48.,  -61.,   22.],       [ -15.,  -75.,   21.],       [  15.,  -78.,   17.],       [  51.,  -62.,   20.],       [  73.,  -31.,   20.],       [  86.,   -4.,   17.],       [  73.,   33.,   24.],       [  57.,   67.,   24.],       [  23.,   85.,   24.],       [ -25.,   85.,   15.],       [ -53.,   63.,   24.],       [ -74.,   35.,   18.],       [ -89.,   -1.,   21.],       [ -74.,  -39.,   21.],       [ -53.,  -68.,   18.],       [ -15.,  -86.,   21.],       [  24.,  -86.,   20.],       [  55.,  -66.,   15.],       [  78.,  -37.,   22.],       [  97.,   -1.,   24.],       [  86.,   39.,   16.],       [  61.,   73.,   19.],       [  15.,   87.,   18.],       [ -22.,   92.,   15.],       [ -62.,   72.,   16.],       [ -84.,   43.,   19.],       [ -97.,    1.,   22.],       [ -81.,  -41.,   24.],       [ -57.,  -78.,   19.],       [ -23.,  -88.,   17.],       [  19.,  -88.,   24.],       [  54.,  -69.,   17.],       [  84.,  -38.,   19.],       [  96.,    1.,   24.],       [  85.,   44.,   22.],       [  63.,   82.,   20.],       [  27.,   93.,   21.],       [ -22.,   97.,   19.],       [ -61.,   77.,   19.],       [ -89.,   39.,   15.],       [-105.,   -2.,   22.],       [ -90.,  -46.,   22.],       [ -58.,  -74.,   20.],       [ -21.,  -95.,   19.],       [  18.,  -99.,   16.],       [  64.,  -78.,   20.],       [  88.,  -45.,   15.],       [  51.,   -1.,   27.],       [  49.,   22.,   20.],       [  27.,   38.,   28.],       [  15.,   45.,   21.],       [  -7.,   49.,   21.],       [ -35.,   44.,   22.],       [ -47.,   21.,   27.],       [ -55.,   -1.,   25.],       [ -48.,  -20.,   23.],       [ -33.,  -41.,   25.],       [ -10.,  -46.,   20.],       [   8.,  -45.,   26.],       [  35.,  -38.,   25.],       [  49.,  -19.,   24.],       [  60.,   -2.,   22.],       [  49.,   20.,   22.],       [  31.,   43.,   26.],       [   9.,   50.,   29.],       [  -9.,   52.,   24.],       [ -35.,   44.,   28.],       [ -51.,   21.,   29.],       [ -62.,   -3.,   27.],       [ -52.,  -30.,   22.],       [ -36.,  -44.,   20.],       [ -12.,  -56.,   22.],       [  10.,  -53.,   28.],       [  39.,  -49.,   21.],       [  56.,  -26.,   21.],       [  61.,   -1.,   21.],       [  58.,   24.,   26.],       [  37.,   55.,   20.],       [   9.,   63.,   29.],       [ -10.,   60.,   29.],       [ -39.,   47.,   26.],       [ -61.,   28.,   29.],       [ -62.,    1.,   27.],       [ -56.,  -27.,   23.],       [ -36.,  -51.,   22.],       [ -11.,  -66.,   23.],       [  10.,  -59.,   25.],       [  38.,  -56.,   20.],       [  52.,  -25.,   28.],       [  66.,    1.,   27.],       [  62.,   34.,   22.],       [  40.,   57.,   26.],       [  13.,   65.,   22.],       [ -16.,   68.,   29.],       [ -43.,   53.,   26.],       [ -63.,   33.,   27.],       [ -73.,    4.,   27.],       [ -62.,  -28.,   28.],       [ -46.,  -60.,   20.],       [ -12.,  -75.,   26.],       [  19.,  -74.,   29.],       [  40.,  -57.,   25.],       [  64.,  -34.,   24.],       [  82.,   -5.,   20.],       [  71.,   32.,   29.],       [  50.,   63.,   21.],       [  15.,   76.,   25.],       [ -19.,   79.,   21.],       [ -47.,   56.,   24.],       [ -70.,   37.,   26.],       [ -80.,   -2.,   23.],       [ -68.,  -37.,   22.],       [ -51.,  -60.,   26.],       [ -16.,  -81.,   26.],       [  20.,  -73.,   26.],       [  45.,  -62.,   26.],       [  71.,  -30.,   24.],       [  84.,   -5.,   24.],       [  81.,   37.,   26.],       [  50.,   71.,   25.],       [  22.,   79.,   20.],       [ -17.,   88.,   29.],       [ -55.,   62.,   24.],       [ -78.,   42.,   22.],       [ -82.,    0.,   21.],       [ -79.,  -38.,   26.],       [ -54.,  -65.,   25.],       [ -21.,  -88.,   23.],       [  19.,  -83.,   25.],       [  48.,  -70.,   24.],       [  82.,  -34.,   27.],       [  89.,    0.,   24.],       [  82.,   41.,   28.],       [  56.,   69.,   29.],       [  15.,   93.,   25.],       [ -17.,   91.,   27.],       [ -57.,   73.,   22.],       [ -84.,   37.,   21.],       [ -92.,   -5.,   20.],       [ -86.,  -38.,   27.],       [ -62.,  -76.,   24.],       [ -24.,  -89.,   28.],       [  23.,  -89.,   27.],       [  61.,  -72.,   29.],       [  79.,  -38.,   27.],       [ 104.,   -2.,   24.],       [  91.,   47.,   26.],       [  60.,   79.,   21.],       [  24.,   98.,   27.],       [ -28.,  100.,   26.],       [ -60.,   79.,   29.],       [ -89.,   44.,   20.],       [-103.,    4.,   26.],       [ -92.,  -39.,   22.],       [ -66.,  -80.,   29.],       [ -24., -100.,   21.],       [  20.,  -98.,   27.],       [  58.,  -80.,   24.],       [  93.,  -43.,   23.],       [  52.,    1.,   31.],       [  47.,   20.,   31.],       [  31.,   34.,   32.],       [  11.,   48.,   32.],       [ -13.,   48.,   27.],       [ -29.,   34.,   34.],       [ -47.,   19.,   30.],       [ -49.,    4.,   25.],       [ -48.,  -23.,   25.],       [ -31.,  -37.,   26.],       [ -10.,  -44.,   25.],       [  10.,  -47.,   25.],       [  33.,  -35.,   29.],       [  41.,  -21.,   34.],       [  59.,    4.,   34.],       [  47.,   29.,   28.],       [  32.,   48.,   32.],       [   7.,   53.,   30.],       [ -16.,   58.,   31.],       [ -36.,   45.,   25.],       [ -54.,   24.,   34.],       [ -60.,   -2.,   32.],       [ -47.,  -29.,   32.],       [ -34.,  -48.,   34.],       [ -11.,  -57.,   32.],       [   9.,  -61.,   30.],       [  35.,  -49.,   29.],       [  49.,  -28.,   29.],       [  66.,   -4.,   30.],       [  60.,   27.,   30.],       [  41.,   46.,   34.],       [  16.,   64.,   33.],       [ -18.,   64.,   25.],       [ -43.,   54.,   33.],       [ -57.,   31.,   26.],       [ -63.,   -2.,   33.],       [ -58.,  -25.,   31.],       [ -45.,  -48.,   31.],       [ -16.,  -64.,   32.],       [  15.,  -64.,   28.],       [  43.,  -55.,   26.],       [  54.,  -29.,   34.],       [  73.,    1.,   25.],       [  60.,   32.,   28.],       [  45.,   52.,   28.],       [  14.,   68.,   26.],       [ -17.,   73.,   31.],       [ -45.,   51.,   27.],       [ -69.,   27.,   29.],       [ -71.,    3.,   27.],       [ -66.,  -34.,   26.],       [ -40.,  -53.,   33.],       [ -17.,  -66.,   34.],       [  20.,  -70.,   25.],       [  44.,  -56.,   30.],       [  63.,  -35.,   30.],       [  76.,   -3.,   30.],       [  73.,   30.,   26.],       [  51.,   65.,   34.],       [  21.,   75.,   28.],       [ -14.,   73.,   25.],       [ -52.,   65.,   25.],       [ -75.,   36.,   30.],       [ -82.,   -4.,   34.],       [ -72.,  -38.,   28.],       [ -47.,  -58.,   28.],       [ -14.,  -82.,   27.],       [  17.,  -79.,   30.],       [  44.,  -63.,   28.],       [  70.,  -31.,   33.],       [  86.,   -4.,   33.],       [  77.,   39.,   31.],       [  49.,   66.,   30.],       [  20.,   84.,   30.],       [ -20.,   86.,   29.],       [ -56.,   68.,   29.],       [ -77.,   42.,   34.],       [ -84.,    1.,   25.],       [ -80.,  -35.,   29.],       [ -55.,  -69.,   34.],       [ -16.,  -83.,   33.],       [  21.,  -87.,   26.],       [  50.,  -68.,   29.],       [  77.,  -38.,   25.],       [  97.,   -3.,   26.],       [  88.,   43.,   26.],       [  59.,   75.,   33.],       [  17.,   88.,   28.],       [ -18.,   90.,   29.],       [ -61.,   75.,   29.],       [ -87.,   38.,   25.],       [ -96.,    2.,   26.],       [ -84.,  -46.,   33.],       [ -55.,  -75.,   33.],       [ -21.,  -94.,   34.],       [  16.,  -92.,   26.],       [  60.,  -73.,   34.],       [  86.,  -42.,   29.],       [ 102.,   -1.,   30.],       [  90.,   38.,   34.],       [  62.,   77.,   29.],       [  23.,   97.,   32.],       [ -23.,  101.,   32.],       [ -58.,   82.,   27.],       [ -93.,   45.,   34.],       [-104.,    2.,   32.],       [ -90.,  -47.,   32.],       [ -66.,  -76.,   28.],       [ -25.,  -96.,   33.],       [  20.,  -99.,   28.],       [  57.,  -75.,   26.],       [  90.,  -41.,   34.],       [  51.,    0.,   36.],       [  42.,   21.,   30.],       [  26.,   42.,   38.],       [   6.,   47.,   35.],       [ -10.,   49.,   31.],       [ -35.,   38.,   31.],       [ -50.,   19.,   31.],       [ -54.,   -4.,   31.],       [ -47.,  -26.,   31.],       [ -29.,  -35.,   35.],       [ -13.,  -47.,   31.],       [  13.,  -46.,   33.],       [  34.,  -44.,   32.],       [  41.,  -23.,   36.],       [  58.,   -3.,   35.],       [  49.,   23.,   35.],       [  39.,   49.,   33.],       [   9.,   53.,   37.],       [ -13.,   54.,   38.],       [ -41.,   42.,   36.],       [ -52.,   28.,   39.],       [ -58.,   -2.,   38.],       [ -57.,  -22.,   37.],       [ -36.,  -48.,   33.],       [ -14.,  -55.,   32.],       [  16.,  -53.,   30.],       [  35.,  -50.,   30.],       [  49.,  -27.,   35.],       [  66.,   -2.,   35.],       [  62.,   23.,   36.],       [  35.,   52.,   31.],       [   9.,   64.,   38.],       [ -10.,   63.,   36.],       [ -37.,   50.,   39.],       [ -60.,   30.,   35.],       [ -61.,    3.,   30.],       [ -56.,  -27.,   36.],       [ -43.,  -55.,   31.],       [ -19.,  -63.,   31.],       [  12.,  -66.,   37.],       [  36.,  -55.,   38.],       [  58.,  -30.,   34.],       [  69.,   -2.,   33.],       [  66.,   30.,   37.],       [  48.,   60.,   31.],       [  11.,   65.,   31.],       [ -19.,   69.,   37.],       [ -40.,   55.,   39.],       [ -69.,   30.,   33.],       [ -68.,   -3.,   38.],       [ -61.,  -28.,   32.],       [ -44.,  -52.,   30.],       [ -14.,  -72.,   38.],       [  14.,  -68.,   34.],       [  39.,  -53.,   32.],       [  62.,  -32.,   39.],       [  79.,   -5.,   30.],       [  71.,   35.,   31.],       [  47.,   58.,   33.],       [  19.,   77.,   30.],       [ -15.,   79.,   34.],       [ -54.,   57.,   37.],       [ -71.,   36.,   35.],       [ -82.,    2.,   32.],       [ -71.,  -30.,   34.],       [ -54.,  -57.,   34.],       [ -15.,  -77.,   35.],       [  15.,  -75.,   34.],       [  47.,  -59.,   37.],       [  67.,  -34.,   38.],       [  87.,    2.,   34.],       [  79.,   37.,   32.],       [  55.,   71.,   34.],       [  19.,   86.,   35.],       [ -17.,   78.,   38.],       [ -54.,   64.,   30.],       [ -73.,   32.,   32.],       [ -86.,    4.,   37.],       [ -82.,  -42.,   34.],       [ -57.,  -64.,   37.],       [ -18.,  -87.,   36.],       [  18.,  -85.,   36.],       [  55.,  -70.,   31.],       [  75.,  -42.,   33.],       [  94.,   -3.,   31.],       [  88.,   40.,   31.],       [  53.,   70.,   39.],       [  17.,   86.,   32.],       [ -17.,   94.,   35.],       [ -58.,   76.,   38.],       [ -88.,   42.,   30.],       [ -92.,    0.,   34.],       [ -81.,  -41.,   33.],       [ -62.,  -73.,   33.],       [ -22.,  -87.,   39.],       [  20.,  -94.,   30.],       [  53.,  -68.,   31.],       [  84.,  -36.,   32.],       [  97.,   -1.,   34.],       [  93.,   42.,   32.],       [  64.,   76.,   36.],       [  19.,   94.,   36.],       [ -24.,  101.,   36.],       [ -60.,   80.,   36.],       [ -94.,   45.,   37.],       [ -96.,   -1.,   35.],       [ -92.,  -43.,   31.],       [ -65.,  -81.,   30.],       [ -27.,  -94.,   39.],       [  22., -100.,   31.],       [  60.,  -75.,   30.],       [  89.,  -43.,   38.],       [  47.,    3.,   35.],       [  46.,   18.,   37.],       [  33.,   36.,   40.],       [  11.,   46.,   42.],       [  -7.,   43.,   40.],       [ -29.,   40.,   36.],       [ -50.,   23.,   39.],       [ -55.,    4.,   37.],       [ -41.,  -26.,   36.],       [ -35.,  -42.,   36.],       [ -12.,  -50.,   41.],       [   6.,  -49.,   42.],       [  30.,  -36.,   38.],       [  46.,  -26.,   38.],       [  58.,   -1.,   43.],       [  50.,   29.,   42.],       [  33.,   42.,   40.],       [  14.,   58.,   37.],       [ -12.,   57.,   38.],       [ -36.,   47.,   35.],       [ -55.,   28.,   38.],       [ -55.,   -1.,   40.],       [ -55.,  -24.,   35.],       [ -33.,  -49.,   44.],       [ -10.,  -53.,   44.],       [   8.,  -53.,   40.],       [  30.,  -43.,   38.],       [  48.,  -30.,   42.],       [  61.,    1.,   41.],       [  59.,   28.,   43.],       [  35.,   53.,   44.],       [  15.,   60.,   39.],       [ -19.,   60.,   44.],       [ -43.,   49.,   36.],       [ -63.,   29.,   38.],       [ -61.,    4.,   43.],       [ -58.,  -33.,   44.],       [ -42.,  -52.,   44.],       [ -16.,  -66.,   37.],       [  11.,  -62.,   44.],       [  37.,  -46.,   36.],       [  59.,  -24.,   35.],       [  76.,    1.,   44.],       [  63.,   30.,   43.],       [  45.,   56.,   35.],       [  16.,   66.,   38.],       [ -15.,   69.,   38.],       [ -43.,   54.,   42.],       [ -67.,   27.,   42.],       [ -71.,   -5.,   36.],       [ -62.,  -28.,   37.],       [ -40.,  -52.,   38.],       [ -17.,  -66.,   42.],       [  12.,  -68.,   36.],       [  47.,  -58.,   37.],       [  60.,  -29.,   40.],       [  78.,    2.,   40.],       [  66.,   37.,   40.],       [  45.,   65.,   38.],       [  14.,   78.,   38.],       [ -13.,   79.,   40.],       [ -52.,   66.,   44.],       [ -69.,   33.,   37.],       [ -77.,   -3.,   39.],       [ -69.,  -37.,   38.],       [ -54.,  -62.,   44.],       [ -21.,  -82.,   36.],       [  16.,  -75.,   44.],       [  44.,  -65.,   44.],       [  68.,  -34.,   35.],       [  81.,    3.,   35.],       [  74.,   33.,   36.],       [  56.,   63.,   36.],       [  19.,   79.,   35.],       [ -19.,   81.,   37.],       [ -53.,   62.,   43.],       [ -80.,   35.,   43.],       [ -83.,   -3.,   35.],       [ -78.,  -38.,   36.],       [ -50.,  -64.,   35.],       [ -18.,  -88.,   43.],       [  14.,  -88.,   39.],       [  54.,  -72.,   39.],       [  81.,  -40.,   35.],       [  91.,   -5.,   42.],       [  83.,   39.,   44.],       [  62.,   70.,   37.],       [  16.,   92.,   40.],       [ -26.,   93.,   41.],       [ -57.,   68.,   36.],       [ -79.,   42.,   37.],       [ -98.,    4.,   37.],       [ -87.,  -42.,   38.],       [ -61.,  -70.,   35.],       [ -21.,  -91.,   41.],       [  20.,  -90.,   42.],       [  58.,  -69.,   40.],       [  79.,  -38.,   39.],       [  98.,   -5.,   35.],       [  90.,   42.,   35.],       [  63.,   77.,   35.],       [  21.,   93.,   38.],       [ -20.,   93.,   40.],       [ -66.,   80.,   39.],       [ -93.,   44.,   37.],       [ -99.,    0.,   38.],       [ -90.,  -45.,   35.],       [ -60.,  -78.,   41.],       [ -24., -100.,   42.],       [  22., -102.,   35.],       [  66.,  -77.,   37.],       [  86.,  -39.,   44.]])
        basis0 = BSplineBasis(5, np.array([ -3.3,  -1.8,  -1.2,   0. ,   0. ,   0.6,   1.8,   3.1,   4.2,   5.1,   5.9,   7.2,   7.7,
         8.8,   9.7,  11.2,  11.8,  13. ,  13. ,  13.6,  14.8,  16.1]),2)
        basis1 = BSplineBasis(5, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  1. ,  2.1,  2.8,  4. ,  4. ,  4. ,  4. ,  4. ]))
        basis2 = BSplineBasis(6, np.array([ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ,  1.2,  1.8,  3.3,  4. ,  4. ,  4. ,  4. ,  4. ,  4. ]))
        vol  = Volume(basis0, basis1, basis2, controlpoints,False)
        vol2 = vol.clone()
        vol2.raise_order(1, 1, 2)
        self.assertEqual(vol2.order(direction=0), 6)
        self.assertEqual(vol2.order(direction=1), 6)
        self.assertEqual(vol2.order(direction=2), 8)

        u    = np.linspace(vol.start(0), vol.end(0), 7)
        v    = np.linspace(vol.start(1), vol.end(1), 7)
        w    = np.linspace(vol.start(2), vol.end(2), 7)
        pt   = vol(u,v,w)
        pt2  = vol2(u,v,w)
        self.assertAlmostEqual(np.linalg.norm(pt-pt2), 0.0)


if __name__ == '__main__':
    unittest.main()
